@@ -130,11 +130,12 @@ SYS_LIBRARIES = -ll -lm         /* these are the default libraries */
 
 /* ------------------- end of user configuration ------------------------- */
 
-PATCHLEVEL = BETA
-RELEASE = 1.6.$(PATCHLEVEL)
+PATCHLEVEL = 1.ALPHA
+RELEASE = 1.7.$(PATCHLEVEL)
 
 COMPRESS = gzip
 SUFFIX   = gz
+CDEBUGFLAGS = -Wall
 
 DEFINES = -DRELEASE=\"$(RELEASE)\" \
  -DPCBLIBDIR=\"$(PCBLIBDIR)\" \
