@@ -44,9 +44,12 @@
 /* ---------------------------------------------------------------------------
  * some default values
  */
-#define	DEFAULT_SIZE		"7000x5000"	/* default layout size (EURO) */
+#define	DEFAULT_SIZE		"7000x5000"	/* default layout size */
 #define	DEFAULT_MEDIASIZE	"a4"		/* default output media */
 #define	DEFAULT_CELLSIZE	50		/* default cell size for symbols */
+#define CLICK_TIME		200		/* default time for click expiration */
+#define SCROLL_TIME 25				/* time between scrolls when drawing beyond border */
+#define COLUMNS			8		/* number of columns for found pin report */
 
 /* ---------------------------------------------------------------------------
  * frame between the groundplane and the copper
@@ -58,24 +61,26 @@
  */
 #define	MAX_LAYER		8	/* max number of layer, check source */
 					/* code for more changes, a *lot* more changes */
+#define NUM_STYLES		4
 #define	MIN_LINESIZE		1	/* thickness of lines in 1/1000'' */
 #define	MAX_LINESIZE		250
 #define	MIN_TEXTSCALE		20	/* scaling of text objects in percent */
 #define	MAX_TEXTSCALE		1000
 #define	MIN_PINORVIASIZE	30	/* size of a pin or via in mils */
 #define	MIN_PINORVIAHOLE	10	/* size of a pins or vias drilling hole */
-#define	MAX_PINORVIASIZE	250
+#define	MAX_PINORVIASIZE	400
 #define	MIN_PINORVIACOPPER	20	/* min difference outer-inner diameter */
 #define	MIN_PADSIZE		10	/* size of a pad */
 #define	MAX_PADSIZE		250
 #define	MIN_GRID		1	/* grid in 1/1000'' */
 #define	MAX_GRID		200
+#define	CHUNK			1	/* step size for size sliders */
 #define	MIN_ZOOM		0	/* min zoom (shift operations) */
-#define	MAX_ZOOM		4	/* max zoom (shift operations) */
+#define	MAX_ZOOM		6	/* max zoom (shift operations) */
 #define	MAX_FONTPOSITION	127	/* upper limit of characters in my font */
 
 #define	MAX_COORD		20000	/* coordinate limits */
-#define	MIN_SIZE		1000	/* lowest width and height */
+#define	MIN_SIZE		1200	/* lowest width and height */
 #define	MAX_BUFFER		5	/* number of pastebuffers */
 					/* additional changes in menu.c are */
 					/* also required to select more buffers */
@@ -103,10 +108,14 @@
 						/* polygon points */
 #define	MAX_ELEMENTNAMES		3	/* number of supported names of */
 						/* an element */
-#define	MAX_LIBRARY_LINE_LENGTH		256	/* maximum line length in the */
+#define	MAX_LIBRARY_LINE_LENGTH		255	/* maximum line length in the */
 						/* library-description file */
+#define MAX_NETLIST_LINE_LENGTH		255	/* maximum line length for netlist files */
 #define	MAX_MODESTACK_DEPTH		16	/* maximum depth of mode stack */
 #define	MAX_CROSSHAIRSTACK_DEPTH	16	/* maximum depth of state stack */
 #define	MIN_GRID_DISTANCE		4	/* minimum distance between point */
 						/* to enable grid drawing */
+	/* size of diamond element mark */
+#define EMARK_SIZE	10
+#define GBX_MAXAPERTURECOUNT	256
 #endif
