@@ -34,6 +34,9 @@
 #include "global.h"
 
 void	ActionMovePointer(Widget, XEvent *, String *, Cardinal *);
+void	ActionAtomic(Widget, XEvent *, String *, Cardinal *);
+void	ActionAdjustStyle(Widget, XEvent *, String *, Cardinal *);
+void	ActionRouteStyle(Widget, XEvent *, String *, Cardinal *);
 void	EventMoveCrosshair(XMotionEvent *);
 void	ActionMarkCrosshair(Widget, XEvent *, String *, Cardinal *);
 void	ActionToggleHideName(Widget, XEvent *, String *, Cardinal *);
@@ -51,7 +54,9 @@ void	ActionAddRats(Widget, XEvent *, String *, Cardinal *);
 void	ActionReport(Widget, XEvent *, String *, Cardinal *);
 void	ActionChangeSize(Widget, XEvent *, String *, Cardinal *);
 void	ActionChange2ndSize(Widget, XEvent *, String *, Cardinal *);
+void	ActionChangeClearSize(Widget, XEvent *, String *, Cardinal *);
 void	ActionChangeName(Widget, XEvent *, String *, Cardinal *);
+void	ActionChangeJoin(Widget, XEvent *, String *, Cardinal *);
 void	ActionChangeSquare(Widget, XEvent *, String *, Cardinal *);
 void	ActionChangeOctagon(Widget, XEvent *, String *, Cardinal *);
 void	ActionChangeHole(Widget, XEvent *, String *, Cardinal *);
@@ -70,5 +75,12 @@ void	ActionPolygon(Widget, XEvent *, String *, Cardinal *);
 void	ActionSwitchDrawingLayer(Widget, XEvent *, String *, Cardinal *);
 void	ActionEditLayerGroups(Widget, XEvent *, String *, Cardinal *);
 void	ActionMoveToCurrentLayer(Widget, XEvent *, String *, Cardinal *);
+void	ActionDRCheck(Widget, XEvent *, String *, Cardinal *);
+void	ActionFlip(Widget, XEvent *, String *, Cardinal *);
+void	ActionListAct(Widget, XEvent *, String *, Cardinal *);
+void	ActionToggleThermal(Widget, XEvent *, String *, Cardinal *);
+void	ActionButton3(Widget, XEvent *, String *, Cardinal *);
+void	ActionSetSame(Widget, XEvent *, String *, Cardinal *);
+void	AdjustAttachedObjects(void);
 
 #endif
