@@ -56,12 +56,14 @@ static	char	*rcsid = "$Id: buffer.c,v 1.2 1998/02/21 10:27:14 cad Exp $";
 static	void	*AddViaToBuffer(PinTypePtr);
 static	void	*AddLineToBuffer(LayerTypePtr, LineTypePtr);
 static	void	*AddArcToBuffer(LayerTypePtr, ArcTypePtr);
+static	void	*AddRatToBuffer(RatTypePtr);
 static	void	*AddTextToBuffer(LayerTypePtr, TextTypePtr);
 static	void	*AddPolygonToBuffer(LayerTypePtr, PolygonTypePtr);
 static	void	*AddElementToBuffer(ElementTypePtr);
 static	void	*MoveViaToBuffer(PinTypePtr);
 static	void	*MoveLineToBuffer(LayerTypePtr, LineTypePtr);
 static	void	*MoveArcToBuffer(LayerTypePtr, ArcTypePtr);
+static	void	*MoveRatToBuffer(RatTypePtr);
 static	void	*MoveTextToBuffer(LayerTypePtr, TextTypePtr);
 static	void	*MovePolygonToBuffer(LayerTypePtr, PolygonTypePtr);
 static	void	*MoveElementToBuffer(ElementTypePtr);
@@ -83,7 +85,8 @@ static	ObjectFunctionType	AddBufferFunctions = {
 			NULL,
 			NULL,
 			NULL,
-			AddArcToBuffer },
+			AddArcToBuffer,
+			AddRatToBuffer },
 							MoveBufferFunctions = {
 			MoveLineToBuffer,
 			MoveTextToBuffer,
@@ -95,15 +98,28 @@ static	ObjectFunctionType	AddBufferFunctions = {
 			NULL,
 			NULL,
 			NULL,
-			MoveArcToBuffer };
+			MoveArcToBuffer,
+			MoveRatToBuffer };
+
+static int ExtraFlag = 0;
 
 /* ---------------------------------------------------------------------------
  * copies a via to paste buffer
  */
 static void *AddViaToBuffer(PinTypePtr Via)
 {
-	return(CreateNewVia(Dest, Via->X, Via->Y, Via->Thickness,
-		Via->DrillingHole, Via->Name, Via->Flags));
+	return(CreateNewVia(Dest, Via->X, Via->Y, Via->Thickness, Via->Clearance,
+		Via->DrillingHole, Via->Name, Via->Flags & ~(FOUNDFLAG | ExtraFlag)));
+}
+
+/* ---------------------------------------------------------------------------
+ * copies a rat-line to paste buffer
+ */
+static void *AddRatToBuffer(RatTypePtr Rat)
+{
+	return(CreateNewRat(Dest, Rat->Point1.X, Rat->Point1.Y,
+		Rat->Point2.X, Rat->Point2.Y, Rat->group1, Rat->group2,
+		Rat->Thickness, Rat->Flags & ~(FOUNDFLAG | ExtraFlag)));
 }
 
 /* ---------------------------------------------------------------------------
@@ -114,7 +130,8 @@ static void *AddLineToBuffer(LayerTypePtr Layer, LineTypePtr Line)
 	LayerTypePtr	layer = &Dest->Layer[GetLayerNumber(Source, Layer)];
 	
 	return(CreateNewLineOnLayer(layer, Line->Point1.X, Line->Point1.Y,
-		Line->Point2.X, Line->Point2.Y, Line->Thickness, Line->Flags));
+		Line->Point2.X, Line->Point2.Y, Line->Thickness, Line->Clearance,
+		Line->Flags & ~(FOUNDFLAG | ExtraFlag)));
 }
 
 /* ---------------------------------------------------------------------------
@@ -126,7 +143,8 @@ static void *AddArcToBuffer(LayerTypePtr Layer, ArcTypePtr Arc)
 	
 	return(CreateNewArcOnLayer(layer, Arc->X, Arc->Y,
 		Arc->Width, Arc->StartAngle, Arc->Delta,
-		Arc->Thickness, Arc->Flags));
+		Arc->Thickness, Arc->Clearance,
+		Arc->Flags & ~(FOUNDFLAG | ExtraFlag)));
 }
 
 /* ---------------------------------------------------------------------------
@@ -137,7 +155,7 @@ static void *AddTextToBuffer(LayerTypePtr Layer, TextTypePtr Text)
 	LayerTypePtr	layer = &Dest->Layer[GetLayerNumber(Source, Layer)];
 	
 	return(CreateNewText(layer, &PCB->Font, Text->X, Text->Y,
-		Text->Direction, Text->Scale, Text->TextString, Text->Flags));
+		Text->Direction, Text->Scale, Text->TextString, Text->Flags & ~ExtraFlag));
 }
 
 /* ---------------------------------------------------------------------------
@@ -150,6 +168,7 @@ static void *AddPolygonToBuffer(LayerTypePtr Layer, PolygonTypePtr Polygon)
 	
 	polygon = GetPolygonMemory(layer);
 	CopyPolygonLowLevel(polygon, Polygon);
+	polygon->Flags = polygon->Flags & ~(FOUNDFLAG | ExtraFlag);
 	return(polygon);
 }
 
@@ -161,7 +180,20 @@ static void *AddElementToBuffer(ElementTypePtr Element)
 	ElementTypePtr	element;
 	
 	element = GetElementMemory(Dest);
-	CopyElementLowLevel(element, Element);
+	CopyElementLowLevel(Dest, element, Element, True);
+	element->Flags &= ~ExtraFlag;
+	if (ExtraFlag)
+	{
+		ELEMENTTEXT_LOOP(element,
+			text->Flags &= ~ExtraFlag;
+		);
+		PIN_LOOP(element,
+			pin->Flags &= ~ExtraFlag;
+		);
+		PAD_LOOP(element,
+			pad->Flags &= ~ExtraFlag;
+		);
+	}
 	return(element);
 }
 
@@ -175,10 +207,26 @@ static void *MoveViaToBuffer(PinTypePtr Via)
 	via = GetViaMemory(Dest);
 	*via = *Via;
 	*Via = Source->Via[--Source->ViaN];
+	via->Flags &= ~(WARNFLAG | FOUNDFLAG);
 	memset(&Source->Via[Source->ViaN], 0, sizeof(PinType));
 	return(via);
 }
 
+/* ---------------------------------------------------------------------------
+ * moves a rat-line to paste buffer
+ */
+static void *MoveRatToBuffer(RatTypePtr Rat)
+{
+	RatTypePtr	rat;
+
+	rat = GetRatMemory(Dest);
+	*rat = *Rat;
+	*Rat = Source->Rat[--Source->RatN];
+	Rat->Flags &= ~FOUNDFLAG;
+	memset(&Source->Rat[Source->RatN], 0, sizeof(RatType));
+	return(rat);
+}
+
 /* ---------------------------------------------------------------------------
  * moves a line to buffer  
  */
@@ -188,6 +236,7 @@ static void *MoveLineToBuffer(LayerTypePtr Layer, LineTypePtr Line)
 
 	line = GetLineMemory(&Dest->Layer[GetLayerNumber(Source, Layer)]);
 	*line = *Line;
+	line->Flags &= ~FOUNDFLAG;
 	*Line = Layer->Line[--Layer->LineN];
 	memset(&Layer->Line[Layer->LineN], 0, sizeof(LineType));
 	return(line);
@@ -201,7 +250,8 @@ static void *MoveArcToBuffer(LayerTypePtr Layer, ArcTypePtr Arc)
 	ArcTypePtr	arc;
 
 	arc = GetArcMemory(&Dest->Layer[GetLayerNumber(Source, Layer)]);
-	*arc = *Arc;
+	*arc = *Arc; 
+	arc->Flags &= ~FOUNDFLAG;
 	*Arc = Layer->Arc[--Layer->ArcN];
 	memset(&Layer->Arc[Layer->ArcN], 0, sizeof(ArcType));
 	return(arc);
@@ -227,11 +277,10 @@ static void *MoveTextToBuffer(LayerTypePtr Layer, TextTypePtr Text)
 static void *MovePolygonToBuffer(LayerTypePtr Layer, PolygonTypePtr Polygon)
 {
 	PolygonTypePtr	polygon;
-	Cardinal	index;
-	PointTypePtr	point;
 	
 	polygon = GetPolygonMemory(&Dest->Layer[GetLayerNumber(Source, Layer)]);
 	*polygon = *Polygon;
+	polygon->Flags &= ~FOUNDFLAG;
 	*Polygon = Layer->Polygon[--Layer->PolygonN];
 	memset(&Layer->Polygon[Layer->PolygonN], 0, sizeof(PolygonType));
 	return(polygon);
@@ -246,7 +295,12 @@ static void *MoveElementToBuffer(ElementTypePtr Element)
 
 	element = GetElementMemory(Dest);
 	*element = *Element;
-	FreeNetlist();
+	PIN_LOOP(element,
+		pin->Flags &= ~(WARNFLAG | FOUNDFLAG);
+	);
+	PAD_LOOP(element,
+		pad->Flags &= ~(WARNFLAG | FOUNDFLAG);
+	);
 	*Element = Source->Element[--Source->ElementN];
 	memset(&Source->Element[Source->ElementN], 0, sizeof(ElementType));
 	return(element);
@@ -276,20 +330,31 @@ void ClearBuffer(BufferTypePtr Buffer)
  * copies all selected and visible objects to the paste buffer
  * returns True if any objects have been removed
  */
-void AddSelectedToBuffer(BufferTypePtr Buffer)
+void AddSelectedToBuffer(BufferTypePtr Buffer, Position X, Position Y, Boolean LeaveSelected)
 {
 		/* switch crosshair off because adding objects to the pastebuffer
 		 * may change the 'valid' area for the cursor
 		 */
+	if (!LeaveSelected)
+		ExtraFlag = SELECTEDFLAG;
 	HideCrosshair(True);
 	Source = PCB->Data;
 	Dest = Buffer->Data;
 	SelectedOperation(&AddBufferFunctions, False);
 
-		/* set origin to current position */
-	Buffer->X = Crosshair.X;
-	Buffer->Y = Crosshair.Y;
+		/* set origin to passed or current position */
+	if (X || Y)
+	{
+		Buffer->X = X;
+		Buffer->Y = Y;
+	}
+	else
+	{
+		Buffer->X = Crosshair.X;
+		Buffer->Y = Crosshair.Y;
+	}
 	RestoreCrosshair(True);
+	ExtraFlag = 0;
 }
 
 /* ---------------------------------------------------------------------------
@@ -316,7 +381,7 @@ Boolean LoadElementToBuffer(BufferTypePtr Buffer,
 
 			/* always add elements using top-side coordinates */
 		if (Settings.ShowSolderSide)
-			MirrorElementCoordinates(element);
+			MirrorElementCoordinates(element, 0);
 
 			/* set buffer offset to 'mark' position */
 		PASTEBUFFER->X = element->MarkX;
@@ -337,39 +402,29 @@ Boolean LoadElementToBuffer(BufferTypePtr Buffer,
 Boolean	ConvertBufferToElement(BufferTypePtr Buffer)
 {
 	ElementTypePtr	Element;
-	Cardinal	group, i, savedmax;
+	Cardinal	group, i;
+	Cardinal	pin_n = 1;
 	Boolean		hasParts = False;
 
-	ELEMENT_LOOP(Buffer->Data, FreeElementMemory(element));
-	Buffer->Data->ElementN = 0;
-	Element = CreateNewElement(Buffer->Data, NULL, &PCB->Font, NOFLAG,
-			NULL, NULL, NULL, PASTEBUFFER->X, PASTEBUFFER->Y, 0, 100, NOFLAG);
+	Element = CreateNewElement(PCB->Data, NULL, &PCB->Font, NOFLAG,
+			NULL, NULL, NULL, PASTEBUFFER->X, PASTEBUFFER->Y, 0, 100, NOFLAG, False);
 	if (!Element)
 		return(False);
-	group = GetLayerGroupNumberByNumber(MAX_LAYER + SOLDER_LAYER);
-	for (i = 0; i < PCB->LayerGroups.Number[group]; i++)
-	{
-		LayerTypePtr	silklayer;
-		Cardinal	number = PCB->LayerGroups.Entries[group][i];
-	
-		if (number >= MAX_LAYER)
-			continue;
-		silklayer = &Buffer->Data->Layer[number];
-		LINE_LOOP(silklayer,
-			CreateNewLineInElement(Element, line->Point1.X,
-				line->Point1.Y, line->Point2.X, line->Point2.Y, line->Thickness);
-			hasParts = True;
-		);
-		ARC_LOOP(silklayer,
-			CreateNewArcInElement(Element, arc->X, arc->Y, arc->Width,
-				arc->Height, arc->StartAngle, arc->Delta, arc->Thickness);
-			hasParts = True;
-		);
-	}
+	LINE_LOOP(&Buffer->Data->Layer[MAX_LAYER + COMPONENT_LAYER],
+		CreateNewLineInElement(Element, line->Point1.X,
+			line->Point1.Y, line->Point2.X, line->Point2.Y, line->Thickness);
+		hasParts = True;
+	);
+	ARC_LOOP(&Buffer->Data->Layer[MAX_LAYER + COMPONENT_LAYER],
+		CreateNewArcInElement(Element, arc->X, arc->Y, arc->Width,
+			arc->Height, arc->StartAngle, arc->Delta, arc->Thickness);
+		hasParts = True;
+	);
 	group = GetLayerGroupNumberByNumber(MAX_LAYER + COMPONENT_LAYER);
 	for (i = 0; i < PCB->LayerGroups.Number[group]; i++)
 	{
 		LayerTypePtr	padlayer;
+		char		num[8];
 		Cardinal	number = PCB->LayerGroups.Entries[group][i];
 	
 		if (number >= MAX_LAYER)
@@ -378,44 +433,51 @@ Boolean	ConvertBufferToElement(BufferTypePtr Buffer)
 		LINE_LOOP(padlayer,
 			if (line->Point1.X == line->Point2.X || line->Point1.Y == line->Point2.Y)
 			{
+				sprintf(num, "%d", pin_n++);
 				CreateNewPad(Element, line->Point1.X,
 				line->Point1.Y, line->Point2.X, line->Point2.Y, line->Thickness,
-				NULL, NOFLAG);
+				line->Clearance, line->Thickness + line->Clearance, NULL, num,
+				NOFLAG);
 				hasParts = True;
 			}
 		);
 	}
 	VIA_LOOP(Buffer->Data,
-		CreateNewPin(Element, via->X, via->Y, via->Thickness,
-			via->DrillingHole, via->Name,
-			via->Flags & (~(VIAFLAG | FOUNDFLAG | SELECTEDFLAG)) | PINFLAG);
-		hasParts = True;
+		{
+			char		num[8];
+
+			if (via->Name)
+				CreateNewPin(Element, via->X, via->Y, via->Thickness,
+					via->Clearance, via->Mask, via->DrillingHole,
+					NULL, via->Name, (via->Flags &
+					~(VIAFLAG | FOUNDFLAG | SELECTEDFLAG
+						| WARNFLAG)) | PINFLAG);
+			else
+			{	
+				sprintf(num, "%d", pin_n++);
+				CreateNewPin(Element, via->X, via->Y, via->Thickness,
+					via->Clearance, via->Mask, via->DrillingHole,
+					NULL, num, (via->Flags &
+					~(VIAFLAG | FOUNDFLAG | SELECTEDFLAG
+						| WARNFLAG)) | PINFLAG);
+			}
+			hasParts = True;
+		}
 	);
+	if (!hasParts)
+	{
+		DestroyObject(PCB->Data, ELEMENT_TYPE, Element, Element, Element);
+		return(False);
+	}
 	Element->MarkX = Buffer->X;
 	Element->MarkY = Buffer->Y;
 	SetElementBoundingBox(Element, &PCB->Font);
-		/* hide the element so it isn't released */
-	if (hasParts)
-	{
-		Buffer->Data->ElementN = 0;
-		savedmax = Buffer->Data->ElementMax;
-	}
 	ClearBuffer(Buffer);
-		/* now put it back */
-	if (hasParts)
-	{
-		Buffer->Data->ElementN = 1;
-		Buffer->Data->ElementMax = savedmax;
-		Buffer->Data->Element = Element;
-		SetBufferBoundingBox(Buffer);
-		return(True);
-	}
-	else
-		return(False);
+	MoveObjectToBuffer(Buffer->Data, PCB->Data, ELEMENT_TYPE, Element, Element, Element);
+	SetBufferBoundingBox(Buffer);
+	return(True);
 }
 	
-	
-
 /* ---------------------------------------------------------------------------
  * load PCB into buffer
  * parse the file with enabled 'PCB mode' (see parser)
@@ -485,11 +547,103 @@ void InitBuffers(void)
 {
 	int	i;
 
+	for (i = 0; i < MAX_BUFFER; i++)
+		Buffers[i].Data = CreateNewBuffer();
+}
+
+/* ---------------------------------------------------------------------------
+ * flip components/tracks from one side to the other
+ */
+void SwapBuffers(void)
+{
+	int		i, j, k;
+	Cardinal	sgroup, cgroup;
+	LayerType	swap;
+
 	for (i = 0; i < MAX_BUFFER; i++)
 	{
-		Buffers[i][0].Data = CreateNewBuffer();
-		Buffers[i][1].Data = CreateNewBuffer();
+		ELEMENT_LOOP(Buffers[i].Data,
+			MirrorElementCoordinates(element, 0)
+		);
+			/* set buffer offset to 'mark' position */
+		Buffers[i].X = SWAP_X(Buffers[i].X);
+		Buffers[i].Y = SWAP_Y(Buffers[i].Y);
+		VIA_LOOP(Buffers[i].Data,
+			via->X = SWAP_X(via->X);
+			via->Y = SWAP_Y(via->Y);
+		);
+		ALLLINE_LOOP(Buffers[i].Data,
+			line->Point1.X = SWAP_X(line->Point1.X);
+			line->Point1.Y = SWAP_Y(line->Point1.Y);
+			line->Point2.X = SWAP_X(line->Point2.X);
+			line->Point2.Y = SWAP_Y(line->Point2.Y);
+		);
+		ALLARC_LOOP(Buffers[i].Data,
+			arc->X = SWAP_X(arc->X);
+			arc->Y = SWAP_Y(arc->Y);
+			arc->StartAngle = SWAP_ANGLE(arc->StartAngle);
+			arc->Delta = SWAP_DELTA(arc->Delta);
+		);
+		ALLPOLYGON_LOOP(Buffers[i].Data,
+			POLYGONPOINT_LOOP(polygon, 
+				point->X = SWAP_X(point->X);
+				point->Y = SWAP_Y(point->Y);
+			);
+			SetPolygonBoundingBox(polygon);
+		);
+		ALLTEXT_LOOP(Buffers[i].Data,
+			text->X = SWAP_X(text->X);
+			text->Y = SWAP_Y(text->Y);
+			TOGGLE_FLAG(ONSOLDERFLAG, text);
+			SetTextBoundingBox(&PCB->Font, text);
+		);
+			/* swap silkscreen layers */
+		swap = Buffers[i].Data->Layer[MAX_LAYER + SOLDER_LAYER];
+		Buffers[i].Data->Layer[MAX_LAYER + SOLDER_LAYER] =
+			Buffers[i].Data->Layer[MAX_LAYER + COMPONENT_LAYER];
+		Buffers[i].Data->Layer[MAX_LAYER + COMPONENT_LAYER] = swap;
+		
+			/* swap layer groups when balanced */
+		sgroup = GetLayerGroupNumberByNumber(MAX_LAYER + SOLDER_LAYER);
+		cgroup = GetLayerGroupNumberByNumber(MAX_LAYER + COMPONENT_LAYER);
+		if (PCB->LayerGroups.Number[cgroup] == PCB->LayerGroups.Number[sgroup])
+		{
+			for (j = k = 0; j < PCB->LayerGroups.Number[sgroup]; j++)
+			{
+				int		t1, t2, f1, f2;
+				Cardinal	cnumber = PCB->LayerGroups.Entries[cgroup][k];
+				Cardinal	snumber = PCB->LayerGroups.Entries[sgroup][j];
+	
+				if (snumber >= MAX_LAYER)
+					continue;
+				swap = Buffers[i].Data->Layer[snumber];
+				
+				while (cnumber >= MAX_LAYER)
+				{
+					k++;
+					cnumber = PCB->LayerGroups.Entries[cgroup][k];
+				}
+				Buffers[i].Data->Layer[snumber] = Buffers[i].Data->Layer[cnumber];
+				Buffers[i].Data->Layer[cnumber] = swap;
+				k++;
+					/* move the thermal flags with the layers */
+				t1 = L0THERMFLAG << snumber;
+				t2 = L0THERMFLAG << cnumber;
+				ALLPIN_LOOP(Buffers[i].Data,
+					f1 = (TEST_FLAG(t1, pin)) ? t2 : 0;
+					f2 = (TEST_FLAG(t2, pin)) ? t1 : 0;
+					pin->Flags = (pin->Flags & ~t1 & ~t2) | f1 | f2;
+				);
+				VIA_LOOP(Buffers[i].Data,
+					f1 = (TEST_FLAG(t1, via)) ? t2 : 0;
+					f2 = (TEST_FLAG(t2, via)) ? t1 : 0;
+					via->Flags = (via->Flags & ~t1 & ~t2) | f1 | f2;
+				);
+			}
+		}
+		SetBufferBoundingBox(&Buffers[i]);
 	}
+	SetCrosshairRangeToBuffer();
 }
 
 /* ----------------------------------------------------------------------
