@@ -38,12 +38,13 @@
  */
 void	SetBufferBoundingBox(BufferTypePtr);
 void	ClearBuffer(BufferTypePtr);
-void	AddSelectedToBuffer(BufferTypePtr);
+void	AddSelectedToBuffer(BufferTypePtr, Position, Position, Boolean);
 Boolean	LoadElementToBuffer(BufferTypePtr, char *, Boolean);
 Boolean	ConvertBufferToElement(BufferTypePtr);
 Boolean	LoadLayoutToBuffer(BufferTypePtr, char *);
 void	RotateBuffer(BufferTypePtr, BYTE);
 void	SelectPasteBuffer(int);
+void	SwapBuffers(void);
 void	InitBuffers(void);
 void	*MoveObjectToBuffer(DataTypePtr, DataTypePtr, int,
 			void *, void *, void *);
