@@ -36,11 +36,15 @@ static	char	*rcsid = "$Id: change.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 
 #include "change.h"
 #include "control.h"
+#include "crosshair.h"
 #include "data.h"
 #include "dialog.h"
 #include "draw.h"
+#include "error.h"
 #include "mymem.h"
 #include "misc.h"
+#include "mirror.h"
+#include "output.h"
 #include "polygon.h"
 #include "rats.h"
 #include "search.h"
@@ -52,12 +56,17 @@ static	char	*rcsid = "$Id: change.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
  * some local prototypes
  */
 static	void	*ChangePinSize(ElementTypePtr, PinTypePtr);
+static	void	*ChangePinClearSize(ElementTypePtr, PinTypePtr);
 static	void	*ChangePadSize(ElementTypePtr, PadTypePtr);
+static	void	*ChangePadClearSize(ElementTypePtr, PadTypePtr);
 static	void	*ChangePin2ndSize(ElementTypePtr, PinTypePtr);
 static	void	*ChangeViaSize(PinTypePtr);
 static	void	*ChangeVia2ndSize(PinTypePtr);
+static	void	*ChangeViaClearSize(PinTypePtr);
 static	void	*ChangeLineSize(LayerTypePtr, LineTypePtr);
+static	void	*ChangeLineClearSize(LayerTypePtr, LineTypePtr);
 static	void	*ChangeArcSize(LayerTypePtr, ArcTypePtr);
+static	void	*ChangeArcClearSize(LayerTypePtr, ArcTypePtr);
 static	void	*ChangeTextSize(LayerTypePtr, TextTypePtr);
 static	void	*ChangeElementSize(ElementTypePtr);
 static	void	*ChangeElementNameSize(ElementTypePtr);
@@ -72,9 +81,12 @@ static	void	*ChangePinSquare(ElementTypePtr, PinTypePtr);
 static	void	*ChangePinOctagon(ElementTypePtr, PinTypePtr);
 static	void	*ChangeViaOctagon(PinTypePtr);
 static  void    *ChangePadSquare(ElementTypePtr, PadTypePtr);
+static	void	*ChangeViaThermal(PinTypePtr);
+static	void	*ChangePinThermal(ElementTypePtr, PinTypePtr);
+static  void    *ChangeLineJoin(LayerTypePtr, LineTypePtr);
+static  void    *ChangeArcJoin(LayerTypePtr, ArcTypePtr);
 static  void    *ChangePolyClear(LayerTypePtr, PolygonTypePtr);
 
-
 /* ---------------------------------------------------------------------------
  * some local identifiers
  */
@@ -92,7 +104,8 @@ static	ObjectFunctionType	ChangeSizeFunctions = {
 	ChangePadSize,
 	NULL,
 	NULL,
-	ChangeArcSize };
+	ChangeArcSize,
+	NULL };
 static	ObjectFunctionType	Change2ndSizeFunctions = {
 	NULL,
 	NULL,
@@ -104,6 +117,33 @@ static	ObjectFunctionType	Change2ndSizeFunctions = {
 	NULL,
 	NULL,
 	NULL,
+	NULL,
+	NULL };
+static	ObjectFunctionType	ChangeThermalFunctions = {
+	NULL,
+	NULL,
+	NULL,
+	ChangeViaThermal,
+	NULL,
+	NULL,
+	ChangePinThermal,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL };
+static	ObjectFunctionType	ChangeClearSizeFunctions = {
+	ChangeLineClearSize,
+	NULL,
+	NULL,
+	ChangeViaClearSize,
+	NULL,
+	NULL,
+	ChangePinClearSize,
+	ChangePadClearSize,
+	NULL,
+	NULL,
+	ChangeArcClearSize,
 	NULL };
 static	ObjectFunctionType	ChangeNameFunctions = {
 	NULL,
@@ -116,6 +156,7 @@ static	ObjectFunctionType	ChangeNameFunctions = {
 	ChangePadName,
 	NULL,
 	NULL,
+	NULL,
 	NULL };
 static	ObjectFunctionType	ChangeSquareFunctions = {
 	NULL,
@@ -128,8 +169,21 @@ static	ObjectFunctionType	ChangeSquareFunctions = {
 	ChangePadSquare,
 	NULL,
 	NULL,
+	NULL,
+	NULL };
+static	ObjectFunctionType	ChangeJoinFunctions = {
+	ChangeLineJoin,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	ChangeArcJoin,
 	NULL };
-
 static	ObjectFunctionType	ChangeOctagonFunctions = {
 	NULL,
 	NULL,
@@ -141,8 +195,49 @@ static	ObjectFunctionType	ChangeOctagonFunctions = {
 	NULL,
 	NULL,
 	NULL,
+	NULL,
 	NULL };
 
+/* ---------------------------------------------------------------------------
+ * changes the thermal on a via
+ * returns TRUE if changed
+ */
+static void *ChangeViaThermal(PinTypePtr Via)
+{
+	int     LayerThermFlag;
+	int     LayerPIPFlag = L0PIPFLAG << INDEXOFCURRENT;
+
+        if (TEST_FLAG(LayerPIPFlag, Via) && !TEST_FLAG(HOLEFLAG, Via))
+        {
+		AddObjectToFlagUndoList(VIA_TYPE, Via, Via, Via);
+		LayerThermFlag = L0THERMFLAG << INDEXOFCURRENT;
+		TOGGLE_FLAG(LayerThermFlag, Via);
+		ClearPin(Via, VIA_TYPE, 0);
+		return(Via);
+	}
+	return(NULL);
+}
+
+/* ---------------------------------------------------------------------------
+ * changes the thermal on a pin 
+ * returns TRUE if changed
+ */
+static void *ChangePinThermal(ElementTypePtr element, PinTypePtr Pin)
+{
+	int     LayerThermFlag;
+	int     LayerPIPFlag = L0PIPFLAG << INDEXOFCURRENT;
+
+        if (TEST_FLAG(LayerPIPFlag, Pin) && !TEST_FLAG(HOLEFLAG, Pin))
+        {
+		AddObjectToFlagUndoList(VIA_TYPE, element, Pin, Pin);
+		LayerThermFlag = L0THERMFLAG << INDEXOFCURRENT;
+		TOGGLE_FLAG(LayerThermFlag, Pin);
+		ClearPin(Pin, PIN_TYPE, 0);
+		return(Pin);
+	}
+	return(NULL);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the size of a via
  * returns TRUE if changed
@@ -192,6 +287,23 @@ static void *ChangeVia2ndSize(PinTypePtr Via)
 	return(NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance size of a via 
+ * returns TRUE if changed
+ */
+static void *ChangeViaClearSize(PinTypePtr Via)
+{
+	Dimension	value = (Absolute) ? Absolute : Via->Clearance +Delta;
+
+	value = MIN(MAX_LINESIZE, MAX(value, Settings.Bloat * 2));
+	AddObjectToClearSizeUndoList(VIA_TYPE, Via, Via, Via);
+	EraseVia(Via);
+	Via->Clearance = value;
+	DrawVia(Via, 0);
+	return(Via);
+}
+
+
 /* ---------------------------------------------------------------------------
  * changes the size of a pin
  * returns TRUE if changed
@@ -215,6 +327,23 @@ static void *ChangePinSize(ElementTypePtr Element, PinTypePtr Pin)
 	return(NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance size of a pin
+ * returns TRUE if changed
+ */
+static void *ChangePinClearSize(ElementTypePtr Element, PinTypePtr Pin)
+{
+	Dimension	value = (Absolute) ? Absolute : Pin->Clearance +Delta;
+
+	value = MIN(MAX_LINESIZE, MAX(value, Settings.Bloat * 2));
+	Element = Element;		/* get rid of 'unused...' warnings */
+	AddObjectToClearSizeUndoList(PIN_TYPE, Element, Pin, Pin);
+	ErasePin(Pin);
+	Pin->Clearance = value;
+	DrawPin(Pin, 0);
+	return(Pin);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the size of a pad
  * returns TRUE if changed
@@ -235,6 +364,27 @@ static void *ChangePadSize(ElementTypePtr Element, PadTypePtr Pad)
 	return(NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance size of a pad
+ * returns TRUE if changed
+ */
+static void *ChangePadClearSize(ElementTypePtr Element, PadTypePtr Pad)
+{
+	Dimension	value = (Absolute) ? Absolute : Pad->Clearance +Delta;
+
+	value = MIN(MAX_LINESIZE, MAX(value, Settings.Bloat * 2));
+	if (value <= MAX_PADSIZE && value >= MIN_PADSIZE &&
+		value != Pad->Thickness)
+	{
+		AddObjectToClearSizeUndoList(PAD_TYPE, Element, Pad, Pad);
+		ErasePad(Pad);
+		Pad->Clearance = value;
+		DrawPad(Pad, 0);
+		return(Pad);
+	}
+	return(NULL);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the drilling hole of a pin
  * returns TRUE if changed
@@ -282,6 +432,26 @@ static void *ChangeLineSize(LayerTypePtr Layer, LineTypePtr Line)
 	return(NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance size of a line
+ * returns TRUE if changed
+ */
+static void *ChangeLineClearSize(LayerTypePtr Layer, LineTypePtr Line)
+{
+	Dimension	value = (Absolute) ? Absolute : Line->Clearance +Delta;
+
+	value = MIN(MAX_LINESIZE, MAX(value, Settings.Bloat * 2));
+	if (value != Line->Clearance)
+	{
+		AddObjectToClearSizeUndoList(LINE_TYPE, Layer, Line, Line);
+		EraseLine(Line);
+		Line->Clearance = value;
+		DrawLine(Layer, Line, 0);
+		return(Line);
+	}
+	return(NULL);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the size of an arc
  * returns TRUE if changed
@@ -302,6 +472,26 @@ static void *ChangeArcSize(LayerTypePtr Layer, ArcTypePtr Arc)
 	return(NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance size of an arc
+ * returns TRUE if changed
+ */
+static void *ChangeArcClearSize(LayerTypePtr Layer, ArcTypePtr Arc)
+{
+	Dimension	value = (Absolute) ? Absolute : Arc->Clearance +Delta;
+
+	value = MIN(MAX_LINESIZE, MAX(value, Settings.Bloat * 2));
+	if (value != Arc->Clearance)
+	{
+		AddObjectToClearSizeUndoList(ARC_TYPE, Layer, Arc, Arc);
+		EraseArc(Arc);
+		Arc->Clearance = value;
+		DrawArc(Layer, Arc, 0);
+		return(Arc);
+	}
+	return(NULL);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the scaling factor of a text object
  * returns TRUE if changed
@@ -425,11 +615,18 @@ static void *ChangeElementName(ElementTypePtr Element)
 {
 	char	*old = ELEMENT_NAME(PCB, Element);
 
+	if (NAME_INDEX(PCB) == NAMEONPCB_INDEX)
+	{
+		if (UniqueElementName(PCB->Data, NewName) != NewName)
+		{
+			Message("Error: The name \"%s\" is not unique!!!\n", NewName);
+			return((char *) -1);
+		}
+	}
 	EraseElementName(Element);
 	ELEMENT_NAME(PCB, Element) = NewName;
 	SetTextBoundingBox(&PCB->Font, &ELEMENT_TEXT(PCB, Element));
 	DrawElementName(Element, 0);
-	FreeNetlist();
 	return(old);
 }
 
@@ -462,6 +659,19 @@ Boolean ChangeLayoutName(char *Name)
 	return(True);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the side of the board an element is on
+ * returns TRUE
+ */
+Boolean ChangeElementSide(ElementTypePtr Element, Position yoff)
+{
+	EraseElement(Element);
+	AddObjectToMirrorUndoList(ELEMENT_TYPE, Element, Element, Element, yoff);
+	MirrorElementCoordinates(Element, yoff);
+	DrawElement(Element,0);
+	return(True);
+}
+	
 /* ---------------------------------------------------------------------------
  * changes the name of a layer; memory has to be already allocated
  */
@@ -472,6 +682,30 @@ Boolean ChangeLayerName(LayerTypePtr Layer, char *Name)
 	return(True);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance flag of a line
+ */
+static void *ChangeLineJoin(LayerTypePtr Layer, LineTypePtr Line)
+{
+	EraseLine(Line);
+	AddObjectToFlagUndoList(LINE_TYPE, Layer, Line, Line);
+	TOGGLE_FLAG(CLEARLINEFLAG, Line);
+	DrawLine(Layer, Line, 0);
+	return(Line);
+}
+
+/* ---------------------------------------------------------------------------
+ * changes the clearance flag of an arc
+ */
+static void *ChangeArcJoin(LayerTypePtr Layer, ArcTypePtr Arc)
+{
+	EraseArc(Arc);
+	AddObjectToFlagUndoList(ARC_TYPE, Layer, Arc, Arc);
+	TOGGLE_FLAG(CLEARLINEFLAG, Arc);
+	DrawArc(Layer, Arc, 0);
+	return(Arc);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the square flag of an element's first pin
  */
@@ -564,7 +798,7 @@ Boolean ChangeHole(PinTypePtr Via)
 		Via->DrillingHole = Via->Thickness - MIN_PINORVIACOPPER;
 	}
 	DrawVia(Via, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(True);
 }
 
@@ -575,228 +809,235 @@ static  void  *ChangePolyClear(LayerTypePtr Layer, PolygonTypePtr Polygon)
 {     
       AddObjectToFlagUndoList(POLYGON_TYPE, Layer, Polygon, Polygon);
       TOGGLE_FLAG(CLEARPOLYFLAG, Polygon);
-      UpdatePIPFlags(NULL, NULL, Layer, True);
+      UpdatePIPFlags(NULL, NULL, Layer, Polygon, True);
       DrawPolygon(Layer, Polygon, 0);
       return(Polygon);
 }
 
 /* ----------------------------------------------------------------------
- * changes the size of all selected and visible pins
+ * changes the side of all selected and visible elements 
  * returns True if anything has changed
  */
-Boolean ChangeSelectedPinSize(Position Difference, Boolean fixIt)
+Boolean ChangeSelectedElementSide(void)
 {
 	Boolean	change = False;
 
 		/* setup identifiers */
-        Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	if (PCB->PinOn)
-		ALLPIN_LOOP(PCB->Data,
-			if (TEST_FLAG(SELECTEDFLAG, pin))
-				change |= (ChangePinSize(element, pin) != NULL);
+	if (PCB->PinOn && PCB->ElementOn)
+		ELEMENT_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, element))
+			{
+				change |= ChangeElementSide(element, 0);
+				UpdatePIPFlags(NULL, element, NULL, NULL, True);
+			}
 		);
         if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
                 IncrementUndoSerialNumber();
 	}
 	return(change);
 }
 
 /* ----------------------------------------------------------------------
- * changes the size of all selected and visible pads
- * returns True if anything has changed
+ * changes the thermals on all selected and visible pins
+ * and/or vias. Returns True if anything has changed
  */
-Boolean ChangeSelectedPadSize(Position Difference, Boolean fixIt)
+Boolean ChangeSelectedThermals(int types)
 {
 	Boolean	change = False;
 
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	if (PCB->PinOn)
-	{
-		ALLPAD_LOOP(PCB->Data,
-			if (TEST_FLAG(SELECTEDFLAG, pad) &&
-				((TEST_FLAG(ONSOLDERFLAG, pad) != 0) == SWAP_IDENT ||
-				PCB->InvisibleObjectsOn))
-				change |= (ChangePadSize(element, pad) != NULL);
-		);
-	}
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
-
-/* ----------------------------------------------------------------------
- * changes the 2nd size (drilling hole) of all selected and visible pins
- * returns True if anything has changed
- */
-Boolean ChangeSelectedPin2ndSize(Position Difference, Boolean fixIt)
-{
-	Boolean	change = False;
-
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	if (PCB->PinOn)
+	if ((types & PIN_TYPE) && PCB->PinOn)
 		ALLPIN_LOOP(PCB->Data,
 			if (TEST_FLAG(SELECTEDFLAG, pin))
-				change |= (ChangePin2ndSize(element, pin) != NULL);
+				change |= (ChangePinThermal(element, pin) != NULL);
 		);
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
 
-/* ----------------------------------------------------------------------
- * changes the size of all selected and visible vias
- * returns True if anything has changed
- */
-Boolean ChangeSelectedViaSize(Position Difference, Boolean fixIt)
-{
-	Boolean	change = False;
-
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	if (PCB->ViaOn)
+	if ((types & VIA_TYPE) && PCB->ViaOn)
 		VIA_LOOP(PCB->Data,
 			if (TEST_FLAG(SELECTEDFLAG, via))
-				change |= (ChangeViaSize(via) != NULL);
+				change |= (ChangeViaThermal(via) != NULL);
 		);
-	if (change)
+        if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
+		Draw();
+                IncrementUndoSerialNumber();
 	}
 	return(change);
 }
 
 /* ----------------------------------------------------------------------
- * changes the 2nd size (drilling hole) of all selected and visible vias
+ * changes the size of all selected and visible object types 
  * returns True if anything has changed
  */
-Boolean ChangeSelectedVia2ndSize(Position Difference, Boolean fixIt)
+Boolean ChangeSelectedSize(int types, Position Difference, Boolean fixIt)
 {
 	Boolean	change = False;
 
 		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
+        Absolute = (fixIt) ? Difference : 0;
 	Delta = Difference;
-	if (PCB->ViaOn)
-		VIA_LOOP(PCB->Data,
-			if (TEST_FLAG(SELECTEDFLAG, via))
-				change |= (ChangeVia2ndSize(via) != NULL);
+
+	if ((types & PIN_TYPE) && PCB->PinOn)
+		ALLPIN_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, pin))
+				change |= (ChangePinSize(element, pin) != NULL);
 		);
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
 
-/* ----------------------------------------------------------------------
- * changes the size of all selected and visible lines
- * returns True if anything has changed
- */
-Boolean ChangeSelectedLineSize(Position Difference, Boolean fixIt)
-{
-	Boolean	change = False;
-
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	VISIBLELINE_LOOP(PCB->Data,
-		if (TEST_FLAG(SELECTEDFLAG, line))
-			change |= (ChangeLineSize(layer, line) != NULL);
-	);
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
-
-/* ----------------------------------------------------------------------
- * changes the size of all selected and visible text objects
- * returns True if anything has changed
- */
-Boolean ChangeSelectedTextSize(Position Difference, Boolean fixIt)
-{
-	Boolean	change = False;
-
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	VISIBLETEXT_LOOP(PCB,
-		if (TEST_FLAG(SELECTEDFLAG, text))
-			change |= (ChangeTextSize(layer, text) != NULL);
-	);
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
-
-/* ----------------------------------------------------------------------
- * changes the size of all selected and visible Element outlines 
- * returns True if anything has changed
- */
-Boolean ChangeSelectedElementSize(Position Difference, Boolean fixIt)
-{
-	Boolean	change = False;
-
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	if (PCB->ElementOn)
-	{
-		ELEMENT_LOOP(PCB->Data,
-			 if (TEST_FLAG(SELECTEDFLAG, element))
-			change |= (ChangeElementSize(element) != NULL);
+	if ((types & PAD_TYPE) && PCB->PinOn)
+		ALLPAD_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, pad) &&
+				(FRONT(pad) || PCB->InvisibleObjectsOn))
+				change |= (ChangePadSize(element, pad) != NULL);
+		);
+	
+	if (types & LINE_TYPE)
+		VISIBLELINE_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, line))
+				change |= (ChangeLineSize(layer, line) != NULL);
 		);
-	}
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
 
-/* ----------------------------------------------------------------------
- * changes the size of all selected and visible Element Names
- * returns True if anything has changed
- */
-Boolean ChangeSelectedNameSize(Position Difference, Boolean fixIt)
-{
-	Boolean	change = False;
+	if (types & TEXT_TYPE)
+		VISIBLETEXT_LOOP(PCB,
+			if (TEST_FLAG(SELECTEDFLAG, text))
+				change |= (ChangeTextSize(layer, text) != NULL);
+		);
+	if (types & ARC_TYPE)
+		VISIBLEARC_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, arc))
+				change |= (ChangeArcSize(layer, arc) != NULL);
+		);
 
-		/* setup identifiers */
-	Absolute = (fixIt) ? Difference : 0;
-	Delta = Difference;
-	if (PCB->ElementOn)
-	{
+	if ((types & ELEMENTNAME_TYPE) && PCB->ElementOn)
 		ELEMENT_LOOP(PCB->Data,
 			 if (TEST_FLAG(SELECTEDFLAG, &ELEMENT_TEXT(PCB, element)))
 			change |= (ChangeElementNameSize(element) != NULL);
 		);
-	}
+
+	if ((types & ELEMENT_TYPE) && PCB->ElementOn)
+		ELEMENT_LOOP(PCB->Data,
+			 if (TEST_FLAG(SELECTEDFLAG, element))
+			change |= (ChangeElementSize(element) != NULL);
+		);
+	if ((types & VIA_TYPE) && PCB->ViaOn)
+		VIA_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, via))
+				change |= (ChangeViaSize(via) != NULL);
+		);
+
 	if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
+		IncrementUndoSerialNumber();
+	}
+	return(change);
+}
+
+/* ----------------------------------------------------------------------
+ * changes the clearance size of all selected and visible objects
+ * returns True if anything has changed
+ */
+Boolean ChangeSelectedClearSize(int types, Position Difference, Boolean fixIt)
+{
+	Boolean	change = False;
+
+		/* setup identifiers */
+	Absolute = (fixIt) ? Difference : 0;
+	Delta = Difference;
+	if ((types & PAD_TYPE) && PCB->PinOn)
+		ALLPAD_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, pad) &&
+				(FRONT(pad) || PCB->InvisibleObjectsOn))
+				change |= (ChangePadClearSize(element, pad) != NULL);
+		);
+
+	if ((types & PIN_TYPE) && PCB->PinOn)
+		ALLPIN_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, pin))
+				change |= (ChangePinClearSize(element, pin) != NULL);
+		);
+
+	if ((types & VIA_TYPE) && PCB->ViaOn)
+		VIA_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, via))
+				change |= (ChangeViaClearSize(via) != NULL);
+		);
+
+	if (types & LINE_TYPE)
+		VISIBLELINE_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, line) &&
+		    	  TEST_FLAG(CLEARLINEFLAG, line))
+				change |= (ChangeLineClearSize(layer, line) != NULL);
+		);
+
+	if (types & ARC_TYPE)
+		VISIBLEARC_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, arc) &&
+		    	  TEST_FLAG(CLEARLINEFLAG, arc))
+				change |= (ChangeArcClearSize(layer, arc) != NULL);
+	);
+
+	if (change)
+      	{
+		Draw();
+		IncrementUndoSerialNumber();
+	}
+	return(change);
+}
+
+/* --------------------------------------------------------------------------
+ * changes the 2nd size (drilling hole) of all selected and visible objects
+ * returns True if anything has changed
+ */
+Boolean ChangeSelected2ndSize(int types, Position Difference, Boolean fixIt)
+{
+	Boolean	change = False;
+
+		/* setup identifiers */
+	Absolute = (fixIt) ? Difference : 0;
+	Delta = Difference;
+	if ((types & PIN_TYPE) && PCB->PinOn)
+		ALLPIN_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, pin))
+				change |= (ChangePin2ndSize(element, pin) != NULL);
+		);
+
+	if ((types & VIA_TYPE) && PCB->ViaOn)
+		VIA_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, via))
+				change |= (ChangeVia2ndSize(via) != NULL);
+		);
+
+	if (change)
+      	{
+		Draw();
+		IncrementUndoSerialNumber();
+	}
+	return(change);
+}
+
+/* ----------------------------------------------------------------------
+ * changes the clearance flag (join) of all selected and visible lines
+ * and/or arcs. Returns True if anything has changed
+ */
+Boolean ChangeSelectedJoin(int types)
+{
+	Boolean	change = False;
+
+	if (types & LINE_TYPE)
+		VISIBLELINE_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, line))
+				change |= (ChangeLineJoin(layer, line) != NULL);
+		);
+
+	if (types & ARC_TYPE)
+		VISIBLEARC_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, arc))
+				change |= (ChangeArcJoin(layer, arc) != NULL);
+		);
+	if (change)
+      	{
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(change);
@@ -806,11 +1047,11 @@ Boolean ChangeSelectedNameSize(Position Difference, Boolean fixIt)
  * changes the square-flag of all selected and visible pins or pads
  * returns True if anything has changed
  */
-Boolean ChangeSelectedPinSquare(void)
+Boolean ChangeSelectedSquare(int types)
 {
 	Boolean	change = False;
 
-	if (PCB->PinOn)
+	if ((types & PIN_TYPE) && PCB->PinOn)
 		ALLPIN_LOOP(PCB->Data,
 			if (TEST_FLAG(SELECTEDFLAG, pin))
 			{
@@ -818,6 +1059,8 @@ Boolean ChangeSelectedPinSquare(void)
 				change = True;
 			}
 		);
+
+	if ((types & PAD_TYPE) && PCB->PinOn)
 		ALLPAD_LOOP(PCB->Data,
 			if (TEST_FLAG(SELECTEDFLAG, pad))
 			{
@@ -825,9 +1068,19 @@ Boolean ChangeSelectedPinSquare(void)
 				change = True;
 			}
 		);
+
+	if ((types & ELEMENT_TYPE) && PCB->PinOn)
+		ELEMENT_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, element) && element->Pin)
+			{
+				ChangeElementSquare(element);
+				change = True;
+			}
+		);
+
 	if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(change);
@@ -869,7 +1122,7 @@ Boolean ChangeSelectedOctagon(int Types)
 		);
 	if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(change);
@@ -890,37 +1143,13 @@ Boolean ChangeSelectedHole(void)
 		);
 	if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(change);
 }
 
 
-/* ----------------------------------------------------------------------
- * changes the square-flag of pin one of all selected and visible elements
- * returns True if anything has changed
- */
-Boolean ChangeSelectedElementSquare(void)
-{
-	Boolean	change = False;
-
-	if (PCB->PinOn)
-		ELEMENT_LOOP(PCB->Data,
-			if (TEST_FLAG(SELECTEDFLAG, element) && element->Pin)
-			{
-				ChangeElementSquare(element);
-				change = True;
-			}
-		);
-	if (change)
-      	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		IncrementUndoSerialNumber();
-	}
-	return(change);
-}
-
 /* ---------------------------------------------------------------------------
  * changes the size of the passed object
  * Returns True if anything is changed
@@ -936,7 +1165,45 @@ Boolean ChangeObjectSize(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 	change = (ObjectOperation(&ChangeSizeFunctions, Type,Ptr1,Ptr2,Ptr3) != NULL);
 	if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
+		IncrementUndoSerialNumber();
+	}
+	return(change);
+}
+
+/* ---------------------------------------------------------------------------
+ * changes the clearance size of the passed object
+ * Returns True if anything is changed
+ */
+Boolean ChangeObjectClearSize(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
+	Position Difference, Boolean fixIt)
+{
+	Boolean change;
+
+		/* setup identifier */
+	Absolute = (fixIt) ? Difference : 0;
+	Delta = Difference;
+	change = (ObjectOperation(&ChangeClearSizeFunctions, Type,Ptr1,Ptr2,Ptr3) != NULL);
+	if (change)
+      	{
+		Draw();
+		IncrementUndoSerialNumber();
+	}
+	return(change);
+}
+
+/* ---------------------------------------------------------------------------
+ * changes the thermal of the passed object
+ * Returns True if anything is changed
+ */
+Boolean ChangeObjectThermal(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
+{
+	Boolean change;
+
+	change = (ObjectOperation(&ChangeThermalFunctions, Type,Ptr1,Ptr2,Ptr3) != NULL);
+	if (change)
+      	{
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(change);
@@ -957,7 +1224,7 @@ Boolean ChangeObject2ndSize(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 	change = (ObjectOperation(&Change2ndSizeFunctions,Type,Ptr1,Ptr2,Ptr3) !=NULL);
 	if (change)
       	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(change);
@@ -975,11 +1242,26 @@ void *ChangeObjectName(int Type, void *Ptr1, void *Ptr2, void *Ptr3, char *Name)
 	void *result;
 		/* setup identifier */
 	NewName = Name;
-	if (result = ObjectOperation(&ChangeNameFunctions, Type, Ptr1, Ptr2, Ptr3));
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	if ((result = ObjectOperation(&ChangeNameFunctions, Type, Ptr1, Ptr2, Ptr3)));
+		Draw();
 	return(result);
 }
 
+/* ---------------------------------------------------------------------------
+ * changes the clearance-flag of the passed object
+ * Returns True if anything is changed
+ */
+Boolean ChangeObjectJoin(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
+{
+	if (ObjectOperation(&ChangeJoinFunctions, Type,Ptr1,Ptr2,Ptr3) !=NULL)
+	{
+		Draw();
+		IncrementUndoSerialNumber();
+		return(True);
+	}
+	return(False);
+}
+
 /* ---------------------------------------------------------------------------
  * changes the square-flag of the passed object
  * Returns True if anything is changed
@@ -988,7 +1270,7 @@ Boolean ChangeObjectSquare(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 {
 	if (ObjectOperation(&ChangeSquareFunctions, Type,Ptr1,Ptr2,Ptr3) !=NULL)
 	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 		return(True);
 	}
@@ -1003,7 +1285,7 @@ Boolean ChangeObjectOctagon(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 {
 	if (ObjectOperation(&ChangeOctagonFunctions, Type,Ptr1,Ptr2,Ptr3) !=NULL)
 	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 		return(True);
 	}
@@ -1048,11 +1330,39 @@ void *QueryInputAndChangeObjectName(int Type,
 	if (name)
 	{
 		char	*old = ChangeObjectName(Type, Ptr1, Ptr2, Ptr3, name);
-
-		AddObjectToChangeNameUndoList(Type, Ptr1, Ptr2, Ptr3, old);
-		IncrementUndoSerialNumber();
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		if (old != (char *) -1)
+		{
+			AddObjectToChangeNameUndoList(Type, Ptr1, Ptr2, Ptr3, old);
+			IncrementUndoSerialNumber();
+		}
+		Draw();
 		return(Ptr3);
 	}
 	return(NULL);
 }
+
+/* ---------------------------------------------------------------------------
+ * changes the maximum size of a layout
+ * adjusts the scrollbars
+ * releases the saved pixmap if necessary
+ * and adjusts the cursor confinement box
+ */
+void ChangePCBSize(Dimension Width, Dimension Height)
+{
+	PCB->MaxWidth =  Width;
+	PCB->MaxHeight = Height;
+
+		/* crosshair range is different if pastebuffer-mode
+		 * is enabled
+		 */
+	if (Settings.Mode == PASTEBUFFER_MODE)
+		SetCrosshairRange(
+			PASTEBUFFER->X -PASTEBUFFER->BoundingBox.X1,
+			PASTEBUFFER->Y -PASTEBUFFER->BoundingBox.Y1,
+			MAX(0, Width -(PASTEBUFFER->BoundingBox.X2-PASTEBUFFER->X)),
+			MAX(0, Height -(PASTEBUFFER->BoundingBox.Y2-PASTEBUFFER->Y)));
+	else
+		SetCrosshairRange(0, 0, (Position) Width, (Position) Height);
+	ScaleOutput(Output.Width, Output.Height);
+	UpdateAll();
+}
