@@ -41,39 +41,49 @@
 
 #define	CHANGESIZE_TYPES        \
 	(POLYGON_TYPE | VIA_TYPE | PIN_TYPE | PAD_TYPE | LINE_TYPE | \
-	 ARC_TYPE | TEXT_TYPE | ELEMENTNAME_TYPE)
+	 ARC_TYPE | TEXT_TYPE | ELEMENTNAME_TYPE | ELEMENT_TYPE)
 
 #define	CHANGE2NDSIZE_TYPES     \
 	(VIA_TYPE | PIN_TYPE)
 
+#define CHANGECLEARSIZE_TYPES	\
+	(PIN_TYPE | PAD_TYPE | VIA_TYPE | LINE_TYPE | ARC_TYPE)
+
 #define	CHANGESQUARE_TYPES     \
 	(ELEMENT_TYPE | PIN_TYPE | PAD_TYPE)
 
 #define	CHANGEOCTAGON_TYPES     \
 	(ELEMENT_TYPE | PIN_TYPE | VIA_TYPE)
 
+#define CHANGEJOIN_TYPES	\
+	(ARC_TYPE | LINE_TYPE)
+
+#define CHANGETHERMAL_TYPES	\
+	(PIN_TYPE | VIA_TYPE)
+
 Boolean	ChangeLayoutName(char *);
 Boolean	ChangeLayerName(LayerTypePtr, char *);
-Boolean	ChangeSelectedPinSize(Position, Boolean);
-Boolean	ChangeSelectedPadSize(Position, Boolean);
-Boolean	ChangeSelectedPin2ndSize(Position, Boolean);
-Boolean	ChangeSelectedViaSize(Position, Boolean);
-Boolean	ChangeSelectedVia2ndSize(Position, Boolean);
-Boolean	ChangeSelectedLineSize(Position, Boolean); 
-Boolean	ChangeSelectedTextSize(Position, Boolean); 
-Boolean	ChangeSelectedElementSize(Position, Boolean); 
-Boolean	ChangeSelectedNameSize(Position, Boolean); 
-Boolean	ChangeSelectedPinSquare(void); 
-Boolean	ChangeSelectedElementSquare(void); 
+Boolean	ChangeSelectedSize(int, Position, Boolean);
+Boolean	ChangeSelectedClearSize(int, Position, Boolean);
+Boolean	ChangeSelected2ndSize(int, Position, Boolean);
+Boolean	ChangeSelectedJoin(int); 
+Boolean	ChangeSelectedSquare(int); 
+Boolean	ChangeSelectedThermals(int); 
 Boolean ChangeSelectedHole(void);
 Boolean ChangeSelectedOctagon(int);
+Boolean ChangeSelectedElementSide(void);
+Boolean ChangeElementSide(ElementTypePtr, Position);
 Boolean ChangeHole(PinTypePtr);
 Boolean	ChangeObjectSize(int, void *, void *, void *, Position, Boolean);
+Boolean	ChangeObjectThermal(int, void *, void *, void *);
+Boolean	ChangeObjectClearSize(int, void *, void *, void *, Position, Boolean);
 Boolean	ChangeObject2ndSize(int, void *, void *, void *, Position, Boolean);
+Boolean	ChangeObjectJoin(int, void *, void *, void *);
 Boolean	ChangeObjectSquare(int, void *, void *, void *);
 Boolean	ChangeObjectOctagon(int, void *, void *, void *);
 void	*ChangeObjectName(int, void *, void *, void *, char *);
 void	*QueryInputAndChangeObjectName(int, void *, void *, void *);
+void	ChangePCBSize(Dimension, Dimension);
 
 #endif
 
