@@ -40,6 +40,7 @@
  * the default X cursor shape for the output window
  */
 #define	DEFAULT_CURSORSHAPE	XC_crosshair
+#define XC_Clockwise		257
 
 /* ---------------------------------------------------------------------------
  * the layer-numbers of the two additional special layers
@@ -52,7 +53,7 @@
  * misc constants
  */
 #define	MARK_SIZE		50		/* mark size of elements */
-#define	UNDO_WARNING_SIZE	(300*1024)	/* warning limit of undo */
+#define	UNDO_WARNING_SIZE	(1024*1024)	/* warning limit of undo */
 #define	USERMEDIANAME		"user defined"	/* label of default media */
 
 /* ---------------------------------------------------------------------------
@@ -62,6 +63,7 @@
 #define	M_PI			3.14159265358979323846
 #endif
 #define	M180			(M_PI/180.0)
+#define RAD_TO_DEG		(180.0/M_PI)
 #define	TAN_22_5_DEGREE_2	0.207106781	/* 0.5*tan(22.5) */
 #define COS_22_5_DEGREE		0.923879533	/* cos(22.5) */
 #define	TAN_30_DEGREE		0.577350269	/* tan(30) */
@@ -86,6 +88,7 @@
 #define	RUBBERBANDMOVE_MODE	107	/* move objects and attached lines */
 #define THERMAL_MODE            108     /* toggle thermal layer flag */
 #define ARC_MODE                109     /* draw arcs */
+#define ARROW_MODE		110	/* selection with arrow mode */
 
 /* ---------------------------------------------------------------------------
  * object flags
@@ -103,16 +106,18 @@
 #define HIDENAMEFLAG		0x0010	/* hide the element name */
 #define	DISPLAYNAMEFLAG		0x0020	/* display the names of pins/pads */
 					/* of an element */
+#define CLEARLINEFLAG		0x0020	/* line doesn't touch polygons */
 #define	SELECTEDFLAG		0x0040	/* object has been selected */
 #define	ONSOLDERFLAG		0x0080	/* element is on bottom side */
 #define	SQUAREFLAG		0x0100	/* pin is square, not round */
 #define RUBBERENDFLAG		0x0200	/* indicates one end already rubber */
-					/* banding same as PinOneFlag */
-					/* since lines won't use it */
+					/* banding same as warn flag*/
+					/* since pins/pads won't use it */
 #define WARNFLAG		0x0200          /* Warning for pin/via/pad */
 #define USETHERMALFLAG		0x0400          /* draw pin, via with thermal fingers */
 #define ONSILKFLAG		0x0400          /* text is to be drawn on silkscreen layer */
 #define OCTAGONFLAG		0x0800		/* draw pin/via as octagon instead of round */
+#define DRCFLAG			0x1000		/* flag like FOUND flag for DRC checking */
 #define ALLTHERMFLAGS		0xff0000        /* 8 flags indicating pin/via connects to polys */
 #define L0THERMFLAG           0x010000        /* Pin/Via connects to Polygons on Layer 0 */
 #define L1THERMFLAG           0x020000        /* Pin/Via connects to Polygons on Layer 1 */
@@ -143,6 +148,7 @@
 #define	NAMEONPCBFLAG		0x0040	/* display name of an element */
 #define	ABSOLUTEFLAG		0x0080	/* grid is relative to (0,0) */
 #define	ALLDIRECTIONFLAG	0x0100	/* enable 'all-direction' lines */
+#define SWAPSTARTDIRFLAG        0x0200  /* switch starting angle after each click */
 
 /* ---------------------------------------------------------------------------
  * object types
