@@ -37,6 +37,7 @@ static	char	*rcsid = "$Id: control.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 #include "control.h"
 #include "data.h"
 #include "draw.h"
+#include "output.h"
 #include "misc.h"
 #include "menu.h"
 #include "set.h"
@@ -91,9 +92,9 @@ static	void		CB_SetMode(Widget, XtPointer, XtPointer);
 /* ---------------------------------------------------------------------------
  * some local identifiers
  */
-static	Widget	OnOffWidgets[MAX_LAYER+4];	/* widgets for on/off selection */
+static	Widget	OnOffWidgets[MAX_LAYER+6];	/* widgets for on/off selection */
 
-static	PopupEntryType	DrawingLayerMenuEntries[MAX_LAYER+1];
+static	PopupEntryType	DrawingLayerMenuEntries[MAX_LAYER+2];
 static	PopupMenuType	DrawingLayerMenu =
 	{ "drawingLayer", NULL, DrawingLayerMenuEntries,
 		CBPOPUP_SetDrawingLayer, NULL, NULL };
@@ -123,7 +124,9 @@ static	ModeButtonType	ModeButtons[] = {
 		InsertPointModeIcon_bits, InsertPointModeIcon_width,
 		InsertPointModeIcon_height, NULL },
 	{ "thermalIcon", "thermal", THERMAL_MODE, ThermalModeIcon_bits,
-		ThermalModeIcon_width, ThermalModeIcon_height, NULL}};
+		ThermalModeIcon_width, ThermalModeIcon_height, NULL},
+	{ "selectIcon", "select", ARROW_MODE, SelModeIcon_bits,
+		SelModeIcon_width, SelModeIcon_height, NULL}};
 
 /* ---------------------------------------------------------------------------
  * adds a label without border, surrounded by additional space
@@ -152,13 +155,13 @@ static Widget AddLabel(String Label, Widget Parent, Widget Top, Widget Left)
  */
 static void AddDrawingLayerSelection(Widget Parent)
 {
-	static	char	name[MAX_LAYER][20];
+	static	char	name[MAX_LAYER + 1][20];
 			int		i;
 			Widget	label;
 			Pixel	background;
 
 		/* fill struct and initialize menu button */
-	for (i = 0; i < MAX_LAYER; i++)
+	for (i = 0; i < MAX_LAYER + 1; i++)
 	{
 			/* the widgets name is used for the 'CheckEntry()' funtion
 			 * and must be declared static therefore
@@ -169,6 +172,7 @@ static void AddDrawingLayerSelection(Widget Parent)
 		DrawingLayerMenuEntries[i].Callback = CB_SetDrawingLayer;
 		DrawingLayerMenuEntries[i].ClientData = (XtPointer) i;
 	}
+	
 
 		/* init routine exits on NULL pointer */
 	DrawingLayerMenuEntries[i].Name = NULL;
@@ -189,7 +193,7 @@ static void AddDrawingLayerSelection(Widget Parent)
  */
 static void AddOnOffSelection(Widget Parent)
 {
-	static	char	name[MAX_LAYER+4][20];
+	static	char	name[MAX_LAYER+5][20];
 			Widget	last;
 			int		i;
 			char	*text;
@@ -198,7 +202,7 @@ static void AddOnOffSelection(Widget Parent)
 		/* insert a label at the top */
 	last = AddLabel("on/off", Parent, NULL, NULL);
 
-	for (i = 0; i < MAX_LAYER+4; i++)
+	for (i = 0; i < MAX_LAYER+5; i++)
 	{
 		switch(i)
 		{
@@ -220,12 +224,19 @@ static void AddOnOffSelection(Widget Parent)
 				strcpy(name[i], "vias");
 				break;
 
-			case MAX_LAYER+3:		/* settings for invisible objects */
+			case MAX_LAYER+3:		/* settings for objects on opposite side */
 				color = PCB->InvisibleObjectsColor;
 				text = "far side";
 				strcpy(name[i], "invisible");
 				break;
 
+			case MAX_LAYER+4:		/* settings for rats */
+				color = PCB->RatColor;
+				text = "rat lines";
+				strcpy(name[i], "rats");
+				break;
+
+
 			default:				/* layers */
 				color = PCB->Data->Layer[i].Color;
 				text = "";
@@ -262,13 +273,16 @@ static void UpdateDrawingLayerSelection(void)
 			XtNlabel, UNKNOWN(PCB->Data->Layer[i].Name),
 			XtNforeground, PCB->Data->Layer[i].Color,
 			NULL);
+	XtVaSetValues(DrawingLayerMenuEntries[i].W,
+		XtNlabel, UNKNOWN(PCB->Data->Layer[i].Name),
+		XtNforeground, PCB->ElementColor,
+		NULL);
 
-		/* set the label of the menu-nutton itself and reset a flag */
+		/* set the label of the menu-button itself and reset a flag */
 	XtVaSetValues(DrawingLayerMenuButton.W,
 		XtNlabel, UNKNOWN(CURRENT->Name),
 		XtNbackground, CURRENT->Color,
 		NULL);
-	RedrawOnEnter = False;
 }
 
 /* ---------------------------------------------------------------------------
@@ -292,6 +306,7 @@ static void UpdateOnOffSelection(void)
 	XtVaSetValues(OnOffWidgets[i++], XtNstate, PCB->PinOn, NULL);
 	XtVaSetValues(OnOffWidgets[i++], XtNstate, PCB->ViaOn, NULL);
 	XtVaSetValues(OnOffWidgets[i++], XtNstate, PCB->InvisibleObjectsOn, NULL);
+	XtVaSetValues(OnOffWidgets[i++], XtNstate, PCB->RatOn, NULL);
 }
 
 /* ----------------------------------------------------------------------
@@ -303,6 +318,8 @@ static Cardinal GetGroupOfLayer(Cardinal Layer)
 	int	group,
 		i;
 
+	if (Layer == MAX_LAYER)
+		return(Layer);
 	for (group = 0; group < MAX_LAYER; group++)
 		for (i = 0; i < PCB->LayerGroups.Number[group]; i++)
 			if (PCB->LayerGroups.Entries[group][i] == Layer)
@@ -320,6 +337,18 @@ int ChangeGroupVisibility(Cardinal Layer, Boolean On, Boolean ChangeStackOrder)
 			i,
 			changed = 1;	/* at least the current layer changes */
 
+		/* special case of silk layer */
+	if (Layer == MAX_LAYER)
+	{
+		PCB->ElementOn = On;
+		PCB->SilkActive = On && ChangeStackOrder;
+		SILKLAYER->On = On;
+		BACKSILKLAYER->On = (On && PCB->InvisibleObjectsOn);
+		UpdateOnOffSelection();
+		return(0);
+	}
+	PCB->SilkActive = (On && ChangeStackOrder) ? False : PCB->SilkActive;
+
 		/* decrement 'i' to keep stack in order of layergroup */
 	if ((group = GetGroupOfLayer(Layer)) < MAX_LAYER)
 		for (i = PCB->LayerGroups.Number[group]; i;) 
@@ -379,7 +408,7 @@ static void CBPOPUP_SetDrawingLayer(Widget W,
 	char	name[10];
 
 	RemoveCheckFromMenu(&DrawingLayerMenu);
-	sprintf(name, "layer%-i", LayerStack[0]+1);
+	sprintf(name, "layer%-i", MIN(INDEXOFCURRENT, MAX_LAYER) + 1);
 	CheckEntry(&DrawingLayerMenu, name);
 }
 
@@ -392,12 +421,15 @@ static void CBPOPUP_SetDrawingLayer(Widget W,
 static void CB_SetDrawingLayer(Widget W,
 	XtPointer ClientData, XtPointer CallData)
 {
+	LayerTypePtr lay;
+
 	ChangeGroupVisibility((int) ClientData, True, True);
+	lay = CURRENT;
 	XtVaSetValues(DrawingLayerMenuButton.W,
-		XtNlabel, UNKNOWN(CURRENT->Name),
-		XtNbackground, CURRENT->Color,
+		XtNlabel, UNKNOWN(lay->Name),
+		XtNbackground, lay->Color,
 		NULL);
-	RedrawOnEnter = True;
+	UpdateAll();
 }
 
 /* ---------------------------------------------------------------------------
@@ -405,7 +437,7 @@ static void CB_SetDrawingLayer(Widget W,
  */
 static void CB_SetOnOff(Widget W, XtPointer ClientData, XtPointer CallData)
 {
-	Boolean	state;
+	Boolean	state, Redraw = False;
 	int		layer = (int) ClientData;
 
 		/* get new state of widget */
@@ -414,48 +446,69 @@ static void CB_SetOnOff(Widget W, XtPointer ClientData, XtPointer CallData)
 		/* set redraw flag if object exists */
 	switch(layer)
 	{
-		case MAX_LAYER:			/* settings for elements */
-			PCB->ElementOn = state;
-			RedrawOnEnter = (PCB->Data->ElementN != 0);
-			break;
-
 		case MAX_LAYER+1:		/* settings for pins */
 			PCB->PinOn = state;
-			RedrawOnEnter = (PCB->Data->ElementN != 0);
+			Redraw |= (PCB->Data->ElementN != 0);
 			break;
 
 		case MAX_LAYER+2:		/* settings for vias */
 			PCB->ViaOn = state;
-			RedrawOnEnter = (PCB->Data->ViaN != 0);
+			Redraw |= (PCB->Data->ViaN != 0);
 			break;
 
 		case MAX_LAYER+3:		/* settings for invisible objects */
 			PCB->InvisibleObjectsOn = state;
-			RedrawOnEnter = (PCB->Data->ElementN != 0);
+			BACKSILKLAYER->On = (state && PCB->ElementOn);
+			Redraw = True;
 			break;
 
-		default:				/* layers; current one can't be switched off */
+		case MAX_LAYER+4:		/* settings for rat lines */
+			PCB->RatOn = state;
+			Redraw |= (PCB->Data->RatN != 0);
+			break;
+
+		default:
 		{
 			int		i, group;
 
-				/* check if current one is in that group;
-				 * if YES, do nothing because it cannot be changed
+				/* check if active layer is in the group;
+				 * if YES, make a different one active if possible
 				 */
-			if ((group = GetGroupOfLayer(layer)) < MAX_LAYER)
-				for (i = 0; i < PCB->LayerGroups.Number[group]; i++)
-					if (PCB->LayerGroups.Entries[group][i] == LayerStack[0])
-					{
-							/* reset state to 'On' */
-						XtVaSetValues(W, XtNstate, True, NULL);
-						return;
-					}
+			if ((group = GetGroupOfLayer(layer)) ==
+				GetGroupOfLayer(MIN(MAX_LAYER, INDEXOFCURRENT)))
+			{
+				/* find next visible layer and make it
+			 	 * active so this one can be turned off
+			 	 */
+				for (i = (layer + 1) % (MAX_LAYER + 1);
+				     i != layer; i = (i + 1) % (MAX_LAYER + 1))
+				     	if (PCB->Data->Layer[i].On == True &&
+					   GetGroupOfLayer(i) != group)
+						break;
+				if (i != layer)
+				{
+					ChangeGroupVisibility((int) i, True, True);
+					XtVaSetValues(DrawingLayerMenuButton.W,
+						XtNlabel, UNKNOWN(CURRENT->Name),
+						XtNbackground, CURRENT->Color,
+						NULL);
+				}
+				else
+				{
+						/* everything else off, we can't turn this off too */
+					XtVaSetValues(W, XtNstate, True, NULL);
+					return;
+				}
+			}
 
 				/* switch layer group on/off */
 			ChangeGroupVisibility(layer, state, False);
-			RedrawOnEnter = True;
+			Redraw = True;
 			break;
 		}
 	}
+	if (Redraw)
+		UpdateAll();
 }
 
 /* ---------------------------------------------------------------------------
@@ -476,9 +529,10 @@ void InitControlPanel(Widget Parent, Widget Top, Widget Left)
 		XtNfromVert, Top,
 		LAYOUT_TOP,
 		NULL);
+	InitOutputPanner(masterform, NULL, NULL);
 	visible = XtVaCreateManagedWidget("onOffSelection", formWidgetClass,
 		masterform,
-		XtNfromVert, NULL,
+		XtNfromVert, Output.panner,
 		XtNfromHoriz, NULL,
 		NULL);
 	drawing = XtVaCreateManagedWidget("currentSelection", formWidgetClass,
@@ -514,12 +568,36 @@ static void CB_SetMode(Widget W, XtPointer ClientData, XtPointer CallData)
 	SetMode(button ? button->Mode : NO_MODE);
 }
 
+/* ---------------------------------------------------------------------------
+ * updates the mode selection buttons to reflect the currently set mode
+ */
+void UpdateModeSelection(void)
+{
+		/* reset all buttons if no mode is selected;
+		 * the first buttons widget identifies the radio group whereas
+		 * radioData points to the array member itself
+		 */
+	if (Settings.Mode == NO_MODE)
+		XawToggleUnsetCurrent(ModeButtons[0].W);
+	else
+	{
+		int	i;
+
+		for (i = 0; i < ENTRIES(ModeButtons); i++)
+			if (Settings.Mode == ModeButtons[i].Mode)
+			{
+				XawToggleSetCurrent(ModeButtons[0].W,
+					(XtPointer) &ModeButtons[i]);
+				break;
+			}
+	}
+}
+
 /* ---------------------------------------------------------------------------
  * initializes mode buttons
  */
 Widget InitModeButtons(Widget Parent, Widget Top, Widget Left)
 {
-	Screen	*screen = XtScreen(Parent);
 	int		i;
 
 	for (i = 0; i < ENTRIES(ModeButtons); i++)
@@ -527,10 +605,8 @@ Widget InitModeButtons(Widget Parent, Widget Top, Widget Left)
 		Pixmap	bitmap;
 
 			/* create background pixmap */
-		bitmap = XCreatePixmapFromBitmapData(Dpy, XtWindow(Parent),
-			ModeButtons[i].Bitmap, ModeButtons[i].Width, ModeButtons[i].Height,
-			BlackPixelOfScreen(screen), WhitePixelOfScreen(screen),
-			DefaultDepthOfScreen(screen));
+		bitmap = XCreateBitmapFromData(Dpy, XtWindow(Parent),
+			ModeButtons[i].Bitmap, ModeButtons[i].Width, ModeButtons[i].Height);
 
 			/* create radio button; use label only if pixmap creation failed
 			 * layout has to be set here else X11R4 fails
@@ -565,7 +641,7 @@ Widget InitModeButtons(Widget Parent, Widget Top, Widget Left)
 			CB_SetMode, (XtPointer) &ModeButtons[i]);
 
 			/* update position for next widget */
-		if (i % BUTTONS_PER_ROW == BUTTONS_PER_ROW-1)
+		if (i % BUTTONS_PER_ROW == BUTTONS_PER_ROW - 1)
 		{
 			Left = NULL;
 			Top = ModeButtons[i].W;
@@ -575,28 +651,3 @@ Widget InitModeButtons(Widget Parent, Widget Top, Widget Left)
 	}
 	return(ModeButtons[i-1].W);
 }
-
-/* ---------------------------------------------------------------------------
- * updates the mode selection buttons to reflect the currently set mode
- */
-void UpdateModeSelection(void)
-{
-		/* reset all buttons if no mode is selected;
-		 * the first buttons widget identifies the radio group whereas
-		 * radioData points to the array member itself
-		 */
-	if (Settings.Mode == NO_MODE)
-		XawToggleUnsetCurrent(ModeButtons[0].W);
-	else
-	{
-		int	i;
-
-		for (i = 0; i < ENTRIES(ModeButtons); i++)
-			if (Settings.Mode == ModeButtons[i].Mode)
-			{
-				XawToggleSetCurrent(ModeButtons[0].W,
-					(XtPointer) &ModeButtons[i]);
-				break;
-			}
-	}
-}
