@@ -44,6 +44,7 @@ static	char	*rcsid = "$Id: copy.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 #include "misc.h"
 #include "move.h"
 #include "polygon.h"
+#include "rats.h"
 #include "select.h"
 #include "undo.h"
 
@@ -72,7 +73,8 @@ static	ObjectFunctionType	CopyFunctions = {
 	NULL,
 	NULL,
 	NULL,
-	CopyArc };
+	CopyArc,
+	NULL };
 
 /* ---------------------------------------------------------------------------
  * copies data from one polygon to another
@@ -85,7 +87,7 @@ PolygonTypePtr CopyPolygonLowLevel(PolygonTypePtr Dest, PolygonTypePtr Src)
 		CreateNewPointInPolygon(Dest, point->X, point->Y);
 	);
 	SetPolygonBoundingBox(Dest);
-	Dest->Flags = Src->Flags & (~(FOUNDFLAG | SELECTEDFLAG));
+	Dest->Flags = Src->Flags & (~(FOUNDFLAG));
 	return(Dest);
 }
 
@@ -93,19 +95,21 @@ PolygonTypePtr CopyPolygonLowLevel(PolygonTypePtr Dest, PolygonTypePtr Src)
  * copies data from one element to another and creates the destination 
  * if necessary
  */
-ElementTypePtr CopyElementLowLevel(ElementTypePtr Dest, ElementTypePtr Src)
+ElementTypePtr CopyElementLowLevel(DataTypePtr Data, ElementTypePtr Dest, ElementTypePtr Src,
+	Boolean uniqueName)
 {
 		/* release old memory if necessary */
 	if (Dest)
 		FreeElementMemory(Dest);
 
 		/* both coordinates and flags are the same */
-	Dest = CreateNewElement(PCB->Data, Dest, &PCB->Font,
-		Src->Flags & (~(FOUNDFLAG | SELECTEDFLAG)),
+	Dest = CreateNewElement(Data, Dest, &PCB->Font,
+		Src->Flags & (~(FOUNDFLAG)),
 		DESCRIPTION_NAME(Src), NAMEONPCB_NAME(Src), VALUE_NAME(Src),
 		DESCRIPTION_TEXT(Src).X, DESCRIPTION_TEXT(Src).Y,
 		DESCRIPTION_TEXT(Src).Direction, DESCRIPTION_TEXT(Src).Scale,
-		DESCRIPTION_TEXT(Src).Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+		DESCRIPTION_TEXT(Src).Flags & (~(FOUNDFLAG)),
+		uniqueName);
 
 		/* abort on error */
 	if (!Dest)
@@ -117,13 +121,13 @@ ElementTypePtr CopyElementLowLevel(ElementTypePtr Dest, ElementTypePtr Src)
 	);
 	PIN_LOOP(Src,
 		CreateNewPin(Dest, pin->X, pin->Y, pin->Thickness,
-			pin->DrillingHole, pin->Name,
-			pin->Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+			pin->Clearance, pin->Mask, pin->DrillingHole, pin->Name,
+			pin->Number, pin->Flags & (~(FOUNDFLAG)));
 	);
 	PAD_LOOP(Src,
 		CreateNewPad(Dest, pad->Point1.X, pad->Point1.Y, pad->Point2.X, pad->Point2.Y,
-			pad->Thickness, pad->Name,
-			pad->Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+			pad->Thickness, pad->Clearance, pad->Mask, pad->Name, pad->Number,
+			pad->Flags & (~(FOUNDFLAG)));
 	);
 	ARC_LOOP(Src,
 		CreateNewArcInElement(Dest, arc->X, arc->Y, arc->Width, arc->Height,
@@ -145,14 +149,14 @@ static void *CopyVia(PinTypePtr Via)
 	PinTypePtr	via;
 
 	via = CreateNewVia(PCB->Data, Via->X + DeltaX, Via->Y + DeltaY,
-			Via->Thickness, Via->DrillingHole, Via->Name,
-			Via->Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+			Via->Thickness, Via->Clearance, Via->DrillingHole, Via->Name,
+			Via->Flags & (~(FOUNDFLAG)));
 	if (!via)
 		return(via);
-        UpdatePIPFlags(via, (ElementTypePtr) via, NULL, False);
+        UpdatePIPFlags(via, (ElementTypePtr) via, NULL, NULL, False);
 	DrawVia(via, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	AddObjectToCopyUndoList(VIA_TYPE, via, via, via);
+	Draw();
+	AddObjectToCreateUndoList(VIA_TYPE, via, via, via);
 	return(via);
 }
 
@@ -166,12 +170,12 @@ static void *CopyLine(LayerTypePtr Layer, LineTypePtr Line)
 	line = CreateNewLineOnLayer(Layer, Line->Point1.X + DeltaX,
 		Line->Point1.Y + DeltaY, Line->Point2.X + DeltaX,
 		Line->Point2.Y + DeltaY, Line->Thickness,
-		Line->Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+		Line->Clearance, Line->Flags & (~(FOUNDFLAG)));
 	if (!line)
 		return(line);
 	DrawLine(Layer, line, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	AddObjectToCopyUndoList(LINE_TYPE, Layer, line, line);
+	Draw();
+	AddObjectToCreateUndoList(LINE_TYPE, Layer, line, line);
 	return(line);
 }
 
@@ -183,14 +187,14 @@ static void *CopyArc(LayerTypePtr Layer, ArcTypePtr Arc)
 	ArcTypePtr	arc;
 	
 	arc = CreateNewArcOnLayer(Layer, Arc->X + DeltaX,
-		Arc->Y + DeltaY, Arc->Width,
-		Arc->StartAngle, Arc->Delta, Arc->Thickness,
-		Arc->Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+		Arc->Y + DeltaY, Arc->Width, Arc->StartAngle,
+		Arc->Delta, Arc->Thickness, Arc->Clearance,
+		Arc->Flags & (~(FOUNDFLAG)));
 	if (!arc)
 		return(arc);
 	DrawArc(Layer, arc, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	AddObjectToCopyUndoList(ARC_TYPE, Layer, arc, arc);
+	Draw();
+	AddObjectToCreateUndoList(ARC_TYPE, Layer, arc, arc);
 	return(arc);
 }
 
@@ -204,10 +208,10 @@ static void *CopyText(LayerTypePtr Layer, TextTypePtr Text)
 	text = CreateNewText(Layer, &PCB->Font, Text->X + DeltaX,
 		Text->Y + DeltaY, Text->Direction,
 		Text->Scale, Text->TextString,
-		Text->Flags & (~(FOUNDFLAG | SELECTEDFLAG)));
+		Text->Flags & (~(FOUNDFLAG)));
 	DrawText(Layer, text, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	AddObjectToCopyUndoList(TEXT_TYPE, Layer, text, text);
+	Draw();
+	AddObjectToCreateUndoList(TEXT_TYPE, Layer, text, text);
 	return(text);
 }
 
@@ -222,9 +226,9 @@ static void *CopyPolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 	CopyPolygonLowLevel(polygon, Polygon);
 	MovePolygonLowLevel(polygon, DeltaX, DeltaY);
 	DrawPolygon(Layer, polygon, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-        UpdatePIPFlags(NULL, NULL, Layer, True);
-	AddObjectToCopyUndoList(POLYGON_TYPE, Layer, polygon, polygon);
+	Draw();
+        UpdatePIPFlags(NULL, NULL, Layer, polygon, True);
+	AddObjectToCreateUndoList(POLYGON_TYPE, Layer, polygon, polygon);
 	return(polygon);
 }
 
@@ -234,26 +238,25 @@ static void *CopyPolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 static void *CopyElement(ElementTypePtr Element)
 {
 	Boolean didDraw = False;
-	ElementTypePtr	element = CopyElementLowLevel(NULL, Element);
+	ElementTypePtr	element = CopyElementLowLevel(PCB->Data, NULL, Element, True);
 
 	MoveElementLowLevel(element, DeltaX, DeltaY);
 	if (PCB->ElementOn &&
-		((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-		PCB->InvisibleObjectsOn))
+		(FRONT(element) || PCB->InvisibleObjectsOn))
 	{
 		DrawElementName(element, 0);
 		DrawElementPackage(element, 0);
 		didDraw = True;
 	}
-	UpdatePIPFlags(NULL, element, NULL, False);
-	AddObjectToCopyUndoList(ELEMENT_TYPE, element, element, element);
+	UpdatePIPFlags(NULL, element, NULL, NULL, False);
+	AddObjectToCreateUndoList(ELEMENT_TYPE, element, element, element);
 	if (PCB->PinOn)
 	{
 		DrawElementPinsAndPads(element, 0);
 		didDraw = True;
 	}
 	if (didDraw)
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	return(element);
 }
 
@@ -265,24 +268,16 @@ Boolean CopyPastebufferToLayout(Position X, Position Y)
 {
 	Cardinal	i;
 	Boolean		changed = False;
-	BoxType		bounds;
 
 		/* set movement vector */
 	DeltaX = X -PASTEBUFFER->X,
 	DeltaY = Y -PASTEBUFFER->Y;
 
-		/* set up for fast drawing */
-	bounds.X1 = PASTEBUFFER->BoundingBox.X1 + DeltaX;
-	bounds.X2 = PASTEBUFFER->BoundingBox.X2 + DeltaX;
-	bounds.Y1 = PASTEBUFFER->BoundingBox.Y1 + DeltaY;
-	bounds.Y2 = PASTEBUFFER->BoundingBox.Y2 + DeltaY;
-	AddBlockToUndoList(&bounds);
-	BlockDrawing = True;
 		/* paste all layers */
-	for (i = MAX_LAYER-1; i != -1; i--)
+	for (i = 0; i < MAX_LAYER + 2; i++)
 	{
-		LayerTypePtr	sourcelayer = &PASTEBUFFER->Data->Layer[LayerStack[i]],
-						destlayer = &PCB->Data->Layer[LayerStack[i]];
+		LayerTypePtr	sourcelayer = &PASTEBUFFER->Data->Layer[i],
+				destlayer = &PCB->Data->Layer[i];
 
 		if (destlayer->On)
 		{
@@ -302,8 +297,7 @@ Boolean CopyPastebufferToLayout(Position X, Position Y)
 	if (PCB->PinOn && PCB->ElementOn)
 	{
 		ELEMENT_LOOP(PASTEBUFFER->Data,
-			if ((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-				PCB->InvisibleObjectsOn)
+			if (FRONT(element) || PCB->InvisibleObjectsOn)
 			{
 				CopyElement(element);
 				changed = True;
@@ -317,15 +311,11 @@ Boolean CopyPastebufferToLayout(Position X, Position Y)
 		changed |= (PASTEBUFFER->Data->ViaN != 0);
 		VIA_LOOP(PASTEBUFFER->Data, CopyVia(via););
 	}
-	BlockDrawing = False;
 	if (changed)
 	{
-		AddBlockToUndoList(NULL);
+		Draw();
 		IncrementUndoSerialNumber(); 
-		DrawBlock(&bounds);
 	}
-	else
-		UndoFalseBlock();
 	return(changed);
 }
 
