@@ -42,8 +42,9 @@
 
 
 PolygonTypePtr	CopyPolygonLowLevel(PolygonTypePtr, PolygonTypePtr);
-ElementTypePtr	CopyElementLowLevel(ElementTypePtr, ElementTypePtr);
-Boolean			CopyPastebufferToLayout(Position, Position);
-void			*CopyObject(int, void *, void *, void *, Position, Position);
+ElementTypePtr	CopyElementLowLevel(DataTypePtr, ElementTypePtr,
+			ElementTypePtr, Boolean);
+Boolean		CopyPastebufferToLayout(Position, Position);
+void		*CopyObject(int, void *, void *, void *, Position, Position);
 
 #endif
