@@ -36,12 +36,16 @@
 DataTypePtr		CreateNewBuffer(void);
 PCBTypePtr		CreateNewPCB(Boolean);
 PinTypePtr		CreateNewVia(DataTypePtr, Position, Position,
-				Dimension, Dimension, char *, int);
+				Dimension, Dimension, Dimension, char *, int);
 LineTypePtr		CreateNewLineOnLayer(LayerTypePtr, Position,
-				Position, Position, Position, Dimension, int);
+				Position, Position, Position, Dimension,
+				Dimension, int);
+RatTypePtr		CreateNewRat(DataTypePtr, Position,
+				Position, Position, Position, Cardinal,
+				Cardinal, Dimension, int);
 ArcTypePtr		CreateNewArcOnLayer(LayerTypePtr, Position,
 				Position, Dimension, int,
-				int, Dimension, int);
+				int, Dimension, Dimension, int);
 PolygonTypePtr		CreateNewPolygonFromRectangle(LayerTypePtr,
 				Position, Position, Position, Position, int);
 TextTypePtr		CreateNewText(LayerTypePtr, FontTypePtr, Position,
@@ -51,20 +55,24 @@ PointTypePtr		CreateNewPointInPolygon(PolygonTypePtr,
 				Position, Position);
 ElementTypePtr		CreateNewElement(DataTypePtr, ElementTypePtr,
 				FontTypePtr, int, char *, char *, char *,
-				Position, Position, BYTE, int, int);
+				Position, Position, BYTE, int, int, Boolean);
 LineTypePtr		CreateNewLineInElement(ElementTypePtr, Position,
 				Position, Position, Position, Dimension);
 ArcTypePtr		CreateNewArcInElement(ElementTypePtr, Position,
 				Position, Dimension, Dimension,
 				int, int, Dimension);
 PinTypePtr		CreateNewPin(ElementTypePtr, Position, Position,
-				Dimension, Dimension, char *, int);
+				Dimension, Dimension, Dimension, Dimension,
+				char *, char *, int);
 PadTypePtr		CreateNewPad(ElementTypePtr, Position, Position,
-				Position, Position, Dimension, char *, int);
+				Position, Position, Dimension, Dimension,
+				Dimension, char *, char *, int);
 LineTypePtr		CreateNewLineInSymbol(SymbolTypePtr, Position,
 				Position, Position, Position, Dimension);
 void			CreateDefaultFont(void);
 RubberbandTypePtr	CreateNewRubberbandEntry(LayerTypePtr,
 				LineTypePtr, PointTypePtr);
+LibraryMenuTypePtr	CreateNewNet(LibraryTypePtr, char *, char *);
+LibraryEntryTypePtr	CreateNewConnection(LibraryMenuTypePtr, char *);
 
 #endif
