@@ -97,6 +97,7 @@ static void CreateTMPPolygon(PolygonTypePtr Polygon, Position DX, Position DY)
  */
 static void DrawCrosshair(void)
 {
+	XSetForeground(Dpy, Crosshair.GC, Settings.CrossColor);
 	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
 		TO_SCREEN_X(Crosshair.X), 0,
 		TO_SCREEN_X(Crosshair.X), MAX_COORD);
@@ -104,6 +105,7 @@ static void DrawCrosshair(void)
 	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
 		0, TO_SCREEN_Y(Crosshair.Y), MAX_COORD,
 		TO_SCREEN_Y(Crosshair.Y));
+	XSetForeground(Dpy, Crosshair.GC, Settings.CrosshairColor);
 }
 
 /*-----------------------------------------------------------
@@ -152,23 +154,27 @@ static void	XORDrawAttachedArc(Dimension thick)
 	arc.Width = arc.Height = wy;
 	bx = GetArcEnds(&arc);
 	arc.X -= wy;
-	arc.Y -= wy;
+	arc.Y -= TO_SCREEN_SIGN_Y(wy);
 	sa = (sa - 180) * 64;
 	dir *= 64;
 	XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
-		TO_SCREEN_X(arc.X - wid), TO_SCREEN_Y(arc.Y - wid),
-		TO_SCREEN(2*wy + thick), TO_SCREEN(2*wy + thick), sa, dir);
+		TO_SCREEN_X(arc.X - wid), TO_SCREEN_Y(arc.Y - TO_SCREEN_SIGN_Y(wid)),
+		TO_SCREEN(2*wy + thick), TO_SCREEN(2*wy + thick), TO_SCREEN_ANGLE(sa),
+		TO_SCREEN_DELTA(dir));
 	if (TO_SCREEN(wid) && (2*wy - thick) > 0 && TO_SCREEN(2*wy - thick))
 	{
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(arc.X + wid), TO_SCREEN_Y(arc.Y + wid),
-			TO_SCREEN(2*wy - thick), TO_SCREEN(2*wy - thick), sa, dir);
+			TO_SCREEN_X(arc.X + wid), TO_SCREEN_Y(arc.Y + TO_SCREEN_SIGN_Y(wid)),
+			TO_SCREEN(2*wy - thick), TO_SCREEN(2*wy - thick), TO_SCREEN_ANGLE(sa),
+			TO_SCREEN_DELTA(dir));
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(bx->X1 - wid), TO_SCREEN_Y(bx->Y1 - wid),
-			TO_SCREEN(thick), TO_SCREEN(thick), sa, -11520*SGN(dir));
+			TO_SCREEN_X(bx->X1 - wid), TO_SCREEN_Y(bx->Y1 - TO_SCREEN_SIGN_Y(wid)),
+			TO_SCREEN(thick), TO_SCREEN(thick), TO_SCREEN_ANGLE(sa),
+			TO_SCREEN_DELTA(-11520*SGN(dir)));
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(bx->X2 - wid), TO_SCREEN_Y(bx->Y2 - wid),
-			TO_SCREEN(thick), TO_SCREEN(thick), sa + dir, 11520*SGN(dir));
+			TO_SCREEN_X(bx->X2 - wid), TO_SCREEN_Y(bx->Y2 - TO_SCREEN_SIGN_Y(wid)),
+			TO_SCREEN(thick), TO_SCREEN(thick), TO_SCREEN_ANGLE(sa + dir),
+			TO_SCREEN_DELTA(11520*SGN(dir)));
 	}
 }
 
@@ -184,26 +190,29 @@ static void	XORDrawAttachedLine(Position x1, Position y1, Position x2,
 
 	dx = x2 - x1;
 	dy = y2 - y1;
-	h = wid / sqrt((float) (dx * dx + dy * dy));
-	ox = dy * h;
-	oy = - dx * h;
+	if (dx != 0 || dy != 0)
+		h = 0.5 * thick / sqrt((float) ((float) dx * dx + (float) dy * dy));
+	else
+		h = 0.0;
+	ox = dy * h + 0.5 * SGN(dy);
+	oy = - (dx * h + 0.5 * SGN(dx));
 	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
 		TO_SCREEN_X(x1 + ox), TO_SCREEN_Y(y1 + oy),
 		TO_SCREEN_X(x2 + ox), TO_SCREEN_Y(y2 + oy));
 	if (TO_SCREEN(abs(ox)) || TO_SCREEN(abs(oy)))
 	{
-		Position angle = atan(((float) dx)/dy) * 3666.9298888;
+		Position angle = TO_SCREEN_ANGLE(atan2((float) dx, (float) dy) * 3666.9298888);
 		XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
 			TO_SCREEN_X(x1 - ox), TO_SCREEN_Y(y1 - oy),
 			TO_SCREEN_X(x2 - ox), TO_SCREEN_Y(y2 - oy));
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(x1 - wid), TO_SCREEN_Y(y1 - wid),
+			TO_SCREEN_X(x1 - wid), TO_SCREEN_Y(y1 - TO_SCREEN_SIGN_Y(wid)),
 			TO_SCREEN(thick), TO_SCREEN(thick), angle,
-			dy < 0 ? -11520 : 11520);
+			TO_SCREEN_DELTA(11520));
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(x2 - wid), TO_SCREEN_Y(y2 - wid),
+			TO_SCREEN_X(x2 - wid), TO_SCREEN_Y(y2 - TO_SCREEN_SIGN_Y(wid)),
 			TO_SCREEN(thick), TO_SCREEN(thick), angle,
-			dy < 0 ? 11520 : -11520);
+			TO_SCREEN_DELTA(-11520));
 	}
 }
 
@@ -247,7 +256,27 @@ static void XORDrawElement(ElementTypePtr Element, Position DX, Position DY)
 				TO_SCREEN_Y(DY +pad->Point1.Y),
 				TO_SCREEN_X(DX +pad->Point2.X),
 				TO_SCREEN_Y(DY +pad->Point2.Y)););
-
+		/* mark */
+	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+		TO_SCREEN_X(Element->MarkX + DX - EMARK_SIZE),
+		TO_SCREEN_Y(Element->MarkY + DY),
+		TO_SCREEN_X(Element->MarkX + DX),
+		TO_SCREEN_Y(Element->MarkY + DY - EMARK_SIZE));
+	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+		TO_SCREEN_X(Element->MarkX + DX + EMARK_SIZE),
+		TO_SCREEN_Y(Element->MarkY + DY),
+		TO_SCREEN_X(Element->MarkX + DX),
+		TO_SCREEN_Y(Element->MarkY + DY - EMARK_SIZE));
+	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+		TO_SCREEN_X(Element->MarkX + DX - EMARK_SIZE),
+		TO_SCREEN_Y(Element->MarkY + DY),
+		TO_SCREEN_X(Element->MarkX + DX),
+		TO_SCREEN_Y(Element->MarkY + DY + EMARK_SIZE));
+	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+		TO_SCREEN_X(Element->MarkX + DX + EMARK_SIZE),
+		TO_SCREEN_Y(Element->MarkY + DY),
+		TO_SCREEN_X(Element->MarkX + DX),
+		TO_SCREEN_Y(Element->MarkY + DY + EMARK_SIZE));
 }
 
 /* ---------------------------------------------------------------------------
@@ -263,12 +292,17 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
 	y = Crosshair.Y -Buffer->Y;
 
 		/* draw all visible layers */
-	for (i = 0; i < MAX_LAYER; i++)
+	for (i = 0; i < MAX_LAYER + 2; i++)
 		if (PCB->Data->Layer[i].On)
 		{
 			LayerTypePtr	layer = &Buffer->Data->Layer[i];
 
 			LINE_LOOP(layer,
+/*
+				XORDrawAttachedLine(x +line->Point1.X,
+					y +line->Point1.Y, x +line->Point2.X,
+					y +line->Point2.Y, line->Thickness);
+*/
 				XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
 					TO_SCREEN_X(x +line->Point1.X),
 					TO_SCREEN_Y(y +line->Point1.Y),
@@ -287,10 +321,12 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
 			TEXT_LOOP(layer,
 				{
 					BoxTypePtr	box = &text->BoundingBox;
+					Position	y0;
 
+					y0 = Settings.ShowSolderSide ? box->Y2 : box->Y1;
 					XDrawRectangle(Dpy, Output.OutputWindow, Crosshair.GC,
 						TO_SCREEN_X(x +box->X1),
-						TO_SCREEN_Y(y +box->Y1),
+						TO_SCREEN_Y(y +y0),
 						TO_SCREEN(box->X2 -box->X1),
 						TO_SCREEN(box->Y2 -box->Y1));
 				}
@@ -310,8 +346,7 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
 		/* draw elements if visible */
 	if (PCB->PinOn && PCB->ElementOn)
 		ELEMENT_LOOP(Buffer->Data,
-			if ((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-			PCB->InvisibleObjectsOn)
+			if (FRONT(element) || PCB->InvisibleObjectsOn)
 				XORDrawElement(element, x, y);
 		);
 
@@ -320,7 +355,7 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
 		VIA_LOOP(Buffer->Data,
 			XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
 				TO_SCREEN_X(x +via->X -via->Thickness/2),
-				TO_SCREEN_Y(y +via->Y -via->Thickness/2),
+				TO_SCREEN_Y(y +via->Y -TO_SCREEN_SIGN_Y(via->Thickness/2)),
 				TO_SCREEN(via->Thickness), TO_SCREEN(via->Thickness),
 				0, 360*64);
 		);
@@ -347,7 +382,7 @@ static void XORDrawInsertPointObject(void)
 }
 
 /* ---------------------------------------------------------------------------
- * draws the attched object while in RUBBERBAND_MOVE, MOVE_MODE or COPY_MODE
+ * draws the attched object while in MOVE_MODE or COPY_MODE
  */
 static void XORDrawMoveOrCopyObject(void)
 {
@@ -364,7 +399,7 @@ static void XORDrawMoveOrCopyObject(void)
 
 			XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
 				TO_SCREEN_X(via->X +dx -via->Thickness/2),
-				TO_SCREEN_Y(via->Y +dy -via->Thickness/2),
+				TO_SCREEN_Y(via->Y +dy) -TO_SCREEN(via->Thickness/2),
 				TO_SCREEN(via->Thickness),
 				TO_SCREEN(via->Thickness),0,360*64);
 			break;
@@ -374,11 +409,9 @@ static void XORDrawMoveOrCopyObject(void)
 		{
 			LineTypePtr	line = (LineTypePtr) Crosshair.AttachedObject.Ptr2;
 
-			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(line->Point1.X +dx),
-				TO_SCREEN_Y(line->Point1.Y +dy),
-				TO_SCREEN_X(line->Point2.X +dx),
-				TO_SCREEN_Y(line->Point2.Y +dy));
+			XORDrawAttachedLine(line->Point1.X +dx, line->Point1.Y +dy,
+				line->Point2.X +dx, line->Point2.Y +dy,
+				line->Thickness);
 			break;
 		}
 
@@ -474,9 +507,19 @@ static void XORDrawMoveOrCopyObject(void)
 			break;
 		}
 
-			/* element names are moved like normal text objects */
-		case TEXT_TYPE:
 		case ELEMENTNAME_TYPE:
+		{
+		/* locate the element "mark" and draw an association line from crosshair to it */
+			ElementTypePtr	element = (ElementTypePtr) Crosshair.AttachedObject.Ptr1;
+
+			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+				TO_SCREEN_X(element->MarkX),
+				TO_SCREEN_Y(element->MarkY),
+				TO_SCREEN_X(Crosshair.X),
+				TO_SCREEN_Y(Crosshair.Y));
+			/* fall through to move the text as a box outline */
+		}
+		case TEXT_TYPE:
 		{
 			TextTypePtr	text = (TextTypePtr) Crosshair.AttachedObject.Ptr2;
 			BoxTypePtr	box = &text->BoundingBox;
@@ -547,7 +590,7 @@ static void DrawAttached(Boolean BlockToo)
 		case VIA_MODE:
 			XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
 				TO_SCREEN_X(Crosshair.X -Settings.ViaThickness/2),
-				TO_SCREEN_Y(Crosshair.Y -Settings.ViaThickness/2),
+				TO_SCREEN_Y(Crosshair.Y) - TO_SCREEN(Settings.ViaThickness/2),
 				TO_SCREEN(Settings.ViaThickness),
 				TO_SCREEN(Settings.ViaThickness),
 				0, 360*64);
@@ -602,7 +645,6 @@ static void DrawAttached(Boolean BlockToo)
 
 		case COPY_MODE:
 		case MOVE_MODE:
-		case RUBBERBANDMOVE_MODE:
 			XORDrawMoveOrCopyObject();
 			break;
 
@@ -640,7 +682,9 @@ void CrosshairOn(Boolean BlockToo)
 	if (!Crosshair.On)
 	{
 		Crosshair.On = True;
+		XSync(Dpy, False);
 		DrawAttached(BlockToo);
+		DrawMark(True);
 	}
 }
 
@@ -652,7 +696,9 @@ void CrosshairOff(Boolean BlockToo)
 	if (Crosshair.On)
 	{
 		Crosshair.On = False;
+		XSync(Dpy, False);
 		DrawAttached(BlockToo);
+		DrawMark(True);
 	}
 }
 
@@ -686,33 +732,20 @@ void RestoreCrosshair(Boolean BlockToo)
  */
 static void FitCrosshairIntoGrid(Position X, Position Y)
 {
-	Position	x1, y1, x2, y2, height, width,
-			dx1, dy1, dx2, dy2, ox, oy;
+	Position	x2, y2;
 	
-		/* When the widget has just been changed, the Output structure
-		 * can have stale values, so we get the fresh ones
-		 */
-	XtVaGetValues(Output.Output, XtNx, &ox, XtNy, &oy, XtNheight, &height, XtNwidth, &width, NULL);
-	oy = - oy;
-	ox = - ox;
 		/* get PCB coordinates from visible display size.
 		 * If the bottom view mode is active, y2 might be less then y1
 		 */
-	dx1 = TO_PCB_X(ox);
-	dy1 = TO_PCB_Y(oy);
-	dx2 = TO_PCB_X(ox +width -1);
-	dy2 = TO_PCB_Y(oy +height -1);
-	x1 = MIN(dx1, dx2);
-	y1 = MIN(dy1, dy2);
-	x2 = MAX(dx1, dx2);
-	y2 = MAX(dy1, dy2);
+	x2 = TO_PCB_X(Output.Width -1);
+	y2 = MAX(TO_PCB_Y(0), TO_PCB_Y(Output.Height -1));
 
-		/* check position agains window size and agains valid
+		/* check position agains window size and against valid
 		 * coordinates determined by the size of an attached
 		 * object or buffer
 		 */
-	Crosshair.X = MIN(x2, MAX(x1, X));
-	Crosshair.Y = MIN(y2, MAX(y1, Y));
+	Crosshair.X = MIN(x2, MAX(0, X));
+	Crosshair.Y = MIN(y2, MAX(0, Y));
 	Crosshair.X = MIN(Crosshair.MaxX, MAX(Crosshair.MinX, Crosshair.X));
 	Crosshair.Y = MIN(Crosshair.MaxY, MAX(Crosshair.MinY, Crosshair.Y));
 	
@@ -721,28 +754,28 @@ static void FitCrosshairIntoGrid(Position X, Position Y)
 		 * In this case we just set it to the center of the window or
 		 * with respect to the grid (if possible)
 		 */
-	if (Crosshair.X < x1 || Crosshair.X > x2)
+	if (Crosshair.X < 0 || Crosshair.X > x2)
 	{
-		if (x2 -x1 +1 >= PCB->Grid)
+		if (x2 +1 >= PCB->Grid)
 				/* there must be a point that matches the grid 
 				 * so we just have to look for it with some integer
 				 * calculations
 				 */
-			Crosshair.X = GRIDFIT_X(x1 +PCB->Grid, PCB->Grid);
+			Crosshair.X = GRIDFIT_X(PCB->Grid, PCB->Grid);
 		else
-			Crosshair.X = (x1+x2)/2;
+			Crosshair.X = (x2)/2;
 	}
 	else
 			/* check if the new position matches the grid */
 		Crosshair.X = GRIDFIT_X(Crosshair.X, PCB->Grid);
 
 		/* do the same for the second coordinate */
-	if (Crosshair.Y < y1 || Crosshair.Y > y2)
+	if (Crosshair.Y < 0 || Crosshair.Y > y2)
 	{
-		if (y2 -y1 +1 >= PCB->Grid)
-			Crosshair.Y = GRIDFIT_Y(y1 +PCB->Grid, PCB->Grid);
+		if (y2 +1 >= PCB->Grid)
+			Crosshair.Y = GRIDFIT_Y(PCB->Grid, PCB->Grid);
 		else
-			Crosshair.Y = (y1+y2)/2;
+			Crosshair.Y = (y2)/2;
 	}
 	else
 		Crosshair.Y = GRIDFIT_Y(Crosshair.Y, PCB->Grid);
@@ -779,6 +812,23 @@ void SetCrosshairRange(Position MinX, Position MinY,
 	MoveCrosshairRelative(0, 0);
 }
 
+/* --------------------------------------------------------------------------
+ * draw the marker position
+ * if argument is True, draw only if it is visible, otherwise draw it regardless
+ */
+void	DrawMark(Boolean ifvis)
+{
+	if (Marked.status || !ifvis)
+	{
+		XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(Marked.X - MARK_SIZE), TO_SCREEN_Y(Marked.Y - MARK_SIZE),
+			TO_SCREEN_X(Marked.X + MARK_SIZE), TO_SCREEN_Y(Marked.Y + MARK_SIZE));
+		XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(Marked.X + MARK_SIZE), TO_SCREEN_Y(Marked.Y - MARK_SIZE),
+			TO_SCREEN_X(Marked.X - MARK_SIZE), TO_SCREEN_Y(Marked.Y + MARK_SIZE));
+	}
+}
+
 /* ---------------------------------------------------------------------------
  * initializes crosshair stuff
  * clears the struct, allocates to graphical contexts and
