@@ -30,8 +30,6 @@ static	char	*rcsid = "$Id: data.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
  */
 #include "global.h"
 #include "dev_ps.h"
-#include "dev_gerber.h"
-#include "dev_gerberX.h"
 #include "dev_rs274x.h"
 
 /* ---------------------------------------------------------------------------
@@ -40,23 +38,32 @@ static	char	*rcsid = "$Id: data.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 XtAppContext	Context;		/* application context */
 Display		*Dpy;			/* my display */
 CrosshairType	Crosshair;		/* information about cursor settings */
-MarkType      Marked;                         /* a cross-hair mark */
+MarkType	Marked;                         /* a cross-hair mark */
 OutputType	Output;			/* some widgets ... used for drawing */
 PCBTypePtr	PCB;			/* pointer to layout struct */
-char		*Progname;		/* the programs name */
+Pixmap		Offscreen = BadAlloc;	/* off screen drawing pixmap */
+Pixmap		Offmask = BadAlloc;	/* off screen mask pixmap */
+char		*Progname;		/* the program's name */
 SettingType	Settings;		/* some resources */
-Boolean		RedrawOnEnter;		/* a flag, indicating that settings */
-					/* have changed */
 int		LayerStack[MAX_LAYER];	/* determines the order in */
 					/* the layers are drawn */
 Atom		WMDeleteWindowAtom;	/* all toplevel windows get */
 					/* 'Delete' messages */
-BufferType	Buffers[MAX_BUFFER][2];	/* my buffers */
+BufferType	Buffers[MAX_BUFFER];	/* my buffers */
 LibraryType	Library;		/* the library */
-XExposeEvent	Erasure;	/* whether something has been erased */
 Region		UpRegion;	/* the region of updated drawing */
 Region		FullRegion;	/* the full screen clip region */
-Boolean		BlockDrawing = False; /* set to true if the clip region will be too complex */
+Boolean		Bumped = False;		/* whether the undo serial number has changed */
+Window		LogWindID;
+Position	Xorig, Yorig;		/* origin offset for drawing directly in pixmap */
+Boolean		render;			/* wether or not to re-render the pixmap */
+Pixmap		*Stipples,
+		XC_clock_source,
+		XC_clock_mask;
+int		ShiftKeyIndex;
+char		ShiftKeyMask;
+int		addedLines;
+Position	vxl, vxh, vyl, vyh;	/* approximate visible pcb coordinates */
 
 /* ---------------------------------------------------------------------------
  * set all keys which generate a new line not 'no operation'.
@@ -80,12 +87,4 @@ DeviceInfoType	PrintingDevice[] = {
 	{ GBX_Queryh, NULL },
 /*	{ GBX_Query, NULL }, */
 	{ NULL, NULL }};
-Pixmap	Stipples[] = {
-	NULL, NULL, NULL, NULL,
-	NULL, NULL, NULL, NULL,
-	NULL};
-
-int	ShiftKeyIndex;
-char	ShiftKeyMask;
-int	addedLines;
 
