@@ -43,21 +43,25 @@ extern	CrosshairType	Crosshair;
 extern  MarkType	Marked;
 extern	OutputType	Output;
 extern	PCBTypePtr	PCB;
+extern	Pixmap		Offscreen, Offmask;
 extern	char		*Progname;
 extern	SettingType	Settings;
 extern	Boolean		RedrawOnEnter;
 extern	int		LayerStack[MAX_LAYER];
 extern	String		InputTranslations;
 extern	Atom		WMDeleteWindowAtom;
-extern	BufferType	Buffers[MAX_BUFFER][2];
+extern	BufferType	Buffers[MAX_BUFFER];
 extern	LibraryType	Library;
 extern	DeviceInfoType	PrintingDevice[];
-extern	Pixmap		Stipples[];
+extern	Pixmap		*Stipples, XC_clock_source, XC_clock_mask;
 extern	int		ShiftKeyIndex;
 extern	char		ShiftKeyMask;
 extern	int		addedLines;
-extern	XExposeEvent	Erasure;
 extern  Region		UpRegion;
 extern	Region		FullRegion;
-extern  Boolean		BlockDrawing;
+extern	Boolean		Bumped;
+extern	Window		LogWindID;
+extern	Position	Xorig, Yorig;
+extern	Boolean		render;
+extern	Position	vxl, vxh, vyl, vyh;
 #endif
