@@ -68,26 +68,23 @@ static	void	PS_Postamble(void);
 static	void	PS_EPS_Exit(Boolean);
 static	void	PS_Init(PrintInitTypePtr);
 static	void	EPS_Init(PrintInitTypePtr);
-static	void	PS_SetColor(XColor, int);
-static	void	PS_PrintLayer(LayerTypePtr, int, Boolean, int);
-static	void	PS_PrintLine(LineTypePtr, int);
-static	void	PS_PrintArc(ArcTypePtr, int);
-static	void	PS_PrintPolygon(PolygonTypePtr, int);
-static	void	PS_PrintText(TextTypePtr, int);
-static	void	PS_PrintTextLowLevel(TextTypePtr, int);
-static	void	PS_PrintElementPackage(ElementTypePtr, int);
+static	void	PS_SetColor(XColor);
+static	void	PS_Invert(int);
+static	void	PS_PrintLine(LineTypePtr, Boolean);
+static	void	PS_PrintArc(ArcTypePtr, Boolean);
+static	void	PS_PrintPolygon(PolygonTypePtr);
+static	void	PS_PrintText(TextTypePtr);
+static	void	PS_PrintTextLowLevel(TextTypePtr);
+static	void	PS_PrintElementPackage(ElementTypePtr);
 static	void	PS_PrintPad(PadTypePtr, int);
 static	void	PS_PrintPinOrVia(PinTypePtr, int);
-static	void	PS_PrintClearPinOrViaOnGroundplane(PinTypePtr, int);
-static	void	PS_PrintPadMask(PadTypePtr, int);
-static	void	PS_PrintPinOrViaMask(PinTypePtr, int);
-static	void	PS_FilledRectangle(Position, Position,
-			Position, Position, int);
 static	void	PS_Outline(Position, Position,
-			Position, Position, int);
+			Position, Position);
 static	void	PS_Alignment(Position, Position,
-			Position, Position, int);
+			Position, Position);
 static	void	PS_DrillHelper(PinTypePtr, int);
+static	void	PS_FilledRectangle(Position, Position,
+			Position, Position);
 
 /* ----------------------------------------------------------------------
  * some local identifiers
@@ -97,6 +94,8 @@ static	void	PS_DrillHelper(PinTypePtr, int);
  *   A:         draws an element arc
  *   Alignment: draws the boards outline
  *   B:         draws a filled box
+ *   CA:	clears an arc
+ *   CL:	clears a line
  *   CLRB       clears a box
  *   CLRPA:     clears a pad
  *   CLRPV:     clears an octagon 
@@ -125,31 +124,26 @@ static	PrintDeviceType	PS_QueryConstants = {
 	PS_Preamble,
 	PS_Postamble,
 	PS_SetColor,
-	NULL,				/* no drill information */
-	PS_PrintLayer,
-	PS_PrintElementPackage,
+	PS_Invert,
+	PS_PrintLine,
+	PS_PrintArc,
+	PS_PrintPolygon,
 	PS_PrintText,
 	PS_PrintPad,
 	PS_PrintPinOrVia,
-	PS_PrintPadMask,
-	PS_PrintPinOrViaMask,
-	PS_PrintClearPinOrViaOnGroundplane,
-	PS_FilledRectangle,
+	PS_PrintElementPackage,
+	NULL,				/* no drill information */
 	PS_Outline,
 	PS_Alignment,
 	PS_DrillHelper,
+	NULL,				/* no group IDs */
 	True,				/* driver can handle color */
-	True,				/* and inverted output */
-	True,				/* handles ground planes */
 	False,				/* handles no drill information */
-	True,				/* handles mask layers */
 	True,				/* handles different media */
 	True,				/* allows mirroring */
-	True,				/* allows rotate */
-	False,				/* needs preprint calls */
-	False },			/* needs an AppMap */
+	True },				/* allows rotate */
 
-			EPS_QueryConstants = {
+	EPS_QueryConstants = {
 	"encapsulated PostScript",
 	"eps",
 	EPS_Init,
@@ -157,29 +151,24 @@ static	PrintDeviceType	PS_QueryConstants = {
 	EPS_Preamble,
 	EPS_Postamble,
 	PS_SetColor,
-	NULL,				/* no drill information */
-	PS_PrintLayer,
-	PS_PrintElementPackage,
+	PS_Invert,
+	PS_PrintLine,
+	PS_PrintArc,
+	PS_PrintPolygon,
 	PS_PrintText,
 	PS_PrintPad,
 	PS_PrintPinOrVia,
-	PS_PrintPadMask,
-	PS_PrintPinOrViaMask,
-	PS_PrintClearPinOrViaOnGroundplane,
-	PS_FilledRectangle,
+	PS_PrintElementPackage,
+	NULL,				/* no drill information */
 	PS_Outline,
 	PS_Alignment,
 	PS_DrillHelper,
-	True,
-	True,
+	NULL,				/* no group ID */
 	True,
 	False,
-	True,
 	False,		/* encapsulated doesn't allow media changes */
 	True,
-	True,				/* allows rotate */
-	False,		/* needs preprint calls */
-	False};
+	True };				/* allows rotate */
 
 static	PrintInitType	PS_Flags;
 
@@ -225,7 +214,7 @@ static	char	*PS_Functions[] = {
 	"% pin or via, x, y and thickness are on the stack",
 	"	/drillinghole exch def /thickness exch def /y exch def /x exch def",
 	"	x y thickness P",
-	"% draw drilling whole",
+	"% draw drilling hole",
 	"	gsave White 0 setlinewidth",
 	"   newpath x y drillinghole 2 div 0 360 arc closepath fill stroke",
 	"   grestore",
@@ -248,7 +237,7 @@ static	char	*PS_Functions[] = {
 	"	newpath x thickness 2 div sub y thickness 2 div sub moveto",
 	"	thickness 0 rlineto 0 thickness rlineto",
 	"	thickness neg 0 rlineto closepath fill stroke",
-	"% draw drilling whole",
+	"% draw drilling hole",
 	"	gsave White 0 setlinewidth",
 	"   newpath x y drillinghole 2 div 0 360 arc closepath fill stroke",
 	"   grestore",
@@ -270,6 +259,14 @@ static	char	*PS_Functions[] = {
 	"	grestore",
 	"} def",
 	"",
+	"/CL {",
+	"% line, get x1, y1, x2, y2 and thickness from stack",
+	"	/thick exch def /y2 exch def /x2 exch def /y1 exch def /x1 exch def",
+	"	gsave White thick setlinewidth",
+	"	x1 y1 moveto x2 y2 lineto stroke",
+	"	grestore",
+	"} def",
+	"",
 	"/B {",
 	"% filled box, get x1, y1, x2 and y2 from stack",
 	"	/y2 exch def /x2 exch def /y1 exch def /x1 exch def",
@@ -296,6 +293,19 @@ static	char	*PS_Functions[] = {
 	"	grestore",
 	"} def",
 	"",
+	"/CA {",
+	"% arc for elements, get x, y, width, height, thickness",
+	"% startangle and delta-angle from stack",
+	"  /delta exch def /start exch def /thickness exch def",
+	"  /height exch def /width exch def /y exch def /x exch def",
+	"% draw it",
+	"	gsave White thickness setlinewidth /save DictMatrix currentmatrix def",
+	"% scaling is less then zero because the coord system has to be swapped",
+	"	x y translate width neg height scale",
+	"	0 0 1 start start delta add arc save setmatrix stroke",
+	"	grestore",
+	"} def",
+	"",
 	"/CLRPV {",
 	"% clears a pin/via for groundplane; x,y and thickness are on stack",
 	"   /thickness exch def /y exch def /x exch def",
@@ -362,9 +372,18 @@ static	char	*PS_Functions[] = {
 	"% alignment targets, get x1, y1, x2, y2 and distance from stack",
 	"	/dis exch def /y2 exch def /x2 exch def /y1 exch def /x1 exch def",
 	"   gsave 0.175 setlinewidth",
-	"   newpath x1 y1 dis add moveto 0 dis neg rlineto dis 0 rlineto stroke",
+	"   newpath x1 y1 dis add moveto",
+	"      0 dis 2 mul neg rlineto",
+	"      dis neg dis rmoveto",
+	"      dis 2 mul 0 rlineto",
+	"   stroke",
+	"   newpath x1 y1 dis 0 90 arcn stroke",
 	"   newpath x1 y2 dis sub moveto 0 dis rlineto dis 0 rlineto stroke",
-	"   newpath x2 y2 dis sub moveto 0 dis rlineto dis neg 0 rlineto stroke",
+	"   newpath x2 y2 dis sub moveto",
+	"      0 2 dis mul rlineto",
+	"      dis dup neg rmoveto",
+	"      2 dis mul neg 0 rlineto"
+	"   stroke",
 	"   newpath x2 y1 dis add moveto 0 dis neg rlineto dis neg 0 rlineto stroke",
 	"   grestore",
 	"} def",
@@ -434,6 +453,18 @@ static char *EPS_Preamble(PrintInitTypePtr Flags, char *Description)
 static void EPS_Init(PrintInitTypePtr wasted)
 {
 }
+static void PS_Invert(int mode)
+{
+	if (mode == 1)
+	{
+		if (PS_Flags.InvertFlag)
+			PrintStringArray(PS_ColorFunctions,
+				ENTRIES(PS_ColorFunctions), PS_Flags.FP);
+		else
+			PrintStringArray(PS_InvertColorFunctions,
+				ENTRIES(PS_InvertColorFunctions), PS_Flags.FP);
+	}
+}
 
 /* ----------------------------------------------------------------------
  * prints PostScript or enc. PostScript header with function definition
@@ -642,78 +673,56 @@ static void PS_EPS_Exit(Boolean CreateEPS)
 	fputs("%%EOF\n", PS_Flags.FP);
 }
 
-/* ----------------------------------------------------------------------
- * prints layer data
- */
-static void PS_PrintLayer(LayerTypePtr Layer, int GroupNumber,
-	Boolean SilkscreenTextFlag, int unused)
-{
-        int ThermLayerFlag, LayerPIPFlag;
-
-	fprintf(PS_Flags.FP, "%% LayerGroup: %i, Layer: \"%s\" (#%i)\n",
-		GroupNumber,
-		UNKNOWN(Layer->Name),
-		GetLayerNumber(PCB->Data, Layer));
-	LINE_LOOP(Layer, PS_PrintLine(line, unused));
-	ARC_LOOP(Layer, PS_PrintArc(arc, unused));
-        TEXT_LOOP(Layer, {
-                if (!TEST_FLAG(ONSILKFLAG, text))
-                        PS_PrintTextLowLevel(text, 0);
-        });
-        POLYGON_LOOP(Layer, PS_PrintPolygon(polygon, 0));
-          ThermLayerFlag = L0THERMFLAG << GetLayerNumber(PCB->Data, Layer);
-          LayerPIPFlag = L0PIPFLAG << GetLayerNumber(PCB->Data, Layer);
-          ALLPIN_LOOP(PCB->Data,
-                  if (TEST_FLAG(LayerPIPFlag, pin) && !TEST_FLAG(HOLEFLAG, pin))
-                {
-                        if (TEST_FLAG(ThermLayerFlag, pin))
-                                SET_FLAG(USETHERMALFLAG, pin);
-                        PS_PrintClearPinOrViaOnGroundplane(pin, 0);
-                }
-          );
-          VIA_LOOP(PCB->Data,
-                  if (TEST_FLAG(LayerPIPFlag, via) && !TEST_FLAG(HOLEFLAG, via))
-                {
-                        if (TEST_FLAG(ThermLayerFlag, via))
-                                SET_FLAG(USETHERMALFLAG, via);
-                        PS_PrintClearPinOrViaOnGroundplane(via, 0);
-                }
-          );
- 
-}
-
 /* ----------------------------------------------------------------------
  * prints a line
  */
-static void PS_PrintLine(LineTypePtr Line, int unused)
+static void PS_PrintLine(LineTypePtr Line, Boolean Clear)
 {
-	fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
-		(int) Line->Point1.X,
-		(int) Line->Point1.Y,
-		(int) Line->Point2.X,
-		(int) Line->Point2.Y,
-		(int) Line->Thickness);
+	if (Clear)
+		fprintf(PS_Flags.FP, "%d %d %d %d %d CL\n",
+			(int) Line->Point1.X,
+			(int) Line->Point1.Y,
+			(int) Line->Point2.X,
+			(int) Line->Point2.Y,
+			(int) Line->Thickness + (int) Line->Clearance);
+	else
+		fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
+			(int) Line->Point1.X,
+			(int) Line->Point1.Y,
+			(int) Line->Point2.X,
+			(int) Line->Point2.Y,
+			(int) Line->Thickness);
 }
 
 /* ---------------------------------------------------------------------------
  * prints an arc 
  */
-static void PS_PrintArc(ArcTypePtr arc, int unused)
+static void PS_PrintArc(ArcTypePtr arc, Boolean Clear)
 {
-	fprintf(PS_Flags.FP, "%d %d %d %d %d %d %d A\n",
-		(int) arc->X,
-		(int) arc->Y,
-		(int) arc->Width,
-		(int) arc->Height,
-		(int) arc->Thickness,
-		arc->StartAngle,
-		arc->Delta);
+	if (Clear)
+		fprintf(PS_Flags.FP, "%d %d %d %d %d %d %d CA\n",
+			(int) arc->X,
+			(int) arc->Y,
+			(int) arc->Width,
+			(int) arc->Height,
+			(int) arc->Thickness + (int) arc->Clearance,
+			(arc->Delta < 0) ? arc->StartAngle + arc->Delta : arc->StartAngle,
+			(arc->Delta < 0) ? -arc->Delta : arc->Delta);
+	else
+		fprintf(PS_Flags.FP, "%d %d %d %d %d %d %d A\n",
+			(int) arc->X,
+			(int) arc->Y,
+			(int) arc->Width,
+			(int) arc->Height,
+			(int) arc->Thickness,
+			(arc->Delta < 0) ? arc->StartAngle + arc->Delta : arc->StartAngle,
+			(arc->Delta < 0) ? -arc->Delta : arc->Delta);
 }
 
 /* ---------------------------------------------------------------------------
  * prints a filled polygon
  */
-static void PS_PrintPolygon(PolygonTypePtr Ptr, int unused)
+static void PS_PrintPolygon(PolygonTypePtr Ptr)
 {
 	int i = 0;
 
@@ -730,12 +739,12 @@ static void PS_PrintPolygon(PolygonTypePtr Ptr, int unused)
 }
 
 /* ----------------------------------------------------------------------
- * lowlevel routine to print a text
+ * lowlevel routine to print text
  * the routine is identical to DrawText() in module draw.c except
  * that DrawLine() and DrawRectangle() are replaced by their corresponding
  * printing routines
  */
-static void PS_PrintTextLowLevel(TextTypePtr Text, int unused)
+static void PS_PrintTextLowLevel(TextTypePtr Text)
 {
 	Position		x = 0,
 				width;
@@ -782,7 +791,7 @@ static void PS_PrintTextLowLevel(TextTypePtr Text, int unused)
 				newline.Point1.Y += Text->Y;
 				newline.Point2.X += Text->X;
 				newline.Point2.Y += Text->Y;
-				PS_PrintLine(&newline, unused);
+				PS_PrintLine(&newline, False);
 			}
 
 				/* move on to next cursor position */
@@ -824,18 +833,17 @@ static void PS_PrintTextLowLevel(TextTypePtr Text, int unused)
  * Albert John FitzPatrick III <ajf_nylorac@acm.org>
  * see ../CHANGES for details
  */
-static void PS_PrintText(TextTypePtr Text, int unused)
+static void PS_PrintText(TextTypePtr Text)
 {
-		/* FIXME: This code has not been tested. */
-	PS_PrintTextLowLevel(Text, unused);
+	PS_PrintTextLowLevel(Text);
 }
 
 /* ----------------------------------------------------------------------
  * prints package outline
  */
-static void PS_PrintElementPackage(ElementTypePtr Element, int unused)
+static void PS_PrintElementPackage(ElementTypePtr Element)
 {
-	ELEMENTLINE_LOOP(Element, PS_PrintLine(line, unused););
+	ELEMENTLINE_LOOP(Element, PS_PrintLine(line, False););
 	ARC_LOOP(Element, 
 		fprintf(PS_Flags.FP, "%d %d %d %d %d %d %d A\n",
 			(int) arc->X,
@@ -847,112 +855,109 @@ static void PS_PrintElementPackage(ElementTypePtr Element, int unused)
 			arc->Delta);
 	);
 	if (!TEST_FLAG(HIDENAMEFLAG, Element))
-		PS_PrintTextLowLevel(&ELEMENT_TEXT(PCB, Element), unused);
+		PS_PrintTextLowLevel(&ELEMENT_TEXT(PCB, Element));
 }
 
 /* ----------------------------------------------------------------------
  * prints a pad
  */
-static void PS_PrintPad(PadTypePtr Pad, int unused)
+static void PS_PrintPad(PadTypePtr Pad, int mode)
 {
+	Dimension scrunch = 0;
+
+	switch (mode)
+	{
+		case 0:
+		case 3:
+			scrunch = Pad->Thickness;
+			break;
+		case 1:
+			scrunch = (Pad->Thickness + Pad->Clearance);
+			break;
+		case 2:
+			scrunch = Pad->Mask;
+			break;
+	}
+			
 	if (TEST_FLAG(SQUAREFLAG, Pad))
 	{
-		Dimension scrunch = Pad->Thickness/2;
-	fprintf(PS_Flags.FP, "%d %d %d %d B\n",
+		scrunch /= 2;
+	fprintf(PS_Flags.FP, "%d %d %d %d %s\n",
 		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point2.X  : Pad->Point1.X) - scrunch,
 		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point2.Y  : Pad->Point1.Y) - scrunch,
 		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point1.X  : Pad->Point2.X) + scrunch,
-		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point1.Y  : Pad->Point2.Y) + scrunch);
+		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point1.Y  : Pad->Point2.Y) + scrunch,
+		(mode == 1 || mode == 2) ? "CLRB" : "B");
 	}
 	else
-	fprintf(PS_Flags.FP, "%d %d %d %d %d PA\n",
+	fprintf(PS_Flags.FP, "%d %d %d %d %d %s\n",
 		(int) Pad->Point1.X,
 		(int) Pad->Point1.Y,
 		(int) Pad->Point2.X,
 		(int) Pad->Point2.Y,
-		(int) Pad->Thickness);
+		(int) scrunch,
+		(mode == 1 || mode == 2) ? "CLRPA" : "PA");
 }
 
 /* ----------------------------------------------------------------------
  * prints a via or pin
  */
-static void PS_PrintPinOrVia(PinTypePtr Ptr, int unused)
+static void PS_PrintPinOrVia(PinTypePtr Ptr, int mode)
 {
-	fprintf(PS_Flags.FP, "%d %d %d %d %s\n",
-		(int) Ptr->X,
-		(int) Ptr->Y,
-		(int) Ptr->Thickness,
-		(int) Ptr->DrillingHole,
-		TEST_FLAG(SQUAREFLAG, Ptr) ? "PVSQ" : TEST_FLAG(OCTAGONFLAG, Ptr) ? "PV" : "PVR");
-}
+	int size = 0;
 
-/* ----------------------------------------------------------------------
- * clears the area around a via or pin on the groundplane
- */
-static void PS_PrintClearPinOrViaOnGroundplane(PinTypePtr Ptr, int unused)
-{
-        Dimension       half = Ptr->Thickness/2 + GROUNDPLANEFRAME;
-
-	fprintf(PS_Flags.FP, "%d %d %d %s\n",
-		(int) Ptr->X,
-		(int) Ptr->Y,
-		(int) (Ptr->Thickness +2 *GROUNDPLANEFRAME),
-		TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" : TEST_FLAG(OCTAGONFLAG, Ptr) ? "CLRPV" : "CLRPVR");
-        if (TEST_FLAG(USETHERMALFLAG, Ptr)) {
-                if (!TEST_FLAG(SQUAREFLAG, Ptr))
-                        half = (7 * half) / 10;
-                fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
-                        Ptr->X - half, Ptr->Y - half,
-                        Ptr->X + half, Ptr->Y + half,
-                        GROUNDPLANEFRAME);
-                fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
-                        Ptr->X - half, Ptr->Y + half,
-                        Ptr->X + half, Ptr->Y - half,
-                        GROUNDPLANEFRAME);
-                CLEAR_FLAG(USETHERMALFLAG, Ptr);
-        }
-}
-
-/* ----------------------------------------------------------------------
- * prints a pad mask
- */
-static void PS_PrintPadMask(PadTypePtr Pad, int unused)
-{
-	if (TEST_FLAG(SQUAREFLAG, Pad))
+	switch (mode)
 	{
-		Dimension scrunch = Pad->Thickness/2 + GROUNDPLANEFRAME;
-	fprintf(PS_Flags.FP, "%d %d %d %d CLRB\n",
-		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point2.X  : Pad->Point1.X) - scrunch,
-		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point2.Y  : Pad->Point1.Y) - scrunch,
-		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point1.X  : Pad->Point2.X) + scrunch,
-		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point1.Y  : Pad->Point2.Y) + scrunch);
+		case 0:
+			size = Ptr->Thickness;
+			break;
+		case 1:
+			size = Ptr->Thickness + Ptr->Clearance;
+			break;
+		case 2:
+			size = Ptr->Mask;
+			break;
 	}
-	else
-	fprintf(PS_Flags.FP, "%d %d %d %d %d CLRPA\n",
-		(int) Pad->Point1.X,
-		(int) Pad->Point1.Y,
-		(int) Pad->Point2.X,
-		(int) Pad->Point2.Y,
-		(int) Pad->Thickness + 2*GROUNDPLANEFRAME);
-}
 
-/* ----------------------------------------------------------------------
- * prints a via or pin mask
- */
-static void PS_PrintPinOrViaMask(PinTypePtr Ptr, int unused)
-{
-	fprintf(PS_Flags.FP, "%d %d %d %s\n",
-		(int) Ptr->X,
-		(int) Ptr->Y,
-		(int) Ptr->Thickness + 2*GROUNDPLANEFRAME,
-		TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" : TEST_FLAG(OCTAGONFLAG, Ptr) ? "CLRPV" : "CLRPVR");
+	if (mode)
+		fprintf(PS_Flags.FP, "%d %d %d %s\n",
+			(int) Ptr->X,
+			(int) Ptr->Y,
+			size,
+			TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" 
+			: TEST_FLAG(OCTAGONFLAG, Ptr) ? "CLRPV" : "CLRPVR");
+	else
+	{
+        	if (TEST_FLAG(USETHERMALFLAG, Ptr))
+		{
+			int size2 = (size + Ptr->Clearance)/2;
+                	if (!TEST_FLAG(SQUAREFLAG, Ptr))
+                        	size2 = (7 * size2) / 10;
+                	fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
+                        	Ptr->X - size2, Ptr->Y - size2,
+                        	Ptr->X + size2, Ptr->Y + size2,
+                        	Ptr->Clearance/2);
+                	fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
+                        	Ptr->X - size2, Ptr->Y + size2,
+                        	Ptr->X + size2, Ptr->Y - size2,
+                        	Ptr->Clearance/2);
+                	CLEAR_FLAG(USETHERMALFLAG, Ptr);
+        	}
+		fprintf(PS_Flags.FP, "%d %d %d %d %s\n",
+			(int) Ptr->X,
+			(int) Ptr->Y,
+			size,
+			(int) Ptr->DrillingHole,
+			TEST_FLAG(SQUAREFLAG, Ptr) ? "PVSQ" 
+			: TEST_FLAG(OCTAGONFLAG, Ptr) ? "PV" : "PVR");
+	}
 }
 
 /* ---------------------------------------------------------------------------
  * draws a filled rectangle for the ground plane
  */
 static void PS_FilledRectangle(Position X1, Position Y1,
-	Position X2, Position Y2, int unused)
+	Position X2, Position Y2)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d FILL\n",
 		(int) X1,
@@ -966,7 +971,7 @@ static void PS_FilledRectangle(Position X1, Position Y1,
  * the upper/left and lower/right corner are passed
  */
 static void PS_Outline(Position X1, Position Y1,
-	Position X2, Position Y2, int unused)
+	Position X2, Position Y2)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d Outline\n",
 		(int) X1,
@@ -980,7 +985,7 @@ static void PS_Outline(Position X1, Position Y1,
  * the upper/left and lower/right corner are passed
  */
 static void PS_Alignment(Position X1, Position Y1,
-	Position X2, Position Y2, int unused)
+	Position X2, Position Y2)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d %d Alignment\n",
 		(int) X1,
@@ -1008,7 +1013,7 @@ static void PS_DrillHelper(PinTypePtr Ptr, int unused)
  * set PostScript identifier to the calculated value or to black by 
  * default
  */
-static void PS_SetColor(XColor RGB, int unused)
+static void PS_SetColor(XColor RGB)
 {
 	fprintf(PS_Flags.FP,
 		"/Color {%.3f %.3f %.3f mysetrgbcolor} def Color\n",
