@@ -99,19 +99,23 @@ static void CenterDialog(Widget Popup, Boolean CenterX, Boolean CenterY)
 		NULL);	
 	
 		/* make sure position is inside our screen */
-	if (CenterX)
-		if ((root_x -= (width/2)) < 0)
+	if (CenterX) {
+		if ((root_x -= (width/2)) < 0) {
 			root_x = 0;
-		else
+		} else {
 			if (root_x > WidthOfScreen(XtScreen(Popup)) -width)
 				root_x =  WidthOfScreen(XtScreen(Popup)) -width;
+		}	
+	}
 
-	if (CenterY)
-		if ((root_y -= (height/2)) < 0)
+	if (CenterY) {
+		if ((root_y -= (height/2)) < 0) {
 			root_y = 0;
-		else
+		} else {
 			if (root_y > HeightOfScreen(XtScreen(Popup)) -height)
 				root_y =  HeightOfScreen(XtScreen(Popup)) -height;
+		}
+	}	
 
 	XtVaSetValues(Popup,
 		XtNx, root_x,
@@ -131,6 +135,9 @@ int DialogEventLoop(int *Code)
 	for(*Code = -1; *Code == -1;)
 	{
 		XtAppNextEvent(Context, &event);
+			/* exit if window manager says delete window */
+		if (event.type == ClientMessage)
+			return(*Code);
 		XtDispatchEvent(&event);
 	}
 	return(*Code);
@@ -204,10 +211,14 @@ Widget CreateDialogBox(char *LabelText,
  */
 void StartDialog(Widget Popup)
 {
-	SaveOutputWindow();
+	Atom	protocol;
+
 	XtRealizeWidget(Popup);
 	CenterDialog(Popup, True, True);
 	XtPopup(Popup, XtGrabExclusive);
+	protocol = XInternAtom(Dpy, "WM_DELETE_WINDOW", True);
+	if (protocol != None)
+		XSetWMProtocols(Dpy, XtWindow(Popup), &protocol, 1);
 }
 
 /* ---------------------------------------------------------------------------
@@ -261,7 +272,7 @@ void MessagePrompt(char *MessageText)
 			Output.MasterForm,
 			XtNlabel, EMPTY(MessageText),
 			XtNfromHoriz, Output.Control,
-			XtNfromVert, Output.Viewport,
+			XtNfromVert, Output.Porthole,
 			LAYOUT_BOTTOM,
 			NULL);
 	}
@@ -319,7 +330,7 @@ char *GetUserInput(char *MessageText, char *OutputString)
 
 		/* calculate size */
 	XtVaGetValues(Output.Message, XtNwidth, &messageWidth, NULL);
-	XtVaGetValues(Output.Viewport, XtNwidth, &viewportWidth, NULL);
+	XtVaGetValues(Output.Porthole, XtNwidth, &viewportWidth, NULL);
 	
 		/* the input field itself */
 	inputfield = XtVaCreateManagedWidget("inputField", asciiTextWidgetClass,
@@ -327,7 +338,7 @@ char *GetUserInput(char *MessageText, char *OutputString)
 		XtNresizable, True,
 		XtNeditType, XawtextEdit,
 		XtNfromHoriz, Output.Message,
-		XtNfromVert, Output.Viewport,
+		XtNfromVert, Output.Porthole,
 		XtNstring, EMPTY(OutputString),
 		XtNwidth, MAX(100, viewportWidth-messageWidth),
 		LAYOUT_BOTTOM_RIGHT,
@@ -383,10 +394,12 @@ void AboutDialog(void)
 		"This is PCB, an interactive\n"
 		"printed circuit board editor\n"
 		"version "RELEASE"\n\n"
-		"by Thomas Nau (c) 1994, 1995, 1996\n\n"
+		"by Thomas Nau (c) 1994, 1995, 1996, 1997, 1998\n\n"
 		"If you have problems, hints or\n"
 		"suggestions, send mail to:\n"
-		"Thomas.Nau@rz.uni-ulm.de\n\n",
+		"Thomas.Nau@rz.uni-ulm.de\n"
+		"         or\n"
+		"haceaton@aplcomm.jhuapl.edu\n\n",
 		&button, 1,"About PCB");
 	if ((dialog = XtNameToWidget(popup, "dialog")) != NULL)
 	{
