@@ -68,6 +68,8 @@ static	Window	LogWindow;				/* the logging window */
 /* ---------------------------------------------------------------------------
  * output of message in a dialog window or log window
  */
+#if (!defined(__GLIBC__) || (__GLIBC__ < 2) || \
+((__GLIBC__ == 2) && defined(__powerpc__) && (__GLIBC_MINOR__ == 0)))
 void Message(Format, va_alist)
 char *Format;
 va_dcl
@@ -75,8 +77,15 @@ va_dcl
 	va_list	args;
 	char	s[1024];
 	XEvent	event;
-
 	va_start(args);
+#else
+void Message(char *Format, ...)
+{
+	va_list	args;
+	char	s[1024];
+	XEvent	event;
+	va_start(args, Format);
+#endif
 	vsprintf(s, Format, args);
 	va_end(args);
 
@@ -156,17 +165,25 @@ void RestoreStderr(void)
 /* ---------------------------------------------------------------------------
  * output of fatal error message
  */
+#if (!defined(__GLIBC__) || (__GLIBC__ < 2) || \
+((__GLIBC__ == 2) && defined(__powerpc__) && (__GLIBC_MINOR__ == 0)))
 void MyFatal(Format, va_alist)
 char *Format;
 va_dcl
 {
 	va_list	args;
 
+	va_start(args);
+#else
+void MyFatal(char *Format, ...)
+{
+	va_list	args;
+
+	va_start(args, Format);
+#endif
 		/* try to save the layout and do some cleanup */
 	RestoreStderr();
 	EmergencySave();
-
-	va_start(args);
 	fprintf(stderr, "%s (%i): fatal, ", Progname, (int) getpid());
 	vfprintf(stderr, Format, args);
 	fflush(stderr);
