@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  PCB, interactive printed circuit board design
- *  Copyright (C) 1994,1995,1996 Thomas Nau
+ *  Copyright (C) 1994,1995,1996,1997,1998 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@ static	char	*rcsid = "$Id: file.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 #include "global.h"
 
+#include "change.h"
 #include "create.h"
 #include "crosshair.h"
 #include "data.h"
@@ -52,7 +53,9 @@ static	char	*rcsid = "$Id: file.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 #include "file.h"
 #include "mymem.h"
 #include "misc.h"
+#include "netlist.h"
 #include "parse_l.h"
+#include "rats.h"
 #include "remove.h"
 #include "set.h"
 
@@ -72,6 +75,7 @@ static	void	WritePCBInfoHeader(FILE *);
 static	void	WritePCBDataHeader(FILE *);
 static	void	WritePCBFontData(FILE *);
 static	void	WritePCBViaData(FILE *);
+static	void	WritePCBRatData(FILE *);
 static	void	WritePCBElementData(FILE *);
 static	void	WritePCBLayerData(FILE *, Cardinal);
 static	int	WritePCB(FILE *);
@@ -155,6 +159,7 @@ int LoadPCB(char *Filename)
 	{
 		RemovePCB(PCB);
 		PCB = newPCB;
+		InitNetlistWindow(Output.Toplevel);
 		ResetStackAndVisibility();
 
 			/* update cursor location */
@@ -242,8 +247,8 @@ static void WritePCBDataHeader(FILE *FP)
 	PrintQuotedString(FP, EMPTY(PCB->Name));
 	fprintf(FP, " %i %i)\n\n",
 		(int) PCB->MaxWidth, (int) PCB->MaxHeight);
-	fprintf(FP, "Grid(%i %i %i)\n",
-		(int) PCB->Grid, (int) PCB->GridOffsetX, (int) PCB->GridOffsetY);
+	fprintf(FP, "Grid(%i %i %i %i)\n",
+		(int) PCB->Grid, (int) PCB->GridOffsetX, (int) PCB->GridOffsetY, (int) Settings.DrawGrid);
 	fprintf(FP, "Cursor(%i %i %i)\n",
 		(int) Crosshair.X, (int) Crosshair.Y, PCB->Zoom);
 	fprintf(FP, "Flags(0x%08x)\n", (int) PCB->Flags);
@@ -274,7 +279,16 @@ static void WritePCBDataHeader(FILE *FP)
 			if (group != MAX_LAYER-1)
 				fputc(':', FP);
 		}
-	fputs("\")\n\n", FP);
+	fputs("\")\nStyles(\"", FP);
+	for (group = 0; group < NUM_STYLES - 1; group++)
+		fprintf(FP, "%s,%i,%i,%i:", PCB->RouteStyle[group].Name,
+			PCB->RouteStyle[group].Thick,
+			PCB->RouteStyle[group].Diameter,
+			PCB->RouteStyle[group].Hole);
+	fprintf(FP, "%s,%i,%i,%i\")\n\n", PCB->RouteStyle[group].Name,
+		PCB->RouteStyle[group].Thick,
+		PCB->RouteStyle[group].Diameter,
+		PCB->RouteStyle[group].Hole);
 }
 
 /* ---------------------------------------------------------------------------
@@ -314,14 +328,49 @@ static void WritePCBViaData(FILE *FP)
 {
 		/* write information about vias */
 	VIA_LOOP(PCB->Data,
-		fprintf(FP, "Via(%i %i %i %i ",
+		fprintf(FP, "Via(%i %i %i %i %i ",
 			(int) via->X, (int) via->Y,
-			(int) via->Thickness, (int) via->DrillingHole);
+			(int) via->Thickness, (int) via->Clearance,
+			(int) via->DrillingHole);
 		PrintQuotedString(FP, EMPTY(via->Name));
 		fprintf(FP, " 0x%08x)\n", (int) via->Flags);
-
 	);
+}
 
+/* ---------------------------------------------------------------------------
+ * writes rat-line data
+ */
+static void WritePCBRatData(FILE *FP)
+{
+		/* write information about rats */
+	RAT_LOOP(PCB->Data,
+		fprintf(FP, "Rat(%i %i %i %i %i %i ",
+			(int) line->Point1.X, (int) line->Point1.Y,
+			(int) line->group1, (int) line->Point2.X,
+			(int) line->Point2.Y, (int) line->group2);
+		fprintf(FP, " 0x%08x)\n", (int) line->Flags);
+	);
+}
+
+/* ---------------------------------------------------------------------------
+ * writes netlist data
+ */
+static void WritePCBNetlistData(FILE *FP)
+{
+		/* write out the netlist if it exists */
+	if (PCB->NetlistLib.MenuN)
+	{
+		fprintf(FP,"NetList()\n(\n");
+		MENU_LOOP(&PCB->NetlistLib,
+			fprintf(FP,"\tNet(\"%s\" \"%s\")\n\t(\n", &menu->Name[2],
+				UNKNOWN(menu->Style));
+			ENTRY_LOOP(menu,
+				fprintf(FP, "\t\tConnect(\"%s\")\n", entry->ListEntry);
+			);
+			fprintf(FP,"\t)\n");
+		);
+		fprintf(FP,")\n");
+	}
 }
 
 /* ---------------------------------------------------------------------------
@@ -332,7 +381,7 @@ static void WritePCBElementData(FILE *FP)
 	ELEMENT_LOOP(PCB->Data,
 			/* only non empty elements */
 		if (!element->LineN && !element->PinN &&
-			!element->ArcN)
+			!element->ArcN && !element->PadN)
 			continue;
 
 				/* the coordinates and text-flags are the same for
@@ -344,46 +393,57 @@ static void WritePCBElementData(FILE *FP)
 		PrintQuotedString(FP, EMPTY(NAMEONPCB_NAME(element)));
 		fputc(' ', FP);
 		PrintQuotedString(FP, EMPTY(VALUE_NAME(element)));
-		fprintf(FP, " %i %i %i %i 0x%08x)\n(\n",
-				(int) DESCRIPTION_TEXT(element).X,
-				(int) DESCRIPTION_TEXT(element).Y,
+		fprintf(FP, " %i %i %i %i %i %i 0x%08x)\n(\n",
+				(int) element->MarkX, (int) element->MarkY,
+				(int) (DESCRIPTION_TEXT(element).X - element->MarkX),
+				(int) (DESCRIPTION_TEXT(element).Y - element->MarkY),
 				(int) DESCRIPTION_TEXT(element).Direction,
 				(int) DESCRIPTION_TEXT(element).Scale,
 				(int) DESCRIPTION_TEXT(element).Flags);
 
 		PIN_LOOP(element,
-			fprintf(FP, "\tPin(%i %i %i %i ",
-				(int) pin->X, (int) pin->Y,
-				(int) pin->Thickness, (int) pin->DrillingHole);
+			fprintf(FP, "\tPin(%i %i %i %i %i %i ",
+				(int) (pin->X - element->MarkX),
+				(int) (pin->Y - element->MarkY),
+				(int) pin->Thickness, (int) pin->Clearance,
+				(int) pin->Mask, (int) pin->DrillingHole);
 			PrintQuotedString(FP, EMPTY(pin->Name));
+			fprintf(FP, " ");
+			PrintQuotedString(FP, EMPTY(pin->Number));
 			fprintf(FP, " 0x%08x)\n", (int) pin->Flags);
 		);
 		PAD_LOOP(element,
-			fprintf(FP, "\tPad(%i %i %i %i %i ",
-				(int) pad->Point1.X, (int) pad->Point1.Y,
-				(int) pad->Point2.X, (int) pad->Point2.Y,
-				(int) pad->Thickness);
+			fprintf(FP, "\tPad(%i %i %i %i %i %i %i ",
+				(int) (pad->Point1.X - element->MarkX),
+				(int) (pad->Point1.Y - element->MarkY),
+				(int) (pad->Point2.X - element->MarkX),
+				(int) (pad->Point2.Y - element->MarkY),
+				(int) pad->Thickness, (int) pad->Clearance,
+				(int) pad->Mask);
 			PrintQuotedString(FP, EMPTY(pad->Name));
+			fprintf(FP, " ");
+			PrintQuotedString(FP, EMPTY(pad->Number));
 			fprintf(FP, " 0x%08x)\n", (int) pad->Flags);
 		);
 
 		ELEMENTLINE_LOOP(element,
 			fprintf(FP, "\tElementLine (%i %i %i %i %i)\n",
-				(int) line->Point1.X, (int) line->Point1.Y,
-				(int) line->Point2.X, (int) line->Point2.Y,
+				(int) (line->Point1.X - element->MarkX),
+				(int) (line->Point1.Y - element->MarkY),
+				(int) (line->Point2.X - element->MarkX),
+				(int) (line->Point2.Y - element->MarkY),
 				(int) line->Thickness);
 		);
 
 		ARC_LOOP(element,
 			fprintf(FP, "\tElementArc (%i %i %i %i %i %i %i)\n",
-				(int) arc->X, (int) arc->Y,
+				(int) (arc->X - element->MarkX),
+				(int) (arc->Y - element->MarkY),
 				(int) arc->Width, (int) arc->Height,
 				(int) arc->StartAngle, (int) arc->Delta,
 				(int) arc->Thickness);
 		);
-
-		fprintf(FP, "\tMark (%i %i)\n)\n",
-			(int) element->MarkX, (int) element->MarkY);
+		fprintf(FP, "\t)\n");
 	);
 }
 
@@ -403,16 +463,18 @@ static void WritePCBLayerData(FILE *FP, Cardinal Number)
 		fputs(")\n(\n", FP);
 
 		LINE_LOOP(layer,
-			fprintf(FP, "\tLine(%i %i %i %i %i 0x%08x)\n",
+			fprintf(FP, "\tLine(%i %i %i %i %i %i 0x%08x)\n",
 				(int) line->Point1.X, (int) line->Point1.Y,
 				(int) line->Point2.X, (int) line->Point2.Y,
-				(int) line->Thickness, (int) line->Flags);
+				(int) line->Thickness, (int) line->Clearance,
+				(int) line->Flags);
 		);
 		ARC_LOOP(layer,
-			fprintf(FP, "\tArc(%i %i %i %i %i %i %i 0x%08x)\n",
+			fprintf(FP, "\tArc(%i %i %i %i %i %i %i %i 0x%08x)\n",
 				(int) arc->X, (int) arc->Y, (int) arc->Width,
 				(int) arc->Height, (int) arc->Thickness,
-				arc->StartAngle, arc->Delta, (int) arc->Flags);
+				(int) arc->Clearance, (int) arc->StartAngle,
+				(int) arc->Delta, (int) arc->Flags);
 		);
 		TEXT_LOOP(layer,
 			fprintf(FP, "\tText(%i %i %i %i ",
@@ -454,8 +516,10 @@ static int WritePCB(FILE *FP)
 	WritePCBFontData(FP);
 	WritePCBViaData(FP);
 	WritePCBElementData(FP);
-	for (i = 0; i < MAX_LAYER; i++)
+	WritePCBRatData(FP);
+	for (i = 0; i < MAX_LAYER + 2; i++)
 		WritePCBLayerData(FP, i);
+	WritePCBNetlistData(FP);
 
 	return(STATUS_OK);
 }
@@ -515,13 +579,14 @@ static int WritePCBPipe(char *Filename)
 	return(pclose(fp) ? STATUS_ERROR : result);
 }
 
-#if !defined(HAS_ATEXIT) && defined(HAS_ON_EXIT)
+#if defined(HAS_ON_EXIT)
 /* ---------------------------------------------------------------------------
  * just a glue function for systems with on_exit()
  */
 void GlueEmergencySave(int status, caddr_t arg)
 {
-	EmergencySave();
+	if (status)
+		EmergencySave();
 }
 #endif
 
@@ -603,7 +668,7 @@ int ReadLibraryContents(void)
 	static	char		*command = NULL;
 	char			inputline[MAX_LIBRARY_LINE_LENGTH+1];
 	FILE			*resultFP;
-	LibraryMenuTypePtr	menu = Library.Menu;
+	LibraryMenuTypePtr	menu;
 	LibraryEntryTypePtr	entry;
 
 		/* release old command string */
@@ -638,7 +703,7 @@ int ReadLibraryContents(void)
 			/* if the line defines a menu */
 		if (!strncmp(inputline, "TYPE=", 5))
 		{
-			menu = GetLibraryMenuMemory();
+			menu = GetLibraryMenuMemory(&Library);
 			menu->Name = MyStrdup(UNKNOWN(&inputline[5]),
 				"ReadLibraryDescription()");
 		}
@@ -647,7 +712,7 @@ int ReadLibraryContents(void)
 				/* allocate a new menu entry if not already done */
 			if (!menu)
 			{
-				menu = GetLibraryMenuMemory();
+				menu = GetLibraryMenuMemory(&Library);
 				menu->Name = MyStrdup(UNKNOWN((char *) NULL),
 					"ReadLibraryDescription()");
 			}
@@ -673,3 +738,107 @@ int ReadLibraryContents(void)
 
 	return(pclose(resultFP));
 }
+
+#define BLANK(x) ((x) == ' ' || (x) == '\t' || (x) == '\n' \
+		|| (x) == '\0')
+	
+/* ---------------------------------------------------------------------------
+ * Read in a netlist and store it in the netlist menu 
+ */
+ 
+ int	ReadNetlist(char *filename)
+ {
+	static	char		*command = NULL;
+	char			inputline[MAX_NETLIST_LINE_LENGTH + 1];
+	char			temp[MAX_NETLIST_LINE_LENGTH + 1];
+ 	FILE			*fp;
+	LibraryMenuTypePtr	menu;
+	LibraryEntryTypePtr	entry;
+ 	int			i, j, lines, kind;
+ 	Boolean			continued;
+ 	
+	if (!filename)
+		return(1);		/* nothing to do */
+	MyFree(&command);
+	command = EvaluateFilename(Settings.RatCommand,
+			Settings.RatPath, filename, NULL);
+	
+		/* open pipe to stdout of command */
+	if (*command == '\0' || (fp = popen(command, "r")) == NULL)
+	{
+		PopenErrorMessage(command);
+		return(1);
+	}
+	lines = 0;
+		/* kind = 0  is net name
+		 * kind = 1  is route style name
+		 * kind = 2  is connection
+		 */
+	kind = 0;
+ 	while (fgets(inputline, MAX_NETLIST_LINE_LENGTH, fp))
+ 	{
+		size_t	len = strlen(inputline);
+			/* check for maximum length line */
+		if (len)
+		{
+			if (inputline[--len] != '\n')
+				Message("line length (%i) exceeded in netlist file.\n"
+					"additional characters will be ignored.\n",
+					MAX_NETLIST_LINE_LENGTH);
+			else
+				inputline[len] = '\0';
+		}
+		continued = (inputline[len-1] == '\\') ? True : False;
+		if (continued)
+			inputline[len-1] = '\0';
+		lines++;
+		i = 0;
+		while (inputline[i] != '\0')
+		{
+			j = 0;
+				/* skip leading blanks */
+			while(inputline[i] != '\0' && BLANK(inputline[i]))
+				i++;
+			if (kind == 0)
+			{
+			/* add two spaces for included/unincluded */
+				temp[j++] = ' ';
+				temp[j++] = ' ';
+			}
+			while(!BLANK(inputline[i]))
+				temp[j++] = inputline[i++];
+			temp[j] = '\0';
+			while(inputline[i] != '\0' && BLANK(inputline[i]))
+				i++;
+			if (kind == 0)
+			{
+				menu = GetLibraryMenuMemory(&PCB->NetlistLib);
+				menu->Name = MyStrdup(temp, "ReadNetlist()");
+				kind++;
+			}
+			else
+			{
+				if (kind == 1 && strchr(temp, '-') == NULL)
+				{
+					kind++;
+					menu->Style = MyStrdup(temp, "ReadNetlist()");
+				}
+				else
+				{
+					entry = GetLibraryEntryMemory(menu);
+					entry->ListEntry = MyStrdup(temp, "ReadNetlist()");
+				}
+			}
+		}
+		if (!continued)
+			kind = 0;
+	}
+	if (!lines)
+	{
+		Message("Empty netlist file!\n");
+		pclose(fp);
+		return(1);
+	}
+	pclose(fp);
+	return(0);
+}
