@@ -290,7 +290,7 @@ static int FillListsFromDirectory(char *DirName)
 			 */
 		if (!stat(direntry->d_name, &buffer))
 		{
-				/* add entry to crresponding list */
+				/* add entry to corresponding list */
 			if (S_ISDIR(buffer.st_mode))
 			{
 					/* ignore '.' entry */
@@ -383,7 +383,8 @@ char *FileSelectBox(char *MessageText, char *DefaultFile, char *Path)
 			masterform,
 			last;
 	char		currentdir[MAXPATHLEN+1],
-			*file;
+			tempPath[MAXPATHLEN+1],
+			*file, *file2;
 	static	char	result[MAXPATHLEN+1];
 
 		/* save current directory */
@@ -411,12 +412,36 @@ char *FileSelectBox(char *MessageText, char *DefaultFile, char *Path)
 		XtNlabel, MessageText,
 		NULL);
 
+		/* extract any path off default file */
+	for (file = NULL, file2 = DefaultFile; file2 && *file2; file2++)
+		if (*file2 == '/')
+			file = file2;
+	file2 = NULL;
+	if (file)
+	{
+		for (file2 = tempPath ; DefaultFile != file; DefaultFile++, file2++)
+			*file2 = *DefaultFile;
+		*file2 = '\0';
+		if (strcmp(tempPath, currentdir) != 0)
+		{
+			*file2++ = ':';
+			for (file = currentdir; file && *file; file++)
+				*file2++ = *file;
+		}
+		if (strcmp(Path, "."))
+			for (file = Path; file && *file; file++)
+			{
+				if (file == Path)
+					*file2++ = ':';
+				*file2++ = *file;
+			}
+		*file2 = '\0';
+	}
 		/* create a menu button for preset directories, also
 		 * initializes 'CurrentDir'
 		 */
 	last = CreateMenuFromPath(masterform, last, NULL,
-		(Path && *Path) ? Path : ".");
-
+		file2 ? tempPath : (Path && *Path) ? Path : ".");
 		/* the line which displays the current directory */
 	CurrentW = XtVaCreateManagedWidget("current", labelWidgetClass,
 		masterform,
@@ -434,11 +459,12 @@ char *FileSelectBox(char *MessageText, char *DefaultFile, char *Path)
 		XtNfromVert, CurrentW,
 		LAYOUT_TOP,
 		NULL);
+
 	XtAddCallback(XawTextGetSource(InputW), XtNcallback, CB_Text, NULL);
 
 		/* the two selectors and the buttons */
-	last = CreateSelector(masterform, InputW, NULL, &DirectorySelector);
-	last = CreateSelector(masterform, InputW, last, &FileSelector);
+	last = CreateSelector(masterform, InputW, NULL, &DirectorySelector, 1);
+	last = CreateSelector(masterform, InputW, last, &FileSelector, 1);
 	AddButtons(masterform, last, Buttons, ENTRIES(Buttons));
 	
 		/* override the translations for the input widget and
@@ -482,11 +508,12 @@ char *FileSelectBox(char *MessageText, char *DefaultFile, char *Path)
 		XtVaGetValues(InputW, XtNstring, &file, NULL);
 
 			/* append result to current directory */
-		if (file && *file)
-			if (*CurrentDir != '|')
+		if (file && *file) {
+			if (*CurrentDir != '|') {
 				sprintf(result, "%s/%s", CurrentDir, file);
-			else
+			} else 
 				strcpy(result, file);
+		}
 	}
 
 		/* remove event handler, dialog, release resources and
