@@ -50,16 +50,15 @@
 
 /* ---------------------------------------------------------------------------
  * do not change the following definition even if it's not very nice.
- * It allows us too cast PadTypePtr to LineTypePtr and access the pads
- * coordinates as a line. The background is to have only one set of funtions
- * for searching connections
+ * It allows us to cast PadTypePtr and RatTypePtr to LineTypePtr and access
+ * their coordinates as a line.
  */
 #define	LINESTRUCT			\
 	int		ID,		\
 			Flags;		\
 	PointType	Point1,		\
 			Point2;		\
-	Dimension	Thickness;
+	Dimension	Thickness, Clearance;
 
 /* ---------------------------------------------------------------------------
  * some useful values of our widgets
@@ -69,21 +68,21 @@ typedef struct				/* holds information about output window */
 	Widget		Toplevel,	/* toplevel widget */
 			StatusLine,	/* label widget of statusline */
 			CursorPosition,	/* current cursor position */
-			ScrollbarLeft,	/* viewport scrollbars */
-			ScrollbarBottom,
 			Control,	/* control widget */
-			Menu,		/* popup menue */
+			Menu,		/* popup menu */
 			Message,	/* fields for user message */
 			MasterForm,	/* the widgets thats hols all the others */
-			Viewport,	/* viewport widget to scoll output */
+			Porthole,	/* porthole widget to pan output */
+			panner,		/* panner widget for panning */
 			Output;		/* simple widget for drawing */
 	Window		OutputWindow;	/* window ID of simple widget */
 	GC		bgGC,		/* background and foreground; */
 			fgGC,		/* changed from some routines */
+			pmGC,		/* pixmap GC to store clip */
 			GridGC;		/* for the grid */
-	Position	OffsetX,	/* origin offset */
-			OffsetY;
-	Dimension	Width,		/* sizes of output window */
+	Dimension	cw,		/* canvas width/height */
+			ch,
+			Width,		/* sizes of output window (porthole) */
 			Height;
 	Cursor		XCursor;	/* used X cursor */
 	unsigned int	XCursorShape;	/* and its shape (cursorfont) */
@@ -96,8 +95,8 @@ typedef struct				/* holds information about output window */
  */
 typedef struct
 {
-	Cardinal	Number[MAX_LAYER+2],	/* number of entries per groups */
-			Entries[MAX_LAYER+2][MAX_LAYER+2];
+	Cardinal	Number[MAX_LAYER],	/* number of entries per groups */
+			Entries[MAX_LAYER][MAX_LAYER+2];
 } LayerGroupType, *LayerGroupTypePtr;
 
 typedef struct				/* a bounding box */
@@ -107,10 +106,7 @@ typedef struct				/* a bounding box */
 } BoxType, *BoxTypePtr;
 
 /* ---------------------------------------------------------------------------
- * the basic type supported by PCB
- * there are only four combination of coordinates supported:
- * line straight up, right and up, straight right and down and right
- * check 'draw.c' and 'create.c' for details
+ * the basic object types supported by PCB
  */
 typedef struct				/* all objects start with an ID */
 {
@@ -159,7 +155,8 @@ typedef struct				/* holds information about arcs */
 			Y;
 	Dimension	Width,		/* length of axis */
 			Height,
-			Thickness;
+			Thickness,
+			Clearance;
 	int		StartAngle,	/* the two limiting angles in degrees */
 			Delta;	
 	BoxType		BoundingBox;	/* only filled in for arcs on layers */
@@ -185,10 +182,18 @@ typedef struct			/* holds information about one layer */
 			SelectedColor;
 } LayerType, *LayerTypePtr;
 
+typedef struct				/* a rat-line */
+{
+	LINESTRUCT
+	Cardinal group1, group2;	/* the layer group each point is on */
+} RatType, *RatTypePtr;
+
 typedef struct				/* a SMD pad */
 {
 	LINESTRUCT
-	char		*Name;			/* 'Line' */
+	Dimension	Mask;
+	char		*Name, *Number;			/* 'Line' */
+	void		*Spare;
 } PadType, *PadTypePtr;
 
 typedef struct
@@ -198,8 +203,11 @@ typedef struct
 	Position	X,			/* center and diameter */
 			Y;
 	Dimension	Thickness,
+			Clearance,
+			Mask,
 			DrillingHole;
-	char		*Name;
+	char		*Name, *Number;
+	void		*Spare;
 } PinType, *PinTypePtr, **PinTypeHandle;
 
 typedef struct
@@ -256,35 +264,15 @@ typedef struct			/* holds all objects */
 	Cardinal	ViaN,			/* number of vias */
 			ViaMax,			/* max number from malloc() */
 			ElementN,		/* and elements */
-			ElementMax;		/* max number from malloc() */
+			ElementMax,		/* max number from malloc() */
+			RatN,			/* and rat-lines */
+			RatMax;
 	PinTypePtr	Via;			/* pointer to object data */
 	ElementTypePtr	Element;
-	LayerType	Layer[MAX_LAYER];
+	RatTypePtr	Rat;
+	LayerType	Layer[MAX_LAYER + 2];	/* add 2 silkscreen layers */
 } DataType, *DataTypePtr;
 
-typedef struct                        /* holds a connection */
-{
-      Position                X, Y;
-      PinTypePtr              Object; /* ptr to object for flag testing */
-      Cardinal                PinNum;
-      ElementTypePtr          Element; /* ptr to owning element */
-      Boolean                 Reachable; /* flags whether reachable on layer */
-} ConnectionType, *ConnectionTypePtr;
- 
-typedef struct                        /* holds a subnet of connections */
-{
-      Cardinal                ConnectionN,    /* the number of subnets contained */
-                              ConnectionMax;  /* max subnets from malloc */
-      ConnectionTypePtr       Connection;
-} SubnetType, *SubnetTypePtr;
-
-typedef struct                        /* holds a net of subnets */ 
-{            
-      Cardinal        SubnetN,        /* the number of subnets contained */
-                      SubnetMax;      /* max subnets from malloc */
-      SubnetTypePtr   Subnet; 
-} NetType, *NetTypePtr;       
-
 typedef struct                                /* holds drill information */
 {
       Dimension       DrillSize;      /* this drill's diameter */
@@ -306,6 +294,45 @@ typedef struct                                /* holds a range of Drill Infos */
       DrillTypePtr	Drill;		/* plated holes */
 } DrillInfoType, *DrillInfoTypePtr;   
 
+typedef struct
+{
+	Dimension	Thick,		/* line thickness */
+			Diameter,	/* via diameter */
+			Hole;		/* via drill hole */
+	char		*Name;
+} RouteStyleType, *RouteStyleTypePtr;
+
+/* ---------------------------------------------------------------------------
+ * structure used by library routines
+ */
+typedef struct
+{
+	char	*ListEntry;		/* the string for the selection box */
+	char	*AllocatedMemory,	/* pointer to allocated memory; all others */
+					/* point to parts of the string */
+		*Template,		/* m4 template name */
+		*Package,		/* package */
+		*Value,			/* the value field */
+		*Description;		/* some descritional text */
+} LibraryEntryType, *LibraryEntryTypePtr;
+
+typedef struct
+{
+	char			*Name,		/* name of the menu entry */
+				*Style;		/* routing style */
+	Cardinal		EntryN,		/* number of objects */
+				EntryMax;	/* number of reserved memory locations */
+	LibraryEntryTypePtr	Entry;		/* the entries */
+} LibraryMenuType, *LibraryMenuTypePtr;
+
+typedef struct
+{
+	Cardinal		MenuN,
+				MenuMax;
+	LibraryMenuTypePtr	Menu;
+	Window			Wind;
+} LibraryType, *LibraryTypePtr;
+
 typedef struct		/* holds information about board layout */
 {			/* most of the information is saved with layout */
 			/* and initialized from resources when */
@@ -321,15 +348,20 @@ typedef struct		/* holds information about board layout */
 	Boolean		Changed,		/* layout has been changed */
 			ViaOn,			/* visibility flags */
 			ElementOn,
+			RatOn,
 			InvisibleObjectsOn,
-			PinOn;
+			PinOn,
+			SilkActive;		/* active layer is actually silk */
 	Pixel		ViaColor,		/* some colors */
 			ViaSelectedColor,
 			PinColor,
 			PinSelectedColor,
 			ElementColor,
+			RatColor,
 			InvisibleObjectsColor,
+			InvisibleMarkColor,
 			ElementSelectedColor,
+			RatSelectedColor,
 			ConnectedColor,
 			WarnColor;
 	int		Zoom,			/* zoom factor (factor = 1 << zoom) */
@@ -343,8 +375,9 @@ typedef struct		/* holds information about board layout */
 			MaxHeight;
 	FontType	Font;
 	LayerGroupType	LayerGroups;
-	Pixmap		SaveUnder;		/* saves data during dialog handling */
-	DataTypePtr	Data;
+	RouteStyleType	RouteStyle[NUM_STYLES];
+	LibraryType	NetlistLib;
+	DataTypePtr	Data;			/* entire database */
 } PCBType, *PCBTypePtr;
 
 typedef struct				/* information about the paste buffer */
@@ -427,13 +460,17 @@ typedef struct			/* some resources... */
 {
 	Pixel		bgColor,		/* background and cursor color ... */
 			CrosshairColor,	/* different object colors */
+			CrossColor,
 			ViaColor,
 			ViaSelectedColor,
 			PinColor,
 			PinSelectedColor,
 			ElementColor,
+			RatColor,
 			InvisibleObjectsColor,
+			InvisibleMarkColor,
 			ElementSelectedColor,
+			RatSelectedColor,
 			ConnectedColor,
 			OffLimitColor,
 			GridColor,
@@ -443,10 +480,13 @@ typedef struct			/* some resources... */
 	Dimension	ViaThickness,	/* some preset values */
 			ViaDrillingHole,
 			LineThickness,
+			RatThickness,
 			MaxWidth,		/* maximum size of a layout */
 			MaxHeight,
 			TextScale,		/* text scaling in % */
-			AlignmentDistance;
+			AlignmentDistance,
+			Bloat,			/* drc sizes */
+			Shrink;
 	int		Grid,			/* grid 0.001'' */
 			Zoom,			/* number of shift operations for zooming */
 			PinoutZoom,		/* same for pinout windows */
@@ -459,7 +499,6 @@ typedef struct			/* some resources... */
 	long		BackupInterval;	/* time between two backups in seconds */
 	String		DefaultLayerName[MAX_LAYER],
 			FontCommand,	/* commands for file loading... */
-			ApertureCommand,
 			FileCommand,
 			ElementCommand,
 			PrintFile,
@@ -467,10 +506,9 @@ typedef struct			/* some resources... */
 			LibraryContentsCommand,
 			SaveCommand,
 			LibraryFilename,
-			ApertureFile,	/* default aperture definition file */
 			FontFile,		/* name of default font file */
 			Groups,			/* string with layergroups */
-			AperturePath,	/* colon seperated search paths */
+			Routes,			/* string with route styles */
 			FilePath,
 			RatPath,
 			RatCommand,
@@ -483,6 +521,7 @@ typedef struct			/* some resources... */
 			PinoutOffsetY,
 			PinoutTextOffsetX,	/* offset of text from pin center */
 			PinoutTextOffsetY;
+	RouteStyleType	RouteStyle[NUM_STYLES];	/* default routing styles */
 	LayerGroupType	LayerGroups;		/* default layer groups */
 	Boolean		AbsoluteGrid,		/* grid is relative to (0,0) */
 			UseLogWindow,		/* use log window instead of dialog box */
@@ -495,6 +534,7 @@ typedef struct			/* some resources... */
 			StipplePolygons,        /* draw polygons with stipple */
 			AllDirectionLines,	/* enable lines to all directions */
 			RubberBandMode,		/* move, rotate use rubberband connections */
+			SwapStartDirection,	/* change starting direction after each click */
 			ResetAfterElement,	/* reset connections after */
 						/* each element */
 			RingBellWhenFinished;	/* flag if a signal should be */
@@ -519,15 +559,14 @@ typedef struct
 	void	*(*LinePoint)(LayerTypePtr, LineTypePtr, PointTypePtr);
 	void	*(*Point)(LayerTypePtr, PolygonTypePtr, PointTypePtr);
 	void    *(*Arc)(LayerTypePtr, ArcTypePtr);
+	void	*(*Rat)(RatTypePtr);
 } ObjectFunctionType, *ObjectFunctionTypePtr;
 
 /* ---------------------------------------------------------------------------
  * structure used by device drivers
  */
 typedef struct				/* media description */
-{
-	String		Name;		/* name and size (in mil) */
-	Dimension	Width,
+{ String		Name;		/* name and size (in mil) */ Dimension	Width,
 			Height;
 	Position	MarginX,
 			MarginY;
@@ -546,7 +585,7 @@ typedef struct				/* needs and abilities of a driver */
 	BoxType		BoundingBox;	/* bounding box of output */
 } PrintInitType, *PrintInitTypePtr;
 
-typedef struct				/* functions of a driver */
+typedef struct				/* functions of a print driver */
 {
 	char		*Name,								/* name of driver */
 			*Suffix;							/* filename suffix */
@@ -554,30 +593,25 @@ typedef struct				/* functions of a driver */
 	void		(*Exit)(void);				/* exit code */
 	char		*(*Preamble)(PrintInitTypePtr, char *);	/* initializes file */
 	void		(*Postamble)(void);				/* exit code */
-	void		(*SetColor)(XColor, int);			/* set color */
-	void		(*Drill)(PinTypePtr, int);			/* drilling information */
-	void		(*Layer)(LayerTypePtr, int, Boolean, int);	/* printing functions */
-	void		(*ElementPackage)(ElementTypePtr, int);
-	void		(*Text)(TextTypePtr, int);
-	void		(*Pad)(PadTypePtr, int);	
-	void		(*PinOrVia)(PinTypePtr, int);
-	void		(*PadMask)(PadTypePtr, int);
-	void		(*PinOrViaMask)(PinTypePtr, int);
-	void		(*ClearPinOrViaOnGroundplane)(PinTypePtr, int);
-	void		(*FilledRectangle)(Position, Position, Position, Position, int);
-	void		(*Outline)(Position, Position, Position, Position, int);
-	void		(*Alignment)(Position, Position, Position, Position, int);
+	void		(*SetColor)(XColor);			/* set color */
+	void		(*Polarity)(int);			/* control drawing polarity */
+	void		(*Line)(LineTypePtr, Boolean);		/* draw a line (Thick/Clear) */
+	void		(*Arc)(ArcTypePtr, Boolean);		/* draw an arc (Thick/Clear) */
+	void		(*Poly)(PolygonTypePtr);		/* draw a polygon */
+	void		(*Text)(TextTypePtr);			/* draw vector text */
+	void		(*Pad)(PadTypePtr, int);		/* pad thick=0 clear=1 mask=2 */
+	void		(*PinOrVia)(PinTypePtr, int);		/* pin diam=0 clear=1 mask=2 */
+	void		(*ElementPackage)(ElementTypePtr);
+	void		(*Drill)(PinTypePtr, Cardinal);			/* drilling information */
+	void		(*Outline)(Position, Position, Position, Position);
+	void		(*Alignment)(Position, Position, Position, Position);
 	void		(*DrillHelper)(PinTypePtr, int);
+	void		(*GroupID)(int);			/* comments group info */
 	Boolean		HandlesColor,		/* colored output */
-			HandlesInvertion,	/* inverted output */
-			HandlesGroundplane,	/* able to create ground plane */
 			HandlesDrill,		/* able to produce drill info */
-			HandlesMask,		/* able to produce mask layers */
 			HandlesMedia,		/* able to handle different media */
 			HandlesMirror,		/* allows mirror flag to be set */
-			HandlesRotate,		/* allows the rotate flag to be set */
-			NeedsPrePrint,		/* sepecial for gerberX */
-			NeedsAppMap;		/* sepecial for gerber */
+			HandlesRotate;		/* allows the rotate flag to be set */
 } PrintDeviceType, *PrintDeviceTypePtr;
 
 typedef struct
@@ -586,33 +620,52 @@ typedef struct
 	PrintDeviceTypePtr	Info;		/* data returned by Query() */
 } DeviceInfoType, *DeviceInfoTypePtr;
 
+typedef struct                        /* holds a connection */
+{
+	Position	X, Y;			/* coordinate of connection */
+	int		type;			/* type of object in ptr1 - 3 */
+	void		*ptr1, *ptr2;		/* the object of the connection */
+	Cardinal	group;		/* the layer group of the connection */
+	LibraryMenuType	*menu;		/* the netmenu this *SHOULD* belong too */
+} ConnectionType, *ConnectionTypePtr;
+ 
+typedef struct                        /* holds a net of connections */
+{
+      Cardinal                ConnectionN,    /* the number of connections contained */
+                              ConnectionMax;  /* max connections from malloc */
+      ConnectionTypePtr       Connection;
+} NetType, *NetTypePtr;
+
+typedef struct                        /* holds a list of nets */ 
+{            
+      Cardinal        NetN,        /* the number of subnets contained */
+                      NetMax;      /* max subnets from malloc */
+      NetTypePtr      Net; 
+} NetListType, *NetListTypePtr;       
+
+typedef struct                        /* holds a generic list of pointers */ 
+{            
+      Cardinal        PtrN,        /* the number of pointers contained */
+                      PtrMax;      /* max subnets from malloc */
+      void            **Ptr; 
+} PointerListType, *PointerListTypePtr;       
+
 /* ---------------------------------------------------------------------------
- * structure used by library routines
+ * define supported types of undo operations
+ * note these must be separate bits now
  */
-typedef struct
-{
-	char	*ListEntry;		/* the string for the selection box */
-	char	*AllocatedMemory,	/* pointer to allocated memory; all others */
-					/* point to parts of the string */
-		*Template,		/* m4 template name */
-		*Package,		/* package */
-		*Value,			/* the value field */
-		*Description;		/* some descritional text */
-} LibraryEntryType, *LibraryEntryTypePtr;
-
-typedef struct
-{
-	char			*Name;		/* name of the menue entry */
-	Cardinal		EntryN,		/* number of objects */
-				EntryMax;	/* number of reserved memory locations */
-	LibraryEntryTypePtr	Entry;		/* the entries */
-} LibraryMenuType, *LibraryMenuTypePtr;
-
-typedef struct
-{
-	Cardinal		MenuN,
-				MenuMax;
-	LibraryMenuTypePtr	Menu;
-} LibraryType, *LibraryTypePtr;
+#define	UNDO_CHANGENAME			0x0001	/* change of names */
+#define	UNDO_MOVE			0x0002	/* moving objects */
+#define	UNDO_REMOVE			0x0004	/* removing objects */
+#define	UNDO_REMOVE_POINT		0x0008	/* removing polygon/... points */
+#define	UNDO_INSERT_POINT		0x0010	/* inserting polygon/... points */
+#define	UNDO_ROTATE			0x0020	/* rotations */
+#define	UNDO_CREATE			0x0040	/* creation of objects */
+#define	UNDO_MOVETOLAYER		0x0080	/* moving objects to */
+#define UNDO_FLAG			0x0100  /* toggling SELECTED flag */
+#define UNDO_CHANGESIZE			0x0200  /* change size of object */
+#define UNDO_CHANGE2NDSIZE		0x0400  /* change 2ndSize of object */
+#define UNDO_MIRROR			0x0800  /* change side of board */
+#define UNDO_CHANGECLEARSIZE		0x1000	/* change clearance size */
 
 #endif
