@@ -39,6 +39,7 @@ static	char	*rcsid = "$Id: insert.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 #include "misc.h"
 #include "move.h"
 #include "polygon.h"
+#include "search.h"
 #include "select.h"
 #include "set.h"
 #include "undo.h"
@@ -48,6 +49,7 @@ static	char	*rcsid = "$Id: insert.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
  */
 static	void	*InsertPointIntoLine(LayerTypePtr, LineTypePtr);
 static	void	*InsertPointIntoPolygon(LayerTypePtr, PolygonTypePtr);
+static	void	*InsertPointIntoRat(RatTypePtr);
 
 /* ---------------------------------------------------------------------------
  * some local identifiers
@@ -55,6 +57,7 @@ static	void	*InsertPointIntoPolygon(LayerTypePtr, PolygonTypePtr);
 static	Position			InsertX,	/* used by local routines as offset */
 							InsertY;
 static	Cardinal		InsertAt;
+static	Boolean			Forcible;
 static	ObjectFunctionType	InsertFunctions = {
 	InsertPointIntoLine,
 	NULL,
@@ -66,7 +69,31 @@ static	ObjectFunctionType	InsertFunctions = {
 	NULL,
 	NULL,
 	NULL,
-	NULL };
+	NULL,
+	InsertPointIntoRat };
+
+/* ---------------------------------------------------------------------------
+ * inserts a point into a rat-line
+ */
+static void *InsertPointIntoRat(RatTypePtr Rat)
+{
+	LineTypePtr	new;
+
+	new = CreateNewLineOnLayer(CURRENT, Rat->Point1.X, Rat->Point1.Y,
+		InsertX, InsertY, Settings.LineThickness, 2*GROUNDPLANEFRAME,
+		Rat->Flags & ~RATFLAG);
+	AddObjectToCreateUndoList(LINE_TYPE, CURRENT, new, new);
+	EraseRat(Rat);
+	DrawLine(CURRENT, new, 0);
+	new = CreateNewLineOnLayer(CURRENT, Rat->Point2.X, Rat->Point2.Y,
+		InsertX, InsertY, Settings.LineThickness, 2*GROUNDPLANEFRAME,
+		Rat->Flags & ~RATFLAG);
+	AddObjectToCreateUndoList(LINE_TYPE, CURRENT, new, new);
+	DrawLine(CURRENT, new, 0);
+	MoveObjectToRemoveUndoList(RATLINE_TYPE, Rat, Rat, Rat);
+	Draw();
+	return(new);
+}
 
 /* ---------------------------------------------------------------------------
  * inserts a point into a line
@@ -78,26 +105,21 @@ static void *InsertPointIntoLine(LayerTypePtr Layer, LineTypePtr Line)
         if (((Line->Point1.X == InsertX) && (Line->Point1.Y == InsertY)) ||
             ((Line->Point2.X == InsertX) && (Line->Point2.Y == InsertY)))
                 return(NULL);
-                /* Create a new line in the correct place, then modify the original */
-                /* clear the rat-line flag in case a rat is being modified */
-        if (TEST_FLAG(RATFLAG, Line))
-        {
-                AddObjectToFlagUndoList(LINE_TYPE, Layer, Line, Line);
-                CLEAR_FLAG(RATFLAG, Line);
-        }
-        line = CreateNewLineOnLayer(Layer, InsertX, InsertY,
-                Line->Point2.X, Line->Point2.Y, Line->Thickness, Line->Flags & ~RATFLAG);
 	AddObjectToMoveUndoList(LINEPOINT_TYPE, Layer, Line, &Line->Point2,
 			InsertX - Line->Point2.X, InsertY - Line->Point2.Y);
 	EraseLine(Line);
+        if ((line = CreateNewLineOnLayer(Layer, InsertX, InsertY,
+                Line->Point2.X, Line->Point2.Y, Line->Thickness,
+		Line->Clearance, Line->Flags)))
+	{
+        	AddObjectToCreateUndoList(LINE_TYPE, Layer, line, line);
+        	DrawLine(Layer, line, 0);
+	}
         Line->Point2.X = InsertX;
 	Line->Point2.Y = InsertY;
-        AddObjectToCreateUndoList(LINE_TYPE, Layer, line, line);
-	IncrementUndoSerialNumber();
 	DrawLine(Layer, Line, 0);
-        DrawLine(Layer, line, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-        return(line);
+	Draw();
+        return(Line);
 }
 
 /* ---------------------------------------------------------------------------
@@ -107,26 +129,40 @@ static void *InsertPointIntoPolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 {
 	PointType	save;
 	Cardinal	n;
+	LineType	line;
 
+	if (!Forcible)
+	{
+			/*
+		 	* first make sure adding the point is sensible
+		 	*/
+		line.Thickness = 0;
+		if (InsertAt == 0)
+			line.Point1 = Polygon->Points[Polygon->PointN -1];
+		else
+			line.Point1 = Polygon->Points[InsertAt -1];
+		line.Point2 = Polygon->Points[InsertAt];
+		if (IsPointOnLine((float) InsertX, (float) InsertY, 0.0, &line))
+			return(Polygon);
+	}
+		/*
+	 	 * second, shift the points up to make room for the new point
+	 	 */
 	ErasePolygon(Polygon);
-
-		/* first of all get the index where the new point is to be
-		 * inserted; this is necessary because 'CreateNewPointInPolygon()'
-		 * may change some pointers
-		 *
-		 * second move all points of greater index up by one
-		 */
 	save = *CreateNewPointInPolygon(Polygon, InsertX, InsertY);
 	for (n = Polygon->PointN-1; n > InsertAt; n--)
 		Polygon->Points[n] = Polygon->Points[n-1];
 	Polygon->Points[InsertAt] = save;
-	SetPolygonBoundingBox(Polygon);
-	DrawPolygon(Layer, Polygon, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
 	SetChangedFlag(True);
 	AddObjectToInsertPointUndoList(POLYGONPOINT_TYPE, Layer, Polygon,
 		&Polygon->Points[InsertAt]);
-	UpdatePIPFlags(NULL, NULL, Layer, True);
+	if (Forcible || !RemoveExcessPolygonPoints(Layer, Polygon))
+	{
+		SetPolygonBoundingBox(Polygon);
+		DrawPolygon(Layer, Polygon, 0);
+		Draw();
+		UpdatePIPFlags(NULL, NULL, Layer, Polygon, True);
+	}
 	return(Polygon);
 }
 
@@ -134,7 +170,7 @@ static void *InsertPointIntoPolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
  * inserts point into objects
  */
 void *InsertPointIntoObject(int Type, void *Ptr1, void *Ptr2, Cardinal *Ptr3,
-	Position DX, Position DY)
+	Position DX, Position DY, Boolean Force)
 {
 	void	*ptr;
 
@@ -142,6 +178,7 @@ void *InsertPointIntoObject(int Type, void *Ptr1, void *Ptr2, Cardinal *Ptr3,
 	InsertX = DX;
 	InsertY = DY;
 	InsertAt = *Ptr3;
+	Forcible = Force;
 
 		/* the operation insert the points to the undo-list */
 	ptr = ObjectOperation(&InsertFunctions, Type, Ptr1, Ptr2, Ptr3);
