@@ -73,13 +73,12 @@ static	SelectorType	CircuitSelector =
 static Widget CreateTypeSelector(Widget Parent, Widget Top, Widget Left)
 {
 	Widget				selector;
-	Cardinal			i;
-	LibraryMenuTypePtr	libmenu;
 
 		/* create the selector, add all entries and sort them */
-	selector = CreateSelector(Parent, Top, Left, &TypeSelector);
-	for (i = Library.MenuN, libmenu = Library.Menu; i; i--, libmenu++)
-		AddEntryToSelector(libmenu->Name, (XtPointer) libmenu, &TypeSelector);
+	selector = CreateSelector(Parent, Top, Left, &TypeSelector, 1);
+	MENU_LOOP(&Library,
+		AddEntryToSelector(menu->Name, (XtPointer) menu, &TypeSelector);
+	);
 	return(selector);
 }
 
@@ -88,14 +87,12 @@ static Widget CreateTypeSelector(Widget Parent, Widget Top, Widget Left)
  */
 static void UpdateCircuitSelector(LibraryMenuTypePtr Menu)
 {
-	Cardinal			i;
-	LibraryEntryTypePtr	entry;
-
 	FreeSelectorEntries(&CircuitSelector);
-	for (i = Menu->EntryN, entry = Menu->Entry; i; i--, entry++)
+	ENTRY_LOOP(Menu,
 		AddEntryToSelector(
 			MyStrdup(entry->ListEntry, "UpdateCircuitSelector()"),
 			(XtPointer) entry, &CircuitSelector);
+	);
 	UpdateSelector(&CircuitSelector);
 
 		/* update label */
@@ -196,7 +193,7 @@ void InitLibraryWindow(Widget Parent)
 		NULL);
 	type = CreateTypeSelector(masterform, label, NULL);
 	XtVaSetValues(type, LAYOUT_LEFT, NULL);
-	circuit = CreateSelector(masterform, label, type, &CircuitSelector);
+	circuit = CreateSelector(masterform, label, type, &CircuitSelector, 1);
 
 		/* display the whole bunch */
 	XtPopup(popup, XtGrabNone);
@@ -205,6 +202,7 @@ void InitLibraryWindow(Widget Parent)
 	UpdateSelector(&TypeSelector);
 	UpdateCircuitSelector(Library.Menu);
 	
+	Library.Wind = XtWindow(popup);
 		/* register 'delete window' message (will be ignored) */
 	XSetWMProtocols(Dpy, XtWindow(popup), &WMDeleteWindowAtom, 1);
 }
