@@ -40,6 +40,11 @@
 #define	SWAP_IDENT			Settings.ShowSolderSide
 #endif
 
+#ifndef XORIG
+#define XORIG Xorig
+#define YORIG Yorig
+#endif
+
 #define	SWAP_SIGN_X(x)		(x)
 #define	SWAP_SIGN_Y(y)		(-(y))
 #define	SWAP_ANGLE(a)		(-(a))
@@ -51,8 +56,14 @@
 #define	TO_SCREEN(x)		((PCB->Zoom < 0) ? (x) << -PCB->Zoom : (x) >> PCB->Zoom)
 #endif
 
-#define	TO_SCREEN_X(x)		TO_SCREEN(SWAP_IDENT ? SWAP_X(x) : (x))
-#define	TO_SCREEN_Y(y)		TO_SCREEN(SWAP_IDENT ? SWAP_Y(y) : (y))
+#define	TO_SCREEN_X(x)		(TO_SCREEN(SWAP_IDENT ? SWAP_X(x) : (x)) + Xorig)
+#define	TO_SCREEN_Y(y)		(TO_SCREEN(SWAP_IDENT ? SWAP_Y(y) : (y)) + Yorig)
+#define	TO_DRAW_X(x)		(TO_SCREEN(SWAP_IDENT ? SWAP_X(x) : (x)) + XORIG)
+#define	TO_DRAWABS_X(x)		(TO_SCREEN((x)) + XORIG)
+#define	TO_DRAW_Y(y)		(TO_SCREEN(SWAP_IDENT ? SWAP_Y(y) : (y)) + YORIG)
+#define	TO_DRAWABS_Y(y)		(TO_SCREEN((y)) + YORIG)
+#define	TO_MASK_X(x)		TO_SCREEN_X((x))
+#define	TO_MASK_Y(y)		TO_SCREEN_Y((y))
 #define	TO_SCREEN_ANGLE(a)	(SWAP_IDENT ? SWAP_ANGLE((a)) : (a))
 #define	TO_SCREEN_DELTA(d)	(SWAP_IDENT ? SWAP_DELTA((d)) : (d))
 #define	TO_SCREEN_SIGN_X(x)	(SWAP_IDENT ? SWAP_SIGN_X(x) : (x))
@@ -61,9 +72,9 @@
 #ifndef	TO_PCB
 #define	TO_PCB(x)		((PCB->Zoom < 0) ? (x) >> -PCB->Zoom : (x) << PCB->Zoom)
 #endif
-#define	TO_PCB_X(x)		TO_PCB(x)
+#define	TO_PCB_X(x)		TO_PCB(x - Xorig)
 #define	TO_PCB_Y(y)		(SWAP_IDENT ? \
-				PCB->MaxHeight -TO_PCB(y) : TO_PCB(y))
+				PCB->MaxHeight -TO_PCB(y - Yorig) : TO_PCB(y - Yorig))
 
 /* ---------------------------------------------------------------------------
  * misc macros, some might already be defined by <limits.h>
@@ -71,6 +82,8 @@
 #ifndef	MIN
 #define	MIN(a,b)		((a) < (b) ? (a) : (b))
 #define	MAX(a,b)		((a) > (b) ? (a) : (b))
+#endif
+#ifndef SGN
 #define SGN(a)			((a) >= 0 ? 1 : -1)
 #endif
 
@@ -78,13 +91,54 @@
 #define	UNKNOWN(a)		((a) && *(a) ? (a) : "(unknown)")
 #define	EMPTY(a)		((a) ? (a) : "")
 #define XOR(a,b)		(((a) && !(b)) || (!(a) && (b)))
+#define SQUARE(x)		((float) (x) * (float) (x))
+#define TO_RADIANS(degrees)	(M180 * (degrees))
+ 
+/* ---------------------------------------------------------------------------
+ * macros to determine if something is on the visible part of the screen
+ */
+
+#define VELEMENT(e)	((e)->BoundingBox.X1 <= vxh && \
+	(e)->BoundingBox.X2 >= vxl && \
+	(e)->BoundingBox.Y1 <= vyh && \
+	(e)->BoundingBox.Y2 >= vyl)
+#define VELTEXT(e) 	(ELEMENT_TEXT(PCB,(e)).BoundingBox.X1 <= vxh && \
+	ELEMENT_TEXT(PCB,(e)).BoundingBox.X2 >= vxl && \
+	ELEMENT_TEXT(PCB,(e)).BoundingBox.Y1 <= vyh && \
+	ELEMENT_TEXT(PCB,(e)).BoundingBox.Y2 >= vyl)
+#define VTEXT(t) 	((t)->BoundingBox.X1 <= vxh && \
+	(t)->BoundingBox.X2 >= vxl && \
+	(t)->BoundingBox.Y1 <= vyh && \
+	(t)->BoundingBox.Y2 >= vyl)
+#define VLINE(l)	((((l)->Point2.X >= vxl && (l)->Point1.X <= vxh) || \
+	((l)->Point1.X >= vxl && (l)->Point2.X <= vxh)) && \
+	((((l)->Point2.Y >= vyl && (l)->Point1.Y <= vyh) || \
+	((l)->Point1.Y >= vyl && (l)->Point2.Y <= vyh))))
+#define VARC(a) 	((a)->BoundingBox.X1 <= vxh && \
+	(a)->BoundingBox.X2 >= vxl && \
+	(a)->BoundingBox.Y1 <= vyh && \
+	(a)->BoundingBox.Y2 >= vyl)
+#define VPOLY(p) 	((p)->BoundingBox.X1 <= vxh && \
+	(p)->BoundingBox.X2 >= vxl && \
+	(p)->BoundingBox.Y1 <= vyh && \
+	(p)->BoundingBox.Y2 >= vyl)
+#define VVIA(v) 	((v)->X >= vxl && (v)->X <= vxh && \
+			(v)->Y >= vyl && (v)->Y <= vyh)
 
 /* ---------------------------------------------------------------------------
- * returns pointer to current layer or to n-th element on stack
+ * layer macros
  */
 #define	LAYER_ON_STACK(n)	(&PCB->Data->Layer[LayerStack[(n)]])
-#define	CURRENT			(LAYER_ON_STACK(0))
-#define	INDEXOFCURRENT		LayerStack[0]
+#define	CURRENT			(PCB->SilkActive ? &PCB->Data->Layer[MAX_LAYER + \
+				(Settings.ShowSolderSide ? SOLDER_LAYER : COMPONENT_LAYER)] \
+				: LAYER_ON_STACK(0))
+#define	INDEXOFCURRENT		(PCB->SilkActive ? MAX_LAYER + \
+				(Settings.ShowSolderSide ? SOLDER_LAYER : COMPONENT_LAYER) \
+				: LayerStack[0])
+#define SILKLAYER		(&PCB->Data->Layer[MAX_LAYER + \
+				(Settings.ShowSolderSide ? SOLDER_LAYER : COMPONENT_LAYER)])
+#define BACKSILKLAYER		(&PCB->Data->Layer[MAX_LAYER + \
+				(Settings.ShowSolderSide ? COMPONENT_LAYER : SOLDER_LAYER)])
 
 /* ---------------------------------------------------------------------------
  * returns the object ID
@@ -94,7 +148,7 @@
 /* ---------------------------------------------------------------------------
  * access macro for current buffer
  */
-#define	PASTEBUFFER		(&Buffers[Settings.BufferNumber][Settings.ShowSolderSide ? 1 : 0])
+#define	PASTEBUFFER		(&Buffers[Settings.BufferNumber])
 
 /* ---------------------------------------------------------------------------
  * some routines for flag setting, clearing, changeing and testing
@@ -139,12 +193,20 @@
 
 /* ---------------------------------------------------------------------------
  *  Determines if text is actually visible
- */
 #define TEXT_IS_VISIBLE(b, l, t)					\
 		(((l)->On && !TEST_FLAG(ONSILKFLAG, (t))) ||     	\
 		(TEST_FLAG(ONSILKFLAG, (t)) && (b)->ElementOn   	\
 		&& ((b)->InvisibleObjectsOn ||                  	\
 		(TEST_FLAG(ONSOLDERFLAG, (t)) != 0) == SWAP_IDENT)))
+*/
+#define TEXT_IS_VISIBLE(b, l, t) \
+	((l)->On)
+
+/* ---------------------------------------------------------------------------
+ *  Determines if object is on front or back
+ */
+#define FRONT(o)	\
+	((TEST_FLAG(ONSOLDERFLAG, (o)) != 0) == SWAP_IDENT)
 			
 /* ---------------------------------------------------------------------------
  * some loop shortcuts
@@ -159,7 +221,8 @@
 #define	VIA_LOOP(top, command)	{ 		\
 	Cardinal	n, sn;			\
 	PinTypePtr	via;			\
-        for (sn = (top)->ViaN, n = 0; (top)->ViaN > 0 && n < (top)->ViaN ; sn == (top)->ViaN ? n++ : 0)   \
+        for (sn = (top)->ViaN, n = 0; (top)->ViaN > 0 && n < (top)->ViaN ; \
+		n += 1 + (top)->ViaN - sn, sn = (top)->ViaN)   \
 	{					\
 		via = &(top)->Via[n];		\
 		command;			\
@@ -174,16 +237,16 @@
                 command;                                        \
         }}      
 
-#define NET_LOOP(top, command)        {                               \
+#define NET_LOOP(top, command)    {                             \
         Cardinal        n;                                      \
-        SubnetTypePtr   subnet;                                 \
-        for (n = (top)->SubnetN-1; n != -1; n--)                        \
+        NetTypePtr   net;                                       \
+        for (n = (top)->NetN-1; n != -1; n--)                   \
         {                                                       \
-                subnet = &(top)->Subnet[n];                     \
+                net = &(top)->Net[n];                           \
                 command;                                        \
         }}      
 
-#define SUBNET_LOOP(net, command) {                           \
+#define CONNECTION_LOOP(net, command) {                         \
         Cardinal        n;                                      \
         ConnectionTypePtr       connection;                     \
         for (n = (net)->ConnectionN-1; n != -1; n--)            \
@@ -201,6 +264,16 @@
 		command;				\
 	}}
 
+#define RAT_LOOP(top, command)	{			\
+	Cardinal	n;				\
+	RatTypePtr	line;				\
+	for (n = (top)->RatN-1; n != -1; n--)		\
+	{						\
+		line = &(top)->Rat[n];			\
+		command;				\
+	}}
+
+
 #define	ELEMENTTEXT_LOOP(element, command)	{ 	\
 	Cardinal	n;				\
 	TextTypePtr	text;				\
@@ -223,7 +296,7 @@
 	Cardinal	n, sn;				\
 	PinTypePtr	pin;				\
         for (sn = (element)->PinN, n = 0; (element)->PinN > 0 && n < (element)->PinN ; \
-		 sn == (element)->PinN ? n++ : 0)   \
+		n += 1 + (element)->PinN - sn, sn = (element)->PinN)   \
 	{						\
 		pin = &(element)->Pin[n];		\
 		command;				\
@@ -304,35 +377,56 @@
 #define	ALLLINE_LOOP(top, command)	{		\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (top)->Layer;		\
-	for (l = 0; l < MAX_LAYER; l++, layer++)	\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
 		LINE_LOOP(layer, command );		\
 	}
 
 #define ALLARC_LOOP(top, command)	{		\
+	Cardinal		l;			\
+	LayerTypePtr	layer = (top)->Layer;		\
+	for (l =0; l < MAX_LAYER + 2; l++, layer++)		\
+		ARC_LOOP(layer, command );		\
+	}
+
+#define	ALLPOLYGON_LOOP(top, command)	{		\
+	Cardinal		l;			\
+	LayerTypePtr	layer = (top)->Layer;		\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
+		POLYGON_LOOP(layer, command );		\
+	}
+
+#define	COPPERLINE_LOOP(top, command)	{		\
+	Cardinal		l;			\
+	LayerTypePtr	layer = (top)->Layer;		\
+	for (l = 0; l < MAX_LAYER; l++, layer++)	\
+		LINE_LOOP(layer, command );		\
+	}
+
+#define COPPERARC_LOOP(top, command)	{		\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (top)->Layer;		\
 	for (l =0; l < MAX_LAYER; l++, layer++)		\
 		ARC_LOOP(layer, command );		\
 	}
 
-#define	ALLTEXT_LOOP(top, command)	{		\
-	Cardinal		l;			\
-	LayerTypePtr	layer = (top)->Layer;		\
-	for (l = 0; l < MAX_LAYER; l++, layer++)	\
-		TEXT_LOOP(layer, command );		\
-	}
-
-#define	ALLPOLYGON_LOOP(top, command)	{		\
+#define	COPPERPOLYGON_LOOP(top, command)	{		\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (top)->Layer;		\
 	for (l = 0; l < MAX_LAYER; l++, layer++)	\
 		POLYGON_LOOP(layer, command );		\
 	}
 
+#define	ALLTEXT_LOOP(top, command)	{		\
+	Cardinal		l;			\
+	LayerTypePtr	layer = (top)->Layer;		\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
+		TEXT_LOOP(layer, command );		\
+	}
+
 #define	VISIBLELINE_LOOP(top, command)	{		\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (top)->Layer;		\
-	for (l = 0; l < MAX_LAYER; l++, layer++)	\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
 		if (layer->On)				\
 			LINE_LOOP(layer, command );	\
 	}
@@ -340,7 +434,7 @@
 #define	VISIBLEARC_LOOP(top, command)	{		\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (top)->Layer;		\
-	for (l = 0; l < MAX_LAYER; l++, layer++)	\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
 		if (layer->On)				\
 			ARC_LOOP(layer, command );	\
 	}
@@ -348,7 +442,7 @@
 #define	VISIBLETEXT_LOOP(board, command)	{		\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (board)->Data->Layer;		\
-	for (l = 0; l < MAX_LAYER; l++, layer++)	\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
                 TEXT_LOOP(layer,                                        \
                           if (TEXT_IS_VISIBLE((board), layer, text))                      \
                           command );                              \
@@ -357,10 +451,36 @@
 #define	VISIBLEPOLYGON_LOOP(top, command)	{	\
 	Cardinal		l;			\
 	LayerTypePtr	layer = (top)->Layer;		\
-	for (l = 0; l < MAX_LAYER; l++, layer++)	\
+	for (l = 0; l < MAX_LAYER + 2; l++, layer++)	\
 		if (layer->On)				\
 			POLYGON_LOOP(layer, command );	\
 	}
 
+#define POINTER_LOOP(top, command)	{	\
+	Cardinal	n;			\
+	void	**ptr;				\
+	for (n = (top)->PtrN-1; n != -1; n--)	\
+	{					\
+		ptr = &(top)->Ptr[n];		\
+		command;			\
+	}}
+
+#define MENU_LOOP(top, command)	{	\
+	Cardinal	l;			\
+	LibraryMenuTypePtr menu;		\
+	for (l = (top)->MenuN-1; l != -1; l--)	\
+	{					\
+		menu = &(top)->Menu[l];		\
+		command;			\
+	}}
+
+#define ENTRY_LOOP(top, command)	{	\
+	Cardinal	n;			\
+	LibraryEntryTypePtr entry;		\
+	for (n = (top)->EntryN-1; n != -1; n--)	\
+	{					\
+		entry = &(top)->Entry[n];	\
+		command;			\
+	}}
 #endif
 
