@@ -28,6 +28,7 @@ static	char	*rcsid = "$Id: main.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 
 /* main program, initializes some stuff and handles user input
  */
+#define SCROLL_TIME 25
 
 #include <stdlib.h>
 #include <string.h>
@@ -49,6 +50,7 @@ static	char	*rcsid = "$Id: main.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 #include "menu.h"
 #include "misc.h"
 #include "mymem.h"
+#include "output.h"
 #include "remove.h"
 #include "set.h"
 
@@ -59,9 +61,9 @@ static	char	*rcsid = "$Id: main.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 #include <X11/Xaw/Command.h> 
 #include <X11/Xaw/Form.h> 
 #include <X11/Xaw/Label.h> 
+#include <X11/Xaw/Porthole.h>
 #include <X11/Xaw/Scrollbar.h> 
 #include <X11/Xaw/Simple.h> 
-#include <X11/Xaw/Viewport.h> 
 
 /* ---------------------------------------------------------------------------
  * icon data as created by 'bitmap'
@@ -71,15 +73,11 @@ static	char	*rcsid = "$Id: main.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 /* ---------------------------------------------------------------------------
  * some local prototypes
  */
-static	void	GetSizeOfDrawingArea(void);
-static	void	ViewportEvent(Widget, XtPointer, XEvent *, Boolean *);
-static	void	OutputEvent(Widget, XtPointer, XEvent *, Boolean *);
 static	void	CB_Backup(XtPointer, XtIntervalId *);
 static	void	InitShell(int *, char**);
 static	void	InitCursorPosition(Widget, Widget, Widget);
 static	void	InitStatusLine(Widget, Widget, Widget);
-static	void	InitViewport(Widget, Widget, Widget);
-static	void	InitAccelerators(void);
+static	void	InitPorthole(Widget, Widget, Widget);
 static	void	InitGC(void);
 static	void	InitIcon(void);
 static	void	InitHandler(void);
@@ -118,7 +116,7 @@ static	String			Fallback[] = {
 	"*maxPCBWidth:       20000",
 	"*maxPCBHeight:      20000",
 	"*outputMasterForm.viewport.width:  900",
-	"*outputMasterForm.viewport.height: 600",
+	"*outputMasterForm.viewport.height: 620",
 	"*pinoutFont0:    -*-courier-bold-r-*-*-24-*-*-*-*-*-*-*",
 	"*pinoutFont1:    -*-courier-bold-r-*-*-12-*-*-*-*-*-*-*",
 	"*pinoutFont2:    -*-courier-bold-r-*-*-8-*-*-*-*-*-*-*",
@@ -132,26 +130,25 @@ static	String			Fallback[] = {
 /* ---------------------------------------------------------------------------
  * resources to query
  * the number of layers has to be adapted here and in 'global.h'
+ * and elsewhere (Think of the bit flags ...)
  */
 static	XtResource		ToplevelResources[] = {
-	{ "rubberBandMode", "RubberBandMode", XtRBoolean, sizeof(Boolean),
-	  XtOffsetOf(SettingType, RubberBandMode), XtRString, "True" },
 	{ "absoluteGrid", "AbsoluteGrid", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, AbsoluteGrid), XtRString, "True" },
 	{ "alignmentDistance", "AlignmentDistance", XtRDimension, sizeof(Dimension),
 	  XtOffsetOf(SettingType, AlignmentDistance), XtRString, "200" },
 	{ "allDirectionLines", "AllDirectionLines", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, AllDirectionLines), XtRString, "True" },
-	{ "apertureCommand", "ApertureCommand", XtRString, sizeof(String),
-	  XtOffsetOf(SettingType, ApertureCommand), XtRString, "cat %f" },
-	{ "aperturePath", "AperturePath", XtRString, sizeof(String),
-	  XtOffsetOf(SettingType, AperturePath), XtRString, PCBLIBDIR },
 	{ "backupInterval", "BackupInterval", XtRInt, sizeof(long),
 	  XtOffsetOf(SettingType, BackupInterval), XtRString, "300" },
+	{ "bloat", "Bloat", XtRDimension, sizeof(Dimension),
+	  XtOffsetOf(SettingType, Bloat), XtRString, "7" },
 	{ "charactersPerLine", "CharactersPerLine", XtRInt, sizeof(int),
 	  XtOffsetOf(SettingType, CharPerLine), XtRString, "80" },
 	{ "connectedColor", XtCColor, XtRPixel, sizeof(Pixel),
 	  XtOffsetOf(SettingType, ConnectedColor), XtRString, XtDefaultForeground },
+	{ "crossColor", XtCColor, XtRPixel, sizeof(Pixel),
+	  XtOffsetOf(SettingType, CrossColor), XtRString, XtDefaultForeground },
 	{ "crosshairColor", XtCColor, XtRPixel, sizeof(Pixel),
 	  XtOffsetOf(SettingType, CrosshairColor), XtRString, XtDefaultForeground },
 	{ "elementColor", XtCColor, XtRPixel, sizeof(Pixel),
@@ -181,7 +178,9 @@ static	XtResource		ToplevelResources[] = {
 	{ "invisibleObjectsColor", XtCColor, XtRPixel, sizeof(Pixel),
 	  XtOffsetOf(SettingType, InvisibleObjectsColor),
 	  XtRString, XtDefaultForeground },
-
+	{ "invisibleMarkColor", XtCColor, XtRPixel, sizeof(Pixel),
+	  XtOffsetOf(SettingType, InvisibleMarkColor),
+	  XtRString, XtDefaultForeground },
 	{ "layerColor1", XtCColor, XtRPixel, sizeof(Pixel),
 	  XtOffsetOf(SettingType, LayerColor[0]), XtRString, XtDefaultForeground },
 	{ "layerColor2", XtCColor, XtRPixel, sizeof(Pixel),
@@ -291,14 +290,23 @@ static	XtResource		ToplevelResources[] = {
 	  XtOffsetOf(SettingType, PrintFile), XtRString, "%f.output" },
 	{ "raiseLogWindow", "RaiseLogWindow", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, RaiseLogWindow), XtRString, "True" },
+	{ "ratColor", XtCColor, XtRPixel, sizeof(Pixel),
+	  XtOffsetOf(SettingType, RatColor), XtRString, XtDefaultForeground },
+	{ "ratSelectedColor", XtCColor, XtRPixel, sizeof(Pixel),
+	  XtOffsetOf(SettingType, RatSelectedColor),
+	  XtRString, XtDefaultForeground },
 	{ "ratCommand", "RatCommand", XtRString, sizeof(String),
 	  XtOffsetOf(SettingType, RatCommand), XtRString, "cat %f" },
 	{ "ratPath", "RatPath", XtRString, sizeof(String),
 	  XtOffsetOf(SettingType, RatPath), XtRString, "." },
+	{ "ratThickness", XtCThickness, XtRDimension, sizeof(Dimension),
+	  XtOffsetOf(SettingType, RatThickness), XtRString, "5" },
 	{ "resetAfterElement", "ResetAfterElement", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, ResetAfterElement), XtRString, "False" },
 	{ "ringBellWhenFinished", "RingBellWhenFinished",XtRBoolean,sizeof(Boolean),
 	  XtOffsetOf(SettingType, RingBellWhenFinished), XtRString, "True" },
+	{ "routeStyles", "RouteStyles", XtRString, sizeof(String),
+	  XtOffsetOf(SettingType, Routes), XtRString, "50,80,40:20,60,28:10,50,28:30,65,35" },
 	{ "rubberBandMode", "RubberBandMode", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, RubberBandMode), XtRString, "True" },
 	{ "saveCommand", "SaveCommand", XtRString, sizeof(String),
@@ -307,10 +315,14 @@ static	XtResource		ToplevelResources[] = {
 	  XtOffsetOf(SettingType, SaveInTMP), XtRString, "True" },
 	{ "saveLastCommand", "SaveLastCommand", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, SaveLastCommand), XtRString, "False" },
+	{ "shrink", "Shrink", XtRDimension, sizeof(Dimension),
+	  XtOffsetOf(SettingType, Shrink), XtRString, "5" },
 	{ "size", "Size", XtRString, sizeof(String),
 	  XtOffsetOf(SettingType, Size), XtRString, DEFAULT_SIZE },
 	{ "stipplePolygons", "StipplePolygons", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, StipplePolygons), XtRString, "False" },
+	{ "swapStartDirection", "SwapStartDirection", XtRBoolean, sizeof(Boolean),
+	  XtOffsetOf(SettingType, SwapStartDirection), XtRString, "True" },
 	{ "textScale", "TextScale", XtRDimension, sizeof(Dimension),
 	  XtOffsetOf(SettingType, TextScale), XtRString, "100" },
 	{ "useLogWindow", "UseLogWindow", XtRBoolean, sizeof(Boolean),
@@ -355,6 +367,7 @@ static	XrmOptionDescRec	CommandLineOptions[] = {
 	{ "-libpath", "libraryPath", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-llib", "libraryCommand", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-llibcont", "libraryContentsCommand", XrmoptionSepArg, (caddr_t) NULL },
+	{ "-log", "useLogWindow", XrmoptionNoArg, (caddr_t) "False" },
 	{ "-loggeometry", "log.geometry", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-pnl", "pinoutNameLength", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-pz", "pinoutZoom", XrmoptionSepArg, (caddr_t) NULL },
@@ -362,6 +375,7 @@ static	XrmOptionDescRec	CommandLineOptions[] = {
 	{ "+reset", "resetAfterElement", XrmoptionNoArg, (caddr_t) "False" },
 	{ "-ring", "ringBellWhenFinished", XrmoptionNoArg, (caddr_t) "True" },
 	{ "+ring", "ringBellWhenFinished", XrmoptionNoArg, (caddr_t) "False" },
+	{ "-rs", "routeStyles", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-s", "saveLastCommand", XrmoptionNoArg, (caddr_t) "True" },
 	{ "+s", "saveLastCommand", XrmoptionNoArg, (caddr_t) "False" },
 	{ "-save", "saveInTMP", XrmoptionNoArg, (caddr_t) "True" },
@@ -374,9 +388,17 @@ static	XrmOptionDescRec	CommandLineOptions[] = {
  * actions
  */
 static	XtActionsRec	Actions[] = {
+	{ "ButtonThree", ActionButton3 },
+	{ "SetSame", ActionSetSame },
 	{ "MovePointer", ActionMovePointer },
 	{ "ToggleHideName", ActionToggleHideName },
 	{ "ChangeHole", ActionChangeHole },
+	{ "ToggleThermal", ActionToggleThermal },
+	{ "Atomic", ActionAtomic },
+	{ "AdjustStyle", ActionAdjustStyle },
+	{ "RouteStyle", ActionRouteStyle },
+	{ "DRC", ActionDRCheck },
+	{ "Flip", ActionFlip },
 	{ "SetValue", ActionSetValue },
 	{ "FinishInputDialog", ActionFinishInputDialog },
 	{ "Quit", ActionQuit },
@@ -384,17 +406,19 @@ static	XtActionsRec	Actions[] = {
 	{ "Command", ActionCommand },
 	{ "Display", ActionDisplay },
 	{ "Report", ActionReport },
+	{ "ListAct", ActionListAct },
 	{ "Mode", ActionMode },
-	{ "MoveToSilk", ActionMoveToSilk },
 	{ "RemoveSelected", ActionRemoveSelected },
 	{ "DeleteRats", ActionDeleteRats },
 	{ "AddRats", ActionAddRats },
 	{ "MarkCrosshair", ActionMarkCrosshair },
 	{ "ChangeSize", ActionChangeSize },
+	{ "ChangeClearSize", ActionChangeClearSize },
 	{ "ChangeDrillSize", ActionChange2ndSize },
 	{ "ChangeName", ActionChangeName },
 	{ "ChangeSquare", ActionChangeSquare },
 	{ "ChangeOctagon", ActionChangeOctagon },
+	{ "ChangeJoin", ActionChangeJoin },
 	{ "Select", ActionSelect },
 	{ "Unselect", ActionUnselect },
 	{ "Save", ActionSave },
@@ -411,107 +435,6 @@ static	XtActionsRec	Actions[] = {
 	{ "MoveToCurrentLayer", ActionMoveToCurrentLayer },
 	{ "SwitchDrawingLayer", ActionSwitchDrawingLayer }};
 
-/* ----------------------------------------------------------------------
- * sets the size of the drawing area
- * is called by MapNotify and ConfigureNotify events
- *
- * if the function is called during dialog handling (indicated by a valid
- * pixmap) a resize event has occured. This is the only possibility
- * because dialogs are modal to the application. In this case the pixmap
- * is released if the new size is larger than the old one.
- */
-static void GetSizeOfDrawingArea(void)
-{
-	Dimension	width, height,
-			scroll_width, scroll_height,
-			border_left, border_bottom;
-	Position	x, y,
-			maxw = TO_SCREEN(PCB->MaxWidth),
-			maxh = TO_SCREEN(PCB->MaxHeight);
-
-	XtVaGetValues(Output.Viewport,
-		XtNwidth, &width,
-		XtNheight, &height,
-		NULL);
-
-		/* adjust the size of the statusline */
-	XtVaSetValues(Output.StatusLine, XtNwidth, width, NULL);
-
-		/* reduce the viewportsize by the dimensions of the scrollbars */
-	XtVaGetValues(Output.ScrollbarBottom,
-		XtNheight, &scroll_height,
-		XtNborderWidth, &border_bottom,
-		NULL);
-	XtVaGetValues(Output.ScrollbarLeft,
-		XtNwidth, &scroll_width,
-		XtNborderWidth, &border_left,
-		NULL);
-
-		/* calculate new sizes */
-	width -= (scroll_width +border_left);
-	height -= (scroll_height +border_bottom);
-
-		/* get origin (!!! offset is <= 0 !!!) */
-	XtVaGetValues(Output.Output, XtNx, &x, XtNy, &y, NULL);
-	x = -x;
-	y = -y;
-
-		/* release pixmap if necessary (size or offset changed) */
-	if (width > Output.Width || height > Output.Height ||
-		x != Output.OffsetX || y != Output.OffsetY)
-		ReleaseSaveUnderPixmap();
-
-		/* limit the viewport size to the drawing area to prevent scrollbars
-		 * from growing if the window is resized
-		 */
-	Output.OffsetX = x;
-	Output.OffsetY = y;
-	Output.Width = MIN(width, maxw);
-	Output.Height = MIN(height, maxh);
-	if (width > maxw)
-	{
-		Output.Width = maxw;
-		XtVaSetValues(Output.Output, XtNwidth, Output.Width, NULL);
-	}
-	else
-		Output.Width = width;
-	if (height > maxh)
-	{
-		Output.Height = maxh;
-		XtVaSetValues(Output.Output, XtNheight, Output.Height, NULL);
-	}
-	else
-		Output.Height = height;
-}
-
-/* ---------------------------------------------------------------------- 
- * handles all events from viewport window
- */
-static void ViewportEvent(Widget W,
-	XtPointer ClientData, XEvent *Event, Boolean *Flag)
-{
-	switch(Event->type)
-	{
-			/* set flag when viewport is completely visible; needed
-			 * to make sure that 'SaveOutputWindow()' works correctly.
-			 * The viewport event has to be taken because the outputwindow
-			 * is always obscured by the viewports clipping window
-			 */
-		case VisibilityNotify:
-			Output.VisibilityOK = 
-				(((XVisibilityEvent *) Event)->state == VisibilityUnobscured);
-			break;
-
-		case UnmapNotify:
-			Output.VisibilityOK = False;
-			break;
-
-		case ConfigureNotify:		/* get new size of drawing area */
-			GetSizeOfDrawingArea();
-			break;
-	}
-} 
-
 /* ---------------------------------------------------------------------------
  * handles callbacks from timer,
  * used to backup data
@@ -524,98 +447,6 @@ static void CB_Backup(XtPointer ClientData, XtIntervalId *ID)
 	XtAppAddTimeOut(Context, 1000*Settings.BackupInterval, CB_Backup, NULL);
 }
 
-/* ---------------------------------------------------------------------- 
- * handles all events from output window
- * the handling of MapNotify and ConfigureNotify could be done much
- * easier with X11R5 and later but...
- *
- * I also had to set a flag to indicate a valid 'enter window' because
- * modal windows may generate 'LeaveNotify' events without 'EnterNotify'.
- * This behavior disturbs the crosshair-visibility management
- */
-static void OutputEvent(Widget W,
-	XtPointer ClientData, XEvent *Event, Boolean *Flag)
-{
-	int			ksym;
-	XExposeEvent		*event = (XExposeEvent *) Event;
-	static	Boolean		has_entered = False;
-
-	switch(Event->type)
-	{
-		case Expose:	/* expose event means redraw */
-			if (event->send_event)
-			{
-				XSetRegion(Dpy, Output.fgGC, UpRegion);
-				XSetRegion(Dpy, Output.bgGC, UpRegion);
-				XSetRegion(Dpy, Output.GridGC, UpRegion);
-				RedrawOutput();
-				XDestroyRegion(UpRegion);
-				UpRegion = XCreateRegion();
-				XSetRegion(Dpy, Output.fgGC, FullRegion);
-				XSetRegion(Dpy, Output.bgGC, FullRegion);
-				XSetRegion(Dpy, Output.GridGC, FullRegion);
-			}
-			else if (VALID_PIXMAP(PCB->SaveUnder))
-			{
-					/* make sure that cursor is in the same state
-					 * as when a dialog poped up. The state is determined
-					 * by the Enter- and LeaveNotify events
-					 */
-				HideCrosshair(False);
-				if (!XCopyArea(Dpy, PCB->SaveUnder, Output.OutputWindow,
-					Output.fgGC,
-					event->x -Output.OffsetX, event->y -Output.OffsetY,
-					event->width, event->height,
-					event->x, event->y))
-				{
-					RedrawOutput();
-				}
-				RestoreCrosshair(False);
-			}
-			else
-				ClearAndRedrawOutput();
-			break;
-
-		case EnterNotify:			/* enter output area */
-			RestoreCrosshair(False);
-			has_entered = True;
-			if (RedrawOnEnter)		/* control panel was used -> redraw */
-			{
-				ReleaseSaveUnderPixmap();
-				ClearAndRedrawOutput();
-			}
-			RedrawOnEnter = False;
-			break;
-
-		case LeaveNotify:			/* leave output area */
-				/* see also the description at start of this function */
-			if (has_entered)
-				HideCrosshair(False);
-			has_entered = False;	
-			break;
-			
-		case MapNotify:				/* get initial size of drawing area */
-		case ConfigureNotify:		/* get position of origin */
-			GetSizeOfDrawingArea();
-			break;
-
-		case MotionNotify:
-			EventMoveCrosshair((XMotionEvent *) Event);
-			break;
-
-		case KeyPress:
-		case KeyRelease:
-			ksym = XKeycodeToKeysym(Dpy, ((XKeyEvent *) Event)->keycode, 0);
-			if (ksym == XK_Shift_R || ksym == XK_Shift_L)
-			{
-				HideCrosshair(False);
-				AdjustAttachedObjects();
-				RestoreCrosshair(True);
-			}
-			break;
-	}
-} 
-
 /* ---------------------------------------------------------------------------
  * init toplevel shell and get application resources
  */
@@ -626,9 +457,15 @@ static void InitShell(int *Argc, char **Argv)
 	XModifierKeymap		*modMap;
 
 		/* init application toplevel window, get resources */
-	Output.Toplevel = XtAppInitialize(&Context, "Pcb",
+/*
+	Output.Toplevel = XtOpenApplication(&Context, "Pcb",
 		CommandLineOptions, XtNumber(CommandLineOptions),
-		Argc, Argv, Fallback, NULL, 0);
+		Argc, Argv, Fallback, sessionShellWidgetClass, NULL, 0);
+*/
+        Output.Toplevel = XtAppInitialize(&Context, "Pcb",
+                CommandLineOptions, XtNumber(CommandLineOptions),
+                Argc, Argv, Fallback, NULL, 0);
+
 	XtVaSetValues(Output.Toplevel,
 		XtNmappedWhenManaged, False,
 		NULL);
@@ -658,6 +495,7 @@ static void InitShell(int *Argc, char **Argv)
 	Settings.MaxHeight = MIN(MAX_COORD, MAX((Dimension) height, MIN_SIZE));
 	Settings.Volume = MIN(100, MAX(-100, Settings.Volume));
 	ParseGroupString(Settings.Groups, &Settings.LayerGroups);
+	ParseRouteString(Settings.Routes, &Settings.RouteStyle[0]);
 	modMap = XGetModifierMapping(Dpy);
 	ShiftKeyIndex = modMap->modifiermap[ShiftMapIndex] / 8;
 	ShiftKeyMask = 1 << (modMap->modifiermap[ShiftMapIndex] % 8);
@@ -694,25 +532,21 @@ static void InitStatusLine(Widget Parent, Widget Top, Widget Left)
 }
 
 /* ---------------------------------------------------------------------------
- * initialize output and viewport widget
+ * initialize output and porthole widget
  */
-static void InitViewport(Widget Parent, Widget Top, Widget Left)
+static void InitPorthole(Widget Parent, Widget Top, Widget Left)
 {
-		/* viewport widget to handle scrolling */
-	Output.Viewport = XtVaCreateManagedWidget("viewport", viewportWidgetClass,
+		/* porthole widget to handle panning */
+	Output.Porthole = XtVaCreateManagedWidget("porthole", portholeWidgetClass,
 		Parent,
 		XtNfromHoriz, Left,
 		XtNfromVert, Top,
 		LAYOUT_NORMAL,
-		XtNallowHoriz, True,
-		XtNallowVert, True,
-		XtNuseBottom, True,
-		XtNforceBars, True,
 		NULL);
 
 		/* create simple widget to which I draw to (handled by viewport) */
 	Output.Output = XtVaCreateManagedWidget("output", simpleWidgetClass,
-		Output.Viewport,
+		Output.Porthole,
 		XtNresizable, True,
 		XtNheight, TO_SCREEN(Settings.MaxHeight),
 		XtNwidth, TO_SCREEN(Settings.MaxWidth),
@@ -723,27 +557,13 @@ static void InitViewport(Widget Parent, Widget Top, Widget Left)
 		ExposureMask | LeaveWindowMask | EnterWindowMask |
 		StructureNotifyMask | PointerMotionMask | KeyPressMask | KeyReleaseMask,
 		False, (XtEventHandler) OutputEvent, NULL);
-	XtAddEventHandler(Output.Viewport,
+	XtAddEventHandler(Output.Porthole,
 		StructureNotifyMask | VisibilityChangeMask,
-		False, (XtEventHandler) ViewportEvent, NULL);
-}
-
-/* ---------------------------------------------------------------------------
- * installs accelerators for scrollbars
- */
-static void InitAccelerators(void)
-{
-		/* install accelerators for scrollbars */
-	Output.ScrollbarBottom = XtNameToWidget(Output.Viewport, "horizontal");
-	Output.ScrollbarLeft = XtNameToWidget(Output.Viewport, "vertical");
-	if (Output.ScrollbarBottom != NULL)
-		XtInstallAccelerators(Output.Output, Output.ScrollbarBottom);
-	if (Output.ScrollbarLeft != NULL)
-		XtInstallAccelerators(Output.Output, Output.ScrollbarLeft);
+		False, (XtEventHandler) PortholeEvent, NULL);
 }
 
 /* ----------------------------------------------------------------------
- * initializes icon pixmap
+ * initializes icon pixmap and also cursor bit maps
  */
 static void InitIcon(void)
 {
@@ -757,6 +577,10 @@ static void InitIcon(void)
 			BlackPixelOfScreen(screen), WhitePixelOfScreen(screen),
 			DefaultDepthOfScreen(screen)),
 		NULL);
+	XC_clock_source = XCreateBitmapFromData(Dpy, XtWindow(Output.Toplevel), 
+		rotateIcon_bits, rotateIcon_width, rotateIcon_height);
+	XC_clock_mask = XCreateBitmapFromData(Dpy, XtWindow(Output.Toplevel), 
+		rotateMask_bits, rotateMask_width, rotateMask_height);
 }
 
 /* ----------------------------------------------------------------------
@@ -772,15 +596,18 @@ static void InitGC(void)
 	XtVaGetValues(Output.Output, XtNbackground, &Settings.bgColor, NULL);
 	Output.fgGC = XCreateGC(Dpy, Output.OutputWindow, 0, NULL);
 	Output.bgGC = XCreateGC(Dpy, Output.OutputWindow, 0, NULL);
+	Output.pmGC = (GC) 0;
 	Output.GridGC = XCreateGC(Dpy, Output.OutputWindow, 0, NULL);
 	if (!VALID_GC((int) Output.fgGC) ||
 		!VALID_GC((int) Output.bgGC) ||
-		!VALID_GC((int) Output.GridGC))
+		!VALID_GC((int) Output.GridGC ||
+		!VALID_GC((int) Output.pmGC)))
 		MyFatal("can't create default GC\n");
 
 	XSetForeground(Dpy, Output.bgGC, Settings.bgColor);
 	XSetFillRule(Dpy, Output.fgGC, WindingRule);
 	XSetFillStyle(Dpy, Output.fgGC, FillSolid);
+	Stipples = MyCalloc(9, sizeof(Pixmap), "main()");
 	for (i =0; i < 9; i++)
 	{
 		Stipples[i] = XCreateBitmapFromData(Dpy, Output.OutputWindow, 
@@ -832,8 +659,8 @@ static void InitWidgets(void)
 	InitMenu(Output.MasterForm, NULL, NULL);
 	InitControlPanel(Output.MasterForm, Output.Menu, NULL);
 	InitCursorPosition(Output.MasterForm, NULL, Output.Menu);
-	InitViewport(Output.MasterForm, Output.Menu, Output.Control);
-	InitStatusLine(Output.MasterForm, Output.Viewport, NULL);
+	InitPorthole(Output.MasterForm, Output.Menu, Output.Control);
+	InitStatusLine(Output.MasterForm, Output.Porthole, NULL);
 
 		/* set actions and install translations;
 		 * the menu must be initialized before the actions are added
@@ -855,10 +682,9 @@ static void InitWidgets(void)
  */
 	InitModeButtons(Output.MasterForm, Output.Control, NULL);
 	XtDestroyWidget(Output.StatusLine);
-	InitStatusLine(Output.MasterForm, Output.Viewport, NULL);
+	InitStatusLine(Output.MasterForm, Output.Porthole, NULL);
 	Output.OutputWindow = XtWindow(Output.Output);	
 
-	InitAccelerators();
 	InitGC();
 	InitCrosshair();
 	InitIcon();
@@ -867,8 +693,9 @@ static void InitWidgets(void)
 	SetOutputXCursor(XC_crosshair);
 
 		/* set window manager property to get 'delete window' messages */
-	WMDeleteWindowAtom = XInternAtom(Dpy, "WM_DELETE_WINDOW", False);
-	XSetWMProtocols(Dpy, XtWindow(Output.Toplevel), &WMDeleteWindowAtom, 1);
+	WMDeleteWindowAtom = XInternAtom(Dpy, "WM_DELETE_WINDOW", True);
+	if (WMDeleteWindowAtom != None)
+		XSetWMProtocols(Dpy, XtWindow(Output.Toplevel), &WMDeleteWindowAtom, 1);
 }
 
 /* ---------------------------------------------------------------------------
@@ -932,15 +759,12 @@ int main(int argc, char *argv[])
 		/* update zoom and grid... */
 	UpdateSettingsOnScreen();
 	GetSizeOfDrawingArea();
-		/* set up the Erasure client message */
-	Erasure.type = Expose;
-	Erasure.display = Dpy;
-	Erasure.window = Output.OutputWindow;
 	UpRegion = XCreateRegion();
 	FullRegion = XCreateRegion();
 	Big.x = Big.y = 0;
 	Big.width = Big.height = MAX_COORD;
 	XUnionRectWithRegion(&Big, FullRegion, FullRegion);
+	InitErrorLog();
 	switch(--argc)
 	{
 		case 0:		/* only program name */
@@ -971,11 +795,11 @@ int main(int argc, char *argv[])
 		 * If the OS doesn't have at least one of them,
 		 * the critical sections will be handled by parse_l.l
 		 */
-#ifdef HAS_ATEXIT
-	atexit(EmergencySave);
-#else
 #ifdef HAS_ON_EXIT
 	on_exit(GlueEmergencySave, NULL);
+#else
+#ifdef HAS_ATEXIT
+	atexit(EmergencySave);
 #endif
 #endif
 
@@ -984,7 +808,6 @@ int main(int argc, char *argv[])
 		 * (dont know why)
 		 */
 	XMapWindow(Dpy, XtWindow(Output.Toplevel));
-	InitErrorLog();
 
 		/* read the library file and display it if it's not empty */
 	if (!ReadLibraryContents() && Library.MenuN)
@@ -992,7 +815,6 @@ int main(int argc, char *argv[])
 
 		/* start backup timer */
 	XtAppAddTimeOut(Context, 1000*Settings.BackupInterval, CB_Backup, NULL);
-
 	XtAppMainLoop(Context);
 	for (i=0; i < 9; i++)
 		XFreePixmap(Dpy, Stipples[i]);
