@@ -59,18 +59,21 @@ static	char	*rcsid = "$Id: menu.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
  * some local prototypes
  */
 static	void	CBPOPUP_Display(Widget, XtPointer, XtPointer);
+static	void	CBPOPUP_Settings(Widget, XtPointer, XtPointer);
 static	void	CBPOPUP_File(Widget, XtPointer, XtPointer);
 static	void	CBPOPUP_Buffer(Widget, XtPointer, XtPointer);
+static	void	CBPOPUP_Sizes(Widget, XtPointer, XtPointer);
+static	void	CBPOPUP_Window(Widget, XtPointer, XtPointer);
 static	void	CB_Action(Widget, XtPointer, XtPointer);
 static	void	CB_Position(Widget, XtPointer, XtPointer);
 static	void	CB_ElementPosition(Widget, XtPointer, XtPointer);
 static	void	CB_TextPosition(Widget, XtPointer, XtPointer);
 static	void	CB_ObjectPosition(Widget, XtPointer, XtPointer);
-static	void	CB_Sizes(Widget, XtPointer, XtPointer);
 static	void	CB_About(Widget, XtPointer, XtPointer);
 static	void	InitPopupTree(Widget, PopupEntryTypePtr);
 static	void	InitPopupMenu(Widget, PopupMenuTypePtr);
-static	Widget	InitCommandButton(Widget, CommandButtonTypePtr, Widget, Widget);
+static	void	DoWindows(Widget, XtPointer, XtPointer);
+static	Widget	InitializeSizesMenu(Widget, Widget, Widget);
 static	Boolean	InitCheckPixmap(void);
 
 /* ---------------------------------------------------------------------------
@@ -78,10 +81,68 @@ static	Boolean	InitCheckPixmap(void);
  */
 static	Pixmap	Check = BadAlloc;
 
+/*-----------------------------------------------------------------------------
+ * pop up menu for third mouse button
+ */
+static	PopupEntryType	pMenuEntries[] = {
+	{ "header", "Operations on Selections", NULL, NULL, NULL },
+	{ "unselect", "Unselect All", CB_Action, "Unselect,All", NULL },
+	{ "remove", "Remove Selected", CB_Action,
+		"RemoveSelected", NULL },
+	{ "copy", "Copy Selection to Buffer", CB_Action,
+		"PasteBuffer,Clear\n"
+		"PasteBuffer,AddSelected\n"
+		"Mode,PasteBuffer",
+		NULL },
+	{ "cut", "Cut Selection to Buffer", CB_Action,
+		"PasteBuffer,Clear\n"
+		"PasteBuffer,AddSelected\n"
+		"RemoveSelected\n"
+		"Mode,PasteBuffer",
+		NULL },
+	{ "hide", "Toggle Name Visibility", CB_Action,
+		"ToggleHideName,Selected", NULL},
+	{ "header", "Operations on This Object", NULL, NULL, NULL },
+	{ "edit", "Edit Name", CB_Action, "ChangeName,Object", NULL },
+	{ "report", "Object Report", CB_Action, "Report,Object", NULL },
+	{ "rotate", "Rotate Object", CB_Action,
+		"Mode,Save\n"
+		"Mode,Rotate\n"
+		"Mode,Notify\n"
+		"Mode,Restore", NULL },
+	{ "therm", "Toggle Thermal", CB_Action,
+		"Mode,Save\n"
+		"Mode,Thermal\n"
+		"Mode,Notify\n"
+		"Mode,Restore", NULL },
+	{ "lookup", "Lookup Connections", CB_Action,
+		"Connection,Find", NULL },
+	{ "reset", "Reset All Connections", CB_Action,
+		"Connection,Reset\n"
+		"Display,Redraw", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "undo", "Undo Last Operation", CB_Action, "Undo", NULL },
+	{ "redo", "Redo Last Undone Operation", CB_Action, "Redo", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "lmode", "Line Tool", CB_Action, "Mode,Line", NULL },
+	{ "rmode", "Rectangle Tool", CB_Action, "Mode,Rectangle", NULL },
+	{ "amode", "Selection Tool", CB_Action, "Mode,Arrow", NULL },
+	{ "tmode", "Text Tool", CB_Action, "Mode,Text", NULL },
+	{ "nmode", "No Tool", CB_Action, "Mode,None", NULL },
+	{ NULL, NULL, NULL, NULL, NULL }};
+static	PopupMenuType	pMenu =
+	{ "pmenu", NULL, pMenuEntries, NULL, NULL, NULL};
+static	PopupEntryType	p2MenuEntries[] = {
+	{ "nmode", "No Tool", CB_Action, "Mode,None", NULL },
+	{ NULL, NULL, NULL, NULL, NULL }};
+static	PopupMenuType	p2Menu =
+	{ "p2menu", NULL, p2MenuEntries, NULL, NULL, NULL};
+
 /* ---------------------------------------------------------------------------
  * file menu button
  */
 static	PopupEntryType	FileMenuEntries[] = {
+	{ "about", "About...", CB_About, NULL, NULL },
 	{ "save", "save layout", CB_Action, "Save,Layout", NULL },
 	{ "saveas", "save layout as...", CB_Action, "Save,LayoutAs", NULL },
 	{ "load", "load layout", CB_Action, "Load,Layout", NULL },
@@ -91,7 +152,8 @@ static	PopupEntryType	FileMenuEntries[] = {
 	{ "loadlayout", "load layout data to paste-buffer", CB_Action,
 		"PasteBuffer,Clear\n"
 		"Load,LayoutToBuffer", NULL },
-	{ "print", "print layout", CB_Action, "Print", NULL },
+	{ "loadnetlist", "load netlist file", CB_Action, "Load,Netlist", NULL },
+	{ "print", "print layout...", CB_Action, "Print", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "save connection data of", NULL, NULL, NULL },
 	{ "savesingle", "a single element", CB_ElementPosition,
@@ -99,10 +161,6 @@ static	PopupEntryType	FileMenuEntries[] = {
 	{ "saveall", "all elements", CB_Action, "Save,AllConnections", NULL },
 	{ "saveunused", "unused pins", CB_Action, "Save,AllUnusedPins", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
-	{ "header", "change name of", NULL, NULL, NULL },
-	{ "layoutname", "layout", CB_Action, "ChangeName,Layout", NULL },
-	{ "layername", "active layer", CB_Action, "ChangeName,Layer", NULL },
-	{ "line", NULL, NULL, NULL, NULL },
 	{ "new", "start new layout", CB_Action, "New", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "quit", "quit program", CB_Action, "Quit", NULL },
@@ -112,6 +170,42 @@ static	PopupMenuType	FileMenu =
 static	MenuButtonType	FileMenuButton =
 	{ "file", "File", &FileMenu, NULL};
 
+/* ---------------------------------------------------------------------------
+ * edit menu button
+ */
+static	PopupEntryType	EditMenuEntries[] = {
+	{ "undo", "undo last operation", CB_Action, "Undo", NULL },
+	{ "redo", "redo last undone operation", CB_Action, "Redo", NULL },
+	{ "clear", "clear undo-buffer", CB_Action, "Undo,ClearList", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "cut", "cut selection to buffer", CB_Position,
+		"PasteBuffer,Clear\n"
+		"PasteBuffer,AddSelected\n"
+		"RemoveSelected\n"
+		"Mode,PasteBuffer",
+		NULL },
+	{ "copy", "copy selection to buffer", CB_Position,
+		"PasteBuffer,Clear\n"
+		"PasteBuffer,AddSelected\n"
+		"Mode,PasteBuffer",
+		NULL },
+	{ "paste", "paste buffer to layout", CB_Action, "Mode,PasteBuffer", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "unselect", "unselect all", CB_Action, "Unselect,All", NULL },
+	{ "select", "select all", CB_Action, "Select,All", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "header", "Edit Names", NULL, NULL, NULL },
+	{ "edit", "edit text on layout", CB_TextPosition, "ChangeName,Object", NULL },
+	{ "layoutname", "edit name of layout", CB_Action, "ChangeName,Layout", NULL },
+	{ "layername", "edit name of active layer", CB_Action, "ChangeName,Layer", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "boardSize", "change board size...", CB_Action, "AdjustStyle,0", NULL },
+	{ NULL, NULL, NULL, NULL, NULL }};
+static	PopupMenuType	EditMenu =
+	{ "edit",  NULL, EditMenuEntries, NULL, NULL, NULL };
+static	MenuButtonType	EditMenuButton =
+	{ "edits", "Edit", &EditMenu, NULL};
+
 /* ---------------------------------------------------------------------------
  * display menu button
  */
@@ -122,19 +216,15 @@ static	PopupEntryType	DisplayMenuEntries[] = {
 	{ "displayGrid", "display grid", CB_Action, "Display,Grid", NULL },
 	{ "toggleGrid", "use absolute grid", CB_Position,
 		"Display,ToggleGrid", NULL },
-	{ "toggleAllDirections", "'all-direction' lines", CB_Action,
-		"Display,Toggle45Degree", NULL },
-	{ "toggleRubberBandMode", "rubber band mode", CB_Action,
-		"Display,ToggleRubberBandMode", NULL },
-	{ "solderSide", "look at solder-side", CB_Action, "SwapSides", NULL },
+	{ "solderSide", "view solder side", CB_Position, "SwapSides", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "grid setting", NULL, NULL, NULL },
-	{ "grid05", " 5 mil", CB_Position, "SetValue,Grid,5", NULL },
-	{ "grid10", "10 mil", CB_Position, "SetValue,Grid,10", NULL },
-	{ "grid20", "20 mil", CB_Position, "SetValue,Grid,20", NULL },
-	{ "grid25", "25 mil", CB_Position, "SetValue,Grid,25", NULL },
-	{ "grid50", "50 mil", CB_Position, "SetValue,Grid,50", NULL },
-	{ "grid100", "100 mil", CB_Position, "SetValue,Grid,100", NULL },
+	{ "grid05", " 5 mil", CB_Action, "SetValue,Grid,5", NULL },
+	{ "grid10", "10 mil", CB_Action, "SetValue,Grid,10", NULL },
+	{ "grid20", "20 mil", CB_Action, "SetValue,Grid,20", NULL },
+	{ "grid25", "25 mil", CB_Action, "SetValue,Grid,25", NULL },
+	{ "grid50", "50 mil", CB_Action, "SetValue,Grid,50", NULL },
+	{ "grid100", "100 mil", CB_Action, "SetValue,Grid,100", NULL },
 	{ "gridInc", "increment by 5", CB_Action, "SetValue,Grid,+5", NULL },
 	{ "gridDec", "decrement by 5", CB_Action, "SetValue,Grid,-5", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
@@ -150,31 +240,40 @@ static	PopupEntryType	DisplayMenuEntries[] = {
 	{ "onPCB", "name on PCB", CB_Action, "Display,NameOnPCB", NULL },
 	{ "value", "value", CB_Action, "Display,Value", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
+	{ "pinout", "open pinout window", CB_ElementPosition, "Display,Pinout", NULL },
 	{ NULL, NULL, NULL, NULL, NULL }};
 static	PopupMenuType	DisplayMenu =
 	{ "display", NULL, DisplayMenuEntries, CBPOPUP_Display, NULL, NULL};
 static	MenuButtonType	DisplayMenuButton =
-	{ "display", "Display", &DisplayMenu, NULL};
+	{ "display", "Screen", &DisplayMenu, NULL};
+
+/* ----------------------------------------------------------------------
+ * Sizes menu button - contains variable contents
+ */
+static	PopupEntryType 	SizesMenuEntries[NUM_STYLES * 2 + 2];
+static PopupMenuType	SizesMenu =
+	{ "sizes", NULL, SizesMenuEntries, CBPOPUP_Sizes, NULL, NULL };
+static	MenuButtonType SizesMenuButton =
+	{ "sizes", "Sizes", &SizesMenu, NULL};
 
 /* ---------------------------------------------------------------------------
- * object menu button
+ * settings menu button
  */
-static	PopupEntryType	ObjectMenuEntries[] = {
-	{ "header", "elements", NULL, NULL, NULL },
-	{ "pinout", "display pinout", CB_ElementPosition, "Display,Pinout", NULL },
-	{ "changeName", "change name", CB_ElementPosition,
-		"ChangeName,Object", NULL },
-	{ "line", NULL, NULL, NULL, NULL },
-	{ "header", "text objects", NULL, NULL, NULL },
-	{ "edit", "edit", CB_TextPosition, "ChangeName,Object", NULL },
-	{ "line", NULL, NULL, NULL, NULL },
+static	PopupEntryType	SettingsMenuEntries[] = {
 	{ "header", "layer-groups", NULL, NULL, NULL },
-	{ "lgedit", "edit layer-groups", CB_Action, "EditLayerGroups", NULL },
+	{ "lgedit", "edit layer-groupings", CB_Action, "EditLayerGroups", NULL },
+	{ "line", NULL, NULL, NULL, NULL },
+	{ "toggleAllDirections", "'all-direction' lines", CB_Action,
+		"Display,Toggle45Degree", NULL },
+	{ "toggleRubberBandMode", "rubber band mode", CB_Action,
+		"Display,ToggleRubberBandMode", NULL },
+	{ "toggleSwapStartDirection", "auto swap line start angle", CB_Action,
+		"Display,ToggleStartDirection", NULL },
 	{ NULL, NULL, NULL, NULL, NULL }};
-static	PopupMenuType	ObjectMenu =
-	{ "objects",  NULL, ObjectMenuEntries, NULL, NULL, NULL };
-static	MenuButtonType	ObjectMenuButton =
-	{ "objects", "Objects", &ObjectMenu, NULL};
+static	PopupMenuType	SettingsMenu =
+	{ "settings",  NULL, SettingsMenuEntries, CBPOPUP_Settings, NULL, NULL };
+static	MenuButtonType	SettingsMenuButton =
+	{ "settings", "Settings", &SettingsMenu, NULL};
 
 /* ---------------------------------------------------------------------------
  * selection menu button
@@ -200,8 +299,6 @@ static	PopupEntryType	SelectionMenuEntries[] = {
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "remove", "remove selected objects", CB_Action,
 		"RemoveSelected", NULL },
-	{ "movetosilk", "selected text -> silkscreen", CB_Action,
-		"MoveToSilk,SelectedTexts", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "change size of selected objects", NULL, NULL, NULL },
 	{ "decrementline", "lines -10 mil", CB_Action,
@@ -244,7 +341,7 @@ static	PopupEntryType	SelectionMenuEntries[] = {
 static	PopupMenuType	SelectionMenu =
 	{ "selection",  NULL, SelectionMenuEntries, NULL, NULL, NULL };
 static	MenuButtonType	SelectionMenuButton =
-	{ "selection", "Selection", &SelectionMenu, NULL};
+	{ "selection", "Select", &SelectionMenu, NULL};
 
 /* ---------------------------------------------------------------------------
  * paste buffer menu button
@@ -302,22 +399,7 @@ static	PopupEntryType	ConnectionMenuEntries[] = {
 static	PopupMenuType	ConnectionMenu =
 	{ "connections",  NULL, ConnectionMenuEntries, NULL, NULL, NULL };
 static	MenuButtonType	ConnectionMenuButton =
-	{ "connections", "Connections", &ConnectionMenu, NULL};
-
-/* ---------------------------------------------------------------------------
- * undo menu button
- */
-static	PopupEntryType	UndoMenuEntries[] = {
-	{ "undo", "undo last destructive operation", CB_Action, "Undo", NULL },
-	{ "redo", "redo last undone operation", CB_Action, "Redo", NULL },
-	{ "clear", "clear undo-buffer", CB_Action, "Undo,ClearList", NULL },
-	{ NULL, NULL, NULL, NULL, NULL }};
-static	PopupMenuType	UndoMenu =
-	{ "undo",  NULL, UndoMenuEntries, NULL, NULL, NULL };
-static	MenuButtonType	UndoMenuButton =
-	{ "undo", "Undo", &UndoMenu, NULL};
-static	MenuButtonType	RedoMenuButon =
-	{ "redo", "Redo", &UndoMenu, NULL};
+	{ "connections", "Connects", &ConnectionMenu, NULL};
 
 /* ---------------------------------------------------------------------------
  * report menu button
@@ -325,22 +407,110 @@ static	MenuButtonType	RedoMenuButon =
 static	PopupEntryType	ReportMenuEntries[] = {
 	{ "report", "generate object report", CB_Position, "Report,Object", NULL },
 	{ "drillreport", "generate drill summary", CB_Action, "Report,DrillReport", NULL },
+	{ "foundreport", "report found pins/pads", CB_Action, "Report,FoundPins", NULL },
 	{ NULL, NULL, NULL, NULL, NULL }};
 static	PopupMenuType	ReportMenu =
 	{ "report",  NULL, ReportMenuEntries, NULL, NULL, NULL };
 static	MenuButtonType	ReportMenuButton =
-	{ "report", "Report", &ReportMenu, NULL};
+	{ "report", "Info", &ReportMenu, NULL};
+
+/* ---------------------------------------------------------------------------
+ * windows menu button
+ */
+static	PopupEntryType	WindowMenuEntries[] = {
+	{ "layout", "board layout", DoWindows, "1", NULL },
+	{ "library", "library", DoWindows, "2", NULL },
+	{ "log", "message log", DoWindows, "3", NULL },
+	{ "netlist", "netlist", DoWindows, "4", NULL },
+	{ NULL, NULL, NULL, NULL, NULL }};
+static	PopupMenuType	WindowMenu =
+	{ "window",  NULL, WindowMenuEntries, CBPOPUP_Window, NULL, NULL };
+static	MenuButtonType	WindowMenuButton =
+	{ "window", "Window", &WindowMenu, NULL};
+
+static void FillSizesMenu(void)
+{
+	static	char	name[NUM_STYLES * 2 + 1][8];
+	static	char	label[NUM_STYLES * 2 + 1][64];
+	static	char	action[NUM_STYLES * 2 + 1][16];
+	int		i;
+
+	for (i = 0; i < NUM_STYLES; i++)
+	{
+		sprintf(name[i],"size%d", i+1);
+		sprintf(label[i], "use '%s' routing style", PCB->RouteStyle[i].Name);
+		sprintf(action[i], "RouteStyle,%d", i+1);
+		SizesMenuEntries[i].Name = name[i];
+		SizesMenuEntries[i].Label = label[i];
+		SizesMenuEntries[i].Callback = CB_Action;
+		SizesMenuEntries[i].ClientData = (XtPointer) action[i];
+	}
+	for (i = NUM_STYLES; i < 2*NUM_STYLES; i++)
+	{
+		sprintf(name[i],"size%d", i+1);
+		sprintf(label[i], "adjust '%s' sizes ...", PCB->RouteStyle[i - NUM_STYLES].Name);
+		sprintf(action[i], "AdjustStyle,%d", i - NUM_STYLES +1);
+		SizesMenuEntries[i].Name = name[i];
+		SizesMenuEntries[i].Label = label[i];
+		SizesMenuEntries[i].Callback = CB_Action;
+		SizesMenuEntries[i].ClientData = (XtPointer) action[i];
+	}
+	sprintf(name[i],"size%d",i+1);
+	sprintf(label[i],"adjust active sizes ...");
+	sprintf(action[i],"AdjustStyle,0");
+	SizesMenuEntries[i].Name = name[i];
+	SizesMenuEntries[i].Label = label[i];
+	SizesMenuEntries[i].Callback = CB_Action;
+	SizesMenuEntries[i].ClientData = (XtPointer) action[i];
+	i++;
+	SizesMenuEntries[i].Name = NULL;
+	SizesMenuEntries[i].Label = NULL;
+}
+
+void UpdateSizesMenu(void)
+{
+	int	i;
+
+	FillSizesMenu();
+	for (i = 0; i < 2*NUM_STYLES + 1; i++)
+		XtVaSetValues(SizesMenuEntries[i].W, XtNlabel, SizesMenuEntries[i].Label, NULL);
+}
+/* ----------------------------------------------------------------------
+ * Initializes the sizes menu with the routing style names etc.
+ */
+static Widget InitializeSizesMenu(Widget Parent, Widget Top, Widget Last)
+{
+	FillSizesMenu();
+	return(InitMenuButton(Parent, &SizesMenuButton, Top, Last));
+}
 
 /* ----------------------------------------------------------------------
- * command buttons
+ * menu callback for window menu
  */
-static	CommandButtonType	AboutButton =
-	{ "about", "About", CB_About, NULL, NULL },
-							SizesButton =
-	{ "sizes", "Sizes", CB_Sizes, NULL, NULL };
-
-
+static void DoWindows(Widget W, XtPointer ClientData, XtPointer CallData)
+{
+	int	n = atoi( (char *) ClientData);
 
+	switch (n)
+	{
+		case 1:
+			XRaiseWindow(Dpy, XtWindow(Output.Toplevel));
+			break;
+		case 2:
+			XMapRaised(Dpy, Library.Wind);
+			break;
+		case 3:
+			if (Settings.UseLogWindow)
+				XMapRaised(Dpy, LogWindID);
+			break;
+		case 4:
+			if (PCB->NetlistLib.Wind)
+				XMapRaised(Dpy, PCB->NetlistLib.Wind);
+			break;
+	}
+}
+			
+	
 /* ----------------------------------------------------------------------
  * menu callback interface for actions routines that don't need
  * position information
@@ -368,7 +538,7 @@ static void CB_Action(Widget W, XtPointer ClientData, XtPointer CallData)
 		/* first loop over all action routines;
 		 * strtok cannot be used in nested loops because it saves
 		 * a pointer in a private data area which would be corrupted
-		 * by th inner loop
+		 * by the inner loop
 		 */
 	for (current = copy; current; current = next)
 	{
@@ -410,12 +580,20 @@ static void CB_Action(Widget W, XtPointer ClientData, XtPointer CallData)
  */
 static void CB_Position(Widget W, XtPointer ClientData, XtPointer CallData)
 {
+	int TheMode = Settings.Mode;
 	char *asString = (char *) ClientData;
 
 	if (!TEST_FLAG(ABSOLUTEFLAG, PCB) && strncmp("Display,ToggleGrid", asString, 18) == 0)
 		CB_Action(W, ClientData,CallData);
-	else if (GetPosition("move pointer to the appropriate screen position and press a button"))
-		CB_Action(W, ClientData, CallData);
+	else
+	{
+		Settings.Mode = NO_MODE;
+		if (GetPosition(
+			"Move the pointer to the appropriate screen position and press a button."))
+				CB_Action(W, ClientData, CallData);
+		if (Settings.Mode == NO_MODE)
+			Settings.Mode = TheMode;
+	}
 }
 
 /* ----------------------------------------------------------------------
@@ -447,10 +625,33 @@ static void CB_TextPosition(Widget W,
 static void CB_ObjectPosition(Widget W,
 	XtPointer ClientData, XtPointer CallData)
 {
-	if (GetPosition("press a button at an 'connecting-objects' location"))
+	if (GetPosition("press a button at a 'connecting-objects' location"))
 		CB_Action(W, ClientData, CallData);
 }
 
+/* ---------------------------------------------------------------------- 
+ * called before sizes menu is popped up
+ * used to mark the current route style 
+ */
+static void CBPOPUP_Sizes(Widget W, XtPointer ClientData, XtPointer CallData)
+{
+	int	i;
+	char	menuname[5];
+
+	RemoveCheckFromMenu(&SizesMenu);
+	for (i = 0; i < NUM_STYLES; i++)
+	{
+		if (PCB->RouteStyle[i].Thick == Settings.LineThickness &&
+		    PCB->RouteStyle[i].Diameter == Settings.ViaThickness &&
+		    PCB->RouteStyle[i].Hole == Settings.ViaDrillingHole)
+		{
+			sprintf(menuname, "size%d", i+1);
+			CheckEntry(&SizesMenu, menuname);
+			break;
+		}
+	}
+}
+		
 /* ---------------------------------------------------------------------- 
  * called before display menu is popped up
  * used to mark the current grid-mode, zoom value ...
@@ -462,10 +663,6 @@ static void CBPOPUP_Display(Widget W, XtPointer ClientData, XtPointer CallData)
 		GetGridFactor() != 0);
 	if (TEST_FLAG(ABSOLUTEFLAG, PCB))
 		CheckEntry(&DisplayMenu, "toggleGrid");
-	if (TEST_FLAG(ALLDIRECTIONFLAG, PCB))
-		CheckEntry(&DisplayMenu, "toggleAllDirections");
-	if (TEST_FLAG(RUBBERBANDFLAG, PCB))
-		CheckEntry(&DisplayMenu, "toggleRubberBandMode");
 	if (Settings.DrawGrid)
 		CheckEntry(&DisplayMenu, "displayGrid");
 	if (Settings.ShowSolderSide)
@@ -483,6 +680,22 @@ static void CBPOPUP_Display(Widget W, XtPointer ClientData, XtPointer CallData)
 			TEST_FLAG(DESCRIPTIONFLAG, PCB) ? "description" : "value");
 }
 
+/* ---------------------------------------------------------------------- 
+ * called before settings menu is popped up
+ * used to mark the boolean settings
+ */
+static void CBPOPUP_Settings(Widget W, XtPointer ClientData, XtPointer CallData)
+{
+	RemoveCheckFromMenu(&SettingsMenu);
+	if (TEST_FLAG(ALLDIRECTIONFLAG, PCB))
+		CheckEntry(&SettingsMenu, "toggleAllDirections");
+	if (TEST_FLAG(RUBBERBANDFLAG, PCB))
+		CheckEntry(&SettingsMenu, "toggleRubberBandMode");
+	if (TEST_FLAG(SWAPSTARTDIRFLAG, PCB))
+		CheckEntry(&SettingsMenu, "toggleSwapStartDirection");
+}
+
+
 /* ---------------------------------------------------------------------- 
  * called before file menu is popped up
  * enables/disables printing
@@ -506,15 +719,20 @@ static void CBPOPUP_Buffer(Widget W, XtPointer ClientData, XtPointer CallData)
 }
 
 /* ---------------------------------------------------------------------- 
- * callback routine used by sizes button
+ * called before window menu is popped up
+ * enables/disables log window 
  */
-static void CB_Sizes(Widget W, XtPointer ClientData, XtPointer CallData)
+static void CBPOPUP_Window(Widget W, XtPointer ClientData, XtPointer CallData)
 {
-	SizeDialog();
+	XtSetSensitive(XtNameToWidget(WindowMenu.W, "log"),
+		Settings.UseLogWindow);
+	XtSetSensitive(XtNameToWidget(WindowMenu.W, "netlist"),
+		PCB->NetlistLib.Wind != 0);
 }
 
+
 /* ---------------------------------------------------------------------- 
- * callback routine used by about button
+ * callback routine used by about entry
  */
 static void CB_About(Widget W, XtPointer ClientData, XtPointer CallData)
 {
@@ -549,25 +767,6 @@ void CheckEntry(PopupMenuTypePtr MenuPtr, String WidgetName)
 			}
 }
 
-/* ---------------------------------------------------------------------------
- * initializes a command button
- */
-static Widget InitCommandButton(Widget Parent, CommandButtonTypePtr CommandButtonPtr, Widget Top, Widget Left)
-{
-	CommandButtonPtr->W = XtVaCreateManagedWidget(CommandButtonPtr->Name, commandWidgetClass,
-		Parent,
-		XtNlabel, CommandButtonPtr->Label,
-		XtNfromHoriz, Left,
-		XtNfromVert, Top,
-		LAYOUT_TOP,
-		NULL);
-	XtAddCallback(CommandButtonPtr->W, XtNcallback,
-		CommandButtonPtr->Callback, CommandButtonPtr->ClientData);
-
-		/* return the created button widget to position some others */
-	return(CommandButtonPtr->W);
-}
-
 /* ---------------------------------------------------------------------------
  * initializes a menu tree
  * depending on the 'Name' field of the struct either a smeBSB or a
@@ -609,10 +808,12 @@ static void InitPopupTree(Widget Parent, PopupEntryTypePtr EntryPtr)
  */
 static void InitPopupMenu(Widget Parent, PopupMenuTypePtr MenuPtr)
 {
+	Cursor point = XCreateFontCursor(Dpy, XC_left_ptr);
 	MenuPtr->W = XtVaCreatePopupShell(MenuPtr->Name, simpleMenuWidgetClass,
 		Parent,
 		XtNlabel, MenuPtr->Label,
 		XtNsensitive, True,
+		XtNcursor, point,
 		NULL);
 	InitPopupTree(MenuPtr->W, MenuPtr->Entries);
 
@@ -664,16 +865,18 @@ void InitMenu(Widget Parent, Widget Top, Widget Left)
 {
 	Widget	last;
 
-	last = InitCommandButton(Parent, &AboutButton, Top, Left);
-	last = InitMenuButton(Parent, &FileMenuButton, Top, last);
+	last = InitMenuButton(Parent, &FileMenuButton, Top, Left);
+	last = InitMenuButton(Parent, &EditMenuButton, Top, last);
 	last = InitMenuButton(Parent, &DisplayMenuButton, Top, last);
-	last = InitCommandButton(Parent, &SizesButton, Top, last);
-	last = InitMenuButton(Parent, &ObjectMenuButton, Top, last);
+	last = InitializeSizesMenu(Parent, Top, last);
+	last = InitMenuButton(Parent, &SettingsMenuButton, Top, last);
 	last = InitMenuButton(Parent, &SelectionMenuButton, Top, last);
 	last = InitMenuButton(Parent, &BufferMenuButton, Top, last);
 	last = InitMenuButton(Parent, &ConnectionMenuButton, Top, last);
-	last = InitMenuButton(Parent, &UndoMenuButton, Top, last);
 	last = InitMenuButton(Parent, &ReportMenuButton, Top, last);
+	last = InitMenuButton(Parent, &WindowMenuButton, Top, last);
+	InitPopupMenu(Parent, &pMenu);
+	InitPopupMenu(Parent, &p2Menu);
 	Output.Menu = last;
 }
 
