@@ -52,8 +52,10 @@ static	char	*rcsid = "$Id: misc.c,v 1.2 1998/02/21 10:27:14 cad Exp $";
 #include "mymem.h"
 #include "misc.h"
 #include "move.h"
+#include "output.h"
 #include "remove.h"
 #include "rotate.h"
+#include "rubberband.h"
 #include "search.h"
 #include "set.h"
 
@@ -62,7 +64,11 @@ static	char	*rcsid = "$Id: misc.c,v 1.2 1998/02/21 10:27:14 cad Exp $";
 #include <X11/Xaw/Scrollbar.h>
 #include <X11/Xaw/Simple.h>
 
+/*	forward declarations	*/
+static char *BumpName(char *);
 static void RightAngles( int, float *, float *);
+static void GetGridLockCoordinates(int, void *, void *, void *, Position *, Position *);
+
 /* ---------------------------------------------------------------------------
  * prints copyright information
  */
@@ -102,9 +108,6 @@ void Usage(void)
 		"  -backup <seconds>:        time between two backups\n"
 		"  -c <number>:              characters per output-line\n"
 		"  -fontfile <file>:         read default font from this file\n"
-		"  -laperture <command>:     command to copy aperture files to stdout,\n"
-		"                            %%f is set to the filename\n"
-		"                            %%p is set to the seachpath\n"
 		"  -lelement <command>:      command to copy element files to stdout,\n"
 		"                            %%f is set to the filename\n"
 		"                            %%p is set to the seachpath\n"
@@ -125,6 +128,7 @@ void Usage(void)
 		"                            %%f is set to the filename\n"
 		"                            %%p is set to the searchpath\n"
 		"  -loggeometry <geometry>:  set the geometry of the logging window\n"
+		"  -log:                     don't use the log window\n"
 		"  -pnl <value>:             maximum display length of pin names\n"
 		"  -pz <value>:              zoom factor for pinout windows\n"
 		"  -reset:                   reset connections after each element\n"
@@ -315,7 +319,7 @@ Boolean IsDataEmpty(DataTypePtr Data)
 
 	hasNoObjects = (Data->ViaN == 0);
 	hasNoObjects &= (Data->ElementN == 0);
-	for (i = 0; i < MAX_LAYER; i++)
+	for (i = 0; i < MAX_LAYER + 2; i++)
 		hasNoObjects = hasNoObjects &&
 		Data->Layer[i].LineN == 0 && 
 		Data->Layer[i].ArcN == 0 &&
@@ -388,36 +392,23 @@ BoxTypePtr GetDataBoundingBox(DataTypePtr Data)
 
 /* ---------------------------------------------------------------------------
  * centers the displayed PCB around the specified point
- * (X,Y) in screen coordinates
+ * (X,Y) in screen coordinates.
+ * if Delta is true, simply move the center by X, Y
  */
-void CenterDisplay(Position X, Position Y)
+void CenterDisplay(Position X, Position Y, Boolean Delta)
 {
-	Position	old_x, old_y,	/* output window in viewport */
-			new_x, new_y;
-	float		top;		/* top of thumb */
-
-		/* save old values (offset of upper/left corner) */
-	XtVaGetValues(Output.Output, XtNx, &old_x, XtNy, &old_y, NULL);
-
+#ifdef DEBUGDISP
+	Message("CenterDisplay(%d, %d, %s)\n", X, Y, Delta ? "True" : "False" );
+#endif
+	X -= Xorig;
+	Y -= Yorig;
 		/* move origin half a screen width/height to get display centered */
-	X -= (Output.Width /2);
-	top = (float) X /(float) TO_SCREEN(PCB->MaxWidth);
-	top = MIN(MAX(top, 0.0), 1.0);
-	XtCallCallbacks(Output.ScrollbarBottom, XtNjumpProc, (XtPointer) &top);
+	if (!Delta)
+		X -= Output.Width/2; 
 
-	Y -= (Output.Height /2);
-	top = (float) Y /(float) TO_SCREEN(PCB->MaxHeight);
-	top = MIN(MAX(top, 0.0), 1.0);
-	XtCallCallbacks(Output.ScrollbarLeft, XtNjumpProc, (XtPointer) &top);
-
-		/* now get the new values (reread cause of rounding) */
-	XtVaGetValues(Output.Output, XtNx, &new_x, XtNy, &new_y, NULL);
-
-		/* the scrollbar widget triggers an expose event if (x,y)
-		 * has changed. I have to do it if it hasn't changed
-		 */
-	if (new_x == old_x && new_y == old_y)
-		ClearAndRedrawOutput();
+	if (!Delta)
+		Y -= Output.Height/2;
+	Pan(X, Y, True, True);
 }
 
 /* ---------------------------------------------------------------------------
@@ -490,6 +481,68 @@ void SetFontInfo(FontTypePtr Ptr)
 	Ptr->DefaultSymbol.Y2 = Ptr->DefaultSymbol.Y1 +Ptr->MaxHeight;
 } 
 
+static void GetNum(char **s, Dimension *num)
+{
+	*num = atoi(*s);
+	while (isdigit(**s))
+		(*s)++;
+}
+
+
+/* ----------------------------------------------------------------------
+ * parses the routes definition string which is a colon seperated list of
+ * comma seperated Name, Dimension, Dimension, Dimension
+ * e.g. Signal,20,40,20:Power,40,60,28:...
+ */
+int ParseRouteString(char *s, RouteStyleTypePtr routeStyle)
+{
+	int		i, style;
+	char		Name[256];
+
+	memset(routeStyle, 0, NUM_STYLES * sizeof(RouteStyleType));
+	for (style = 0; style < NUM_STYLES; style++, routeStyle++)
+	{
+		while(*s && isspace(*s))
+			s++;
+		for (i = 0; *s && *s != ','; i++)	
+			Name[i] = *s++;
+		Name[i] = '\0';
+		routeStyle->Name = MyStrdup(Name, "ParseRouteString()");
+		if (!isdigit(*++s))
+			goto error;
+		GetNum(&s, &routeStyle->Thick);
+		while(*s && isspace(*s))
+			s++;
+		if (*s++ != ',')
+			goto error;
+		while(*s && isspace(*s))
+			s++;
+		if (!isdigit(*s))
+			goto error;
+		GetNum(&s, &routeStyle->Diameter);
+		while(*s && isspace(*s))
+			s++;
+		if (*s++ != ',')
+			goto error;
+		while(*s && isspace(*s))
+			s++;
+		if (!isdigit(*s))
+			goto error;
+		GetNum(&s, &routeStyle->Hole);
+		if (style < NUM_STYLES - 1)
+		{
+			while(*s && isspace(*s))
+				s++;
+			if (*s++ != ':')
+				goto error;
+		}
+	}
+	return(0);
+error:
+	memset(routeStyle, 0, NUM_STYLES * sizeof(RouteStyleType));
+	return(1);
+}
+
 /* ----------------------------------------------------------------------
  * parses the group definition string which is a colon seperated list of
  * comma seperated layer numbers (1,2,b:4,6,8,t)
@@ -508,8 +561,6 @@ int ParseGroupString(char *s, LayerGroupTypePtr LayerGroup)
 		/* loop over all groups */
 	for (group = 0; s && *s && group < MAX_LAYER; group++)
 	{
-		if (group >= MAX_LAYER)
-			goto error;
 		while (*s && isspace(*s))
 			s++;
 
@@ -725,40 +776,16 @@ Boolean GetPosition(char *MessageText)
 }
 
 /* ----------------------------------------------------------------------
- * releases tmp pixmap used to save output data
+ * releases pixmap used to draw output data
  */
-void ReleaseSaveUnderPixmap(void)
+void ReleaseOffscreenPixmap(void)
 {
-	if (VALID_PIXMAP(PCB->SaveUnder))
-		XFreePixmap(Dpy, PCB->SaveUnder);
+	render = True;
+	if (VALID_PIXMAP(Offscreen))
+		XFreePixmap(Dpy, Offscreen);
 
 		/* mark pixmap unusable */
-	PCB->SaveUnder = BadAlloc;
-}
-
-/* ----------------------------------------------------------------------
- * saves output window to tmp pixmap to reduce redrawing
- */
-void SaveOutputWindow(void)
-{
-		/* release old pixmap and allocate new one */
-	ReleaseSaveUnderPixmap();
-
-		/* create new one only if window isn't obscured */
-	if (Output.VisibilityOK)
-	{
-		PCB->SaveUnder = XCreatePixmap(Dpy, Output.OutputWindow,
-			Output.Width, Output.Height,
-			DefaultDepthOfScreen(XtScreen(Output.Output)));
-
-			/* make sure that cursor is off */
-		if (VALID_PIXMAP(PCB->SaveUnder))
-			XCopyArea(Dpy, Output.OutputWindow,
-				PCB->SaveUnder, Output.fgGC,
-				Output.OffsetX, Output.OffsetY,
-				Output.Width, Output.Height,
-				0, 0);
-	}
+	Offscreen = BadAlloc;
 }
 
 /* ---------------------------------------------------------------------------
@@ -774,7 +801,24 @@ unsigned int SetOutputXCursor(unsigned int Shape)
 	if (Output.OutputWindow)
 	{
 		Output.XCursorShape = Shape;
-		Output.XCursor= XCreateFontCursor(Dpy, Shape);
+		if (Shape == XC_Clockwise)
+		{
+			Screen *screen;
+			XColor blk, wht;
+
+			screen = XtScreen(Output.Toplevel);
+			blk.pixel = BlackPixelOfScreen(screen);
+			wht.pixel = WhitePixelOfScreen(screen);
+			blk.red = 0; blk.green = 0; blk.blue = 0;
+			wht.red = 65535; wht.green = 65535; wht.blue = 65535;
+			blk.flags = DoRed | DoGreen | DoBlue;
+			wht.flags = DoRed | DoGreen | DoBlue;
+			Output.XCursor = XCreatePixmapCursor(Dpy, XC_clock_source,
+				XC_clock_mask, &blk, &wht, rotateIcon_x_hot,
+				rotateIcon_y_hot);
+		}
+		else
+			Output.XCursor= XCreateFontCursor(Dpy, Shape);
 		XDefineCursor(Dpy, Output.OutputWindow, Output.XCursor);
 		XSync(Dpy, False);
 
@@ -793,7 +837,7 @@ int GetLayerNumber(DataTypePtr Data, LayerTypePtr Layer)
 {
 	int	i;
 
-	for (i = 0; i < MAX_LAYER; i++)
+	for (i = 0; i < MAX_LAYER + 2; i++)
 		if (Layer == &Data->Layer[i])
 			break;
 	return(i);
@@ -820,7 +864,7 @@ int GetLayerGroupNumberByNumber(Cardinal Layer)
 			if (PCB->LayerGroups.Entries[group][entry] == Layer)
 				return(group);
 
-		/* since every layer belongs to a group it is save to return
+		/* since every layer belongs to a group it is safe to return
 		 * the value without boundary checking
 		 */
 	return(group);
@@ -953,47 +997,17 @@ void ResetStackAndVisibility(void)
 {
 	Cardinal	i;
 
-	for (i = 0; i < MAX_LAYER; i++)
+	for (i = 0; i < MAX_LAYER + 2; i++)
 	{
-		LayerStack[i] = i;
+		if (i < MAX_LAYER)
+			LayerStack[i] = i;
 		PCB->Data->Layer[i].On = True;
 	}
 	PCB->ElementOn = True;
 	PCB->InvisibleObjectsOn = True;
 	PCB->PinOn = True;
 	PCB->ViaOn = True;
-}
-
-/* ---------------------------------------------------------------------------
- * changes the maximum size of a layout
- * adjusts the scrollbars
- * releases the saved pixmap if necessary
- * and adjusts the cursor confinement box
- */
-void ChangePCBSize(Dimension Width, Dimension Height)
-{
-		/* force a redraw of the forbidden area */
-	ReleaseSaveUnderPixmap();
-	PCB->MaxWidth =  Width;
-	PCB->MaxHeight = Height;
-	Output.Width = TO_SCREEN(Width);
-	Output.Height = TO_SCREEN(Height);
-	XtVaSetValues(Output.Output,
-		XtNwidth, Output.Width,
-		XtNheight, Output.Height,
-		NULL);
-
-		/* crosshair range is different if pastebuffer-mode
-		 * is enabled
-		 */
-	if (Settings.Mode == PASTEBUFFER_MODE)
-		SetCrosshairRange(
-			PASTEBUFFER->X -PASTEBUFFER->BoundingBox.X1,
-			PASTEBUFFER->Y -PASTEBUFFER->BoundingBox.Y1,
-			MAX(0, Width -(PASTEBUFFER->BoundingBox.X2-PASTEBUFFER->X)),
-			MAX(0, Height -(PASTEBUFFER->BoundingBox.Y2-PASTEBUFFER->Y)));
-	else
-		SetCrosshairRange(0, 0, (Position) Width, (Position) Height);
+	PCB->RatOn = True;
 }
 
 /* ----------------------------------------------------------------------
@@ -1060,77 +1074,8 @@ int GetGridFactor(void)
 
 static	void RightAngles(int Angle, float *cosa, float *sina)
 {
-	switch (Angle)
-	{
-		case -360:
-			*cosa = 1.0;
-			*sina = 0;
-			break;
-		case -315:
-			*cosa = SQRT2OVER2;
-			*sina = SQRT2OVER2;
-			break;
-		case -270:
-			*cosa = 0;
-			*sina = 1.0;
-			break;
-		case -225:
-			*cosa = -SQRT2OVER2;
-			*sina = SQRT2OVER2;
-			break;
-		case -180:
-			*cosa = -1.0;
-			*sina = 0;
-			break;
-		case -135:
-			*cosa = -SQRT2OVER2;
-			*sina = -SQRT2OVER2;
-			break;
-		case -90:
-			*cosa = 0;
-			*sina = -1.0;
-			break;
-		case -45:
-			*cosa = SQRT2OVER2;
-			*sina = -SQRT2OVER2;
-			break;
-		case 0:
-			*cosa = 1.0;
-			*sina = 0;
-			break;
-		case 45:
-			*cosa = SQRT2OVER2;
-			*sina = SQRT2OVER2;
-			break;
-		case 90:
-			*cosa = 0;
-			*sina = 1.0;
-			break;
-		case 135:
-			*cosa = -SQRT2OVER2;
-			*sina = SQRT2OVER2;
-			break;
-		case 180:
-			*cosa = -1.0;
-			*sina = 0;
-			break;
-		case 225:
-			*cosa = -SQRT2OVER2;
-			*sina = -SQRT2OVER2;
-			break;
-		case 270:
-			*cosa = 0;
-			*sina = -1.0;
-			break;
-		case 315:
-			*cosa = SQRT2OVER2;
-			*sina = -SQRT2OVER2;
-			break;
-		case 360:
-			*cosa = 1.0;
-			*sina = 0;
-			break;
-	}
+	*cosa = (float) cos((double) Angle * M180);
+	*sina = (float) sin((double) Angle * M180);
 }
 
 BoxTypePtr GetArcEnds(ArcTypePtr Arc)
@@ -1147,3 +1092,163 @@ BoxTypePtr GetArcEnds(ArcTypePtr Arc)
 	return(&box);
 }
 
+static char *BumpName(char *Name)
+{
+	int	num;
+	char	c, *start;
+	static	char temp[256];
+
+	start = Name;
+		/* seek end of string */
+	while (*Name != 0)
+		Name++;
+		/* back up to potential number */
+	for (Name--; isdigit(*Name); Name--);
+	Name++;
+	if (*Name)
+		num = atoi(Name) + 1;
+	else
+		num = 1;
+	c = *Name;
+	*Name = 0;
+	sprintf(temp, "%s%d", start, num);
+		/* if this is not our string, put back the blown character */
+	if (start != temp)
+		*Name = c;
+	return(temp);
+}
+
+/*
+ * make a unique name for the name on board 
+ * this can alter the contents of the input string
+ */
+char	*UniqueElementName(DataTypePtr Data, char *Name)
+{
+	Boolean unique = True;
+		/* null strings are ok */
+	if (!Name || !*Name)
+		return (Name);
+
+	for ( ; ; )
+	{
+		ELEMENT_LOOP(Data,
+			if (NAMEONPCB_NAME(element) && 
+				strcmp(NAMEONPCB_NAME(element), Name) == 0)
+			{
+				Name = BumpName(Name);
+				unique = False;
+				break;
+			}
+		);
+		if (unique)
+			return(Name);
+		unique = True;
+	}
+}
+
+static void GetGridLockCoordinates(int type, void *ptr1,
+		 void *ptr2, void *ptr3, Position *x, Position *y)
+{
+	switch (type)
+	{
+		case VIA_TYPE:
+				*x = ((PinTypePtr) ptr2)->X;
+				*y = ((PinTypePtr) ptr2)->Y;
+				break;
+		case LINE_TYPE:
+				*x = ((LineTypePtr) ptr2)->Point1.X;
+				*y = ((LineTypePtr) ptr2)->Point1.Y;
+				break;
+		case TEXT_TYPE:
+		case ELEMENTNAME_TYPE:
+				*x = ((TextTypePtr) ptr2)->X;
+				*y = ((TextTypePtr) ptr2)->Y;
+				break;
+		case ELEMENT_TYPE:
+				*x = ((ElementTypePtr) ptr2)->MarkX;
+				*y = ((ElementTypePtr) ptr2)->MarkY;
+				break;
+		case POLYGON_TYPE:
+				*x = ((PolygonTypePtr) ptr2)->Points[0].X;
+				*y = ((PolygonTypePtr) ptr2)->Points[0].Y;
+				break;
+				
+		case LINEPOINT_TYPE:
+		case POLYGONPOINT_TYPE:
+				*x = ((PointTypePtr) ptr3)->X;
+				*y = ((PointTypePtr) ptr3)->Y;
+				break;
+		case ARC_TYPE:
+			{
+				BoxTypePtr	box;
+
+				box = GetArcEnds((ArcTypePtr) ptr2);
+				*x = box->X1;
+				*y = box->Y1;
+				break;
+			}
+	}
+}
+
+void	AttachForCopy(Position PlaceX, Position PlaceY)
+{
+	BoxTypePtr	box;
+	Position	mx, my;
+
+	Crosshair.AttachedObject.RubberbandN = 0;
+		/* dither the grab point so that the mark, center, etc
+		 * will end up on a grid coordinate
+		 */
+	GetGridLockCoordinates(Crosshair.AttachedObject.Type,
+		Crosshair.AttachedObject.Ptr1,
+		Crosshair.AttachedObject.Ptr2,
+		Crosshair.AttachedObject.Ptr3,
+		&mx, &my);
+	mx = GRIDFIT_X(mx, PCB->Grid) - mx;
+	my = GRIDFIT_Y(my, PCB->Grid) - my;
+	if (2 * abs(mx) > PCB->Grid)
+		mx = (mx < 0) ? mx + PCB->Grid : mx - PCB->Grid;
+	if (2 * abs(my) > PCB->Grid)
+		my = (my < 0) ? my + PCB->Grid : my - PCB->Grid;
+	Crosshair.AttachedObject.X = PlaceX - mx;
+	Crosshair.AttachedObject.Y = PlaceY - my;
+	Crosshair.AttachedObject.State = STATE_SECOND;
+
+		/* get boundingbox of object and set cursor range */
+	box= GetObjectBoundingBox(Crosshair.AttachedObject.Type,
+		Crosshair.AttachedObject.Ptr1,
+		Crosshair.AttachedObject.Ptr2,
+		Crosshair.AttachedObject.Ptr3);
+	SetCrosshairRange(Crosshair.X -box->X1,
+		Crosshair.Y -box->Y1,
+		PCB->MaxWidth -(box->X2 -Crosshair.X),
+		PCB->MaxHeight -(box->Y2 -Crosshair.Y));
+
+		/* get all attached objects if necessary */
+	if ((Settings.Mode != COPY_MODE) && TEST_FLAG(RUBBERBANDFLAG, PCB))
+		LookupRubberbandLines(Crosshair.AttachedObject.Type,
+			Crosshair.AttachedObject.Ptr1,
+			Crosshair.AttachedObject.Ptr2,
+			Crosshair.AttachedObject.Ptr3);
+	if (Settings.Mode != COPY_MODE &&
+		(Crosshair.AttachedObject.Type == ELEMENT_TYPE ||
+		Crosshair.AttachedObject.Type == VIA_TYPE ||
+		Crosshair.AttachedObject.Type == LINE_TYPE ||
+		Crosshair.AttachedObject.Type == LINEPOINT_TYPE))
+			LookupRatLines(Crosshair.AttachedObject.Type,
+				Crosshair.AttachedObject.Ptr1,
+				Crosshair.AttachedObject.Ptr2,
+				Crosshair.AttachedObject.Ptr3);
+}
+
+Boolean ShiftPressed(void)
+{
+       char    keys[32];
+
+       XQueryKeymap(Dpy, keys);
+       if (keys[ShiftKeyIndex] & ShiftKeyMask)
+               return(True);
+       else
+               return(False);
+}
+
