@@ -41,16 +41,17 @@ void		SetPolygonBoundingBox(PolygonTypePtr);
 void		SetElementBoundingBox(ElementTypePtr, FontTypePtr);
 Boolean		IsDataEmpty(DataTypePtr);
 BoxTypePtr	GetDataBoundingBox(DataTypePtr);
-void		CenterDisplay(Position, Position);
+void		CenterDisplay(Position, Position, Boolean);
 void		SetFontInfo(FontTypePtr);
 void		UpdateSettingsOnScreen(void);
 int		ParseGroupString(char *, LayerGroupTypePtr);
+int		ParseRouteString(char *, RouteStyleTypePtr);
 void		QuitApplication(void);
 char		*EvaluateFilename(char *, char *, char *, char *);
 char		*ExpandFilename(char *, char *);
 void		SetTextBoundingBox(FontTypePtr, TextTypePtr);
 Boolean		GetPosition(char *);
-void		ReleaseSaveUnderPixmap(void);
+void		ReleaseOffscreenPixmap(void);
 void		SaveOutputWindow(void);
 unsigned int	SetOutputXCursor(unsigned int);
 int		GetLayerNumber(DataTypePtr, LayerTypePtr);
@@ -58,11 +59,15 @@ int		GetLayerGroupNumberByPointer(LayerTypePtr);
 int		GetLayerGroupNumberByNumber(Cardinal);
 BoxTypePtr	GetObjectBoundingBox(int, void *, void *, void *);
 void		ResetStackAndVisibility(void);
-void		ChangePCBSize(Dimension, Dimension);
 char		*GetWorkingDirectory(void);
 void		CreateQuotedString(DynamicStringTypePtr, char *);
 int		GetGridFactor(void);
 void		SetArcBoundingBox(ArcTypePtr);
 BoxTypePtr	GetArcEnds(ArcTypePtr);
+char		*UniqueElementName(DataTypePtr, char *);
+void		AttachForCopy(Position, Position);
+Boolean		ShiftPressed(void);
+#define		rotateIcon_x_hot 8
+#define		rotateIcon_y_hot 8
 
 #endif
