@@ -59,6 +59,7 @@ static	void	*MoveLinePoint(LayerTypePtr, LineTypePtr, PointTypePtr);
 static	void	*MovePolygonPoint(LayerTypePtr, PolygonTypePtr, PointTypePtr);
 static	void	*MoveLineToLayer(LayerTypePtr, LineTypePtr);
 static	void	*MoveArcToLayer(LayerTypePtr, ArcTypePtr);
+static	void	*MoveRatToLayer(RatTypePtr);
 static	void	*MoveTextToLayer(LayerTypePtr, TextTypePtr);
 static	void	*MovePolygonToLayer(LayerTypePtr, PolygonTypePtr);
 
@@ -80,7 +81,8 @@ static	ObjectFunctionType	MoveFunctions = {
 	NULL,
 	MoveLinePoint,
 	MovePolygonPoint,
-	MoveArc },
+	MoveArc,
+	NULL },
 							MoveToLayerFunctions = {
 	MoveLineToLayer,
 	MoveTextToLayer,
@@ -92,7 +94,8 @@ static	ObjectFunctionType	MoveFunctions = {
 	NULL,
 	NULL,
 	NULL,
-	MoveArcToLayer };
+	MoveArcToLayer,
+	MoveRatToLayer };
 
 /* ---------------------------------------------------------------------------
  * moves a element by +-X and +-Y
@@ -115,14 +118,12 @@ void MoveElementLowLevel(ElementTypePtr Element, Position DX, Position DY)
  */
 static void *MoveElementName(ElementTypePtr Element)
 {
-	if (PCB->ElementOn &&
-		((TEST_FLAG(ONSOLDERFLAG, Element) != 0) == SWAP_IDENT ||
-		PCB->InvisibleObjectsOn))
+	if (PCB->ElementOn && (FRONT(Element) || PCB->InvisibleObjectsOn))
 	{
 		EraseElementName(Element);
 		ELEMENTTEXT_LOOP(Element, MOVE_TEXT_LOWLEVEL(text, DeltaX, DeltaY));
 		DrawElementName(Element, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	else
 	{
@@ -138,9 +139,7 @@ static void *MoveElement(ElementTypePtr Element)
 {
 	Boolean didDraw = False;
 
-	if (PCB->ElementOn &&
-		((TEST_FLAG(ONSOLDERFLAG, Element) != 0) == SWAP_IDENT ||
-		PCB->InvisibleObjectsOn))
+	if (PCB->ElementOn && (FRONT(Element) || PCB->InvisibleObjectsOn))
 	{
 		EraseElement(Element);
 		MoveElementLowLevel(Element, DeltaX, DeltaY);
@@ -154,14 +153,14 @@ static void *MoveElement(ElementTypePtr Element)
 			EraseElementPinsAndPads(Element);
 		MoveElementLowLevel(Element, DeltaX, DeltaY);
 	}
-	UpdatePIPFlags(NULL, Element, NULL, True);
+	UpdatePIPFlags(NULL, Element, NULL, NULL, True);
 	if (PCB->PinOn)
 	{
 		DrawElementPinsAndPads(Element, 0);
 		didDraw = True;
 	}
 	if (didDraw)
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	return(Element);
 }
 
@@ -177,11 +176,11 @@ static void *MoveVia(PinTypePtr Via)
 	}
 	else
 		MOVE_VIA_LOWLEVEL(Via, DeltaX, DeltaY);
-	UpdatePIPFlags(Via, (ElementTypePtr) Via, NULL, True);
+	UpdatePIPFlags(Via, (ElementTypePtr) Via, NULL, NULL, True);
 	if (PCB->ViaOn)
 	{
 		DrawVia(Via, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	return(Via);
 }
@@ -196,7 +195,7 @@ static void *MoveLine(LayerTypePtr Layer, LineTypePtr Line)
 		EraseLine(Line);
 		MOVE_LINE_LOWLEVEL(Line, DeltaX, DeltaY);
 		DrawLine(Layer, Line, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	else
 		MOVE_LINE_LOWLEVEL(Line, DeltaX, DeltaY);
@@ -214,7 +213,7 @@ static void *MoveArc(LayerTypePtr Layer, ArcTypePtr Arc)
 		MOVE_ARC_LOWLEVEL(Arc, DeltaX, DeltaY);
 		SetArcBoundingBox(Arc);
 		DrawArc(Layer, Arc, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	else
 	{
@@ -234,7 +233,7 @@ static void *MoveText(LayerTypePtr Layer, TextTypePtr Text)
 		EraseText(Text);
 		MOVE_TEXT_LOWLEVEL(Text, DeltaX, DeltaY);
 		DrawText(Layer, Text, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	else
 		MOVE_TEXT_LOWLEVEL(Text, DeltaX, DeltaY);
@@ -263,11 +262,11 @@ static void *MovePolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 		ErasePolygon(Polygon);
 		MovePolygonLowLevel(Polygon, DeltaX, DeltaY);
 		DrawPolygon(Layer, Polygon, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	else
 		MovePolygonLowLevel(Polygon, DeltaX, DeltaY);
-	UpdatePIPFlags(NULL, NULL, Layer, True);
+	UpdatePIPFlags(NULL, NULL, Layer, Polygon, True);
 	return(Polygon);
 }
 
@@ -277,16 +276,32 @@ static void *MovePolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 static void *MoveLinePoint(LayerTypePtr Layer, LineTypePtr Line,
 	PointTypePtr Point)
 {
-	if (Layer->On)
+	if (Layer)
 	{
-		EraseLine(Line);
-		MOVE(Point->X, Point->Y, DeltaX, DeltaY)
-		DrawLine(Layer, Line, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		if (Layer->On)
+		{
+			EraseLine(Line);
+			MOVE(Point->X, Point->Y, DeltaX, DeltaY)
+			DrawLine(Layer, Line, 0);
+			Draw();
+		}
+		else
+			MOVE(Point->X, Point->Y, DeltaX, DeltaY)
+		return(Line);
+	}
+	else		/* must be a rat */
+	{
+		if (PCB->RatOn)
+		{
+			EraseRat((RatTypePtr) Line);
+			MOVE(Point->X, Point->Y, DeltaX, DeltaY)
+			DrawRat((RatTypePtr) Line, 0);
+			Draw();
+		}
+		else
+			MOVE(Point->X, Point->Y, DeltaX, DeltaY)
+		return(Line);
 	}
-	else
-		MOVE(Point->X, Point->Y, DeltaX, DeltaY)
-	return(Line);
 }
 
 /* ---------------------------------------------------------------------------
@@ -299,16 +314,20 @@ static void *MovePolygonPoint(LayerTypePtr Layer, PolygonTypePtr Polygon,
 	{
 		ErasePolygon(Polygon);
 		MOVE(Point->X, Point->Y, DeltaX, DeltaY);
-		SetPolygonBoundingBox(Polygon);
-		DrawPolygon(Layer, Polygon, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		if (!RemoveExcessPolygonPoints(Layer, Polygon))
+		{
+			SetPolygonBoundingBox(Polygon);
+			DrawPolygon(Layer, Polygon, 0);
+			Draw();
+		}
 	}
 	else
 	{
 		MOVE(Point->X, Point->Y, DeltaX, DeltaY);
-		SetPolygonBoundingBox(Polygon);
+		if (!RemoveExcessPolygonPoints(Layer, Polygon))
+			SetPolygonBoundingBox(Polygon);
 	}
-	UpdatePIPFlags(NULL, NULL, Layer, True);
+	UpdatePIPFlags(NULL, NULL, Layer, Polygon, True);
 	return(Point);
 }
 
@@ -353,7 +372,7 @@ static void *MoveArcToLayer(LayerTypePtr Layer, ArcTypePtr Arc)
 	if (Dest == Layer && Layer->On)
 	{
 		DrawArc(Layer, Arc, 0);
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	if (((int) Dest == -1) || Dest == Layer)
 		return(Arc);
@@ -363,10 +382,30 @@ static void *MoveArcToLayer(LayerTypePtr Layer, ArcTypePtr Arc)
 	new = MoveArcToLayerLowLevel(Layer, Arc, Dest);
 	if (Dest->On)
 		DrawArc(Dest, new, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(new);
 }
 
+/* ---------------------------------------------------------------------------
+ * moves a line between layers
+ */
+static void *MoveRatToLayer(RatTypePtr Rat)
+{
+	LineTypePtr	new;
+
+	new = CreateNewLineOnLayer(Dest, Rat->Point1.X, Rat->Point1.Y,
+		Rat->Point2.X, Rat->Point2.Y, Settings.LineThickness,
+		2*GROUNDPLANEFRAME, (Rat->Flags & ~RATFLAG) | CLEARLINEFLAG);
+	if (!new)
+		return(NULL);
+	AddObjectToCreateUndoList(LINE_TYPE, Dest, new, new);
+	if (PCB->RatOn)
+		EraseRat(Rat);
+	MoveObjectToRemoveUndoList(RATLINE_TYPE, Rat, Rat, Rat);
+	DrawLine(Dest, new, 0);
+	Draw();
+	return(new);
+}
 
 /* ---------------------------------------------------------------------------
  * moves a line between layers
@@ -377,15 +416,10 @@ static void *MoveLineToLayer(LayerTypePtr Layer, LineTypePtr Line)
         PinTypePtr      via;
         void            *ptr1, *ptr2, *ptr3;
 
-        if (TEST_FLAG(RATFLAG, Line))
-        {
-                AddObjectToFlagUndoList(LINE_TYPE, Layer, Line, Line);
-                CLEAR_FLAG(RATFLAG, Line);
-        }
 	if (Dest == Layer && Layer->On)
 	{
         	DrawLine(Layer, Line, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
         if (((int) Dest == -1) || Dest == Layer)
                 return(Line);
@@ -396,11 +430,10 @@ static void *MoveLineToLayer(LayerTypePtr Layer, LineTypePtr Line)
 	new = MoveLineToLayerLowLevel(Layer, Line, Dest);
 	if (Dest->On)
 		DrawLine(Dest, new, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	if (!PCB->ViaOn || MoreToCome)
+	Draw();
+	if (!PCB->ViaOn || MoreToCome ||
+            GetLayerGroupNumberByPointer(Layer) == GetLayerGroupNumberByPointer(Dest))
 		return(new);
-        if (GetLayerGroupNumberByPointer(Layer) == GetLayerGroupNumberByPointer(Dest))
-                return (new);
         if ((SearchObjectByPosition(PIN_TYPES, &ptr1, &ptr2, &ptr3,
             new->Point1.X, new->Point1.Y, 0) == NO_TYPE))
                 LINE_LOOP(Layer,
@@ -408,10 +441,12 @@ static void *MoveLineToLayer(LayerTypePtr Layer, LineTypePtr Line)
                       || ((line->Point2.X == new->Point1.X) && (line->Point2.Y == new->Point1.Y)))
                         {
                                 if ((via = CreateNewVia(PCB->Data, new->Point1.X, new->Point1.Y,
-                                    Settings.ViaThickness, Settings.ViaDrillingHole, NULL, VIAFLAG))
+                                    Settings.ViaThickness, 2*GROUNDPLANEFRAME,
+				    Settings.ViaDrillingHole, NULL, VIAFLAG))
                                     != NULL)
                                 {
-                                        UpdatePIPFlags(via, (ElementTypePtr) via, NULL, False);
+                                        UpdatePIPFlags(via, (ElementTypePtr) via,
+						NULL, NULL, False);
                                         AddObjectToCreateUndoList(VIA_TYPE, via, via, via);
                                         DrawVia(via, 0);
                                 } 
@@ -425,17 +460,19 @@ static void *MoveLineToLayer(LayerTypePtr Layer, LineTypePtr Line)
                       || ((line->Point2.X == new->Point2.X) && (line->Point2.Y == new->Point2.Y)))
                         {
                                 if ((via = CreateNewVia(PCB->Data, new->Point2.X, new->Point2.Y,
-                                    Settings.ViaThickness, Settings.ViaDrillingHole, NULL, VIAFLAG))
+                                    Settings.ViaThickness, 2*GROUNDPLANEFRAME,
+				    Settings.ViaDrillingHole, NULL, VIAFLAG))
                                     != NULL)
                                 {
-                                        UpdatePIPFlags(via, (ElementTypePtr) via, NULL, False);
+                                        UpdatePIPFlags(via, (ElementTypePtr) via,
+						NULL, NULL, False);
                                         AddObjectToCreateUndoList(VIA_TYPE, via, via, via);
                                         DrawVia(via, 0);
                                 } 
                         break;
                         }
                 );
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
         return(new);
 }
 
@@ -464,49 +501,23 @@ void *MoveTextToLayerLowLevel(LayerTypePtr Source, TextTypePtr Text,
 /* ---------------------------------------------------------------------------
  * moves a text object between layers
  */
-void *MoveTextToLayer(LayerTypePtr Layer, TextTypePtr Text)
+static void *MoveTextToLayer(LayerTypePtr Layer, TextTypePtr Text)
 {
 	TextTypePtr	new;
 
-                /* Dest == -1 means move to silkscreen layer */
-        if (((int) Dest == -1) || (Dest == Layer))
-                AddObjectToFlagUndoList(TEXT_TYPE, Layer, Text, Text);
-        else 
+	if (Dest != Layer)
+	{                       
 		AddObjectToMoveToLayerUndoList(TEXT_TYPE, Layer, Text, Text);
-        if ((Layer->On && !TEST_FLAG(ONSILKFLAG, Text)) ||
-           (TEST_FLAG(ONSILKFLAG, Text) && (PCB->InvisibleObjectsOn ||
-           (TEST_FLAG(ONSOLDERFLAG, Text) !=0) == SWAP_IDENT)))
-		EraseText(Text);
-        if ((int) Dest == -1)
-        {       
-                SET_FLAG(ONSILKFLAG, Text);
-                if (PCB->ElementOn || (TEST_FLAG(ONSOLDERFLAG, Text) !=0) == SWAP_IDENT)
-		{
-                        DrawText(Layer, Text, 0);
-        		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		}
-                return(Text);
-        }
-        else
-        {
-                CLEAR_FLAG(ONSILKFLAG, Text);
-                if (Dest != Layer)
-                {                       
-			new = MoveTextToLayerLowLevel(Layer, Text, Dest);
-			if (Dest->On)
-			{
-				DrawText(Dest, new, 0);
-        			XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-			}
-			return(new);
-                }
-                if (Dest->On)
-		{
-                        DrawText(Dest, Text, 0);
-        		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-		}
-                return(Text);
-        }  
+		new = MoveTextToLayerLowLevel(Layer, Text, Dest);
+        	if (Layer->On)
+			EraseText(Text);
+		if (Dest->On)
+			DrawText(Dest, new, 0);
+		if (Layer->On || Dest->On)
+			Draw();
+		return(new);
+	}
+	return(Text);
 }
 
 /* ---------------------------------------------------------------------------
@@ -559,11 +570,12 @@ static void *MovePolygonToLayer(LayerTypePtr Layer, PolygonTypePtr Polygon)
                 }
         );
 	new = MovePolygonToLayerLowLevel(Layer, Polygon, Dest);
-	UpdatePIPFlags(NULL, NULL, NULL, True);
+	UpdatePIPFlags(NULL, NULL, Layer, NULL, True);
+	UpdatePIPFlags(NULL, NULL, Dest, new, True);
 	if (Dest->On)
 	{
 		DrawPolygon(Dest, new, 0);
-        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	return(new);
 }
@@ -597,6 +609,8 @@ void *MoveObjectAndRubberband(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 		/* setup offset */
 	DeltaX = DX;
 	DeltaY = DY;
+	if (DX == 0 && DY == 0)
+		return(NULL);
 
 		/* move all the lines... and reset the counter */
 	ptr = Crosshair.AttachedObject.Rubberband;
@@ -646,7 +660,7 @@ Boolean MoveSelectedObjectsToLayer(LayerTypePtr Target)
 	Dest = Target;
 	MoreToCome = True;
 	changed = SelectedOperation(&MoveToLayerFunctions, True);
-	IncrementUndoSerialNumber();
+		/* passing True to above operation causes Undoserial to auto-increment */
 	return(changed);
 }
 
