@@ -68,10 +68,32 @@ RubberbandTypePtr GetRubberbandMemory(void)
 	return(ptr +Crosshair.AttachedObject.RubberbandN++);
 }
 
+void **GetPointerMemory(PointerListTypePtr list)
+{
+	void **ptr = list->Ptr;
+
+		/* realloc new memory if necessary and clear it */
+	if (list->PtrN >= list->PtrMax)
+	{
+		list->PtrMax += STEP_POINT;	
+		ptr = MyRealloc(ptr, list->PtrMax *sizeof(void *),
+			"GetPointerMemory()");
+		list->Ptr = ptr;
+		memset(ptr + list->PtrN, 0, STEP_POINT*sizeof(void *));
+	}
+	return(ptr + list->PtrN++);
+}
+
+void FreePointerListMemory(PointerListTypePtr list)
+{
+	MyFree((char **) &list->Ptr);
+	memset(list, 0, sizeof(PointerListType));
+}
+
 /* ---------------------------------------------------------------------------
  * get next slot for a connection, allocates memory if necessary
  */
-ConnectionTypePtr GetConnectionMemory(SubnetTypePtr Net)
+ConnectionTypePtr GetConnectionMemory(NetTypePtr Net)
 {
 	ConnectionTypePtr con = Net->Connection;
 
@@ -90,20 +112,20 @@ ConnectionTypePtr GetConnectionMemory(SubnetTypePtr Net)
 /* ---------------------------------------------------------------------------
  * get next slot for a subnet, allocates memory if necessary
  */
-SubnetTypePtr GetSubnetMemory(NetTypePtr Net)
+NetTypePtr GetNetMemory(NetListTypePtr Netlist)
 {
-	SubnetTypePtr sub = Net->Subnet;
+	NetTypePtr net = Netlist->Net;
 
 		/* realloc new memory if necessary and clear it */
-	if (Net->SubnetN >= Net->SubnetMax)
+	if (Netlist->NetN >= Netlist->NetMax)
 	{
-		Net->SubnetMax += STEP_POINT;	
-		sub = MyRealloc(sub, Net->SubnetMax *sizeof(SubnetType),
-			"GetSubnetMemory()");
-		Net->Subnet = sub;
-		memset(sub + Net->SubnetN, 0, STEP_POINT*sizeof(SubnetType));
+		Netlist->NetMax += STEP_POINT;	
+		net = MyRealloc(net, Netlist->NetMax *sizeof(NetType),
+			"GetNetMemory()");
+		Netlist->Net = net;
+		memset(net + Netlist->NetN, 0, STEP_POINT*sizeof(NetType));
 	}
-	return(sub + Net->SubnetN++);
+	return(net + Netlist->NetN++);
 }
 
 /* ---------------------------------------------------------------------------
@@ -163,6 +185,25 @@ PinTypePtr GetViaMemory(DataTypePtr Data)
 	return(via +Data->ViaN++);
 }
 
+/* ---------------------------------------------------------------------------
+ * get next slot for a Rat, allocates memory if necessary
+ */
+RatTypePtr GetRatMemory(DataTypePtr Data)
+{
+	RatTypePtr rat = Data->Rat;
+
+		/* realloc new memory if necessary and clear it */
+	if (Data->RatN >= Data->RatMax)
+	{
+		Data->RatMax += STEP_RAT;  
+		rat = MyRealloc(rat, Data->RatMax *sizeof(RatType),
+			"GetRatMemory()");
+		Data->Rat = rat;
+		memset(rat +Data->RatN, 0, STEP_RAT*sizeof(RatType));
+	}
+	return(rat +Data->RatN++);
+}
+
 /* ---------------------------------------------------------------------------
  * get next slot for a line, allocates memory if necessary
  */
@@ -271,7 +312,6 @@ ElementTypePtr GetElementMemory(DataTypePtr Data)
 	if (Data->ElementN >= Data->ElementMax)
 	{
 		Data->ElementMax += STEP_ELEMENT;	
-		FreeNetlist();
 		element = MyRealloc(element, Data->ElementMax *sizeof(ElementType),
 			"GetElementMemory()");
 		Data->Element = element;
@@ -283,21 +323,21 @@ ElementTypePtr GetElementMemory(DataTypePtr Data)
 /* ---------------------------------------------------------------------------
  * get next slot for an library menu, allocates memory if necessary
  */
-LibraryMenuTypePtr GetLibraryMenuMemory(void)
+LibraryMenuTypePtr GetLibraryMenuMemory(LibraryTypePtr lib)
 {
-	LibraryMenuTypePtr	menu = Library.Menu;
+	LibraryMenuTypePtr	menu = lib->Menu;
 
 		/* realloc new memory if necessary and clear it */
-	if (Library.MenuN >= Library.MenuMax)
+	if (lib->MenuN >= lib->MenuMax)
 	{
-		Library.MenuMax += STEP_LIBRARYMENU;	
-		menu = MyRealloc(menu, Library.MenuMax *sizeof(LibraryMenuType),
+		lib->MenuMax += STEP_LIBRARYMENU;	
+		menu = MyRealloc(menu, lib->MenuMax *sizeof(LibraryMenuType),
 			"GetLibraryMenuMemory()");
-		Library.Menu = menu;
-		memset(menu +Library.MenuN, 0,
+		lib->Menu = menu;
+		memset(menu +lib->MenuN, 0,
 			STEP_LIBRARYMENU *sizeof(LibraryMenuType));
 	}
-	return(menu +Library.MenuN++);
+	return(menu +lib->MenuN++);
 }
 
 /* ---------------------------------------------------------------------------
@@ -388,9 +428,15 @@ void *MyCalloc(size_t Number, size_t Size, char *Text)
 {
 	void	*p;
 
+#ifdef MEM_DEBUG
+	fprintf(stderr, "MyCalloc %d by %d from %s ",Number, Size, Text);
+#endif
 	if ((p = calloc(Number, Size)) == NULL)
 		MyFatal("out of memory during calloc() in '%s'()\n",
 			(Text ? Text : "(unknown)"));
+#ifdef MEM_DEBUG
+	fprintf(stderr, "returned 0x%x\n",p);
+#endif
 	return(p);
 }
 
@@ -403,10 +449,16 @@ void *MyRealloc(void *Ptr, size_t Size, char *Text)
 {
 	void	*p;
 
+#ifdef MEM_DEBUG
+	fprintf(stderr, "0x%x Realloc to %d from %s ",Ptr, Size, Text);
+#endif
 	p = Ptr ? realloc(Ptr, Size) : malloc(Size);
 	if (!p)
 		MyFatal("out of memory during realloc() in '%s'()\n",
 			(Text ? Text : "(unknown)"));
+#ifdef MEM_DEBUG
+	fprintf(stderr, "returned 0x%x\n",p);
+#endif
 	return(p);
 }
 
@@ -421,6 +473,9 @@ char *MyStrdup(char *S, char *Text)
 	if (S && ((p = strdup(S)) == NULL))
 		MyFatal("out of memory during strdup() in '%s'\n",
 			(Text ? Text : "(unknown)"));
+#ifdef MEM_DEBUG
+	fprintf(stderr, "strdup returning 0x%x\n", p);
+#endif
 	return(p);	
 }
  
@@ -448,24 +503,25 @@ void FreePolygonMemory(PolygonTypePtr Polygon)
 /* ---------------------------------------------------------------------------
  * frees memory used by a net 
  */
-void FreeNetMemory(NetTypePtr Net)
+void FreeNetListMemory(NetListTypePtr Netlist)
 {
-	if (Net)
+	if (Netlist)
 	{
-		MyFree((char **) &Net->Subnet);
-		memset(Net, 0, sizeof(NetType));
+		NET_LOOP(Netlist, FreeNetMemory(net));
+		MyFree((char **) &Netlist->Net);
+		memset(Netlist, 0, sizeof(NetListType));
 	}
 }
 
 /* ---------------------------------------------------------------------------
  * frees memory used by a subnet 
  */
-void FreeSubnetMemory(SubnetTypePtr Net)
+void FreeNetMemory(NetTypePtr Net)
 {
 	if (Net)
 	{
 		MyFree((char **) &Net->Connection);
-		memset(Net, 0, sizeof(SubnetType));
+		memset(Net, 0, sizeof(NetType));
 	}
 }
 
@@ -494,17 +550,14 @@ void FreePCBMemory(PCBTypePtr PCBPtr)
 
 	if (PCBPtr)
 	{
-			/* release pixmap and names */
-		ReleaseSaveUnderPixmap();
 		MyFree(&PCBPtr->Name);
 		MyFree(&PCBPtr->Filename);
 		MyFree(&PCBPtr->PrintFilename);
 		FreeDataMemory(PCBPtr->Data);
-
 			/* release font symbols */
 		for (i = 0; i <= MAX_FONTPOSITION; i++)
 			MyFree((char **) &PCBPtr->Font.Symbol[i].Line);
-
+		FreeLibraryMemory(&PCBPtr->NetlistLib);
 			/* clear struct */
 		memset(PCBPtr, 0, sizeof(PCBType));
 	}
@@ -520,11 +573,10 @@ void FreeDataMemory(DataTypePtr Data)
 
 	if (Data)
 	{
-		FreeNetlist();
 		VIA_LOOP(Data, MyFree(&via->Name););
 		ELEMENT_LOOP(Data, FreeElementMemory(element););
 
-		for (layer = Data->Layer, i = 0; i < MAX_LAYER; layer++, i++)
+		for (layer = Data->Layer, i = 0; i < MAX_LAYER + 2; layer++, i++)
 		{
 			TEXT_LOOP(layer, MyFree(&text->TextString););
 			MyFree(&layer->Name);
@@ -543,28 +595,20 @@ void FreeDataMemory(DataTypePtr Data)
 /* ---------------------------------------------------------------------------
  * releases the memory that's allocated by the library
  */
-void FreeLibraryMemory(void)
+void FreeLibraryMemory(LibraryTypePtr lib)
 {
-	int					i, j;
-	LibraryMenuTypePtr	menu;
-	LibraryEntryTypePtr	entry;
-	String				*listentry;
-
-	for (i = 0, menu = Library.Menu; i < Library.MenuN; i++, menu++)
-	{
-		entry = menu->Entry;
-		for (j = 0; j < menu->EntryN; j++, entry++, listentry++)
-		{
+	MENU_LOOP(lib,
+		ENTRY_LOOP(menu,
 			SaveFree((void *) entry->AllocatedMemory);
 			SaveFree((void *) entry->ListEntry);
-		}
+		);
 		SaveFree((void *) menu->Entry);
 		SaveFree((void *) menu->Name);
-	}
-	SaveFree((void *) Library.Menu);
+	);
+	SaveFree((void *) lib->Menu);
 
 		/* clear struct */
-	memset(&Library, 0, sizeof(LibraryType));
+	memset(lib, 0, sizeof(LibraryType));
 }
 
 /* ---------------------------------------------------------------------------
@@ -574,6 +618,9 @@ void FreeLibraryMemory(void)
  */
 void SaveFree(void *Ptr)
 {
+#ifdef MEM_DEBUG
+	fprintf(stderr, "Freeing 0x%x\n",Ptr);
+#endif
 	if (Ptr)
 		free(Ptr);
 }
