@@ -54,6 +54,46 @@ static void DoPIPFlags(PinTypePtr, ElementTypePtr, LayerTypePtr, PolygonTypePtr)
 
 static int old_flags, new_flags;
 
+/* --------------------------------------------------------------------------
+ * remove redundant polygon points. Any point that lies on the straight
+ * line between the points on either side of it is redundant.
+ * returns true if any points are removed
+ */
+Boolean RemoveExcessPolygonPoints(LayerTypePtr Layer, PolygonTypePtr Polygon)
+{
+	PointTypePtr	pt1, pt2, pt3;
+	Cardinal	n;
+	LineType	line;
+	Boolean		changed = False;
+	
+	if (Undoing())
+		return(False);
+		/* there are always at least three points in a polygon */
+	pt1 = &Polygon->Points[Polygon->PointN-1];
+	pt2 = &Polygon->Points[0];
+	pt3 = &Polygon->Points[1];
+	for (n = 0; n < Polygon->PointN; n++, pt1++, pt2++, pt3++)
+	{
+			/* wrap around polygon */
+		if (n == 1)
+			pt1 = &Polygon->Points[0];
+		if (n == Polygon->PointN -1)
+			pt3 = &Polygon->Points[0];
+		line.Point1 = *pt1;
+		line.Point2 = *pt3;
+		line.Thickness = 0;
+		if (IsPointOnLine((float) pt2->X, (float) pt2->Y, 0.0, &line))
+		{
+			SaveUndoSerialNumber();
+			RemoveObject(POLYGONPOINT_TYPE, (void *)Layer, (void *) Polygon,
+				(void *) pt2);
+			RestoreUndoSerialNumber();
+			changed = True;
+		}
+	}
+	return(changed);
+}
+
 /* ---------------------------------------------------------------------------
  * returns the index of the polygon point which is the end
  * point of the segment with the lowest distance to the passed
@@ -152,6 +192,7 @@ void ClosePolygon(void)
 			}
 		}
 		CopyAttachedPolygonToLayer();
+		Draw();
 	}
 	else
 		Message("a polygon has to have at least 3 points\n");
@@ -170,7 +211,7 @@ void CopyAttachedPolygonToLayer(void)
         SET_FLAG(CLEARPOLYFLAG, polygon);
 	memset(&Crosshair.AttachedPolygon, 0, sizeof(PolygonType));
 	SetPolygonBoundingBox(polygon);
-        UpdatePIPFlags(NULL, NULL, CURRENT, True);
+        UpdatePIPFlags(NULL, NULL, CURRENT, polygon, True);
 	DrawPolygon(CURRENT, polygon, 0);
 	SetChangedFlag(True);
 
@@ -190,48 +231,51 @@ void CopyAttachedPolygonToLayer(void)
  *  if called with Layer == NULL, search all polygons
  */   
 void UpdatePIPFlags(PinTypePtr Pin, ElementTypePtr Element,
-                  LayerTypePtr Layer, Boolean AddUndo)
+                  LayerTypePtr Layer, PolygonTypePtr Polygon, Boolean AddUndo)
 {     
  
 	if (Element == NULL) 
 	{
-		ALLPIN_LOOP(PCB->Data,{
-			UpdatePIPFlags(pin, element, Layer, AddUndo);
-		});
-		VIA_LOOP(PCB->Data, {
-			UpdatePIPFlags(via, (ElementTypePtr) via, Layer, AddUndo);
-		});
+		ALLPIN_LOOP(PCB->Data,
+			UpdatePIPFlags(pin, element, Layer, Polygon, AddUndo);
+		);
+		VIA_LOOP(PCB->Data,
+			UpdatePIPFlags(via, (ElementTypePtr) via, Layer, 
+				Polygon, AddUndo);
+		);
 	}
-      else if (Pin == NULL)
-      {     
-              PIN_LOOP(Element, {
-                      UpdatePIPFlags(pin, Element, Layer, AddUndo);
-              });
-      }
-      else
-      {
-              old_flags = Pin->Flags;
-              if (Layer == NULL)
-              {
-                      Pin->Flags &= ~(ALLPIPFLAGS | WARNFLAG);
-                      ALLPOLYGON_LOOP(PCB->Data, {
-                              if (TEST_FLAG(CLEARPOLYFLAG, polygon))
-                                      DoPIPFlags(Pin, Element, layer, polygon);
-                      });
-              }
-              else    
-              {  
-                      Pin->Flags &= ~((L0PIPFLAG << GetLayerNumber(PCB->Data, Layer)) |
+	else if (Pin == NULL)
+	{     
+		PIN_LOOP(Element, 
+			UpdatePIPFlags(pin, Element, Layer, Polygon, AddUndo);
+		);
+	}
+	else if (Layer == NULL)
+	{
+		ALLPOLYGON_LOOP(PCB->Data,
+			if (TEST_FLAG(CLEARPOLYFLAG, polygon))
+				UpdatePIPFlags(Pin, Element, layer, polygon, AddUndo);
+		);
+	}
+	else if (Polygon == NULL)
+	{
+		POLYGON_LOOP(Layer,
+			if (TEST_FLAG(CLEARPOLYFLAG, polygon))
+				UpdatePIPFlags(Pin, Element, Layer, polygon, AddUndo);
+		);
+	}
+	else
+	{
+			/* GetLayerNumber this deep is wasted looping */
+		old_flags = Pin->Flags;
+		Pin->Flags &= ~((L0PIPFLAG << GetLayerNumber(PCB->Data, Layer)) |
                                       WARNFLAG);
-                      POLYGON_LOOP(Layer, {
-                              if (TEST_FLAG(CLEARPOLYFLAG, polygon))
-                                      DoPIPFlags(Pin, Element, Layer, polygon);
-                      });
-              }
-              new_flags = Pin->Flags;
-              if (new_flags != old_flags)
-              {
-                      Pin->Flags = old_flags;
+		if (TEST_FLAG(CLEARPOLYFLAG, Polygon))
+			DoPIPFlags(Pin, Element, Layer, Polygon);
+		new_flags = Pin->Flags;
+		if (new_flags != old_flags)
+		{
+			Pin->Flags = old_flags;
 			if (AddUndo)
 			{
 				if (Pin == (PinTypePtr) Element)
@@ -239,9 +283,9 @@ void UpdatePIPFlags(PinTypePtr Pin, ElementTypePtr Element,
 				else 
 					AddObjectToFlagUndoList(PIN_TYPE, Element, Pin, Pin);
 			}
-                      Pin->Flags = new_flags & ((new_flags >> 8) | ~ALLTHERMFLAGS);
-              }       
-      }
+			Pin->Flags = new_flags & ((new_flags >> 8) | ~ALLTHERMFLAGS);
+		}       
+	}
 }                          
        
 static void DoPIPFlags(PinTypePtr Pin, ElementTypePtr Element,
@@ -255,7 +299,7 @@ static void DoPIPFlags(PinTypePtr Pin, ElementTypePtr Element,
       else if (!TEST_FLAG(WARNFLAG, Pin) &&
                IsPointInPolygon(Pin->X, Pin->Y, Pin->Thickness/2+1, Polygon))
       {
-              Message("WARNING:  pin or via too close to polygon\n");
+              Message("WARNING:  pin or via too close to polygon edge\n");
               SET_FLAG(WARNFLAG, Pin);
               DrawPin(Pin, 0);
       }
