@@ -35,11 +35,12 @@
 
 Cardinal	GetLowestDistancePolygonPoint(PolygonTypePtr,
 					Position, Position);
+Boolean		RemoveExcessPolygonPoints(LayerTypePtr, PolygonTypePtr);
 void			GoToPreviousPoint(void);
 void			ClosePolygon(void);
 void			CopyAttachedPolygonToLayer(void);
 void			UpdatePIPFlags(PinTypePtr, ElementTypePtr,
-				       LayerTypePtr, Boolean);
+				       LayerTypePtr, PolygonTypePtr, Boolean);
 
 
 #endif
