@@ -36,15 +36,12 @@ static	char	*rcsid = "$Id: print.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
  */
 #include <math.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "global.h"
 
 #include "data.h"
 #include "dev_ps.h"
-/*
-  #include "dev_gerber.h"
-  #include "dev_gerberX.h"
-*/
 #include "dev_rs274x.h"
 #include "drill.h"
 #include "file.h"
@@ -69,22 +66,23 @@ static	char			*GlobalCommand;
 /* ---------------------------------------------------------------------------
  * some local prototypes
  */
-static	void	SetPrintColor(Pixel, int);
+static	void	SetPrintColor(Pixel);
 static	FILE	*OpenPrintFile(char *);
 static	int	SetupPrintFile(char *, char *);
 static	int	ClosePrintFile(void);
-static	int	FPrintOutline(int);
-static	int	FPrintAlignment(int);
-static	int	PrintLayergroups(Boolean, int);
-static	int	PrintSilkscreen(Boolean, int);
-static	int	PrintDrill(int);
-static	int	PrintMask(int);
+static	int	FPrintOutline(void);
+static	int	FPrintAlignment(void);
+static	int	PrintLayergroups(void);
+static	int	PrintSilkscreen(void);
+static	int	PrintDrill(void);
+static	int	PrintMask(void);
+static	int	PrintPaste(void);
 
 /* ----------------------------------------------------------------------
  * queries color from X11 database and calls device  command
  * black is default on errors
  */
-static void SetPrintColor(Pixel X11Color, int GBAFlag)
+static void SetPrintColor(Pixel X11Color)
 {
 	XColor	rgb;
 	int	result;
@@ -99,10 +97,10 @@ static void SetPrintColor(Pixel X11Color, int GBAFlag)
 		DefaultColormapOfScreen(XtScreen(Output.Toplevel)),
 		&rgb);
 
-	/* create a PS command to set this color */
+	/* default color is black */
 	if (result == BadValue || result == BadColor)
 		rgb.red = rgb.green = rgb.blue = 0;
-	Device->SetColor(rgb, GBAFlag);
+	Device->SetColor(rgb);
 }
 
 /* ---------------------------------------------------------------------------
@@ -169,22 +167,21 @@ static int ClosePrintFile(void)
 /* ---------------------------------------------------------------------------
  * prints or draws outline to a given file
  */
-static int FPrintOutline(int Gerber2Test)
+static int FPrintOutline(void)
 {
-	Device->Outline(0, 0, PCB->MaxWidth, PCB->MaxHeight, Gerber2Test);
+	Device->Outline(0, 0, PCB->MaxWidth, PCB->MaxHeight);
 	return(0);
 }
 
 /* ---------------------------------------------------------------------------
  * prints or draws alignment to a given file
  */
-static int FPrintAlignment(int Gerber2Test)
+static int FPrintAlignment(void)
 {
 	Device->Alignment(DeviceFlags.BoundingBox.X1,
 		DeviceFlags.BoundingBox.Y1,
 		DeviceFlags.BoundingBox.X2,
-		DeviceFlags.BoundingBox.Y2,
-		Gerber2Test);
+		DeviceFlags.BoundingBox.Y2);
 	return(0);
 }
 
@@ -192,26 +189,48 @@ static int FPrintAlignment(int Gerber2Test)
  * prints all layergroups
  * returns != zero on error
  */
-static int PrintLayergroups(Boolean SilkscreenTextFlag, int Gerber2Test)
+static int PrintLayergroups(void)
 {
 	char		extention[12],
 			description[18];
 	Cardinal	group,
-			entry;
+			entry,
+			component,
+			solder;
+	Boolean		noData, Somepolys;
+	int		PIPflag, Tflag;
 
-        ALLLINE_LOOP(PCB->Data, {
-                if (TEST_FLAG(RATFLAG, line))
-                {    
-                        Message("WARNING!!  Rat lines in layout during printing!\n"
-				"You DON'T want this!!!!\n");
-                        break;
-                }
-        });
+	if (PCB->Data->RatN)
+	{    
+		Message("WARNING!!  Rat lines in layout during printing!\n"
+			"You're not DONE with the layout!!!!\n");
+	};
 
-
-  	for (group = 0; group < MAX_LAYER+2; group ++)
+  	for (group = 0; group < MAX_LAYER; group ++)
 		if (PCB->LayerGroups.Number[group]) {
-       			Cardinal	i;
+
+				/* always include solder/component layers */
+			component = GetLayerGroupNumberByNumber(MAX_LAYER + COMPONENT_LAYER);
+			solder = GetLayerGroupNumberByNumber(MAX_LAYER + SOLDER_LAYER);
+			if (component == group || solder == group)
+				noData = False;
+			else
+				noData = True;
+			for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++)
+			{
+				LayerTypePtr	layer;
+				Cardinal	number;
+
+				if ((number = PCB->LayerGroups.Entries[group][entry]) >= MAX_LAYER)
+					continue;
+				layer = &PCB->Data->Layer[number];
+				if (layer->LineN || layer->TextN || layer->ArcN ||
+					layer->PolygonN)
+						noData = False;
+			}
+				/* skip empty layers */
+			if (noData)
+				continue;
 
 			/* setup extention and open new file */
 			sprintf(extention, "%s%i", GlobalDOSFlag ? "" : "group", group+1);
@@ -219,149 +238,147 @@ static int PrintLayergroups(Boolean SilkscreenTextFlag, int Gerber2Test)
 
 			if (SetupPrintFile(extention, description))
 				return(1);
-
-			/* make pre-print calls */
-			if (Gerber2Test == 1) {
-				if (GlobalAlignmentFlag)
-					FPrintAlignment(0);
-				for (i = 0; i < MAX_LAYER; i++)
-					for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++) {
-						LayerTypePtr	layer;
-						Cardinal	number;
-
-						if ((number = PCB->LayerGroups.Entries[group][entry]) != i)
-							continue;
-
-					/* OK, found one; print it and break the look */
-						layer = &PCB->Data->Layer[number];
-						SetPrintColor(layer->Color, 0);
-						Device->Layer(layer, group, SilkscreenTextFlag, 0);
-						break;
-					}
-
-				/* check the component/solder layers for being members
-				 * of this group. Print if TRUE
-				 */
-				for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++) {
-					Cardinal	number;
-
-					number = PCB->LayerGroups.Entries[group][entry] -MAX_LAYER;
-					switch(number) {
-						case COMPONENT_LAYER:
-						case SOLDER_LAYER:
-							if (GlobalOutlineFlag)
-								FPrintOutline(0);
-							SetPrintColor(PCB->PinColor, 0);
-							ALLPAD_LOOP(PCB->Data,
-								if ((TEST_FLAG(ONSOLDERFLAG, pad) != 0) ==
-									(number == SOLDER_LAYER))
-									Device->Pad(pad, 0););
-							break;
-						default:
-							break;
-					}
-				}
-
-				/* now at last all the pins/vias */
-				SetPrintColor(PCB->PinColor, 0);
-				ALLPIN_LOOP(PCB->Data,
-					if (!TEST_FLAG(HOLEFLAG, pin) && !TEST_FLAG(PININPOLYFLAG, pin))
-					Device->PinOrVia(pin,0);
-				);       
-				SetPrintColor(PCB->ViaColor, 0);
-				VIA_LOOP(PCB->Data,
-					if (!TEST_FLAG(HOLEFLAG, via) && !TEST_FLAG(PININPOLYFLAG, via))
-					Device->PinOrVia(via, 0);
-				);
-				ALLPIN_LOOP(PCB->Data,
-					if (!TEST_FLAG(HOLEFLAG, pin) && TEST_FLAG(PININPOLYFLAG, pin)) 
-						Device->ClearPinOrViaOnGroundplane(pin, 0);
-				);
-				VIA_LOOP(PCB->Data,
-					if (!TEST_FLAG(HOLEFLAG, via) && TEST_FLAG(PININPOLYFLAG, via))  
-						Device->ClearPinOrViaOnGroundplane(via, 0);
-				);      
-
-
-				/* print drill-helper if requested */
-				if (GlobalDrillHelperFlag) {
-					SetPrintColor(PCB->PinColor, 0);
-					ALLPIN_LOOP(PCB->Data, Device->DrillHelper(pin, 0););
-					SetPrintColor(PCB->ViaColor, 0);
-					VIA_LOOP(PCB->Data, Device->DrillHelper(via, 0););
-				}
-			}
-
-			/* print all layers in stack-order;
-			 * loop oveal all stackposition and check each layer
+				/* comment on what layers in group */
+			if (Device->GroupID)
+				Device->GroupID(group);
+				/* indicate positive polarity */
+			Device->Polarity(0);
+			/* loop and check each layer
 			 * for being a member of the group
 			 */
 			if (GlobalAlignmentFlag)
-				FPrintAlignment(1);
+				FPrintAlignment();
 
-			for (i = 0; i < MAX_LAYER; i++)
-				for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++) {
+				/* print all polygons */
+			Somepolys = False;
+			for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++)
+			{
+				LayerTypePtr	layer;
+				Cardinal	number;
+
+				number = PCB->LayerGroups.Entries[group][entry];
+				if (number >= MAX_LAYER)
+					continue;
+
+				layer = &PCB->Data->Layer[number];
+				if (layer->PolygonN)
+				{
+					Somepolys = True;
+					SetPrintColor(layer->Color);
+					POLYGON_LOOP(layer, Device->Poly(polygon));
+				}
+			}
+				/* clear all lines, arcs, pins */
+			PIPflag = Tflag = 0;
+			if (Somepolys)
+			{
+				Device->Polarity(2);
+				for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++)
+				{
 					LayerTypePtr	layer;
 					Cardinal	number;
 
-					if ((number = PCB->LayerGroups.Entries[group][entry]) != i)
+					number = PCB->LayerGroups.Entries[group][entry];
+					if (number >= MAX_LAYER)
 						continue;
 
-					/* OK, found one; print it and break the look */
 					layer = &PCB->Data->Layer[number];
-					SetPrintColor(layer->Color, 1);
-					Device->Layer(layer, group, SilkscreenTextFlag, 1);
-					break;
+					PIPflag |= L0PIPFLAG << number;
+					Tflag |= L0THERMFLAG << number;
+					LINE_LOOP(layer,
+						if (TEST_FLAG(CLEARLINEFLAG, line))
+							Device->Line(line, True);
+					);
+					ARC_LOOP(layer,
+						if (TEST_FLAG(CLEARLINEFLAG, arc))
+							Device->Arc(arc, True);
+					);
 				}
-
-			/* check the component/solder layers for being members
-			 * of this group. Print if TRUE
-			 */
-			for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++) {
+				ALLPIN_LOOP(PCB->Data,
+					if (TEST_FLAG(PIPflag, pin))
+						Device->PinOrVia(pin, 1);
+				);
+				VIA_LOOP(PCB->Data,
+					if (TEST_FLAG(PIPflag, via))
+						Device->PinOrVia(via, 1);
+				);
+				if (group == component)
+				{
+					ALLPAD_LOOP(PCB->Data,
+						if (!TEST_FLAG(ONSOLDERFLAG, pad))
+							Device->Pad(pad, 1);
+					);
+				}
+				else if (group == solder)
+					ALLPAD_LOOP(PCB->Data,
+						if (TEST_FLAG(ONSOLDERFLAG, pad))
+							Device->Pad(pad, 1);
+					);
+				Device->Polarity(3);
+			}
+				/* print the lines/arcs/pins/pads/vias */
+			for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++)
+			{
+				LayerTypePtr	layer;
 				Cardinal	number;
 
-				number = PCB->LayerGroups.Entries[group][entry] -MAX_LAYER;
-				switch(number) {
-					case COMPONENT_LAYER:
-					case SOLDER_LAYER:
-						if (GlobalOutlineFlag)
-							FPrintOutline(1);
-						SetPrintColor(PCB->PinColor, 1);
-						ALLPAD_LOOP(PCB->Data,
-							if ((TEST_FLAG(ONSOLDERFLAG, pad) != 0) ==
-								(number == SOLDER_LAYER))
-								Device->Pad(pad, 1););
-						break;
-					default:
-						break;
-				}
-			}
+				number = PCB->LayerGroups.Entries[group][entry];
+				if (number >= MAX_LAYER)
+					continue;
 
-			/* now at last all the pins/vias */
-			SetPrintColor(PCB->PinColor, 1);
+				layer = &PCB->Data->Layer[number];
+				SetPrintColor(layer->Color);
+				LINE_LOOP(layer, Device->Line(line, False));
+				ARC_LOOP(layer, Device->Arc(arc, False));
+				TEXT_LOOP(layer, Device->Text(text));
+			}
+			SetPrintColor(PCB->PinColor);
 			ALLPIN_LOOP(PCB->Data,
-				if (!TEST_FLAG(HOLEFLAG, pin) && !TEST_FLAG(PININPOLYFLAG, pin))
-				Device->PinOrVia(pin,1);
-			);       
-			SetPrintColor(PCB->ViaColor, 1);
-			VIA_LOOP(PCB->Data,
-				if (!TEST_FLAG(HOLEFLAG, via) && !TEST_FLAG(PININPOLYFLAG, via))
-				Device->PinOrVia(via, 1);
-			);
-			ALLPIN_LOOP(PCB->Data,
-				if (!TEST_FLAG(HOLEFLAG, pin) && TEST_FLAG(PININPOLYFLAG, pin)) 
-					Device->ClearPinOrViaOnGroundplane(pin, 1);
+				if (!TEST_FLAG(HOLEFLAG, pin))
+				{
+					if (TEST_FLAG(Tflag, pin))
+						SET_FLAG(USETHERMALFLAG, pin);
+					else
+						CLEAR_FLAG(USETHERMALFLAG, pin);
+					Device->PinOrVia(pin, 0);
+				}
 			);
+			if (group == component)
+			{
+				if (GlobalOutlineFlag)
+					FPrintOutline();
+				ALLPAD_LOOP(PCB->Data,
+					if (!TEST_FLAG(ONSOLDERFLAG, pad))
+						Device->Pad(pad, 0);
+				);
+			}
+			else if (group == solder)
+			{
+				if (GlobalOutlineFlag)
+					FPrintOutline();
+				ALLPAD_LOOP(PCB->Data,
+					if (TEST_FLAG(ONSOLDERFLAG, pad))
+						Device->Pad(pad, 0);
+				);
+			}
+			SetPrintColor(PCB->ViaColor);
 			VIA_LOOP(PCB->Data,
-				if (!TEST_FLAG(HOLEFLAG, via) && TEST_FLAG(PININPOLYFLAG, via))  
-					Device->ClearPinOrViaOnGroundplane(via, 1);
-			);      
+				if (!TEST_FLAG(HOLEFLAG, via))
+				{
+					if (TEST_FLAG(Tflag, via))
+						SET_FLAG(USETHERMALFLAG, via);
+					else
+						CLEAR_FLAG(USETHERMALFLAG, via);
+					Device->PinOrVia(via, 0);
+				}
+			);
 
 			/* print drill-helper if requested */
-			if (GlobalDrillHelperFlag) {
-				SetPrintColor(PCB->PinColor, 1);
+			if (GlobalDrillHelperFlag && Device->DrillHelper)
+			{
+				SetPrintColor(PCB->PinColor);
 				ALLPIN_LOOP(PCB->Data, Device->DrillHelper(pin, 1););
-				SetPrintColor(PCB->ViaColor, 1);
+				SetPrintColor(PCB->ViaColor);
 				VIA_LOOP(PCB->Data, Device->DrillHelper(via, 1););
 			}
 			/* close the device */
@@ -371,66 +388,106 @@ static int PrintLayergroups(Boolean SilkscreenTextFlag, int Gerber2Test)
 }
 
 /* ---------------------------------------------------------------------------
- * prints solder and component side
- * - first component followed by solder side
- * - first all element packages, followed by pads and pins
+ * prints solder and component side silk screens
+ * first print element outlines and names, then silk layer
  * returns != zero on error
  */
-static int PrintSilkscreen(Boolean SilkscreenTextFlag, int Gerber2Test)
+static int PrintSilkscreen(void)
 {
 	static	char	*extention[2] = { "componentsilk", "soldersilk" },
-			*DOSextention[2] = { "cslk", "sslk" },
+			*DOSextention[2] = { "csilk", "ssilk" },
 			*description[2] = {
-				"silkscreen component side",
-				"silkscreen solder side" };
+				"silkscreen, component side",
+				"silkscreen, solder side" };
+	LayerTypePtr	layer;
 	int		i;
+	Boolean		noData;
 
 	/* loop over both sides, start with component */
 	for (i = 0; i < 2; i++) {
+		layer = &PCB->Data->Layer[MAX_LAYER + (i == 0 ? COMPONENT_LAYER : SOLDER_LAYER)];
+		noData = True;
+		ELEMENT_LOOP(PCB->Data,
+			if ((TEST_FLAG(ONSOLDERFLAG, element) == 0) == (i == 0))
+			{
+				noData = False;
+				break;
+			}
+		);
+		if (layer->PolygonN || layer->LineN || layer->ArcN || layer->TextN)
+			noData = False;
+		if (noData)
+			continue;
 		/* start with the component side */
 		if (SetupPrintFile(GlobalDOSFlag ? DOSextention[i] : extention[i],
 			description[i]))
 			return(1);
+		/* positive polarity */
+		Device->Polarity(0);
+		/* print element outlines and canonical, instance or
+		 * value descriptions
+		 */
+		if (GlobalAlignmentFlag)
+			FPrintAlignment();
+
+		SetPrintColor(PCB->ElementColor);
+		ELEMENT_LOOP(PCB->Data,
+			if ((TEST_FLAG(ONSOLDERFLAG, element) == 0) == (i == 0))
+				Device->ElementPackage(element);
+		);
+
+		POLYGON_LOOP(layer, Device->Poly(polygon));
+		LINE_LOOP(layer, Device->Line(line, False));
+		ARC_LOOP(layer, Device->Arc(arc, False));
+		TEXT_LOOP(layer, Device->Text(text));
+		ClosePrintFile();
+	}
+	return(0);
+}
+
+/* ---------------------------------------------------------------------------
+ * prints solder and component side solder paste
+ * returns != zero on error
+ */
+static int PrintPaste(void)
+{
+	static	char	*extention[2] = { "componentpaste", "solderpaste" },
+			*DOSextention[2] = { "cpaste", "spaste" },
+			*description[2] = {
+				"solder paste, component side",
+				"solder paste, solder side" };
+	int		i;
+	Boolean		NoData;
+
+	/* loop over both sides, start with component */
+	for (i = 0; i < 2; i++) {
+		NoData = True;
+		ALLPAD_LOOP(PCB->Data,
+			if ((TEST_FLAG(ONSOLDERFLAG, pad) == 0) == (i == 0))
+			{
+				NoData = False;
+				break;
+			}
+		);
+			/* skip empty files */
+		if (NoData)
+			continue;
+
+			/* start with the component side */
+		if (SetupPrintFile(GlobalDOSFlag ? DOSextention[i] : extention[i],
+			description[i]))
+			return(1);
 
 		/* print element outlines and canonical, instance or
 		 * value descriptions
 		 */
-		if (Gerber2Test == 1) {
-			if (GlobalAlignmentFlag)
-				FPrintAlignment(0);
-
-			SetPrintColor(PCB->ElementColor, 0);
-			ELEMENT_LOOP(PCB->Data,
-				if ((TEST_FLAG(ONSOLDERFLAG, element) == 0) == (i == 0))
-					Device->ElementPackage(element, 0););
-			/* Scan all layers of all groups and place any text in the
-			 * silkscreen file that belongs there
-			 */
-			ALLTEXT_LOOP(PCB->Data, 
-				if (TEST_FLAG(ONSILKFLAG, text) &&
-				   ((TEST_FLAG(ONSOLDERFLAG, text) == 0)
-				   == (i == 0)))
-					Device->Text(text, 0);
-			);
-		}
-
 		if (GlobalAlignmentFlag)
-			FPrintAlignment(1);
+			FPrintAlignment();
 
-		SetPrintColor(PCB->ElementColor, 1);
-		ELEMENT_LOOP(PCB->Data,
-			if ((TEST_FLAG(ONSOLDERFLAG, element) == 0) == (i == 0))
-				Device->ElementPackage(element, 1);
-		);
-
-		/* Scan all layers of all groups and place any text in the
-		 * silkscreen file that belongs there
-		 */
-		ALLTEXT_LOOP(PCB->Data, 
-			if (TEST_FLAG(ONSILKFLAG, text) &&
-			   ((TEST_FLAG(ONSOLDERFLAG, text) == 0)
-			   == (i == 0)))
-				Device->Text(text, 0);
+		SetPrintColor(PCB->ElementColor);
+		ALLPAD_LOOP(PCB->Data,
+			if ((TEST_FLAG(ONSOLDERFLAG, pad) == 0) == (i == 0))
+				Device->Pad(pad, 3);
 		);
 		ClosePrintFile();
 	}
@@ -438,31 +495,54 @@ static int PrintSilkscreen(Boolean SilkscreenTextFlag, int Gerber2Test)
 }
 
 /* ---------------------------------------------------------------------------
- * creates a drill-information file if the device is able to
+ * creates drill-information file(s) if the device is able to
  * returns != zero on error
+ *
+ * if the drill helper is selected, merges plated and unplated in a single file
  */
-static int PrintDrill(int Gerber2Test)
+static int PrintDrill(void)
 {
         DrillInfoTypePtr        AllDrills;
+	Cardinal index;
 	int i;
 
-	/* pass drilling information */
+		/* pass drilling information */
 	if (Device->HandlesDrill) {
-		if (SetupPrintFile(GlobalDOSFlag ? "dril" : "drill",
+		if (SetupPrintFile(GlobalDOSFlag ? "pdrill" : "plated-drill",
 			"drill information"))
 			return(1);
 
-		if (Gerber2Test == 1) {
-			SetPrintColor(PCB->PinColor, 0);
-			ALLPIN_LOOP(PCB->Data, Device->Drill(pin, 0););
-			VIA_LOOP(PCB->Data, Device->Drill(via, 0););
-		}
-
-		SetPrintColor(PCB->PinColor, 1);
+		SetPrintColor(PCB->PinColor);
                 AllDrills = GetDrillInfo(PCB->Data);
+		index = 0;
                 DRILL_LOOP(AllDrills,
+			index++;
                         for (i = 0; i < drill->PinN; i++)
-                                Device->Drill(drill->Pin[i], 0);
+				if (!TEST_FLAG(HOLEFLAG, drill->Pin[i]))
+                                	Device->Drill(drill->Pin[i], index);
+				else if (GlobalDrillHelperFlag)
+					Device->Drill(drill->Pin[i], index);
+                );
+                        /* close the file */
+                ClosePrintFile();
+		if (GlobalDrillHelperFlag)
+		{
+			FreeDrillInfo(AllDrills);
+			return (0);
+		}
+		if (SetupPrintFile(GlobalDOSFlag ? "udrill" : "unplated-drill",
+			"drill information"))
+		{
+			FreeDrillInfo(AllDrills);
+			return(1);
+		}
+		SetPrintColor(PCB->PinColor);
+		index = 0;
+                DRILL_LOOP(AllDrills,
+			index++;
+                        for (i = 0; i < drill->PinN; i++)
+				if (TEST_FLAG(HOLEFLAG, drill->Pin[i]))
+                                	Device->Drill(drill->Pin[i], index);
                 );
                         /* close the file */
                 ClosePrintFile();
@@ -475,49 +555,34 @@ static int PrintDrill(int Gerber2Test)
  * prints solder and component side mask
  * returns != zero on error
  */
-static int PrintMask(int Gerber2Test)
+static int PrintMask(void)
 {
 	static	char	*extention[2] = { "componentmask", "soldermask" },
-			*DOSextention[2] = { "cmsk", "smsk" },
+			*DOSextention[2] = { "cmask", "smask" },
 			*description[2] = {
 				"solder mask component side",
 				"solder mask solder side" };
 	int		i;
-	BoxTypePtr	box;
 
-	/* check capability */
-	if (Device->HandlesMask) {
-		box = GetDataBoundingBox(PCB->Data);
+	/* loop over both sides, start with component */
+	for (i = 0; i < 2; i++) {
+		/* start with the component side */
+		if (SetupPrintFile(GlobalDOSFlag ? DOSextention[i] : extention[i],
+			description[i]))
+			return(1);
 
-		/* loop over both sides, start with component */
-		for (i = 0; i < 2; i++) {
-			/* start with the component side */
-			if (SetupPrintFile(GlobalDOSFlag ? DOSextention[i] : extention[i],
-				description[i]))
-				return(1);
+			/* solder masks are negative */
+		Device->Polarity(1);
+		SetPrintColor(PCB->PinColor);
+		if (GlobalAlignmentFlag)
+			FPrintAlignment();
+		ALLPAD_LOOP(PCB->Data,
+			if ((TEST_FLAG(ONSOLDERFLAG, pad) == 0) == (i == 0))
+				Device->Pad(pad, 2););
+		ALLPIN_LOOP(PCB->Data, Device->PinOrVia(pin, 2););
+		VIA_LOOP(PCB->Data, Device->PinOrVia(via, 2););
 
-			if (Gerber2Test == 1) {
-				Device->FilledRectangle(box->X1, box->Y1, box->X2, box->Y2, 0);
-				SetPrintColor(PCB->PinColor, 0);
-				ALLPAD_LOOP(PCB->Data,
-					if ((TEST_FLAG(ONSOLDERFLAG, pad) == 0) == (i == 0))
-						Device->PadMask(pad, 0););
-				ALLPIN_LOOP(PCB->Data, Device->PinOrViaMask(pin, 0););
-				VIA_LOOP(PCB->Data, Device->PinOrViaMask(via, 0););
-			}
-
-			if (GlobalAlignmentFlag)
-				FPrintAlignment(1);
-			Device->FilledRectangle(box->X1, box->Y1, box->X2, box->Y2, 1);
-			SetPrintColor(PCB->PinColor, 1);
-			ALLPAD_LOOP(PCB->Data,
-				if ((TEST_FLAG(ONSOLDERFLAG, pad) == 0) == (i == 0))
-					Device->PadMask(pad, 1););
-			ALLPIN_LOOP(PCB->Data, Device->PinOrViaMask(pin, 1););
-			VIA_LOOP(PCB->Data, Device->PinOrViaMask(via, 1););
-
-			ClosePrintFile();
-		}
+		ClosePrintFile();
 	}
 	return(0);
 }
@@ -584,37 +649,17 @@ int Print(char *Command, float Scale,
 		DeviceFlags.BoundingBox = *GetDataBoundingBox(PCB->Data);
 
 	if (AlignmentFlag) {
-		DeviceFlags.BoundingBox.X1 -= Settings.AlignmentDistance;
-		DeviceFlags.BoundingBox.Y1 -= Settings.AlignmentDistance;
-		DeviceFlags.BoundingBox.X2 += Settings.AlignmentDistance;
-		DeviceFlags.BoundingBox.Y2 += Settings.AlignmentDistance;
-	}
-
-	/* removes any and all default.map's in a directory */
-	/* that a non-gerberX file is being generated. */
-	if (Device->NeedsAppMap) {
-		unlink("custom.app");
-		unlink("default.map");
+		DeviceFlags.BoundingBox.X1 = -2 * Settings.AlignmentDistance;
+		DeviceFlags.BoundingBox.Y1 = -2 * Settings.AlignmentDistance;
+		DeviceFlags.BoundingBox.X2 = PCB->MaxWidth  + 2 * Settings.AlignmentDistance;
+		DeviceFlags.BoundingBox.Y2 = PCB->MaxHeight + 2 * Settings.AlignmentDistance;
 	}
 
 	Device->init(&DeviceFlags);
-	/* OK, call all necessary subroutines, gerberX */
-	if (Device->NeedsPrePrint) {
-		unlink("gerberX.app");
-		if (PrintLayergroups(SilkscreenTextFlag, 1) ||
-	                PrintSilkscreen(SilkscreenTextFlag, 1) ||
-	                PrintDrill(1) ||
-	                PrintMask(1))
+	/* OK, call all necessary subroutines */
+	if (PrintLayergroups() || PrintSilkscreen() ||
+		PrintDrill() || PrintMask() || PrintPaste())
 			return(1);
-		unlink("gerberX.app");
-	} else {
-		/* OK, call all necessary subroutines */
-		if (PrintLayergroups(SilkscreenTextFlag, 0) ||
-			PrintSilkscreen(SilkscreenTextFlag, 0) ||
-			PrintDrill(0) ||
-			PrintMask(0))
-			return(1);
-	}
 	Device->Exit();
 	return(0);
 }
