@@ -35,10 +35,6 @@ static	char	*rcsid = "$Id: printdialog.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 #include "data.h"
 #include "dev_ps.h"
-/*
-  #include "dev_gerber.h"
-  #include "dev_gerberX.h"
-*/
 #include "dev_rs274x.h"
 #include "dialog.h"
 #include "menu.h"
@@ -161,11 +157,7 @@ static void CB_Device(Widget W, XtPointer ClientData, XtPointer CallData)
 		XtVaSetValues(ColorW, XtNstate, False, NULL);
 	XtVaSetValues(ColorW, XtNsensitive, DeviceSelection->HandlesColor, NULL);
 
-		/* switch off invert button if not supported */
-	if (!DeviceSelection->HandlesInvertion)
-		XtVaSetValues(InvertW, XtNstate, False, NULL);
-	XtVaSetValues(InvertW, XtNsensitive,
-		DeviceSelection->HandlesInvertion, NULL);
+	XtVaSetValues(InvertW, XtNsensitive, True, NULL);
 
 		/* switch off rotate button if not supported */
 	if (!DeviceSelection->HandlesRotate)
@@ -382,7 +374,7 @@ void PrintDialog(void)
 		LAYOUT_TOP,
 		XtNfromVert, ColorW,
 		XtNstate, invertflag,
-		XtNsensitive, DeviceSelection->HandlesInvertion,
+		XtNsensitive, True,
 		NULL);
 	label = XtVaCreateManagedWidget("comment", labelWidgetClass,
 		masterform,
@@ -497,7 +489,7 @@ void PrintDialog(void)
 		/* update menu button according to old settings
 		 * thanks to: John M. Harrison <jmh@nei.mv.com>
 		 */
-	CB_Device(device, DeviceSelection, NULL);
+	CB_Device(device, (XtPointer) DeviceSelection, NULL);
 
 		/* now display dialog window and handle input */
 	StartDialog(Popup);
