@@ -36,7 +36,6 @@
 #include "global.h"
 
 Boolean AddAllRats(Boolean);
-void	ReadNetlist(char *);
-void	FreeNetlist(void);
+Boolean SeekPad(LibraryEntryTypePtr, ConnectionTypePtr);
 
 #endif
