@@ -51,12 +51,14 @@ static	char	*rcsid = "$Id: remove.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
  * some local prototypes
  */
 static	void	*DestroyVia(PinTypePtr);
+static	void	*DestroyRat(RatTypePtr);
 static	void	*DestroyLine(LayerTypePtr, LineTypePtr);
 static	void	*DestroyArc(LayerTypePtr, ArcTypePtr);
 static	void	*DestroyText(LayerTypePtr, TextTypePtr);
 static	void	*DestroyPolygon(LayerTypePtr, PolygonTypePtr);
 static	void	*DestroyElement(ElementTypePtr);
 static	void	*RemoveVia(PinTypePtr);
+static	void	*RemoveRat(RatTypePtr);
 static	void	*DestroyPolygonPoint(LayerTypePtr, PolygonTypePtr,
 					PointTypePtr);
 static	void	*RemovePolygonPoint(LayerTypePtr, PolygonTypePtr,
@@ -78,7 +80,8 @@ static	ObjectFunctionType	RemoveFunctions = {
 	NULL,
 	RemoveLinePoint,
 	RemovePolygonPoint,
-	RemoveArc };
+	RemoveArc,
+	RemoveRat };
 static	ObjectFunctionType	DestroyFunctions = {
 	DestroyLine,
 	DestroyText,
@@ -90,7 +93,8 @@ static	ObjectFunctionType	DestroyFunctions = {
 	NULL,
 	NULL,
 	DestroyPolygonPoint,
-	DestroyArc };
+	DestroyArc,
+	DestroyRat };
 static	DataTypePtr	DestroyTarget;
 static	Boolean	Bulk = False;
 
@@ -109,9 +113,6 @@ void RemovePCB(PCBTypePtr Ptr)
  */
 static void *DestroyVia(PinTypePtr Via)
 {
-		/* erase from screen and memory */
-	if ((DestroyTarget == PCB->Data) && (PCB->ViaOn))
-		EraseVia(Via);
 	*Via = DestroyTarget->Via[--DestroyTarget->ViaN];
 	memset(&DestroyTarget->Via[DestroyTarget->ViaN], 0, sizeof(PinType));
 	SaveFree(Via->Name);
@@ -123,9 +124,6 @@ static void *DestroyVia(PinTypePtr Via)
  */
 static void *DestroyLine(LayerTypePtr Layer, LineTypePtr Line)
 {
-		/* erase from screen */
-	if ((DestroyTarget == PCB->Data) && (Layer->On))
-		EraseLine(Line);
 	*Line = Layer->Line[--Layer->LineN];
 	memset(&Layer->Line[Layer->LineN], 0, sizeof(LineType));
 	return(NULL);
@@ -136,9 +134,6 @@ static void *DestroyLine(LayerTypePtr Layer, LineTypePtr Line)
  */
 static void *DestroyArc(LayerTypePtr Layer, ArcTypePtr Arc)
 {
-		/* erase from screen */
-	if ((DestroyTarget == PCB->Data) && (Layer->On))
-		EraseArc(Arc);
 	*Arc = Layer->Arc[--Layer->ArcN];
 	memset(&Layer->Arc[Layer->ArcN], 0, sizeof(ArcType));
 	return(NULL);
@@ -149,9 +144,6 @@ static void *DestroyArc(LayerTypePtr Layer, ArcTypePtr Arc)
  */
 static void *DestroyPolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 {
-		/* erase from screen */
-	if ((DestroyTarget == PCB->Data) && (Layer->On))
-		ErasePolygon(Polygon);
 	FreePolygonMemory(Polygon);
 	*Polygon = Layer->Polygon[--Layer->PolygonN];
 	memset(&Layer->Polygon[Layer->PolygonN], 0, sizeof(PolygonType));
@@ -172,9 +164,6 @@ static void *DestroyPolygonPoint(LayerTypePtr Layer,
  */
 static void *DestroyText(LayerTypePtr Layer, TextTypePtr Text)
 {
-		/* erase from screen */
-	if ((DestroyTarget == PCB->Data) && (Layer->On))
-		EraseText(Text);
 	SaveFree(Text->TextString);
 	*Text = Layer->Text[--Layer->TextN];
 	memset(&Layer->Text[Layer->TextN], 0, sizeof(TextType));
@@ -186,17 +175,23 @@ static void *DestroyText(LayerTypePtr Layer, TextTypePtr Text)
  */
 static void *DestroyElement(ElementTypePtr Element)
 {
-		/* erase from screen */
-	if ((DestroyTarget == PCB->Data) && (PCB->ElementOn || PCB->PinOn) &&
-		   ((TEST_FLAG(ONSOLDERFLAG, Element) != 0) == SWAP_IDENT ||
-		   PCB->InvisibleObjectsOn))
-		EraseElement(Element);
 	FreeElementMemory(Element);
 	*Element = DestroyTarget->Element[--DestroyTarget->ElementN];
 	memset(&DestroyTarget->Element[DestroyTarget->ElementN], 0, sizeof(ElementType));
 	return(NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * destroys a rat
+ */
+static void *DestroyRat(RatTypePtr Rat)
+{
+	*Rat = DestroyTarget->Rat[--DestroyTarget->RatN];
+	memset(&DestroyTarget->Rat[DestroyTarget->RatN], 0, sizeof(RatType));
+	return(NULL);
+}
+
+
 /* ---------------------------------------------------------------------------
  * removes a via
  */
@@ -207,15 +202,32 @@ static void *RemoveVia(PinTypePtr Via)
 	{
 		EraseVia(Via);
 		if (!Bulk)
-	        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	MoveObjectToRemoveUndoList(VIA_TYPE, Via, Via, Via);
 	return(NULL);
 }
+
+/* ---------------------------------------------------------------------------
+ * removes a rat
+ */
+static void *RemoveRat(RatTypePtr Rat)
+{
+		/* erase from screen and memory */
+	if (PCB->RatOn)
+	{
+		EraseRat(Rat);
+		if (!Bulk)
+			Draw();
+	}
+	MoveObjectToRemoveUndoList(RATLINE_TYPE, Rat, Rat, Rat);
+	return(NULL);
+}
+
 /* ---------------------------------------------------------------------------
  * removes a line point 
  */
-void *RemoveLinePoint(LayerTypePtr Layer, LineTypePtr Line, PointTypePtr Point)
+static void *RemoveLinePoint(LayerTypePtr Layer, LineTypePtr Line, PointTypePtr Point)
 {
 	PointType	oldPoint;
 
@@ -256,7 +268,7 @@ void *RemoveLine(LayerTypePtr Layer, LineTypePtr Line)
 	{
 		EraseLine(Line);
 		if (!Bulk)
-	        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	MoveObjectToRemoveUndoList(LINE_TYPE, Layer, Line, Line);
 	return(NULL);
@@ -272,7 +284,7 @@ void *RemoveArc(LayerTypePtr Layer, ArcTypePtr Arc)
 	{
 		EraseArc(Arc);
 		if (!Bulk)
-	        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	MoveObjectToRemoveUndoList(ARC_TYPE, Layer, Arc, Arc);
 	return(NULL);
@@ -288,7 +300,7 @@ void *RemoveText(LayerTypePtr Layer, TextTypePtr Text)
 	{
 		EraseText(Text);
 		if (!Bulk)
-	        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	MoveObjectToRemoveUndoList(TEXT_TYPE, Layer, Text, Text);
 	return(NULL);
@@ -304,10 +316,10 @@ void *RemovePolygon(LayerTypePtr Layer, PolygonTypePtr Polygon)
 	{
 		ErasePolygon(Polygon);
 		if (!Bulk)
-        		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	MoveObjectToRemoveUndoList(POLYGON_TYPE, Layer, Polygon, Polygon);
-	UpdatePIPFlags(NULL, NULL, Layer, True);
+	UpdatePIPFlags(NULL, NULL, Layer, NULL, True);
 	return(NULL);
 }
 
@@ -318,7 +330,7 @@ static void *RemovePolygonPoint(LayerTypePtr Layer,
 	PolygonTypePtr Polygon, PointTypePtr Point)
 {
 	PointTypePtr	ptr;
-	Cardinal	index;
+	Cardinal	index = 0;
 
 		/* insert the polygon-point into the undo list */
 	POLYGONPOINT_LOOP(Polygon,
@@ -340,33 +352,32 @@ static void *RemovePolygonPoint(LayerTypePtr Layer,
 	}
 	Polygon->PointN--;
 	SetPolygonBoundingBox(Polygon);
+	RemoveExcessPolygonPoints(Layer, Polygon);
 
 		/* redraw polygon if necessary */
 	if (Layer->On)
 	{
 		DrawPolygon(Layer, Polygon, 0);
 		if (!Bulk)
-	        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	return(NULL);
 }
 
 /* ---------------------------------------------------------------------------
- * removes a element
+ * removes an element
  */
 void *RemoveElement(ElementTypePtr Element)
 {
 		/* erase from screen */
 	if ((PCB->ElementOn || PCB->PinOn) &&
-		((TEST_FLAG(ONSOLDERFLAG, Element) != 0) == SWAP_IDENT ||
-		PCB->InvisibleObjectsOn))
+		(FRONT(Element) || PCB->InvisibleObjectsOn))
 	{
 		EraseElement(Element);
 		if (!Bulk)
-	        	XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+			Draw();
 	}
 	MoveObjectToRemoveUndoList(ELEMENT_TYPE, Element, Element, Element);
-	FreeNetlist();
 	return(NULL);
 }
 
@@ -380,7 +391,7 @@ Boolean RemoveSelected(void)
 	if (SelectedOperation(&RemoveFunctions, False))
 	{
 		IncrementUndoSerialNumber();
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		Bulk = False;
 		return(True);
 	}
@@ -410,20 +421,17 @@ Boolean DeleteRats(Boolean selected)
  	Boolean changed = False;
  	
 	Bulk = True;
- 	ALLLINE_LOOP(PCB->Data,
- 	{
- 		if (TEST_FLAG(RATFLAG, line))
- 			if ((!selected) || TEST_FLAG(SELECTEDFLAG, line))
- 			{
- 				changed = True;
- 				RemoveLine(layer, line);
- 			}
- 		
-	});
+ 	RAT_LOOP(PCB->Data,
+ 		if ((!selected) || TEST_FLAG(SELECTEDFLAG, line))
+ 		{
+ 			changed = True;
+ 			RemoveRat(line);
+ 		}
+	);
 	Bulk = False;
 	if (changed)
 	{
-		XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(changed);
@@ -431,7 +439,7 @@ Boolean DeleteRats(Boolean selected)
 
 /* ---------------------------------------------------------------------------
  * remove object as referred by pointers and type
- * allocated memory is destroyed
+ * allocated memory is destroyed assumed to already be erased
  */
 void *DestroyObject(DataTypePtr Target, int Type, void *Ptr1, void *Ptr2,
 	void *Ptr3)
