@@ -69,7 +69,8 @@ static	ObjectFunctionType	RotateFunctions = {
 	NULL,
 	RotateLinePoint,
 	NULL,
-	RotateArc };
+	RotateArc,
+	NULL };
 
 /* ---------------------------------------------------------------------------
  * rotates a point in 90 degree steps
@@ -98,15 +99,15 @@ void RotateTextLowLevel(TextTypePtr Text, Position X, Position Y, BYTE Number)
 {
 	BYTE	number;
 
-	number = TEST_FLAG(ONSOLDERFLAG, Text) ? (4 -Number) & 0x03 : Number;
-	RotateBoxLowLevel(&Text->BoundingBox, X, Y, number);
-	ROTATE(Text->X, Text->Y, X, Y, number);
+	number = TEST_FLAG(ONSOLDERFLAG, Text) ? (4 - Number) & 3 : Number;
+	RotateBoxLowLevel(&Text->BoundingBox, X, Y, Number);
+	ROTATE(Text->X, Text->Y, X, Y, Number);
 
 		/* set new direction, 0..3,
 		 * 0-> to the right, 1-> straight up,
 		 * 2-> to the left, 3-> straight down
 		 */
-	Text->Direction = ((Text->Direction +Number) & 0x03);
+	Text->Direction = ((Text->Direction + number) & 0x03);
 }
 
 /* ---------------------------------------------------------------------------
@@ -129,7 +130,7 @@ static void *RotateText(LayerTypePtr Layer, TextTypePtr Text)
 	EraseText(Text);
 	RotateTextLowLevel(Text, CenterX, CenterY, Number);
 	DrawText(Layer, Text, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(Text);
 }
 
@@ -163,24 +164,28 @@ void RotateElementLowLevel(ElementTypePtr Element,
 	Position X, Position Y,
 	BYTE Number)
 {
-	BYTE	number;
+	Boolean OnLayout = False;
 
+	ELEMENT_LOOP(PCB->Data,
+		if (element == Element)
+			OnLayout = True;
+	);
 		/* solder side objects need a different orientation */
-	number = TEST_FLAG(ONSOLDERFLAG, Element) ? (4 -Number) & 0x03 : Number;
 
 		/* the text subroutine decides by itself if the direction
 		 * is to be corrected
 		 */
 	ELEMENTTEXT_LOOP(Element, RotateTextLowLevel(text, X, Y, Number));
-	ELEMENTLINE_LOOP(Element, RotateLineLowLevel(line, X, Y, number););
+	ELEMENTLINE_LOOP(Element, RotateLineLowLevel(line, X, Y, Number););
 	PIN_LOOP(Element, {
-		ROTATE_PIN_LOWLEVEL(pin, X, Y, number);
-		UpdatePIPFlags(pin, Element, NULL, True);
+		ROTATE_PIN_LOWLEVEL(pin, X, Y, Number);
+		if (OnLayout)
+			UpdatePIPFlags(pin, Element, NULL, NULL, True);
 	});
-	PAD_LOOP(Element, ROTATE_PAD_LOWLEVEL(pad, X, Y, number););
-	ARC_LOOP(Element, RotateArcLowLevel(arc, X, Y, number););
-	ROTATE(Element->MarkX, Element->MarkY, X, Y, number);
-	RotateBoxLowLevel(&Element->BoundingBox, X, Y, number);
+	PAD_LOOP(Element, ROTATE_PAD_LOWLEVEL(pad, X, Y, Number););
+	ARC_LOOP(Element, RotateArcLowLevel(arc, X, Y, Number););
+	ROTATE(Element->MarkX, Element->MarkY, X, Y, Number);
+	RotateBoxLowLevel(&Element->BoundingBox, X, Y, Number);
 }
 
 /* ---------------------------------------------------------------------------
@@ -191,7 +196,7 @@ static void *RotateLinePoint(LayerTypePtr Layer, LineTypePtr Line, PointTypePtr
 	EraseLine(Line);
 	RotatePointLowLevel(Point, CenterX, CenterY, Number);
 	DrawLine(Layer, Line, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(Line);
 }
 
@@ -203,7 +208,7 @@ static void *RotateArc(LayerTypePtr Layer, ArcTypePtr Arc)
 	EraseArc(Arc);
 	RotateArcLowLevel(Arc, CenterX, CenterY, Number);
 	DrawArc(Layer, Arc, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(Arc);
 }
 
@@ -215,7 +220,7 @@ static void *RotateElement(ElementTypePtr Element)
 	EraseElement(Element);
 	RotateElementLowLevel(Element, CenterX, CenterY, Number);
 	DrawElement(Element, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(Element);
 }
 
@@ -227,7 +232,7 @@ static void *RotateElementName(ElementTypePtr Element)
 	EraseElementName(Element);
 	ELEMENTTEXT_LOOP(Element, RotateTextLowLevel(text,CenterX,CenterY,Number));
 	DrawElementName(Element, 0);
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	return(Element);
 }
 
@@ -280,13 +285,12 @@ void *RotateObject(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
                 Crosshair.AttachedObject.RubberbandN--;
                 ptr++;
 	}
-
-	AddObjectToRotateUndoList(Type, Ptr1, Ptr2, Ptr3, CenterX, CenterY, Steps);
+	AddObjectToRotateUndoList(Type, Ptr1, Ptr2, Ptr3, CenterX, CenterY, Number);
         ptr2 = ObjectOperation(&RotateFunctions, Type, Ptr1, Ptr2, Ptr3);
 	changed |= (ptr2 != NULL);
 	if (changed)
 	{
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 		IncrementUndoSerialNumber();
 	}
 	return(ptr2);
