@@ -52,6 +52,7 @@ static	void	CheckLinePointForRubberbandConnection(LayerTypePtr,
 					LineTypePtr, PointTypePtr);
 static	void	CheckPolygonForRubberbandConnection(LayerTypePtr,
 					PolygonTypePtr);
+static void	CheckLinePointForRat(LayerTypePtr, PointTypePtr);
 
 /* ---------------------------------------------------------------------------
  * checks all visible lines which belong to the same layergroup as the
@@ -90,10 +91,9 @@ static void CheckPadForRubberbandConnection(PadTypePtr Pad)
 				if (line->Point1.X >= minx && line->Point1.X <= maxx &&
 					line->Point1.Y >= miny && line->Point1.Y <= maxy)
 					CreateNewRubberbandEntry(layer, line, &line->Point1);
-				else
-					if (line->Point2.X >= minx && line->Point2.X <= maxx &&
-						line->Point2.Y >= miny && line->Point2.Y <= maxy)
-						CreateNewRubberbandEntry(layer, line, &line->Point2);
+				else if (line->Point2.X >= minx && line->Point2.X <= maxx &&
+					line->Point2.Y >= miny && line->Point2.Y <= maxy)
+					CreateNewRubberbandEntry(layer, line, &line->Point2);
 			);
 		}
 	}
@@ -108,31 +108,17 @@ static void CheckPadForRat(PadTypePtr Pad)
 		(TEST_FLAG(ONSOLDERFLAG, Pad) ? SOLDER_LAYER : COMPONENT_LAYER);
 	group = GetLayerGroupNumberByNumber(i);
 
-		/* check all visible layers in the same group */
-	for (i = 0; i < PCB->LayerGroups.Number[group]; i++)
-	{
-		Cardinal	number = PCB->LayerGroups.Entries[group][i];
-		LayerTypePtr	layer;
-
-			/* skip solder/component layers */
-		if (number >= MAX_LAYER)
-			continue;
-
-			/* check all lines of the group member whether visible or not */
-		layer = &PCB->Data->Layer[number];
-		LINE_LOOP(layer,
-			if (TEST_FLAG(RATFLAG, line))
-			{
-				if (line->Point1.X == Pad->Point1.X &&
-					line->Point1.Y == Pad->Point1.Y)
-					CreateNewRubberbandEntry(layer, line, &line->Point1);
-				else
-					if (line->Point2.X == Pad->Point1.X &&
-						line->Point2.Y == Pad->Point1.Y)
-						CreateNewRubberbandEntry(layer, line, &line->Point2);
-			}
-		);
-	}
+	RAT_LOOP(PCB->Data,
+		if (line->Point1.X == Pad->Point1.X &&
+		    line->Point1.Y == Pad->Point1.Y && line->group1 == group)
+			CreateNewRubberbandEntry(NULL, (LineTypePtr) line, &line->Point1);
+		else
+			if (line->Point2.X == Pad->Point1.X &&
+				line->Point2.Y == Pad->Point1.Y &&
+			    line->group2 == group)
+				CreateNewRubberbandEntry(NULL, (LineTypePtr) line, &line->Point2);
+		
+	);
 }
 
 /* ---------------------------------------------------------------------------
@@ -149,40 +135,50 @@ static void CheckPinForRubberbandConnection(PinTypePtr Pin)
 			dx, dy;
 
 	VISIBLELINE_LOOP(PCB->Data,
-		if (!TEST_FLAG(RATFLAG, line))
+			/* save sqrt computation */
+		radius = (float) (Pin->Thickness +line->Thickness) /2.0;
+		radius *= radius;
+		dx = line->Point1.X -Pin->X;
+		dy = line->Point1.Y -Pin->Y;
+		if (dx*dx+dy*dy <= radius)
+			CreateNewRubberbandEntry(layer, line, &line->Point1);
+		else
 		{
-				/* save sqrt computation */
-			radius = (float) (Pin->Thickness +line->Thickness) /2.0;
-			radius *= radius;
-			dx = line->Point1.X -Pin->X;
-			dy = line->Point1.Y -Pin->Y;
+			dx = line->Point2.X -Pin->X;
+			dy = line->Point2.Y -Pin->Y;
 			if (dx*dx+dy*dy <= radius)
-				CreateNewRubberbandEntry(layer, line, &line->Point1);
-			else
-			{
-				dx = line->Point2.X -Pin->X;
-				dy = line->Point2.Y -Pin->Y;
-				if (dx*dx+dy*dy <= radius)
-					CreateNewRubberbandEntry(layer, line, &line->Point2);
-			}
+				CreateNewRubberbandEntry(layer, line, &line->Point2);
 		}
 	);
 }
 
 static void CheckPinForRat(PinTypePtr Pin)
 {
-	ALLLINE_LOOP(PCB->Data,
-		if (TEST_FLAG(RATFLAG, line))
-		{
-			if (line->Point1.X == Pin->X && line->Point1.Y == Pin->Y)
-				CreateNewRubberbandEntry(layer, line, &line->Point1);
-			else
-				if (line->Point2.X == Pin->X && line->Point2.Y == Pin->Y)
-					CreateNewRubberbandEntry(layer, line, &line->Point2);
-		}
+	RAT_LOOP(PCB->Data,
+		if (line->Point1.X == Pin->X && line->Point1.Y == Pin->Y)
+			CreateNewRubberbandEntry(NULL, (LineTypePtr) line, &line->Point1);
+		else
+			if (line->Point2.X == Pin->X && line->Point2.Y == Pin->Y)
+				CreateNewRubberbandEntry(NULL, (LineTypePtr) line, &line->Point2);
 	);
 }
 
+static void CheckLinePointForRat(LayerTypePtr Layer, PointTypePtr Point)
+{
+	Cardinal	group = GetLayerGroupNumberByPointer(Layer);
+
+	RAT_LOOP(PCB->Data,
+		if (line->group1 == group &&
+		    line->Point1.X == Point->X && line->Point1.Y == Point->Y)
+			CreateNewRubberbandEntry(NULL, (LineTypePtr) line, &line->Point1);
+		else
+			if (line->group2 == group &&
+		    	    line->Point2.X == Point->X && line->Point2.Y == Point->Y)
+				CreateNewRubberbandEntry(NULL, (LineTypePtr) line, &line->Point2);
+	);
+}
+
+
 
 /* ---------------------------------------------------------------------------
  * checks all visible lines which belong to the same group as the passed line.
@@ -213,7 +209,7 @@ static void CheckLinePointForRubberbandConnection(LayerTypePtr Layer,
 			register	float	radius,
 						dx, dy;
 
-				/* the following code just stupidly compares the endpoints
+				/* the following code just compares the endpoints
 				 * of the lines
 				 */
 			LINE_LOOP(layer,
@@ -275,6 +271,8 @@ static void CheckPolygonForRubberbandConnection(LayerTypePtr Layer,
 				 * of the lines
 				 */
 			LINE_LOOP(layer,
+				if (TEST_FLAG(CLEARLINEFLAG, line))
+					continue;
 				thick = (line->Thickness+1)/2;
 				if (IsPointInPolygon(line->Point1.X, line->Point1.Y,
 					thick, Polygon))
@@ -311,15 +309,11 @@ void LookupRubberbandLines(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 				 * of faiures that are immediately recognized
 				 */
 			PIN_LOOP(element,
-				{
 				CheckPinForRubberbandConnection(pin);
-				CheckPinForRat(pin);
-				});
+			);
 			PAD_LOOP(element,
-				{
 				CheckPadForRubberbandConnection(pad);
-				CheckPadForRat(pad);
-				});
+			);
 			break;
 		}
 
@@ -327,27 +321,64 @@ void LookupRubberbandLines(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 		{
 			LayerTypePtr	layer = (LayerTypePtr) Ptr1;
 			LineTypePtr	line = (LineTypePtr) Ptr2;
-			CheckLinePointForRubberbandConnection(layer, line, &line->Point1);
-			CheckLinePointForRubberbandConnection(layer, line, &line->Point2);
+			if (GetLayerNumber(PCB->Data, layer) < MAX_LAYER)
+			{
+				CheckLinePointForRubberbandConnection(layer, line, &line->Point1);
+				CheckLinePointForRubberbandConnection(layer, line, &line->Point2);
+			}
 			break;
 		}
 
 		case LINEPOINT_TYPE:
-			CheckLinePointForRubberbandConnection((LayerTypePtr) Ptr1,
-				(LineTypePtr) Ptr2, (PointTypePtr) Ptr3);
+			if (GetLayerNumber(PCB->Data, (LayerTypePtr) Ptr1) < MAX_LAYER)
+				CheckLinePointForRubberbandConnection((LayerTypePtr) Ptr1,
+					(LineTypePtr) Ptr2, (PointTypePtr) Ptr3);
+			break;
+
+		case VIA_TYPE:
+			CheckPinForRubberbandConnection((PinTypePtr) Ptr1);
 			break;
 
 		case POLYGON_TYPE:
-			CheckPolygonForRubberbandConnection((LayerTypePtr) Ptr1,
-				(PolygonTypePtr) Ptr2);
+			if (GetLayerNumber(PCB->Data, (LayerTypePtr) Ptr1) < MAX_LAYER)
+				CheckPolygonForRubberbandConnection((LayerTypePtr) Ptr1,
+					(PolygonTypePtr) Ptr2);
 			break;
 	}
 }
 
-void LookupRatLines(ElementTypePtr element)
+void LookupRatLines(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 {
+	switch(Type)
+	{
+		case ELEMENT_TYPE:
+		{
+			ElementTypePtr	element = (ElementTypePtr) Ptr1;
+
 			PIN_LOOP(element,
 				CheckPinForRat(pin););
 			PAD_LOOP(element,
 				CheckPadForRat(pad););
+			break;
+		}
+
+		case LINE_TYPE:
+		{
+			LayerTypePtr	layer = (LayerTypePtr) Ptr1;
+			LineTypePtr	line = (LineTypePtr) Ptr2;
+
+			CheckLinePointForRat(layer, &line->Point1);
+			CheckLinePointForRat(layer, &line->Point2);
+			break;
+		}
+
+		case LINEPOINT_TYPE:
+			CheckLinePointForRat((LayerTypePtr) Ptr1,
+				(PointTypePtr) Ptr3);
+			break;
+
+		case VIA_TYPE:
+			CheckPinForRat((PinTypePtr) Ptr1);
+			break;
+	}
 }
