@@ -39,6 +39,7 @@ static	char	*rcsid = "$Id: search.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 
 #include "data.h"
 #include "draw.h"
+#include "error.h"
 #include "find.h"
 #include "misc.h"
 #include "search.h"
@@ -50,6 +51,7 @@ static	Position		PosX,		/* search position for subroutines */
 				PosY;
 static	Dimension		SearchRadius;
 static	LayerTypePtr	SearchLayer;
+static  Boolean		ScreenOnly = False;
 
 /* ---------------------------------------------------------------------------
  * some local prototypes
@@ -58,8 +60,8 @@ static	Boolean	SearchLineByPosition(LayerTypePtr *, LineTypePtr *,
 					LineTypePtr *);
 static	Boolean	SearchArcByPosition(LayerTypePtr *, ArcTypePtr *,
 					ArcTypePtr *);
-static	Boolean	SearchRatLineByPosition(LayerTypePtr *, LineTypePtr *,
-					LineTypePtr *);
+static	Boolean	SearchRatLineByPosition(RatTypePtr *, RatTypePtr *,
+					RatTypePtr *);
 static	Boolean	SearchTextByPosition(LayerTypePtr *, TextTypePtr *,
 					TextTypePtr *);
 static	Boolean	SearchPolygonByPosition(LayerTypePtr *, PolygonTypePtr *,
@@ -77,6 +79,7 @@ static	Boolean	SearchPointByPosition(LayerTypePtr *, PolygonTypePtr *,
 					PointTypePtr *);
 static Boolean SearchElementByPosition(ElementTypePtr *, ElementTypePtr *,
 					 ElementTypePtr *, Boolean);
+static Boolean IsPointInBox(Position, Position, BoxTypePtr, Cardinal);
 
 /* ---------------------------------------------------------------------------
  * searches a via
@@ -87,8 +90,26 @@ static Boolean SearchViaByPosition(PinTypePtr *Via, PinTypePtr *Dummy1,
 		/* search only if via-layer is visible */
 	if (PCB->ViaOn)
 		VIA_LOOP(PCB->Data,
-			if (abs(via->X - PosX) <= via->Thickness/SearchRange + SearchRadius &&
-		    	abs(via->Y - PosY) <= via->Thickness/SearchRange + SearchRadius)
+			if (ScreenOnly && !VVIA(via))
+				continue;
+			if (TEST_FLAG(SQUAREFLAG, via))
+			{
+				BoxType b;
+
+				b.X1 = via->X - via->Thickness/SearchRange;
+				b.X2 = via->X + via->Thickness/SearchRange;
+				b.Y1 = via->Y - via->Thickness/SearchRange;
+				b.Y2 = via->Y + via->Thickness/SearchRange;
+				if (IsPointInBox(PosX, PosY, &b, SearchRadius))
+				{
+					*Via = *Dummy1 = *Dummy2 = via;
+					return(True);
+				}
+			}
+			else if ((via->X - PosX)*(via->X - PosX) +
+			(via->Y - PosY)*(via->Y - PosY) <=
+			(via->Thickness/SearchRange + SearchRadius)
+			*(via->Thickness/SearchRange + SearchRadius))
 			{
 				*Via = *Dummy1 = *Dummy2 = via;
 				return(True);
@@ -107,11 +128,29 @@ static Boolean SearchPinByPosition(ElementTypePtr *Element, PinTypePtr *Pin,
 		/* search only if pin-layer is visible */
 	if (PCB->PinOn)
 		ELEMENT_LOOP(PCB->Data,
+			if (ScreenOnly && !VELEMENT(element))
+				continue;
 			PIN_LOOP(element,
-				if (abs(pin->X - PosX) <= pin->Thickness/SearchRange  + SearchRadius &&
-					abs(pin->Y - PosY) <= pin->Thickness/SearchRange + SearchRadius )
+				if (TEST_FLAG(SQUAREFLAG, pin))
+				{
+					BoxType b;
+
+					b.X1 = pin->X - pin->Thickness/SearchRange;
+					b.X2 = pin->X + pin->Thickness/SearchRange;
+					b.Y1 = pin->Y - pin->Thickness/SearchRange;
+					b.Y2 = pin->Y + pin->Thickness/SearchRange;
+					if (IsPointInBox(PosX, PosY, &b, SearchRadius))
+					{
+						*Element = element;
+						*Pin = *Dummy = pin;
+						return(True);
+					}
+				}
+				else if ((pin->X - PosX)*(pin->X - PosX) +
+				(pin->Y - PosY)*(pin->Y - PosY) <=
+				(pin->Thickness/SearchRange + SearchRadius)
+				*(pin->Thickness/SearchRange + SearchRadius))
 				{
-						/* bug-fix by Ulrich Pegelow (ulrpeg@bigcomm.gun.de) */
 					*Element = element;
 					*Pin = *Dummy = pin;
 					return(True);
@@ -129,14 +168,14 @@ static Boolean SearchPadByPosition(ElementTypePtr *Element, PadTypePtr *Pad,
 	PadTypePtr *Dummy, Boolean BackToo)
 {
 		/* search only if pin-layer is visible */
-	if (PCB->PinOn)
-		ALLPAD_LOOP(PCB->Data,
-			if ((TEST_FLAG(ONSOLDERFLAG, pad) != 0) == SWAP_IDENT ||
-				(BackToo & PCB->InvisibleObjectsOn))
+	if (!PCB->PinOn)
+		return(False);
+	ELEMENT_LOOP(PCB->Data,
+		if (ScreenOnly && !VELEMENT(element))
+			continue;
+		PAD_LOOP(element,
+			if (FRONT(pad) || (BackToo & PCB->InvisibleObjectsOn))
 			{
-					/* the cast isn't very nice but working, check
-					 * global.h for details
-					 */
 				if (TEST_FLAG(SQUAREFLAG, pad))
 				{
 					if (IsPointInSquarePad(PosX, PosY, SearchRadius, pad))
@@ -148,6 +187,9 @@ static Boolean SearchPadByPosition(ElementTypePtr *Element, PadTypePtr *Pad,
 				}
 				else
 				{
+					/* the cast isn't very nice but working, check
+					 * global.h for details
+					 */
 					if (IsPointOnLine(PosX, PosY, SearchRadius, (LineTypePtr) pad))
 					{
 						*Element = element;
@@ -157,6 +199,7 @@ static Boolean SearchPadByPosition(ElementTypePtr *Element, PadTypePtr *Pad,
 				}
 			}
 		);
+	);
 	return(False);
 }
 
@@ -168,7 +211,9 @@ static Boolean SearchLineByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
 {
 	*Layer = SearchLayer;
 	LINE_LOOP(*Layer,
-		if (!TEST_FLAG(RATFLAG, line) && IsPointOnLine(PosX, PosY, SearchRadius, line))
+		if (ScreenOnly && !VLINE(line))
+			continue;
+		if (IsPointOnLine(PosX, PosY, SearchRadius, line))
 		{
 			*Line = *Dummy = line;
 			return(True);
@@ -178,16 +223,17 @@ static Boolean SearchLineByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
 }
 
 /* ---------------------------------------------------------------------------
- * searches rat line on the SearchLayer 
+ * searches rat lines if they are visible
  */
-static Boolean SearchRatLineByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
-	LineTypePtr *Dummy)
+static Boolean SearchRatLineByPosition(RatTypePtr *Line, RatTypePtr *Dummy1,
+		RatTypePtr *Dummy2)
 {
-	*Layer = SearchLayer;
-	LINE_LOOP(*Layer,
-		if (TEST_FLAG(RATFLAG, line) && IsPointOnLine(PosX, PosY, SearchRadius, line))
+	RAT_LOOP(PCB->Data,
+		if (ScreenOnly && !VLINE(line))
+			continue;
+		if (IsPointOnLine(PosX, PosY, SearchRadius, (LineTypePtr) line))
 		{
-			*Line = *Dummy = line;
+			*Line = *Dummy1 = *Dummy2 = line;
 			return(True);
 		}
 	);
@@ -202,6 +248,8 @@ static Boolean SearchArcByPosition(LayerTypePtr *Layer, ArcTypePtr *Arc,
 {
 	*Layer = SearchLayer;
 	ARC_LOOP(*Layer,
+		if (ScreenOnly && !VARC(arc))
+			continue;
 		if(IsPointOnArc(PosX, PosY, SearchRadius, arc))
 		{
 			*Arc = *Dummy = arc;
@@ -219,6 +267,8 @@ static Boolean SearchTextByPosition(LayerTypePtr *Layer, TextTypePtr *Text,
 {
 	*Layer = SearchLayer;
 	TEXT_LOOP(*Layer,
+		if (ScreenOnly && !VTEXT(text))
+			continue;
 		if (TEXT_IS_VISIBLE(PCB, *Layer, text) &&
 		    POINT_IN_BOX(PosX, PosY, &text->BoundingBox))
 		{
@@ -237,6 +287,8 @@ static Boolean SearchPolygonByPosition(LayerTypePtr *Layer,
 {
 	*Layer = SearchLayer;
 	POLYGON_LOOP(*Layer,
+		if (ScreenOnly && !VPOLY(polygon))
+			continue;
 		if (IsPointInPolygon(PosX, PosY, SearchRadius, polygon))
 		{
 			*Polygon = *Dummy = polygon;
@@ -258,30 +310,29 @@ static Boolean SearchLinePointByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
 	least = (MAX_LINE_POINT_DISTANCE + SearchRadius) * (MAX_LINE_POINT_DISTANCE + SearchRadius);
 	*Layer = SearchLayer;
 	LINE_LOOP(*Layer,
-		if (!TEST_FLAG(RATFLAG, line))
+		if (ScreenOnly && !VLINE(line))
+			continue;
+			/* some stupid code to check both points */
+		d = (PosX - line->Point1.X)*(PosX - line->Point1.X)+
+			(PosY - line->Point1.Y)*(PosY - line->Point1.Y);
+		if (d < least)
 		{
-				/* some stupid code to check both points */
-			d = (PosX - line->Point1.X)*(PosX - line->Point1.X)+
-				(PosY - line->Point1.Y)*(PosY - line->Point1.Y);
-			if (d < least)
-			{
-				least = d;
-				*Line = line;
-				*Point = &line->Point1;
-				found = True;
-			}
-
-			d = (PosX - line->Point2.X)*(PosX - line->Point2.X)+
-				(PosY - line->Point2.Y)*(PosY - line->Point2.Y);
-			if (d < least)
-			{
-				least = d;
-				*Line = line;
-				*Point = &line->Point2;
-				found = True;
-			}
+			least = d;
+			*Line = line;
+			*Point = &line->Point1;
+			found = True;
 		}
-	    );
+
+		d = (PosX - line->Point2.X)*(PosX - line->Point2.X)+
+			(PosY - line->Point2.Y)*(PosY - line->Point2.Y);
+		if (d < least)
+		{
+			least = d;
+			*Line = line;
+			*Point = &line->Point2;
+			found = True;
+		}
+	);
 		/* return with nearest */
 	if (found) return(True);
 	return(False);
@@ -300,6 +351,8 @@ static Boolean SearchPointByPosition(LayerTypePtr *Layer,
 	least = (SearchRadius + MAX_POLYGON_POINT_DISTANCE) * (SearchRadius + MAX_POLYGON_POINT_DISTANCE);
 	*Layer = SearchLayer;
 	POLYGON_LOOP(*Layer,
+		if (ScreenOnly && !VPOLY(polygon))
+			continue;
 		POLYGONPOINT_LOOP(polygon,
 		{
 			d = (point->X - PosX)*(point->X - PosX)+
@@ -331,8 +384,10 @@ static Boolean SearchElementNameByPosition(ElementTypePtr *Element,
 	if (PCB->ElementOn)
 	{
 		ELEMENT_LOOP(PCB->Data,
-			if (((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-				(BackToo && PCB->InvisibleObjectsOn)) && !TEST_FLAG(HIDENAMEFLAG, element))
+			if (ScreenOnly && !VELTEXT(element))
+				continue; 
+			if ((FRONT(element) || (BackToo && PCB->InvisibleObjectsOn))
+			   && !TEST_FLAG(HIDENAMEFLAG, element))
 			{
 				text = &ELEMENT_TEXT(PCB, element);
 				if (POINT_IN_BOX(PosX, PosY, &text->BoundingBox))
@@ -367,8 +422,9 @@ static Boolean SearchElementByPosition(ElementTypePtr *Element,
 			 * we have to check all of them
 			 */
 		ELEMENT_LOOP(PCB->Data,
-			if ((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-				(BackToo && PCB->InvisibleObjectsOn))
+			if (ScreenOnly && ! VELEMENT(element))
+				continue;
+			if (FRONT(element) || (BackToo && PCB->InvisibleObjectsOn))
 			{
 				found = POINT_IN_BOX(PosX, PosY, &element->BoundingBox);
 				if (!TEST_FLAG(HIDENAMEFLAG, element))
@@ -395,9 +451,9 @@ static Boolean SearchElementByPosition(ElementTypePtr *Element,
 }
 
 /* ---------------------------------------------------------------------------
- * checks if a line end is on a PV
+ * checks if a rat-line end is on a PV
  */
-Boolean IsPointOnLineEnd(Position X, Position Y, LineTypePtr Line)
+Boolean IsPointOnLineEnd(Position X, Position Y, RatTypePtr Line)
 {
 	if (((X == Line->Point1.X) && (Y == Line->Point1.Y)) ||
 	   ((X == Line->Point2.X) && (Y == Line->Point2.Y)))
@@ -447,14 +503,16 @@ Boolean IsPointOnLine(float X, float Y, float Radius, LineTypePtr Line)
 	dy = (float) (Line->Point2.Y -Line->Point1.Y);
 	dx1 = (float) (Line->Point1.X -X);
 	dy1 = (float) (Line->Point1.Y -Y);
-	if ((l = dx*dx +dy*dy) == 0.0)
-		return(False);
-
 	d = dx*dy1 -dy*dx1;
 
 		/* check distance from PV to line */
 	Radius += ((float) Line->Thickness)/2.0;
 	Radius *= Radius;
+	if ((l = dx*dx +dy*dy) == 0.0)
+	{
+		l = (Line->Point1.X - X)*(Line->Point1.X -X) + (Line->Point1.Y - Y)*(Line->Point1.Y -Y);
+		return ((l <= Radius) ? True : False);
+	}
 	if (d*d > Radius*l)
 		return(False);
 
@@ -472,7 +530,7 @@ Boolean IsPointOnLine(float X, float Y, float Radius, LineTypePtr Line)
 }
 
 /* ---------------------------------------------------------------------------
- * checks if a line crosses a square
+ * checks if a line crosses a rectangle
  */
 Boolean IsLineInRectangle(Position X1, Position Y1,
 	Position X2, Position Y2, LineTypePtr Line)
@@ -484,6 +542,11 @@ Boolean IsLineInRectangle(Position X1, Position Y1,
 				maxx = MAX(Line->Point1.X, Line->Point2.X) +thick,
 				maxy = MAX(Line->Point1.Y, Line->Point2.Y) +thick;
 
+		/* first, see if point 1 is inside the rectangle */
+		/* in case the whole line is inside the rectangle */
+	if (X1 < Line->Point1.X && X2 > Line->Point1.X &&
+	    Y1 < Line->Point1.Y && Y2 > Line->Point1.Y)
+		return(True);
 		/* construct a set of dummy lines and check each of them */
 	line.Thickness = 0;
 	line.Flags = NOFLAG;
@@ -577,16 +640,46 @@ Boolean IsArcInRectangle(Position X1, Position Y1,
 
 /* ---------------------------------------------------------------------------
  * check if a point lies inside a square Pad
+ * fixed 10/30/98 - radius is can't expand both edges of a square box
  */
 Boolean IsPointInSquarePad(Position X, Position Y, Cardinal Radius, PadTypePtr Pad)
 {
+	register	Dimension t2 = Pad->Thickness/2;
 	BoxType		padbox;
 
-	padbox.X1 = MIN(Pad->Point1.X, Pad->Point2.X) - Pad->Thickness/2 - Radius;
-	padbox.X2 = MAX(Pad->Point1.X, Pad->Point2.X) + Pad->Thickness/2 + Radius;
-	padbox.Y1 = MIN(Pad->Point1.Y, Pad->Point2.Y) - Pad->Thickness/2 - Radius;
-	padbox.Y2 = MAX(Pad->Point1.Y, Pad->Point2.Y) + Pad->Thickness/2 + Radius;
-	return(POINT_IN_BOX(X, Y, &padbox));
+	padbox.X1 = MIN(Pad->Point1.X, Pad->Point2.X) - t2;
+	padbox.X2 = MAX(Pad->Point1.X, Pad->Point2.X) + t2;
+	padbox.Y1 = MIN(Pad->Point1.Y, Pad->Point2.Y) - t2;
+	padbox.Y2 = MAX(Pad->Point1.Y, Pad->Point2.Y) + t2;
+	return(IsPointInBox(X, Y, &padbox, Radius));
+}
+
+static Boolean IsPointInBox(Position X, Position Y, BoxTypePtr box, Cardinal Radius)
+{
+	LineType line;
+
+	if (POINT_IN_BOX(X, Y, box))
+		return(True);
+	line.Thickness = 0;
+	line.Flags = NOFLAG;
+
+	line.Point1.X = box->X1;
+	line.Point1.Y = box->Y1;
+	line.Point2.X = box->X2;
+	line.Point2.Y = box->Y1;
+	if (IsPointOnLine(X, Y, Radius, &line))
+		return(True);
+	line.Point2.X = box->X1;
+	line.Point2.Y = box->Y2;
+	if (IsPointOnLine(X, Y, Radius, &line))
+		return(True);
+	line.Point1.X = box->X2;
+	line.Point1.Y = box->Y2;
+	if (IsPointOnLine(X, Y, Radius, &line))
+		return(True);
+	line.Point2.X = box->X2;
+	line.Point2.Y = box->Y1;
+	return(IsPointOnLine(X, Y, Radius, &line));
 }
 
 /* ---------------------------------------------------------------------------
@@ -674,24 +767,73 @@ Boolean IsRectangleInPolygon(Position X1, Position Y1,
 
 Boolean IsPointOnArc(float X, float Y, float Radius, ArcTypePtr Arc)
 {
-	register float r, wt, sr, x1, y1, x2, y2;
-	BoxTypePtr box;
 
-	r = (X - Arc->X)*(X - Arc->X) + (Y - Arc->Y)*(Y - Arc->Y);
-		/* sr is the search radius */
-	sr = 0.5 * Arc->Thickness + Radius;
-	wt = Arc->Width * sr * 2.0;
-	if (abs(r - (Arc->Width * Arc->Width + sr * sr)) > wt)
-		return(False);
-	box = GetArcEnds(Arc);
-	if (X >= MIN(box->X1, box->X2) && X <= MAX(box->X1, box->X2)
-		&& Y >= MIN(box->Y1, box->Y2) && Y <= MAX(box->Y1, box->Y2))
+	register float x, y, dx, dy, r1, r2, a, d, l;
+        register Position pdx, pdy;
+	int ang1, ang2, delta;
+
+        pdx = X - Arc->X;
+        pdy = Y - Arc->Y;
+        l = pdx*pdx + pdy*pdy;
+                /* concentric arcs, simpler intersection conditions */
+        if (l == 0.0) {
+                if (Arc->Width <= Radius + 0.5 * Arc->Thickness)
+			return(True);
+                else
+			return(False);
+	}
+        r1 = Arc->Width;
+        r1 *= r1;
+        r2 = Radius + 0.5 * Arc->Thickness;
+	r2 *= r2;
+        a = 0.5 * (r1 - r2 + l)/l;
+        r1 = r1/l;
+                /* add a tiny positive number for round-off problems */
+        d = r1 - a*a + 1e-5;
+                /* the circles are too far apart to touch */
+        if (d < 0)
+                return(False);
+			/* project the points of intersection */
+        d = sqrt(d);
+        x = a * pdx;
+        y = a * pdy;
+        dy = -d * pdx;
+        dx = d * pdy;
+		/* argh calculate the angles, and put them in a standard range */
+	while (Arc->StartAngle < 0)
+		Arc->StartAngle += 360;
+	if (Arc->Delta < 0)
+	{
+		Arc->StartAngle += Arc->Delta;
+		Arc->Delta = - Arc->Delta;
+		while (Arc->StartAngle < 0)
+			Arc->StartAngle += 360;
+	}
+	ang1 = RAD_TO_DEG * atan2((y + dy), -(x + dx));
+	if (ang1 < 0)
+		ang1 += 360;
+	ang2 = RAD_TO_DEG * atan2((y - dy), -(x - dx));
+	if (ang2 < 0)
+		ang2 += 360;
+	delta = ang2 - ang1;
+	if (delta > 180)
+		delta -=360;
+	else if (delta < -180)
+		delta +=360;
+	if (delta < 0)
+	{
+		ang1 += delta;
+		delta = - delta;
+		while (ang1 < 0)
+			ang1 += 360;
+	}
+	if (ang1 >= Arc->StartAngle && ang1 <= Arc->StartAngle + Arc->Delta)
 		return(True);
-		/* check the end points for the search radius */
-	sr *= sr;
-	if ((X - box->X1)*(X - box->X1) + (Y - box->Y1)*(Y - box->Y1) <= sr)
+	if (Arc->StartAngle >= ang1 && Arc->StartAngle <= ang1 + delta)
 		return(True);
-	if ((X - box->X2)*(X - box->X2) + (Y - box->Y2)*(Y - box->Y2) <= sr)
+	if (Arc->StartAngle + Arc->Delta >= 360 && ang1 <= Arc->StartAngle + Arc->Delta - 360)
+		return(True);
+	if (ang1 + delta >= 360 && Arc->StartAngle <= ang1 + delta - 360)
 		return(True);
 	return(False);
 }
@@ -711,9 +853,10 @@ int SearchObjectByPosition(int Type,
 	void **Result1, void **Result2, void **Result3,
 	Position X, Position Y, Dimension Radius)
 {
-	Cardinal i;
+	void *r1, *r2, *r3;
+	int i;
 	int HigherBound = 0;
-	Boolean HigherAvail = False;
+	int HigherAvail = NO_TYPE;
 		/* setup local identifiers */
 	PosX = X;
 	PosY = Y;
@@ -725,36 +868,41 @@ int SearchObjectByPosition(int Type,
 		return(VIA_TYPE);
 
 	if (Type & PIN_TYPE &&
-		SearchPinByPosition((ElementTypePtr *) Result1,
-			(PinTypePtr *) Result2, (PinTypePtr *) Result3))
-		HigherAvail = True;
+		SearchPinByPosition((ElementTypePtr *) &r1,
+			(PinTypePtr *) &r2, (PinTypePtr *) &r3))
+		HigherAvail = PIN_TYPE;
 	
 	if (!HigherAvail && Type & PAD_TYPE &&
-		SearchPadByPosition((ElementTypePtr *) Result1,
-			(PadTypePtr *) Result2, (PadTypePtr *) Result3, False))
-		HigherAvail = True;
+		SearchPadByPosition((ElementTypePtr *) &r1,
+			(PadTypePtr *) &r2, (PadTypePtr *) &r3, False))
+		HigherAvail = PAD_TYPE;
 	
 	if (!HigherAvail && Type & ELEMENTNAME_TYPE &&
-		SearchElementNameByPosition((ElementTypePtr *) Result1,
-			(TextTypePtr *) Result2, (TextTypePtr *) Result3, False))
+		SearchElementNameByPosition((ElementTypePtr *) &r1,
+			(TextTypePtr *) &r2, (TextTypePtr *) &r3, False))
 	{
-		BoxTypePtr box = &(*(TextTypePtr *) Result2)->BoundingBox;
+		BoxTypePtr box = &((TextTypePtr) r2)->BoundingBox;
 		HigherBound = (box->X2 - box->X1) * (box->Y2 - box->Y1);
-		HigherAvail = True;
+		HigherAvail = ELEMENTNAME_TYPE;
 	}
 
 	if (!HigherAvail && Type & ELEMENT_TYPE &&
-		SearchElementByPosition((ElementTypePtr *) Result1,
-			(ElementTypePtr *) Result2, (ElementTypePtr *) Result3, False))
+		SearchElementByPosition((ElementTypePtr *) &r1,
+			(ElementTypePtr *) &r2, (ElementTypePtr *) &r3, False))
 	{
-		BoxTypePtr box = &(*(ElementTypePtr *) Result1)->BoundingBox;
+		BoxTypePtr box = &((ElementTypePtr) r1)->BoundingBox;
 		HigherBound = (box->X2 - box->X1) * (box->Y2 - box->Y1);
-		HigherAvail = True;
+		HigherAvail = ELEMENT_TYPE;
 	}
 
-	for (i = 0; i < MAX_LAYER; i++)
+	for (i = -1; i < MAX_LAYER + 1; i++)
 	{
-		SearchLayer = LAYER_ON_STACK(i);
+		if (i < 0)
+			SearchLayer = SILKLAYER;
+		else if (i < MAX_LAYER)	
+			SearchLayer = LAYER_ON_STACK(i);
+		else
+			SearchLayer = BACKSILKLAYER;
 		if (SearchLayer->On)
 		{
 			if (Type & POLYGONPOINT_TYPE &&
@@ -762,9 +910,30 @@ int SearchObjectByPosition(int Type,
 					(PolygonTypePtr *) Result2, (PointTypePtr *) Result3))
 				return(POLYGONPOINT_TYPE);
 
+			if (Type & LINEPOINT_TYPE &&
+				SearchLinePointByPosition((LayerTypePtr *) Result1,
+					(LineTypePtr *) Result2,
+					(PointTypePtr *) Result3))
+				return(LINEPOINT_TYPE);
+	
+			if ((HigherAvail & (PIN_TYPE | PAD_TYPE)) == 0 && Type & LINE_TYPE &&
+				SearchLineByPosition((LayerTypePtr *) Result1,
+					(LineTypePtr *) Result2, (LineTypePtr *) Result3))
+				return(LINE_TYPE);
+
+			if ((HigherAvail & (PIN_TYPE | PAD_TYPE)) == 0 && Type & ARC_TYPE &&
+				SearchArcByPosition((LayerTypePtr *) Result1,
+					(ArcTypePtr *) Result2, (ArcTypePtr *) Result3))
+				return(ARC_TYPE);
+
+			if ((HigherAvail & (PIN_TYPE | PAD_TYPE)) == 0 && Type & TEXT_TYPE &&
+				SearchTextByPosition((LayerTypePtr *) Result1,
+					(TextTypePtr *) Result2, (TextTypePtr *) Result3))
+				return(TEXT_TYPE);
+
 			if (Type & POLYGON_TYPE &&
 				SearchPolygonByPosition((LayerTypePtr *) Result1,
-					(PolygonTypePtr *) Result2, (PolygonTypePtr *) Result3))
+					(PolygonTypePtr *) Result2, (PolygonTypePtr *) Result3)) {
 				if (HigherAvail)
 				{
 					BoxTypePtr box = &(*(PolygonTypePtr *) Result2)->BoundingBox;
@@ -776,35 +945,43 @@ int SearchObjectByPosition(int Type,
 				}
 				else
 					return(POLYGON_TYPE);
-
-			if (Type & LINEPOINT_TYPE &&
-				SearchLinePointByPosition((LayerTypePtr *) Result1,
-					(LineTypePtr *) Result2,
-					(PointTypePtr *) Result3))
-				return(LINEPOINT_TYPE);
-	
-			if (Type & LINE_TYPE &&
-				SearchLineByPosition((LayerTypePtr *) Result1,
-					(LineTypePtr *) Result2, (LineTypePtr *) Result3))
-				return(LINE_TYPE);
-
-			if (Type & ARC_TYPE &&
-				SearchArcByPosition((LayerTypePtr *) Result1,
-					(ArcTypePtr *) Result2, (ArcTypePtr *) Result3))
-				return(ARC_TYPE);
-
-			if (Type & TEXT_TYPE &&
-				SearchTextByPosition((LayerTypePtr *) Result1,
-					(TextTypePtr *) Result2, (TextTypePtr *) Result3))
-				return(TEXT_TYPE);
+			}
 		}
 	}
-
-	if (HigherAvail && Type & PIN_TYPE &&
-		SearchPinByPosition((ElementTypePtr *) Result1,
-			(PinTypePtr *) Result2, (PinTypePtr *) Result3))
+		/* return any previously found objects */
+	if (HigherAvail & PIN_TYPE)
+	{
+		*Result1 = r1; *Result2 = r2; *Result3 = r3;
 		return(PIN_TYPE);
+	}
 	
+	if (HigherAvail & PAD_TYPE)
+	{
+		*Result1 = r1; *Result2 = r2; *Result3 = r3;
+		return(PAD_TYPE);
+	}
+	
+	if (HigherAvail & ELEMENTNAME_TYPE)
+	{
+		*Result1 = r1; *Result2 = r2; *Result3 = r3;
+		return(ELEMENTNAME_TYPE);
+	}
+
+	if (Type & RATLINE_TYPE &&
+	    SearchRatLineByPosition((RatTypePtr *) Result1,
+	    (RatTypePtr *) Result2, (RatTypePtr *) Result3))
+		return(RATLINE_TYPE);
+
+	if (HigherAvail & ELEMENT_TYPE)
+	{
+		*Result1 = r1; *Result2 = r2; *Result3 = r3;
+		return(ELEMENT_TYPE);
+	}
+
+		/* search the 'invisible objects' last */
+	if (!PCB->InvisibleObjectsOn)
+		return(NO_TYPE);
+
 	if (Type & PAD_TYPE &&
 		SearchPadByPosition((ElementTypePtr *) Result1,
 			(PadTypePtr *) Result2, (PadTypePtr *) Result3, True))
@@ -815,18 +992,6 @@ int SearchObjectByPosition(int Type,
 			(TextTypePtr *) Result2, (TextTypePtr *) Result3, True))
 		return(ELEMENTNAME_TYPE);
 
-	if (Type & RATLINE_TYPE)
-		for (i = 0; i < MAX_LAYER; i++)
-		{
-			SearchLayer = LAYER_ON_STACK(i);
-			if (SearchLayer->On)
-			{
-				if (SearchRatLineByPosition((LayerTypePtr *) Result1,
-						(LineTypePtr *) Result2, (LineTypePtr *) Result3))
-					return(LINE_TYPE);
-			}
-		}
-
 	if (Type & ELEMENT_TYPE &&
 		SearchElementByPosition((ElementTypePtr *) Result1,
 			(ElementTypePtr *) Result2, (ElementTypePtr *) Result3, True))
@@ -912,6 +1077,28 @@ int SearchObjectByID(DataTypePtr Base,
 		}
 	);
 
+	RAT_LOOP(Base,
+		if (line->ID == ID)
+		{
+			*Result1 = *Result2 = *Result3 = (void *) line;
+			return(RATLINE_TYPE);
+		}
+		if (line->Point1.ID == ID)
+		{
+			*Result1 = (void *) NULL;
+			*Result2 = (void *) line;
+			*Result3 = (void *) &line->Point1;
+			return(LINEPOINT_TYPE);
+		}
+		if (line->Point2.ID == ID)
+		{
+			*Result1 = (void *) NULL;
+			*Result2 = (void *) line;
+			*Result3 = (void *) &line->Point2;
+			return(LINEPOINT_TYPE);
+		}
+	);
+
 		/* check pins and elementnames too */
 	ELEMENT_LOOP(Base,
 		if (element->ID == ID)
@@ -919,6 +1106,22 @@ int SearchObjectByID(DataTypePtr Base,
 			*Result1 = *Result2 = *Result3 = (void *) element;
 			return(ELEMENT_TYPE);
 		}
+		ELEMENTLINE_LOOP(element,
+			if (line->ID == ID)
+			{
+				*Result1 = (void *) element;
+				*Result2 = *Result3 = (void *) line;
+				return(ELEMENTLINE_TYPE);
+			}
+		);
+		ARC_LOOP(element,
+			if (arc->ID == ID)
+			{
+				*Result1 = (void *) element;
+				*Result2 = *Result3 = (void *) arc;
+				return(ELEMENTARC_TYPE);
+			}
+		);
 		ELEMENTTEXT_LOOP(element,
 			if (text->ID == ID)
 			{
@@ -945,6 +1148,7 @@ int SearchObjectByID(DataTypePtr Base,
 		);
 	);
 
+	Message("hace: Internal error, search for ID %d failed\n", ID);
 	return(NO_TYPE);
 }
 
@@ -967,3 +1171,20 @@ ElementTypePtr SearchElementByName(DataTypePtr Base, char *Name)
 	return result;
 }
 
+/* ---------------------------------------------------------------------------
+ * searches the cursor position for the type 
+ */
+int SearchScreen(int Type, void **Result1, void **Result2, void **Result3)
+{
+	Window		root, child;
+	int		ans, rootX, rootY, childX, childY;
+	unsigned int	mask;
+
+	XQueryPointer(Dpy, Output.OutputWindow, &root, &child,
+		&rootX, &rootY, &childX, &childY, &mask);
+	ScreenOnly = True;
+	ans = SearchObjectByPosition(Type, Result1, Result2, Result3,
+		TO_PCB_X(childX), TO_PCB_Y(childY), TO_PCB(2));
+	ScreenOnly = False;
+	return (ans);
+}
