@@ -54,6 +54,8 @@ static	char	*rcsid = "$Id: select.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 Boolean SelectObject(void)
 {
 	void	*ptr1, *ptr2, *ptr3;
+	LayerTypePtr layer;
+
 	Boolean	changed = True;
 	
 	switch(SearchObjectByPosition(SELECT_TYPES, &ptr1, &ptr2, &ptr3,
@@ -66,28 +68,71 @@ Boolean SelectObject(void)
 			break;
 
 		case LINE_TYPE:
+		{
+			LineType swap, *line = (LineTypePtr) ptr2;
+
+			layer = (LayerTypePtr) ptr1;
                         AddObjectToFlagUndoList(LINE_TYPE, ptr1, ptr2, ptr2);
-			TOGGLE_FLAG(SELECTEDFLAG, (LineTypePtr) ptr2);
-			DrawLine((LayerTypePtr) ptr1, (LineTypePtr) ptr2, 0);
+			TOGGLE_FLAG(SELECTEDFLAG, line);
+			DrawLine(layer, line, 0);
+				/* move the line to the top of the data base */
+			swap = layer->Line[0];
+			layer->Line[0] = *line;
+			*line = swap;
 			break;
+		}
+
+		case RATLINE_TYPE:
+		{
+			RatTypePtr	rat = (RatTypePtr) ptr2;
+
+			AddObjectToFlagUndoList(RATLINE_TYPE, ptr1, ptr1, ptr1);
+			TOGGLE_FLAG(SELECTEDFLAG, rat);
+			DrawRat(rat, 0);
+			break;
+		}
 
 		case ARC_TYPE:
+		{
+			ArcType	swap, *arc = (ArcTypePtr) ptr2;
+
+			layer = (LayerTypePtr) ptr1;
                         AddObjectToFlagUndoList(ARC_TYPE, ptr1, ptr2, ptr2);
-			TOGGLE_FLAG(SELECTEDFLAG, (ArcTypePtr) ptr2);
-			DrawArc((LayerTypePtr) ptr1, (ArcTypePtr) ptr2, 0);
+			TOGGLE_FLAG(SELECTEDFLAG, arc);
+			DrawArc(layer, arc, 0);
+			swap = layer->Arc[0];
+			layer->Arc[0] = *arc;
+			*arc = swap;
 			break;
+		}
 
 		case TEXT_TYPE:
+		{
+			TextType swap, *text = (TextTypePtr) ptr2;
+
+			layer = (LayerTypePtr) ptr1;
                         AddObjectToFlagUndoList(TEXT_TYPE, ptr1, ptr2, ptr2);
-			TOGGLE_FLAG(SELECTEDFLAG, (TextTypePtr) ptr2);
-			DrawText((LayerTypePtr) ptr1, (TextTypePtr) ptr2, 0);
+			TOGGLE_FLAG(SELECTEDFLAG, text);
+			DrawText(layer, text, 0);
+			swap = layer->Text[0];
+			layer->Text[0] = *text;
+			*text = swap;
 			break;
+		}
 
 		case POLYGON_TYPE:
+		{
+			PolygonType swap, *poly = (PolygonTypePtr) ptr2;
+
+			layer = (LayerTypePtr) ptr1;
                         AddObjectToFlagUndoList(POLYGON_TYPE, ptr1, ptr2, ptr2);
-			TOGGLE_FLAG(SELECTEDFLAG, (PolygonTypePtr) ptr2);
-			DrawPolygon((LayerTypePtr) ptr1, (PolygonTypePtr) ptr2, 0);
+			TOGGLE_FLAG(SELECTEDFLAG, poly);
+			DrawPolygon(layer, poly, 0);
+			swap = layer->Polygon[0];
+			layer->Polygon[0] = *poly;
+			*poly = swap;
 			break;
+		}
 
 		case PIN_TYPE:
                         AddObjectToFlagUndoList(PIN_TYPE, ptr1, ptr2, ptr2);
@@ -149,7 +194,7 @@ Boolean SelectObject(void)
 			break;
 		}
 	}
-        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	Draw();
 	IncrementUndoSerialNumber();
 	return(changed);
 }
@@ -161,11 +206,20 @@ Boolean SelectObject(void)
  */
 Boolean SelectBlock(BoxTypePtr Box, Boolean Flag)
 {
-	XRectangle	Drawn;
 	Boolean		changed = False;
 
-	BlockDrawing = True;
-	AddBlockToUndoList(Box);
+	if (PCB->RatOn)
+		RAT_LOOP(PCB->Data,
+			if (LINE_IN_BOX((LineTypePtr) line, Box) &&
+			    TEST_FLAG(SELECTEDFLAG, line) != Flag)
+			{
+				AddObjectToFlagUndoList(RATLINE_TYPE, line, line, line);
+				ASSIGN_FLAG(SELECTEDFLAG, Flag, line);
+				DrawRat(line, 0);
+				changed = True;
+			}
+		);
+	
 		/* check layers */
 	VISIBLELINE_LOOP(PCB->Data, 
 		if (LINE_IN_BOX(line, Box) && TEST_FLAG(SELECTEDFLAG, line) != Flag)
@@ -230,6 +284,7 @@ Boolean SelectBlock(BoxTypePtr Box, Boolean Flag)
 						AddObjectToFlagUndoList(PIN_TYPE,
 							element, pin, pin);
 						ASSIGN_FLAG(SELECTEDFLAG, Flag, pin);
+						DrawPin(pin, 0);
 						changed = True;
 					});
 					PAD_LOOP(element,
@@ -238,6 +293,7 @@ Boolean SelectBlock(BoxTypePtr Box, Boolean Flag)
 						AddObjectToFlagUndoList(PAD_TYPE,
 							element, pad, pad);
 						ASSIGN_FLAG(SELECTEDFLAG, Flag, pad);
+						DrawPad(pad, 0);
 						changed = True;
 					});
 					if (TEST_FLAG(SELECTEDFLAG, element) != Flag)
@@ -263,15 +319,11 @@ Boolean SelectBlock(BoxTypePtr Box, Boolean Flag)
 				changed = True;
 			}
 		);
-	BlockDrawing = False;
 	if (changed)
 	{
-		AddBlockToUndoList(NULL);
+		Draw();
         	IncrementUndoSerialNumber();            
-		DrawBlock(Box);
 	}
-	else
-		UndoFalseBlock();
 	return(changed);
 }
 
@@ -339,6 +391,11 @@ void *ObjectOperation(ObjectFunctionTypePtr F,
 			if (F->ElementName)
 				return(F->ElementName((ElementTypePtr) Ptr1));
 			break;
+
+		case RATLINE_TYPE:
+			if (F->Rat)
+				return(F->Rat((RatTypePtr) Ptr1));
+			break;
 	}
 	return(NULL);
 }
@@ -469,8 +526,7 @@ Boolean SelectedOperation(ObjectFunctionTypePtr F, Boolean Reset)
 			);
 		);
 		
-
-		/* and vias */
+		/* process vias */
 	if (PCB->ViaOn && F->Via)
 		VIA_LOOP(PCB->Data,
 			if (TEST_FLAG(SELECTEDFLAG, via))
@@ -484,7 +540,22 @@ Boolean SelectedOperation(ObjectFunctionTypePtr F, Boolean Reset)
 				changed = True;
 			}
 		);
-	IncrementUndoSerialNumber();
+		/* and rat-lines */
+	if (PCB->RatOn && F->Rat)
+		RAT_LOOP(PCB->Data,
+			if (TEST_FLAG(SELECTEDFLAG, line))
+			{
+				if (Reset) {
+					AddObjectToFlagUndoList(RATLINE_TYPE,
+						line, line, line);
+					CLEAR_FLAG(SELECTEDFLAG, line);
+				}
+				F->Rat(line);
+				changed = True;
+			}
+		);
+	if (Reset && changed)
+		IncrementUndoSerialNumber();
 	return(changed);
 }
 
@@ -499,6 +570,18 @@ Boolean SelectConnection(Boolean Flag)
 {
 	Boolean		changed = False;
 
+	if (PCB->RatOn)
+		RAT_LOOP(PCB->Data,
+			if (TEST_FLAG(FOUNDFLAG, line))
+			{
+				AddObjectToFlagUndoList(RATLINE_TYPE,
+					line, line, line);
+				ASSIGN_FLAG(SELECTEDFLAG, Flag, line);
+				DrawRat(line, 0);
+				changed = True;
+			}
+		);
+			
 	VISIBLELINE_LOOP(PCB->Data,
 		if (TEST_FLAG(FOUNDFLAG, line))
 		{
@@ -530,7 +613,7 @@ Boolean SelectConnection(Boolean Flag)
 		}
 	);
 
-	if (PCB->PinOn)
+	if (PCB->PinOn && PCB->ElementOn)
 	{
 		ALLPIN_LOOP(PCB->Data,
 			if (TEST_FLAG(FOUNDFLAG, pin))
@@ -558,13 +641,15 @@ Boolean SelectConnection(Boolean Flag)
 		VIA_LOOP(PCB->Data,
 			if (TEST_FLAG(FOUNDFLAG, via))
 			{
+				AddObjectToFlagUndoList(VIA_TYPE,
+					via, via, via);
 				ASSIGN_FLAG(SELECTEDFLAG, Flag, via);
 				DrawVia(via, 0);
 				changed = True;
 			}
 	);
 	if (changed)
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	return(changed);
 }
 
@@ -643,6 +728,11 @@ Boolean SelectObjectByName(int Type, char *Pattern)
 						AddObjectToFlagUndoList(PAD_TYPE, element, pad, pad);
 						SET_FLAG(SELECTEDFLAG, pad);
 					);
+					ELEMENTTEXT_LOOP(element,
+						AddObjectToFlagUndoList(ELEMENTNAME_TYPE, element, text, text);
+						TOGGLE_FLAG(SELECTEDFLAG, text);
+					);
+					DrawElementName(element, 0);
 					DrawElement(element, 0);
 					changed = True;
 				}
@@ -690,7 +780,7 @@ Boolean SelectObjectByName(int Type, char *Pattern)
 	if (changed)
 	{
 		IncrementUndoSerialNumber();
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+		Draw();
 	}
 	return(changed);
 }
