@@ -61,6 +61,6 @@ typedef struct							/* a select box */
 void	UpdateSelector(SelectorTypePtr);
 void	AddEntryToSelector(char *, XtPointer, SelectorTypePtr);
 void	FreeSelectorEntries(SelectorTypePtr);
-Widget	CreateSelector(Widget, Widget, Widget, SelectorTypePtr);
+Widget	CreateSelector(Widget, Widget, Widget, SelectorTypePtr, Cardinal);
 
 #endif
