@@ -30,33 +30,39 @@ static	char	*rcsid = "$Id: set.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
  * (except output window and dialogs)
  */
 
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "global.h"
 
+#include "action.h"
 #include "buffer.h"
 #include "crosshair.h"
 #include "control.h"
 #include "data.h"
 #include "draw.h"
+#include "menu.h"
 #include "misc.h"
+#include "output.h"
 #include "set.h"
 
 #include <X11/cursorfont.h>
-#include <X11/Xaw/Form.h>
+#include <X11/Xaw3d/Form.h>
 
 /* ---------------------------------------------------------------------------
  * output of cursor position
  */
 void SetCursorStatusLine(void)
 {
-	char	text[40];
+	char	text[64];
 
 	if (Marked.status)
-		sprintf(text, "%-i,%-i  <%-i,%-i>",Crosshair.X, Crosshair.Y,
-			Crosshair.X - Marked.X, Crosshair.Y - Marked.Y);
+		sprintf(text, "%-i,%-i <%-i,%-i> (%-.1f,%-.1f)",Crosshair.X, Crosshair.Y,
+			Crosshair.X - Marked.X, Crosshair.Y - Marked.Y,
+			sqrt(SQUARE(Crosshair.X - Marked.X) + SQUARE(Crosshair.Y - Marked.Y)),
+			RAD_TO_DEG * atan2(Crosshair.Y - Marked.Y, Marked.X - Crosshair.X));
 	else
 	sprintf(text, "%-i,%-i", Crosshair.X, Crosshair.Y);
 	XtVaSetValues(Output.CursorPosition, XtNlabel, text, NULL);
@@ -104,13 +110,11 @@ void SetGrid(int Grid)
 			 * to (0,0) depending on resource
 			 * DrawGrid() uses XOR mode
 			 */
-		if (Settings.DrawGrid)
-			DrawGrid();
 		PCB->GridOffsetX= TEST_FLAG(ABSOLUTEFLAG, PCB) ? 0 : Crosshair.X % Grid;
 		PCB->GridOffsetY= TEST_FLAG(ABSOLUTEFLAG, PCB) ? 0 : Crosshair.Y % Grid;
 		PCB->Grid = Grid;
 		if (Settings.DrawGrid)
-			DrawGrid();
+			UpdateAll();
 		SetStatusLine();
 	}
 } 
@@ -121,7 +125,7 @@ void SetGrid(int Grid)
 void SetZoom(int Zoom)
 {
 	static	String	cmd[] = {"MovePointer", "0", "0"};
-	int	localMin;
+	int		localMin;
 
 	if (MAX(PCB->MaxWidth, PCB->MaxHeight) > (1 << 14))
 		localMin = MIN_ZOOM;
@@ -135,18 +139,14 @@ void SetZoom(int Zoom)
 		/* redraw only if zoom has changed */
 	if (PCB->Zoom != Zoom)
 	{
-			/* saved pixmap is invalid */
-		ReleaseSaveUnderPixmap();
 		PCB->Zoom = Zoom;
+		ScaleOutput(Output.Width, Output.Height);
+		Pan(TO_SCREEN_X(Crosshair.X) - Output.Width/2 - Xorig,
+			TO_SCREEN_Y(Crosshair.Y) - Output.Height/2 - Yorig, False, True);
 
-			/* recalculate size of viewport */
-		XtVaSetValues(Output.Output,
-			XtNwidth, TO_SCREEN(PCB->MaxWidth),
-			XtNheight, TO_SCREEN(PCB->MaxHeight),
-			NULL);
-
-		CenterDisplay(TO_SCREEN_X(Crosshair.X), TO_SCREEN_Y(Crosshair.Y));
+			/* move the pointer to the same relative location */
 		XtCallActionProc(Output.Output, cmd[0], NULL, &cmd[1], 2);
+		UpdateAll();
 	}
 		/* always redraw status line (used for init sequence) */
 	SetStatusLine();
@@ -209,12 +209,28 @@ void SetTextScale(Dimension Scale)
  */
 void SetChangedFlag(Boolean New)
 {
-		/* seems that the save output area is invalid now */
-	ReleaseSaveUnderPixmap();
 	PCB->Changed = New;
 	SetStatusLine();
 }
 
+/* ---------------------------------------------------------------------------
+ * sets the crosshair range to the current buffer extents 
+ */
+void SetCrosshairRangeToBuffer(void)
+{
+	if (Settings.Mode == PASTEBUFFER_MODE)
+	{
+		SetBufferBoundingBox(PASTEBUFFER);
+		SetCrosshairRange(
+			PASTEBUFFER->X -PASTEBUFFER->BoundingBox.X1,
+			PASTEBUFFER->Y -PASTEBUFFER->BoundingBox.Y1,
+			PCB->MaxWidth-
+				(PASTEBUFFER->BoundingBox.X2 -PASTEBUFFER->X),
+			PCB->MaxHeight-
+				(PASTEBUFFER->BoundingBox.Y2 -PASTEBUFFER->Y));
+	}
+}
+
 /* ---------------------------------------------------------------------------
  * sets a new buffer number
  */
@@ -225,17 +241,7 @@ void SetBufferNumber(int Number)
 		Settings.BufferNumber = Number;
 
 			/* do an update on the crosshair range */
-		if (Settings.Mode == PASTEBUFFER_MODE)
-		{
-			SetBufferBoundingBox(PASTEBUFFER);
-			SetCrosshairRange(
-				PASTEBUFFER->X -PASTEBUFFER->BoundingBox.X1,
-				PASTEBUFFER->Y -PASTEBUFFER->BoundingBox.Y1,
-				PCB->MaxWidth-
-					(PASTEBUFFER->BoundingBox.X2 -PASTEBUFFER->X),
-				PCB->MaxHeight-
-					(PASTEBUFFER->BoundingBox.Y2 -PASTEBUFFER->Y));
-		}
+		SetCrosshairRangeToBuffer();
 		SetStatusLine();
 	}
 }
@@ -248,6 +254,7 @@ void UpdateSettingsOnScreen(void)
 	SetStatusLine();
 	SetCursorStatusLine();
 	UpdateControlPanel();
+	UpdateSizesMenu();
 }
 
 /* ---------------------------------------------------------------------------
@@ -320,6 +327,10 @@ void SetMode(int Mode)
 			SetOutputXCursor(XC_question_arrow);
 			break;
 
+		case ARROW_MODE:
+			SetOutputXCursor(XC_left_ptr);
+			break;
+
 		case POLYGON_MODE:
 			SetOutputXCursor(XC_sb_up_arrow);
 			break;
@@ -328,14 +339,7 @@ void SetMode(int Mode)
 			SetOutputXCursor(XC_hand1);
 
 				/* do an update on the crosshair range */
-			SetBufferBoundingBox(PASTEBUFFER);
-			SetCrosshairRange(
-				PASTEBUFFER->X -PASTEBUFFER->BoundingBox.X1,
-				PASTEBUFFER->Y -PASTEBUFFER->BoundingBox.Y1,
-				PCB->MaxWidth-
-					(PASTEBUFFER->BoundingBox.X2 -PASTEBUFFER->X),
-				PCB->MaxHeight-
-					(PASTEBUFFER->BoundingBox.Y2 -PASTEBUFFER->Y));
+			SetCrosshairRangeToBuffer();
 			break;
 
 		case TEXT_MODE:
@@ -360,7 +364,6 @@ void SetMode(int Mode)
 
 		case COPY_MODE:
 		case MOVE_MODE:
-		case RUBBERBANDMOVE_MODE:
 			SetOutputXCursor(XC_crosshair);
 			break;
 
@@ -378,3 +381,17 @@ void SetMode(int Mode)
 	RestoreCrosshair(True);
 }
 
+void SetRouteStyle(char *name)
+{
+	char	*arg, num[10];
+	Cardinal	i;
+
+	for (i = 0; i < NUM_STYLES; i++)
+		if (name && strcmp(name, PCB->RouteStyle[i].Name) == 0)
+		{
+			arg = &num[0];
+			sprintf(num,"%d", i+1);
+			XtCallActionProc(Output.Output, "RouteStyle", NULL, &arg, 1);
+			break;
+		}
+}
