@@ -45,5 +45,7 @@ void		SetChangedFlag(Boolean);
 void		UpdateSettingsOnScreen(void);
 void		SetBufferNumber(int);
 void		SetMode(int);
+void		SetCrosshairRangeToBuffer(void);
+void		SetRouteStyle(char *);
 
 #endif
