@@ -34,10 +34,12 @@ static	char	*rcsid = "$Id: sizedialog.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 #include "global.h"
 
+#include "change.h"
 #include "crosshair.h"
 #include "data.h"
 #include "dialog.h"
 #include "error.h"
+#include "menu.h"
 #include "misc.h"
 #include "sizedialog.h"
 #include "set.h"
@@ -89,9 +91,9 @@ typedef struct
  */
 static	int			ReturnCode;			/* returncode of buttons */
 static	SliderType	Sliders[] = {
-	{ "linewidth", MIN_LINESIZE, MAX_LINESIZE, 5, 0, NULL, NULL },
-	{ "via hole", MIN_PINORVIAHOLE, 0, 5, 0, NULL, NULL },
-	{ "via size", 0, MAX_PINORVIASIZE, 5, 0, NULL, NULL },
+	{ "linewidth", MIN_LINESIZE, MAX_LINESIZE, CHUNK, 0, NULL, NULL },
+	{ "via hole", MIN_PINORVIAHOLE, 0, CHUNK, 0, NULL, NULL },
+	{ "via size", 0, MAX_PINORVIASIZE, CHUNK, 0, NULL, NULL },
 	{ "text size", MIN_TEXTSCALE, MAX_TEXTSCALE, 10, 0, NULL, NULL },
 	{ "PCB width", MIN_SIZE, MAX_COORD, 100, 0, NULL, NULL },
 	{ "PCB height", MIN_SIZE, MAX_COORD, 100, 0, NULL, NULL }};
@@ -211,7 +213,7 @@ static void CB_JumpProc(Widget W, XtPointer ClientData, XtPointer CallData)
 }
 
 /* ---------------------------------------------------------------------------
- * size dialog
+ * active size dialog
  */
 void SizeDialog(void)
 {
@@ -250,7 +252,7 @@ void SizeDialog(void)
 	}
 
 		/* create a popup shell */
-	popup = XtVaCreatePopupShell("popup", transientShellWidgetClass,
+	popup = XtVaCreatePopupShell("Sizes", transientShellWidgetClass,
 		Output.Toplevel,
 		XtNallowShellResize, True,
 		XtNmappedWhenManaged, False,
@@ -329,3 +331,122 @@ void SizeDialog(void)
 	}
 	EndDialog(popup);
 }
+
+/* ---------------------------------------------------------------------------
+ * style sizes dialog
+ */
+void StyleSizeDialog(int index)
+{
+	Widget				popup,
+					masterform,
+					label,
+					last,
+					inputfield;
+	char				*string;
+	int				i;
+	static	DialogButtonType	buttons[] = {
+		{ "defaultButton", "   OK   ", CB_CancelOrOK,
+			(XtPointer) OK_BUTTON, NULL },
+		{ "cancelButton", "No/Cancel", CB_CancelOrOK,
+			(XtPointer) CANCEL_BUTTON, NULL }};
+	static char			styleName[64];
+
+		/* copy current values to struct */
+	Sliders[LINE_SLIDER].Value = PCB->RouteStyle[index].Thick;
+	Sliders[VIADRILL_SLIDER].Value = PCB->RouteStyle[index].Hole;
+	Sliders[VIADRILL_SLIDER].Max = PCB->RouteStyle[index].Diameter -MIN_PINORVIACOPPER;
+	Sliders[VIASIZE_SLIDER].Value = PCB->RouteStyle[index].Diameter;
+	Sliders[VIASIZE_SLIDER].Min = PCB->RouteStyle[index].Hole +MIN_PINORVIACOPPER;
+
+	sprintf(styleName,"'%s' Sizes",PCB->RouteStyle[index].Name);
+		/* create a popup shell */
+	popup = XtVaCreatePopupShell(styleName, transientShellWidgetClass,
+		Output.Toplevel,
+		XtNallowShellResize, True,
+		XtNmappedWhenManaged, False,
+		NULL);
+
+		/* the form that holds everything */
+	masterform = XtVaCreateManagedWidget("sizeMasterForm", formWidgetClass,
+		popup,
+		XtNresizable, True,
+		NULL);
+	last = XtVaCreateManagedWidget("comment", labelWidgetClass,
+		masterform,
+		XtNfromVert, NULL,
+		XtNlabel, "Routing Style Name",
+		LAYOUT_TOP,
+		NULL);
+	inputfield = XtVaCreateManagedWidget("Name", asciiTextWidgetClass,
+			masterform,
+			XtNfromVert, last,
+			XtNresizable, True,
+			XtNeditType, XawtextEdit,
+			XtNstring, PCB->RouteStyle[index].Name,
+			LAYOUT_TOP,
+			NULL);
+	XtInstallAccelerators(masterform, inputfield);
+
+		/* create the sliders */
+	last = inputfield;
+	for (i = 0; i < 3; i++)
+	{
+		label = XtVaCreateManagedWidget("comment", labelWidgetClass,
+			masterform,
+			XtNfromVert, last,
+			XtNlabel, Sliders[i].Label,
+			LAYOUT_TOP,
+			NULL);
+		Sliders[i].Size = XtVaCreateManagedWidget("size", labelWidgetClass,
+			masterform,
+			XtNfromVert, last,
+			XtNfromHoriz, label,
+			XtNresizable, True,
+			LAYOUT_TOP,
+			NULL);
+		last = XtVaCreateManagedWidget("scrollbar", scrollbarWidgetClass,
+			masterform,
+			LAYOUT_TOP,
+			XtNfromVert, label,
+			XtNlength, SCROLLBAR_LENGTH,
+			XtNminimumThumb, THUMB_LENGTH,
+			XtNorientation, XtorientHorizontal,
+			NULL);
+		Sliders[i].Scrollbar = last;
+
+			/* add callbacks for scrolling */
+		XtAddCallback(Sliders[i].Scrollbar,
+			XtNjumpProc, CB_JumpProc, &Sliders[i]);
+		XtAddCallback(Sliders[i].Scrollbar,
+			XtNscrollProc, CB_ScrollProc, &Sliders[i]);
+	}
+
+		/* now that all sliders are initialized --> do an update */
+	for (i = 0; i < 3; i++)
+		UpdateScrollbar(&Sliders[i], Sliders[i].Value);
+
+		/* add the buttons and install accelerators for them;
+		 * the first one is always default
+		 */
+	AddButtons(masterform, last, buttons, ENTRIES(buttons));
+	XtInstallAccelerators(masterform, buttons[0].W);
+	XtInstallAccelerators(masterform, buttons[1].W);
+
+		/* now display dialog window */
+	StartDialog(popup);
+	DialogEventLoop(&ReturnCode);
+
+		/* get settings */
+	if (ReturnCode == OK_BUTTON)
+	{
+		PCB->RouteStyle[index].Thick = Sliders[LINE_SLIDER].Value;
+		PCB->RouteStyle[index].Diameter = Sliders[VIASIZE_SLIDER].Value;
+		PCB->RouteStyle[index].Hole = Sliders[VIADRILL_SLIDER].Value;
+		XtVaGetValues(inputfield, XtNstring, &string, NULL);
+		SaveFree(PCB->RouteStyle[index].Name);
+		PCB->RouteStyle[index].Name = StripWhiteSpaceAndDup(string);
+		UpdateSizesMenu();
+		SetChangedFlag(True);
+	}
+	EndDialog(popup);
+}
