@@ -54,9 +54,10 @@ static	char	*rcsid = "$Id: undo.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 #include "draw.h"
 #include "error.h"
 #include "insert.h"
-#include "mymem.h"
 #include "misc.h"
+#include "mirror.h"
 #include "move.h"
+#include "mymem.h"
 #include "remove.h"
 #include "rotate.h"
 #include "search.h"
@@ -66,11 +67,6 @@ static	char	*rcsid = "$Id: undo.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 /* ---------------------------------------------------------------------------
  * some local data types
  */
-typedef struct
-{
-	BoxType			*Box;		/* drawing box for block operations */
-} BlockType, *BlockTypePtr;
-
 typedef struct					/* information about a change command */
 {
 	char			*Name;
@@ -84,7 +80,8 @@ typedef struct					/* information about a move command */
 
 typedef struct					/* information about removed polygon points */
 {
-	PointType		Point;		/* data */
+	Position		X, Y;		/* data */
+	int			ID;
 	Cardinal		Index;		/* index in a polygons array of points */
 } RemovedPointType, *RemovedPointTypePtr;
 
@@ -114,7 +111,6 @@ typedef	struct					/* holds information about an operation */
 		MoveToLayerType		MoveToLayer;
 		int			Flags;
 		Dimension		Size;
-		BlockType		Block;
 	} Data;
 } UndoListType, *UndoListTypePtr;
 
@@ -135,19 +131,21 @@ static	Boolean			Locked = False;	/* do not add entries if */
  * some local prototypes
  */
 static	UndoListTypePtr	GetUndoSlot(int, int);
-static	void			DrawRecoveredObject(int, void *, void *, void *);
-static	Boolean			UndoRotate(UndoListTypePtr);
-static	Boolean			UndoChangeName(UndoListTypePtr);
-static	Boolean			UndoCopyOrCreate(UndoListTypePtr);
-static	Boolean			UndoMove(UndoListTypePtr);
-static	Boolean			UndoRemove(UndoListTypePtr);
-static	Boolean			UndoRemovePoint(UndoListTypePtr);
-static	Boolean			UndoInsertPoint(UndoListTypePtr);
-static	Boolean			UndoMoveToLayer(UndoListTypePtr);
-static	Boolean			UndoFlag(UndoListTypePtr);
-static	Boolean			UndoChangeSize(UndoListTypePtr);
-static	Boolean			UndoChange2ndSize(UndoListTypePtr);
-static	Boolean			UndoBlock(UndoListTypePtr);
+static	void		DrawRecoveredObject(int, void *, void *, void *);
+static	Boolean		UndoRotate(UndoListTypePtr);
+static	Boolean		UndoChangeName(UndoListTypePtr);
+static	Boolean		UndoCopyOrCreate(UndoListTypePtr);
+static	Boolean		UndoMove(UndoListTypePtr);
+static	Boolean		UndoRemove(UndoListTypePtr);
+static	Boolean		UndoRemovePoint(UndoListTypePtr);
+static	Boolean		UndoInsertPoint(UndoListTypePtr);
+static	Boolean		UndoMoveToLayer(UndoListTypePtr);
+static	Boolean		UndoFlag(UndoListTypePtr);
+static	Boolean		UndoMirror(UndoListTypePtr);
+static	Boolean		UndoChangeSize(UndoListTypePtr);
+static	Boolean		UndoChange2ndSize(UndoListTypePtr);
+static	Boolean		UndoChangeClearSize(UndoListTypePtr);
+static	int		PerformUndo(UndoListTypePtr);
 
 /* ---------------------------------------------------------------------------
  * adds a command plus some data to the undo list
@@ -187,9 +185,6 @@ static UndoListTypePtr GetUndoSlot(int CommandType, int ID)
 			case UNDO_CHANGENAME:
 				SaveFree(ptr->Data.ChangeName.Name);
 				break;
-			case UNDO_BLOCK:
-				SaveFree(ptr->Data.Block.Box);
-				break;
 			case UNDO_REMOVE:
 				type = SearchObjectByID(RemoveList, &ptr1,
 					&ptr2, &ptr3, ptr->ID);
@@ -242,26 +237,13 @@ static Boolean UndoRotate(UndoListTypePtr Entry)
 	{
 		RotateObject(type, ptr1, ptr2, ptr3,
 			Entry->Data.Rotate.CenterX, Entry->Data.Rotate.CenterY,
-			(4-Entry->Data.Rotate.Steps) & 0x03);
+			(4 - Entry->Data.Rotate.Steps) & 0x03);
 		Entry->Data.Rotate.Steps = (4 - Entry->Data.Rotate.Steps) & 0x03;
 		return(True);
 	}
 	return(False);
 }
 
-/* handles the block drawing aspect of undoing */
-static Boolean UndoBlock(UndoListTypePtr Entry)
-{
-
-	if (Entry->Data.Block.Box)
-	{
-		DrawBlock(Entry->Data.Block.Box);
-		BlockDrawing = False;
-	}
-	else
-		BlockDrawing = True;
-}
-
 /* ---------------------------------------------------------------------------
  * recovers an object from a 'change name' operation
  * returns True if anything has been recovered
@@ -314,6 +296,58 @@ static Boolean UndoChange2ndSize(UndoListTypePtr Entry)
 	return(type != NO_TYPE);
 }
 
+/* ---------------------------------------------------------------------------
+ * recovers an object from a clearance size change operation
+ */
+static Boolean UndoChangeClearSize(UndoListTypePtr Entry)
+{
+	void	*ptr1, *ptr2, *ptr3;
+	int	type;
+	Dimension	swap;
+
+		/* lookup entry by ID */
+	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
+	switch (type)
+	{
+		case VIA_TYPE:
+			swap = ((PinTypePtr) ptr2)->Clearance;
+			EraseVia((PinTypePtr) ptr2);
+			((PinTypePtr) ptr2)->Clearance = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawVia((PinTypePtr) ptr2, 0);
+			break;
+		case PIN_TYPE:
+			swap = ((PinTypePtr) ptr2)->Clearance;
+			ErasePin((PinTypePtr) ptr2);
+			((PinTypePtr) ptr2)->Clearance = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawPin((PinTypePtr) ptr2, 0);
+			break;
+		case LINE_TYPE:
+			swap = ((LineTypePtr) ptr2)->Clearance;
+			EraseLine((LineTypePtr) ptr2);
+			((LineTypePtr) ptr2)->Clearance = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawLine((LayerTypePtr) ptr1, (LineTypePtr) ptr2, 0);
+			break;
+		case ARC_TYPE:
+			swap = ((ArcTypePtr) ptr2)->Clearance;
+			EraseArc((ArcTypePtr) ptr2);
+			((ArcTypePtr) ptr2)->Clearance = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawArc((LayerTypePtr) ptr1, (ArcTypePtr) ptr2, 0);
+			break;
+		case PAD_TYPE:
+			swap = ((PadTypePtr) ptr2)->Clearance;
+			ErasePad((PadTypePtr) ptr2);
+			((PadTypePtr) ptr2)->Clearance = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawPad((PadTypePtr) ptr2, 0);
+			break;
+	}
+	return(type != NO_TYPE);
+}
+
 /* ---------------------------------------------------------------------------
  * recovers an object from a Size change operation
  */
@@ -385,6 +419,13 @@ static Boolean UndoChangeSize(UndoListTypePtr Entry)
 			Entry->Data.Size = swap;
 			DrawElement((ElementTypePtr) ptr1, 0);
 			break;
+		case ELEMENTARC_TYPE:
+			swap = ((ArcTypePtr) ptr2)->Thickness;
+			EraseElement((ElementTypePtr) ptr1);
+			((ArcTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawElement((ElementTypePtr) ptr1, 0);
+			break;
 	}
 	return(type != NO_TYPE);
 }
@@ -397,14 +438,13 @@ static Boolean UndoFlag(UndoListTypePtr Entry)
 	void	*ptr1, *ptr2, *ptr3;
 	int	type, swap, tests;
 
-
 		/* lookup entry by ID */
 	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
 	if (type != NO_TYPE)
 	{
 		swap = tests = ((PinTypePtr) ptr2)->Flags;
 		tests ^= Entry->Data.Flags;
-		if (tests & DRAW_FLAGS & ~SELECTEDFLAG)
+		if (tests & DRAW_FLAGS & ~(SELECTEDFLAG | FOUNDFLAG))
 			EraseObject(type, ptr2);
 		((PinTypePtr) ptr2)->Flags = Entry->Data.Flags;
 		Entry->Data.Flags = swap;
@@ -417,6 +457,30 @@ static Boolean UndoFlag(UndoListTypePtr Entry)
 	return(False);
 }
 
+/* ---------------------------------------------------------------------------
+ * recovers an object from a mirror operation
+ * returns True if anything has been recovered
+ */
+static Boolean UndoMirror(UndoListTypePtr Entry)
+{
+	void	*ptr1, *ptr2, *ptr3;
+	int	type;
+
+		/* lookup entry by ID */
+	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
+	if (type == ELEMENT_TYPE)
+	{
+		ElementTypePtr element = (ElementTypePtr) ptr3;
+
+		EraseElement(element);
+		MirrorElementCoordinates(element, Entry->Data.Move.DY);
+		DrawElement(element,0);
+		return(True);
+	}
+	Message("hace Internal error: UndoMirror on object type %d\n", type);
+	return(False);
+}
+
 /* ---------------------------------------------------------------------------
  * recovers an object from a 'copy' or 'create' operation
  * returns True if anything has been recovered
@@ -529,12 +593,14 @@ static Boolean UndoRemovePoint(UndoListTypePtr Entry)
 				ErasePolygon(polygon);
 			InsertPointIntoObject(POLYGON_TYPE, layer, polygon,
 				&Entry->Data.RemovedPoint.Index,
-				Entry->Data.RemovedPoint.Point.X,
-				Entry->Data.RemovedPoint.Point.Y);
+				Entry->Data.RemovedPoint.X,
+				Entry->Data.RemovedPoint.Y, True);
+			polygon->Points[Entry->Data.RemovedPoint.Index].ID = 
+				Entry->Data.RemovedPoint.ID;
 			if (layer->On)
 				DrawPolygon(layer, polygon, 0);
 			Entry->Type = UNDO_INSERT_POINT;
-			Entry->ID = polygon->Points[Entry->Data.RemovedPoint.Index].ID;
+			Entry->ID = Entry->Data.RemovedPoint.ID;
 			return(True);
 		}
 
@@ -560,8 +626,9 @@ static Boolean UndoInsertPoint(UndoListTypePtr Entry)
 		case POLYGONPOINT_TYPE:		/* removes an inserted polygon point */
 		{
 			pnt = (PointTypePtr) ptr3;
-			Entry->Data.RemovedPoint.Point.X = pnt->X;
-			Entry->Data.RemovedPoint.Point.Y = pnt->Y;
+			Entry->Data.RemovedPoint.X = pnt->X;
+			Entry->Data.RemovedPoint.Y = pnt->Y;
+			Entry->Data.RemovedPoint.ID = pnt->ID;
 			Entry->ID = ((PolygonTypePtr) ptr2)->ID;
 			POLYGONPOINT_LOOP((PolygonTypePtr) ptr2,
 				if (pnt == point)
@@ -583,92 +650,123 @@ static Boolean UndoInsertPoint(UndoListTypePtr Entry)
 /* ---------------------------------------------------------------------------
  * undo of any 'hard to recover' operation
  *
- * returns True if anything is has been done
+ * returns the bitfield for the types of operations that were undone
  */
-Boolean Undo(void)
+int Undo(void)
 {
 	UndoListTypePtr	ptr;
-	Boolean			changed = False;
+	int	Types = 0;
 
 	if (!UndoN)
 	{
-		Message("Nothing to undo - buffer is empty\n");
+		if (!Serial)
+			Message("Nothing to undo - buffer is empty\n");
+		else
+			Serial--;
 		return(False);
 	}
 
 		/* lock undo module to prevent from loops
 		 * and loop over all entries with the same serial number
 		 */
-	LockUndo();
 	ptr = &UndoList[UndoN -1];
+	if (ptr->Serial != Serial -1)
+	{
+		Serial--;
+		return(False);
+	}
+	LockUndo();
 	Serial = ptr->Serial;
 	for (; UndoN && ptr->Serial == Serial; ptr--, UndoN--, RedoN++)
-		switch(ptr->Type)
-		{
-			case UNDO_CHANGENAME:
-				changed |= UndoChangeName(ptr);
-				break;
-
-			case UNDO_CREATE:
-			case UNDO_COPY:
-				changed |= UndoCopyOrCreate(ptr);
-				break;
-
-			case UNDO_MOVE:
-				changed |= UndoMove(ptr);
-				break;
-
-			case UNDO_REMOVE:
-				changed |= UndoRemove(ptr);
-				break;
-
-			case UNDO_REMOVE_POINT:
-				changed |= UndoRemovePoint(ptr);
-				break;
-
-			case UNDO_INSERT_POINT:
-				changed |= UndoInsertPoint(ptr);
-				break;
-
-			case UNDO_ROTATE:
-				changed |= UndoRotate(ptr);
-				break;
-
-			case UNDO_MOVETOLAYER:
-				changed |= UndoMoveToLayer(ptr);
-				break;
-
-			case UNDO_FLAG:
-				changed |= UndoFlag(ptr);
-				break;
-
-			case UNDO_CHANGESIZE:
-				changed |= UndoChangeSize(ptr);
-				break;
-
-			case UNDO_CHANGE2NDSIZE:
-				changed |= UndoChange2ndSize(ptr);
-				break;
-
-			case UNDO_BLOCK:
-				UndoBlock(ptr);
-		}
+		Types |= PerformUndo(ptr);
 		/* release lock */
 	UnlockUndo();
-	if (changed)
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	return(changed);
+	if (Types)
+		Draw();
+	return(Types);
+}
+
+static int PerformUndo(UndoListTypePtr ptr)
+{
+	switch(ptr->Type)
+	{
+		case UNDO_CHANGENAME:
+			if (UndoChangeName(ptr))
+				return(UNDO_CHANGENAME);
+			break;
+
+		case UNDO_CREATE:
+			if (UndoCopyOrCreate(ptr))
+				return(UNDO_CREATE);
+			break;
+
+		case UNDO_MOVE:
+			if (UndoMove(ptr))
+				return(UNDO_MOVE);
+			break;
+
+		case UNDO_REMOVE:
+			if (UndoRemove(ptr))
+				return(UNDO_REMOVE);
+			break;
+
+		case UNDO_REMOVE_POINT:
+			if (UndoRemovePoint(ptr))
+				return(UNDO_REMOVE_POINT);
+			break;
+
+		case UNDO_INSERT_POINT:
+			if (UndoInsertPoint(ptr))
+				return(UNDO_INSERT_POINT);
+			break;
+
+		case UNDO_ROTATE:
+			if (UndoRotate(ptr))
+				return(UNDO_ROTATE);
+			break;
+
+		case UNDO_MOVETOLAYER:
+			if (UndoMoveToLayer(ptr))
+				return(UNDO_MOVETOLAYER);
+			break;
+
+		case UNDO_FLAG:
+			if (UndoFlag(ptr))
+				return(UNDO_FLAG);
+			break;
+
+		case UNDO_CHANGESIZE:
+			if (UndoChangeSize(ptr))
+				return(UNDO_CHANGESIZE);
+			break;
+
+		case UNDO_CHANGECLEARSIZE:
+			if (UndoChangeClearSize(ptr))
+				return(UNDO_CHANGECLEARSIZE);
+			break;
+
+		case UNDO_CHANGE2NDSIZE:
+			if (UndoChange2ndSize(ptr))
+				return(UNDO_CHANGE2NDSIZE);
+			break;
+
+		case UNDO_MIRROR:
+			if (UndoMirror(ptr))
+				return(UNDO_MIRROR);
+			break;
+	}
+	return 0;
 }
 
 /* ---------------------------------------------------------------------------
  * redo of any 'hard to recover' operation
  *
- * returns True if anything is has been done
+ * returns the number of operations redone
  */
-Boolean Redo(void)
+int Redo(void)
 {
 	UndoListTypePtr	ptr;
-	Boolean			changed = False;
+	int	Types = 0;
 
 	if (!RedoN)
 	{
@@ -683,63 +781,13 @@ Boolean Redo(void)
 	ptr = &UndoList[UndoN];
 	Serial = ptr->Serial;
 	for (; RedoN && ptr->Serial == Serial; ptr++, UndoN++, RedoN--)
-		switch(ptr->Type)
-		{
-			case UNDO_CHANGENAME:
-				changed |= UndoChangeName(ptr);
-				break;
-
-			case UNDO_CREATE:
-			case UNDO_COPY:
-				changed |= UndoCopyOrCreate(ptr);
-				break;
-
-			case UNDO_MOVE:
-				changed |= UndoMove(ptr);
-				break;
-
-			case UNDO_REMOVE:
-				changed |= UndoRemove(ptr);
-				break;
-
-			case UNDO_REMOVE_POINT:
-				changed |= UndoRemovePoint(ptr);
-				break;
-
-			case UNDO_INSERT_POINT:
-				changed |= UndoInsertPoint(ptr);
-				break;
-
-			case UNDO_ROTATE:
-				changed |= UndoRotate(ptr);
-				break;
-
-			case UNDO_MOVETOLAYER:
-				changed |= UndoMoveToLayer(ptr);
-				break;
-
-			case UNDO_FLAG:
-				changed |= UndoFlag(ptr);
-				break;
-
-			case UNDO_CHANGESIZE:
-				changed |= UndoChangeSize(ptr);
-				break;
-
-			case UNDO_CHANGE2NDSIZE:
-				changed |= UndoChange2ndSize(ptr);
-				break;
-
-			case UNDO_BLOCK:
-				UndoBlock(ptr);
-				break;
-		}
+		Types |= PerformUndo(ptr);
 	/* Make next serial number current in case we take a new branch */
 	Serial++;
 	UnlockUndo();
-	if (changed)
-	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
-	return(changed);
+	if (Types)
+		Draw();
+	return(Types);
 }
 
 /* ---------------------------------------------------------------------------
@@ -747,7 +795,7 @@ Boolean Redo(void)
  */
 void RestoreUndoSerialNumber(void)
 {
-		Serial = SavedSerial;
+	Serial = SavedSerial;
 }
 
 /* ---------------------------------------------------------------------------
@@ -755,6 +803,7 @@ void RestoreUndoSerialNumber(void)
  */
 void SaveUndoSerialNumber(void)
 {
+	Bumped = False;
 	SavedSerial = Serial;
 }
 
@@ -766,7 +815,11 @@ void SaveUndoSerialNumber(void)
 void IncrementUndoSerialNumber(void)
 {
 	if (!Locked)
+	{
 		Serial++;
+		Bumped = True;
+		SetChangedFlag(True);
+	}
 }
 
 /* ---------------------------------------------------------------------------
@@ -783,8 +836,6 @@ void ClearUndoList(Boolean Force)
 		{
 			if (undo->Type == UNDO_CHANGENAME)
 				SaveFree(undo->Data.ChangeName.Name);
-			if (undo->Type == UNDO_BLOCK)
-				SaveFree(undo->Data.Block.Box);
 		}
 		MyFree((char **) &UndoList);
 		FreeDataMemory(RemoveList);
@@ -798,20 +849,19 @@ void ClearUndoList(Boolean Force)
 }
 
 /* ---------------------------------------------------------------------------
- * marks the block and box boundary for block drawing during undo
+ * adds an object to the list of mirrored objects
  */
-void AddBlockToUndoList(BoxTypePtr Box)
+void AddObjectToMirrorUndoList(int Type, void *Ptr1, void *Ptr2, void *Ptr3, Position yoff)
 {
-	UndoListTypePtr undo;
+	UndoListTypePtr	undo;
 
 	if (!Locked)
 	{
-		undo = GetUndoSlot(UNDO_BLOCK, 0);
-		undo->Data.Block.Box = NULL;
-		if (Box && (undo->Data.Block.Box = malloc(sizeof(BoxType))))
-			*undo->Data.Block.Box = *Box;
+		undo = GetUndoSlot(UNDO_MIRROR, OBJECT_ID(Ptr3));
+		undo->Data.Move.DY = yoff;
 	}
 }
+
 /* ---------------------------------------------------------------------------
  * adds an object to the list of rotated objects
  */
@@ -866,7 +916,9 @@ void AddObjectToRemovePointUndoList(int Type,
 				 * impossible to recover the point
 				 */
 				undo = GetUndoSlot(UNDO_REMOVE_POINT, OBJECT_ID(polygon));
-				undo->Data.RemovedPoint.Point = polygon->Points[index];
+				undo->Data.RemovedPoint.X = polygon->Points[index].X;
+				undo->Data.RemovedPoint.Y = polygon->Points[index].Y;
+				undo->Data.RemovedPoint.ID = polygon->Points[index].ID;
 				undo->Data.RemovedPoint.Index = index;
 			}
 			break;
@@ -886,17 +938,6 @@ void AddObjectToInsertPointUndoList(int Type,
 		undo = GetUndoSlot(UNDO_INSERT_POINT, OBJECT_ID(Ptr3));
 }
 
-/* ---------------------------------------------------------------------------
- * adds an object to the list of copied objects
- */
-void AddObjectToCopyUndoList(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
-{
-	UndoListTypePtr	undo;
-
-	if (!Locked)
-		undo = GetUndoSlot(UNDO_COPY, OBJECT_ID(Ptr3));
-}
-
 /* ---------------------------------------------------------------------------
  * adds an object to the list of moved objects
  */
@@ -1003,6 +1044,35 @@ void AddObjectToSizeUndoList(int Type, void *ptr1, void *ptr2, void *ptr3)
 	}
 }
 
+/* ---------------------------------------------------------------------------
+ * adds an object to the list of objects with Size changes
+ */
+void AddObjectToClearSizeUndoList(int Type, void *ptr1, void *ptr2, void *ptr3)
+{
+	UndoListTypePtr undo;
+
+	if (!Locked)
+	{
+		undo = GetUndoSlot(UNDO_CHANGECLEARSIZE, OBJECT_ID(ptr2));
+		switch (Type)
+		{
+			case PIN_TYPE:
+			case VIA_TYPE:
+				undo->Data.Size = ((PinTypePtr) ptr2)->Clearance;
+				break;
+			case LINE_TYPE:
+				undo->Data.Size = ((LineTypePtr) ptr2)->Clearance;
+				break;
+			case PAD_TYPE:
+				undo->Data.Size = ((PadTypePtr) ptr2)->Clearance;
+				break;
+			case ARC_TYPE:
+				undo->Data.Size = ((ArcTypePtr) ptr2)->Clearance;
+				break;
+		}
+	}
+}
+
 /* ---------------------------------------------------------------------------
  * adds an object to the list of objects with 2ndSize changes
  */
@@ -1018,22 +1088,6 @@ void AddObjectTo2ndSizeUndoList(int Type, void *ptr1, void *ptr2, void *ptr3)
 	}
 }
 
-/* ---------------------------------------------------------------------------
- * aborts a false block start
- */
-void UndoFalseBlock(void)
-{
-	UndoListTypePtr undo = &UndoList[UndoN - 1];
-
-	if (!UndoN || undo->Type != UNDO_BLOCK)
-		Message("hace: Internal error in UndoFalseBlock\n");
-	else
-	{
-		SaveFree(undo->Data.Block.Box);
-		UndoN--;
-	}
-}
-
 /* ---------------------------------------------------------------------------
  * set lock flag
  */
@@ -1049,3 +1103,11 @@ void UnlockUndo(void)
 {
 	Locked = False;
 }
+
+/* ---------------------------------------------------------------------------
+ * return undo lock state
+ */
+Boolean Undoing(void)
+{
+	return(Locked);
+}
