@@ -33,31 +33,13 @@
 
 #include "global.h"
 
-/* ---------------------------------------------------------------------------
- * define several commands which are supprted for undo
- * note there is ABSOLUTELY NO REASON FOR THESE TO OCCUPY SEPARATE BITS
- * it might as well go 1, 2, 3, 4, 5, 6, etc
- */
-#define	UNDO_CHANGENAME				0x0001	/* change of names */
-#define	UNDO_COPY					0x0002	/* copy objects */
-#define	UNDO_MOVE					0x0004	/* moving objects */
-#define	UNDO_REMOVE					0x0008	/* removing objects */
-#define	UNDO_REMOVE_POINT			0x0010	/* removing polygon/... points */
-#define	UNDO_INSERT_POINT			0x0020	/* inserting polygon/... points */
-#define	UNDO_ROTATE					0x0040	/* rotations */
-#define	UNDO_CREATE					0x0080	/* creation of objects */
-#define	UNDO_MOVETOLAYER			0x0100	/* moving objects to */
-#define UNDO_FLAG                             0x0200  /* toggling SELECTED flag */
-#define UNDO_CHANGESIZE                               0x0400  /* change size of object */
-#define UNDO_CHANGE2NDSIZE                    0x0800  /* change 2ndSize of object */
-#define UNDO_BLOCK			0x1000	/* marks a block drawing operation */
 #define DRAW_FLAGS	(RATFLAG | SELECTEDFLAG | SQUAREFLAG | ALLTHERMFLAGS \
-			| HIDENAMEFLAG | HOLEFLAG | OCTAGONFLAG)
+			| HIDENAMEFLAG | HOLEFLAG | OCTAGONFLAG | FOUNDFLAG | CLEARLINEFLAG)
 
 											/* different layers */
 
-Boolean	Undo(void);
-Boolean Redo(void);
+int	Undo(void);
+int	Redo(void);
 void	IncrementUndoSerialNumber(void);
 void	SaveUndoSerialNumber(void);
 void	RestoreUndoSerialNumber(void);
@@ -65,20 +47,20 @@ void	ClearUndoList(Boolean);
 void	MoveObjectToRemoveUndoList(int, void *, void *, void *);
 void	AddObjectToRemovePointUndoList(int, void *, void *, Cardinal);
 void	AddObjectToInsertPointUndoList(int, void *, void *, void *);
-void	AddObjectToCopyUndoList(int, void *, void *, void *);
 void	AddObjectToMoveUndoList(int, void *, void *, void *,
 			Position, Position);
 void	AddObjectToChangeNameUndoList(int, void *,void *, void *, char *);
 void	AddObjectToRotateUndoList(int, void *, void *,void *,
 			Position, Position, BYTE);
 void	AddObjectToCreateUndoList(int, void *, void *,void *);
+void	AddObjectToMirrorUndoList(int, void *, void *,void *, Position);
 void	AddObjectToMoveToLayerUndoList(int, void *, void *, void *);
 void	AddObjectToFlagUndoList(int, void *, void *, void *);
 void	AddObjectToSizeUndoList(int, void *, void *, void *);
 void	AddObjectTo2ndSizeUndoList(int, void *, void *, void *);
-void	AddBlockToUndoList(BoxTypePtr);
-void	UndoFalseBlock(void);
+void	AddObjectToClearSizeUndoList(int, void *, void *, void *);
 void	LockUndo(void);
 void	UnlockUndo(void);
+Boolean	Undoing(void);
  
 #endif
